@@ -1,20 +1,7 @@
 #  Copyright 2013 Linaro Limited
 #  Author Neil Williams <neil.williams@linaro.org>
 #
-#  This program is free software; you can redistribute it and/or modify
-#  it under the terms of the GNU General Public License as published by
-#  the Free Software Foundation; either version 2 of the License, or
-#  (at your option) any later version.
-#
-#  This program is distributed in the hope that it will be useful,
-#  but WITHOUT ANY WARRANTY; without even the implied warranty of
-#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-#  GNU General Public License for more details.
-#
-#  You should have received a copy of the GNU General Public License
-#  along with this program; if not, write to the Free Software
-#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-#  MA 02110-1301, USA.
+# SPDX-License-Identifier: GPL-2.0-or-later
 
 import json
 import logging
@@ -25,7 +12,6 @@ LOG = logging.getLogger("lava-coordinator")
 
 
 class LavaCoordinator:
-
     running = False
     delay = 1
     rpc_delay = 2
@@ -61,7 +47,7 @@ class LavaCoordinator:
                 LOG.info("[BTSP] binding to %s:%s", "0.0.0.0", self.group_port)
                 s.bind(("0.0.0.0", self.group_port))
                 break
-            except socket.error as e:
+            except OSError as e:
                 LOG.warning(
                     "[BTSP] Unable to bind, trying again with delay=%d msg=%s",
                     self.delay,
@@ -452,7 +438,7 @@ class LavaCoordinator:
         messageID = self._getMessageID(json_data)
         if "waitrole" in json_data:
             expected = self.group["roles"][json_data["waitrole"]]
-            expected = expected[0] if type(expected) == list else None
+            expected = expected[0] if isinstance(expected, list) else None
             LOG.debug(
                 "lavaWaitAll waiting for role:%s from %s",
                 json_data["waitrole"],
