@@ -7,20 +7,7 @@ Status check for lava-coordinator
 #  Copyright 2015 Linaro Limited
 #  Author Neil Williams <neil.williams@linaro.org>
 #
-#  This program is free software; you can redistribute it and/or modify
-#  it under the terms of the GNU General Public License as published by
-#  the Free Software Foundation; either version 2 of the License, or
-#  (at your option) any later version.
-#
-#  This program is distributed in the hope that it will be useful,
-#  but WITHOUT ANY WARRANTY; without even the implied warranty of
-#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-#  GNU General Public License for more details.
-#
-#  You should have received a copy of the GNU General Public License
-#  along with this program; if not, write to the Free Software
-#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-#  MA 02110-1301, USA.
+# SPDX-License-Identifier: GPL-2.0-or-later
 
 
 import errno
@@ -74,7 +61,7 @@ def lava_poll(port, host, name, request):
                 sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM, 0)
                 sock.connect((host, port))
                 break
-            except socket.error as exc:
+            except OSError as exc:
                 if exc.errno == errno.ECONNRESET:
                     warnings.append("connection reset by peer: bug 1020")
                 errors.append("not connected, sleeping for 1 second")
@@ -105,14 +92,14 @@ def lava_poll(port, host, name, request):
             if ret_bytes == 0:
                 warnings.append("zero bytes sent for message - connection closed?")
                 continue
-        except socket.error as exc:
+        except OSError as exc:
             errors.append("socket error '%d' on send" % exc.message)
             sock.close()
             continue
         try:
             data = str(sock.recv(8))  # 32bit limit
             data = sock.recv(1024)
-        except socket.error as exc:
+        except OSError as exc:
             errors.append("Exception on receive: %s" % exc)
             continue
         try:
