@@ -2,21 +2,7 @@
 #
 # Author: Neil Williams <neil.williams@linaro.org>
 #
-# This file is part of LAVA.
-#
-# LAVA is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# LAVA is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along
-# with this program; if not, see <http://www.gnu.org/licenses>.
+# SPDX-License-Identifier: GPL-2.0-or-later
 
 from lava_common.decorators import nottest
 
@@ -132,3 +118,8 @@ class ObjectNotPersisted(LAVAServerError):
 class PermissionNameError(LAVAServerError):
     error_help = "PermissionNameError: Unexisting permission codename."
     error_type = "Unexisting permission codename."
+
+
+class RequestBodyTooLargeError(LAVAError):
+    error_help = "RequestBodyTooLarge: Request body exceeds server settings param."
+    error_type = "RequestBodyTooLarge"
