@@ -3,50 +3,38 @@
 # Author: Senthil Kumaran S <senthil.kumaran@linaro.org>
 #         Remi Duraffort <remi.duraffort@linaro.org>
 #
-# This file is part of LAVA Dispatcher.
-#
-# LAVA Dispatcher is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# LAVA Dispatcher is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along
-# with this program; if not, see <http://www.gnu.org/licenses>.
+# SPDX-License-Identifier: GPL-2.0-or-later
 
 import contextlib
 import datetime
 import logging
 import multiprocessing
+import os
 import signal
+import sys
 import time
-from typing import Dict, List, Tuple
 
 import requests
 
-from lava_common.compat import yaml_dump
+from lava_common.exceptions import RequestBodyTooLargeError
 from lava_common.version import __version__
+from lava_common.yaml import yaml_safe_dump
 
 
-def dump(data: Dict) -> str:
+def dump(data: dict) -> str:
     # Set width to a really large value in order to always get one line.
     # But keep this reasonable because the logs will be loaded by CLoader
     # that is limited to around 10**7 chars
-    data_str = yaml_dump(
-        data, default_flow_style=True, default_style='"', width=10**6
+    data_str = yaml_safe_dump(
+        data, default_flow_style=True, default_style='"', width=10**5
     )[:-1]
     # Test the limit and skip if the line is too long
-    if len(data_str) >= 10**6:
+    if len(data_str) >= 10**5:
         if isinstance(data["msg"], str):
             data["msg"] = "<line way too long ...>"
         else:
             data["msg"] = {"skip": "line way too long ..."}
-        data_str = yaml_dump(
+        data_str = yaml_safe_dump(
             data, default_flow_style=True, default_style='"', width=10**6
         )[:-1]
     return data_str
@@ -55,8 +43,15 @@ def dump(data: Dict) -> str:
 def sender(conn, url: str, token: str, max_time: int) -> None:
     HEADERS = {"User-Agent": f"lava {__version__}", "LAVA-Token": token}
     MAX_RECORDS = 1000
+    FAILURE_SLEEP = 5
+    # Record the exception to prevent spamming
+    last_exception_type = None
+    exception_counter = 0
+
+    def post(session, records: list[str], index: int) -> tuple[list[str], int]:
+        nonlocal last_exception_type
+        nonlocal exception_counter
 
-    def post(session, records: List[str], index: int) -> Tuple[List[str], int]:
         # limit the number of records to send in one call
         data, remaining = records[:MAX_RECORDS], records[MAX_RECORDS:]
         with contextlib.suppress(requests.RequestException):
@@ -64,27 +59,96 @@ def sender(conn, url: str, token: str, max_time: int) -> None:
             # background process so waiting is not an issue.
             # Will avoid resending the same request a second time if gunicorn
             # is too slow to answer.
-            ret = session.post(
-                url,
-                data={"lines": "- " + "\n- ".join(data), "index": index},
-                headers=HEADERS,
-            )
+            # In case of exception, print the exception to stderr that will be
+            # forwarded to lava-server by lava-worker. If the same exception is
+            # raised multiple time in a row, record also the number of
+            # occurrences.
+            try:
+                ret = session.post(
+                    url,
+                    data={"lines": "- " + "\n- ".join(data), "index": index},
+                    headers=HEADERS,
+                )
+                if exception_counter > 0:
+                    now = datetime.datetime.utcnow().isoformat()
+                    sys.stderr.write(f"{now}: <{exception_counter} skipped>\n")
+                    last_exception_type = None
+                    exception_counter = 0
+            except Exception as exc:
+                if last_exception_type == type(exc):
+                    exception_counter += 1
+                else:
+                    now = datetime.datetime.utcnow().isoformat()
+                    if exception_counter:
+                        sys.stderr.write(f"{now}: <{exception_counter} skipped>\n")
+                    sys.stderr.write(f"{now}: {str(exc)}\n")
+                    last_exception_type = type(exc)
+                    exception_counter = 0
+                    sys.stderr.flush()
+
+                # Empty response for the rest of the code
+                ret = requests.models.Response()
 
             if ret.status_code == 200:
                 with contextlib.suppress(KeyError, ValueError):
                     count = int(ret.json()["line_count"])
                     data = data[count:]
                     index += count
+            elif ret.status_code == 404:
+                data, remaining = [], []
+                os.kill(os.getppid(), signal.SIGTERM)
+            else:
+                if ret.status_code == 413:
+                    raise RequestBodyTooLargeError
+                # If the request fails, give some time for the server to
+                # recover from the failure.
+                time.sleep(FAILURE_SLEEP)
         return (data + remaining, index)
 
     last_call = time.monotonic()
-    records: List[str] = []
+    records: list[str] = []
     leaving: bool = False
     index: int = 0
 
+    def _reduce_record_size(
+        records: list[str], max_records: int
+    ) -> tuple[list[str], int]:
+        """
+        The method should only be called for handling `RequestBodyTooLargeError`. It
+        minus 100 records for every call. In case only one record left, the record will
+        be replaced by a short "log-upload fail" result line and an error message also
+        will be sent to kill the job.
+        """
+        if max_records == 1:
+            record = records[0]
+            records = [
+                dump(
+                    {
+                        "dt": datetime.datetime.utcnow().isoformat(),
+                        "lvl": "results",
+                        "msg": {
+                            "definition": "lava",
+                            "case": "log-upload",
+                            "result": "fail",
+                        },
+                    }
+                )
+            ]
+
+            sys.stderr.write(
+                "Error: Log post request body exceeds server settings param.\n"
+                f"Log line length: {len(record)}\n"
+                f"Truncated log line: {record[:1024]} ...\n"
+            )
+            sys.stderr.flush()
+        else:
+            max_records = max(1, max_records - 100)
+
+        return (records, max_records)
+
     with requests.Session() as session:
         while not leaving:
-            # Listen for new messages if we don't have  message yet or some
+            # Listen for new messages if we don't have message yet or some
             # messages are already in the socket.
             if len(records) == 0 or conn.poll(max_time):
                 data = conn.recv_bytes()
@@ -98,11 +162,17 @@ def sender(conn, url: str, token: str, max_time: int) -> None:
             if records and (records_limit or time_limit):
                 last_call = time.monotonic()
                 # Send the data
-                (records, index) = post(session, records, index)
+                try:
+                    (records, index) = post(session, records, index)
+                except RequestBodyTooLargeError:
+                    (records, MAX_RECORDS) = _reduce_record_size(records, MAX_RECORDS)
 
         while records:
             # Send the data
-            (records, index) = post(session, records, index)
+            try:
+                (records, index) = post(session, records, index)
+            except RequestBodyTooLargeError:
+                (records, MAX_RECORDS) = _reduce_record_size(records, MAX_RECORDS)
 
 
 class HTTPHandler(logging.Handler):
