@@ -1,24 +1,9 @@
-# -*- coding: utf-8 -*-
 #
 # Copyright (C) 2018 Linaro Limited
 #
 # Author: Rémi Duraffort <remi.duraffort@linaro.org>
 #
-# This file is part of LAVA.
-#
-# LAVA is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# LAVA is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along
-# with this program; if not, see <http://www.gnu.org/licenses>.
+# SPDX-License-Identifier: GPL-2.0-or-later
 
 from voluptuous import Any, Optional, Required
 
@@ -37,6 +22,7 @@ def url(extra=None):
         Optional("md5sum"): str,
         Optional("sha256sum"): str,
         Optional("sha512sum"): str,
+        Optional("use_cache"): bool,
     }
     return Any(
         {**base_url, **extra},
