@@ -1,30 +1,20 @@
-# -*- coding: utf-8 -*-
 #
 # Copyright (C) 2020-present Linaro Limited
 #
 # Author: Remi Duraffort <remi.duraffort@linaro.org>
 #
-# This file is part of LAVA.
-#
-# LAVA is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# LAVA is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, see <http://www.gnu.org/licenses>.
+# SPDX-License-Identifier: GPL-2.0-or-later
 
 import argparse
 import re
 import socket
 from pathlib import Path
+from typing import NoReturn
+
+import sentry_sdk
 
 from lava_common.constants import DOCKER_WORKER_DIR, WORKER_DIR
+from lava_common.version import __version__
 
 
 def get_fqdn() -> str:
@@ -38,6 +28,16 @@ def get_fqdn() -> str:
         raise ValueError("Your FQDN contains invalid characters")
 
 
+def parse_mount(s: str) -> tuple:
+    # Split at ':' and accept one, two or three parameters
+    src, dst, opts, *extra = *s.split(":"), *(None, None)
+    if not src or len(extra) > 2:
+        raise argparse.ArgumentTypeError(
+            "mount should have 1, 2 or 3 parts (separated by ':')"
+        )
+    return (src, dst, opts)
+
+
 def get_parser(docker_worker=False) -> argparse.ArgumentParser:
     if docker_worker:
         description = "LAVA Docker Worker"
@@ -62,6 +62,15 @@ def get_parser(docker_worker=False) -> argparse.ArgumentParser:
             default=False,
             help="Development mode; sets defaults to several options.",
         )
+        parser.add_argument(
+            "-m",
+            "--mount",
+            metavar="SRC[:DST]",
+            type=parse_mount,
+            nargs="*",
+            action="extend",
+            help="Bind mount SRC from the host (as DST in the container, if given). Can be given multiple times",
+        )
     else:
         parser.add_argument(
             "--debug", action="store_true", default=False, help="Debug lava-run"
@@ -88,6 +97,12 @@ def get_parser(docker_worker=False) -> argparse.ArgumentParser:
             default="/etc/lava-dispatcher-host/build",
             help="Path to a directory with a Dockerfile inside for building customized lava-dispatcher docker image.",
         )
+        storage.add_argument(
+            "--use-cache",
+            action="store_true",
+            default=False,
+            help="Use cache when building custom docker worker image.",
+        )
 
     net = parser.add_argument_group("network")
     net.add_argument("--url", required=url_required, help="Base URL of the server")
@@ -134,5 +149,12 @@ def get_parser(docker_worker=False) -> argparse.ArgumentParser:
         choices=["DEBUG", "ERROR", "INFO", "WARN"],
         help="Log level, default to INFO",
     )
+    parser.add_argument(
+        "--sentry-dsn", type=str, default=None, help="Sentry Data Source Name"
+    )
 
     return parser
+
+
+def init_sentry_sdk(dsn: str) -> NoReturn:
+    sentry_sdk.init(dsn=dsn, release=f"lava@{__version__}", traces_sample_rate=1.0)
