@@ -2,32 +2,22 @@
 #
 # Author: Neil Williams <neil.williams@linaro.org>
 #
-# This file is part of LAVA Dispatcher.
-#
-# LAVA Dispatcher is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# LAVA Dispatcher is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along
-# with this program; if not, see <http://www.gnu.org/licenses>.
+# SPDX-License-Identifier: GPL-2.0-or-later
+from __future__ import annotations
 
+import contextlib
 import copy
 import logging
-import shlex
 import subprocess  # nosec - internal
 import time
 import traceback
 import warnings
 from functools import reduce
+from shlex import split as shlex_split
+from typing import TYPE_CHECKING
 
 import pexpect
+from pexpect.popen_spawn import PopenSpawn as PexpectPopenSpawn
 
 from lava_common.decorators import nottest
 from lava_common.exceptions import (
@@ -42,6 +32,13 @@ from lava_common.log import YAMLLogger
 from lava_common.timeout import Timeout
 from lava_dispatcher.utils.strings import seconds_to_str
 
+if TYPE_CHECKING:
+    from typing import Iterator, Optional, TypeVar
+
+    from .job import Job
+
+    TAction = TypeVar("TAction", bound="Action")
+
 
 class InternalObject:
     """
@@ -52,6 +49,33 @@ class InternalObject:
     pass
 
 
+_NOT_FOUND = object()
+
+
+def get_lastest_dict_value(list_of_dictionaries: list[dict], key: str):
+    for d in reversed(list_of_dictionaries):
+        value = d.get(key, _NOT_FOUND)
+        if value is not _NOT_FOUND:
+            return value
+
+    return None
+
+
+def get_lastest_subdict_value(
+    list_of_dictionaries: list[dict[str, dict]], key: str, subkey: str
+):
+    for d in reversed(list_of_dictionaries):
+        sub_dict = d.get(key, _NOT_FOUND)
+        if sub_dict is _NOT_FOUND:
+            continue
+
+        value = sub_dict.get(subkey, _NOT_FOUND)
+        if value is not _NOT_FOUND:
+            return value
+
+    return None
+
+
 class Pipeline:
     """
     Pipelines ensure that actions are run in the correct sequence whilst
@@ -61,8 +85,8 @@ class Pipeline:
     of the per-action log handler.
     """
 
-    def __init__(self, parent=None, job=None, parameters=None):
-        self.actions = []
+    def __init__(self, job: Job, parent=None, parameters=None):
+        self.actions: list[Action] = []
         self.parent = None
         self.parameters = {} if parameters is None else parameters
         self.job = job
@@ -84,12 +108,10 @@ class Pipeline:
         # if isinstance(action, DiagnosticAction):
         #     raise LAVABug("Diagnostic actions need to be triggered, not added to a pipeline.")
 
-    def add_action(self, action, parameters=None):
+    def add_action(self, action: Action, parameters=None):
         self._check_action(action)
         self.actions.append(action)
-        # FIXME: if this is only happening in unit test, this has to be fixed later on
-        # should only be None inside the unit tests
-        action.job = self.job
+
         if self.parent:  # action
             self.parent.pipeline = self
             action.level = "%s.%s" % (self.parent.level, len(self.actions))
@@ -105,47 +127,34 @@ class Pipeline:
         action.populate(parameters)
 
         # Compute the timeout
-        timeouts = []
-        # FIXME: Only needed for the auto-tests
-        if self.job is not None:
-            if self.job.device is not None:
-                # First, the device level overrides
-                timeouts.append(self.job.device.get("timeouts", {}))
-            # Then job level overrides
-            timeouts.append(self.job.parameters.get("timeouts", {}))
-
-        def dict_merge_get(dicts, key):
-            value = None
-            for d in dicts:
-                value = d.get(key, value)
-            return value
-
-        def subdict_merge_get(dicts, key, subkey):
-            value = None
-            for d in dicts:
-                value = d.get(key, {}).get(subkey, value)
-            return value
+        global_timeouts = []
+        # First, the device level overrides
+        global_timeouts.append(self.job.device.get("timeouts", {}))
+        # Then job level overrides
+        global_timeouts.append(self.job.parameters.get("timeouts", {}))
 
         # Set the timeout. The order is:
-        # 1/ the global action timeout
-        # 2/ the individual action timeout
-        # 3/ the action block timeout
-        # 4/ the individual action block timeout
-
-        action._override_action_timeout(dict_merge_get(timeouts, "action"))
+        # 1. global action timeout
         action._override_action_timeout(
-            subdict_merge_get(timeouts, "actions", action.name)
+            get_lastest_dict_value(global_timeouts, "action")
         )
+        # 2. global named action timeout
+        action._override_action_timeout(
+            get_lastest_subdict_value(global_timeouts, "actions", action.name)
+        )
+        # 3. action block timeout
         action._override_action_timeout(parameters.get("timeout"))
-
-        if parameters.get("timeouts"):
+        # 4. action block named action timeout
+        if action_block_timeouts := parameters.get("timeouts"):
             action._override_action_timeout(
-                dict_merge_get([parameters.get("timeouts")], action.name)
+                action_block_timeouts.get(action.name),
             )
 
-        action._override_connection_timeout(dict_merge_get(timeouts, "connection"))
         action._override_connection_timeout(
-            subdict_merge_get(timeouts, "connections", action.name)
+            get_lastest_dict_value(global_timeouts, "connection")
+        )
+        action._override_connection_timeout(
+            get_lastest_subdict_value(global_timeouts, "connections", action.name)
         )
 
         action.parameters = parameters
@@ -158,9 +167,8 @@ class Pipeline:
         """
         desc = []
         for action in self.actions:
-            cls = str(type(action))[8:-2].replace("lava_dispatcher.", "")
             current = {
-                "class": cls,
+                "class": type(action).__name__,
                 "name": action.name,
                 "level": action.level,
                 "max_retries": action.max_retries,
@@ -195,7 +203,7 @@ class Pipeline:
         if self.parent is None and self.errors:
             raise JobError("Invalid job data: %s\n" % self.errors)
 
-    def cleanup(self, connection):
+    def cleanup(self, connection, max_end_time=None):
         """
         Recurse through internal pipelines running action.cleanup(),
         in order of the pipeline levels.
@@ -203,7 +211,10 @@ class Pipeline:
         error = False
         for child in self.actions:
             try:
-                child.cleanup(connection)
+                if max_end_time is not None and child.name == "finalize":
+                    child.cleanup(connection, max_end_time)
+                else:
+                    child.cleanup(connection)
             except Exception as exc:
                 # Just log the exception and continue the cleanup
                 child.logger.error(
@@ -214,29 +225,6 @@ class Pipeline:
         if error:
             raise InfrastructureError("Failed to clean after job")
 
-    def _diagnose(self, connection):
-        """
-        Pipeline Jobs have a number of Diagnostic classes registered - all
-        supported DiagnosticAction classes should be registered with the Job.
-        If an Action.run() function reports a JobError or InfrastructureError,
-        the Pipeline calls Job.diagnose(). The job iterates through the DiagnosticAction
-        classes declared in the diagnostics list, checking if the trigger classmethod
-        matches the requested complaint. Matching diagnostics are run using the current Connection.
-        Actions generate a complaint by appending the return of the trigger classmethod
-        to the triggers list of the Job. This can be done at any point prior to the
-        exception being raised in the run function.
-        The trigger list is cleared after each diagnostic operation is complete.
-        """
-        for complaint in self.job.triggers:
-            diagnose = self.job.diagnose(complaint)
-            if diagnose:
-                connection = diagnose.run(connection, None)
-            else:
-                raise LAVABug("No diagnosis for trigger %s" % complaint)
-        self.job.triggers = []
-        # Diagnosis is not allowed to alter the connection, do not use the return value.
-        return None
-
     def run_actions(self, connection, max_end_time):
         for action in self.actions:
             failed = False
@@ -281,7 +269,6 @@ class Pipeline:
                 # allows retries without setting errors, which make the job incomplete.
                 failed = True
                 action.results = {"fail": str(exc)}
-                self._diagnose(connection)
                 raise
             except Exception as exc:
                 action.logger.exception(traceback.format_exc())
@@ -310,6 +297,28 @@ class Pipeline:
                 connection = new_connection
         return connection
 
+    def _iter_actions(self) -> Iterator[Action]:
+        for action in self.actions:
+            yield action
+            if action.pipeline is not None:
+                yield from action.pipeline._iter_actions()
+
+    def find_action(self, action_class: type[TAction]) -> TAction:
+        for action in self._iter_actions():
+            if isinstance(action, action_class):
+                return action
+
+        raise LookupError(f"Action of type {action_class} not found")
+
+    def find_all_actions(self, action_class: type[TAction]) -> list[TAction]:
+        all_actions: list[TAction] = []
+
+        for action in self._iter_actions():
+            if isinstance(action, action_class):
+                all_actions.append(action)
+
+        return all_actions
+
 
 class CommandLogger:
     """
@@ -321,6 +330,8 @@ class CommandLogger:
         self.logger = logger
 
     def write(self, new_line):
+        if not isinstance(new_line, str):
+            new_line = str(new_line)
         lines = self.line + new_line
 
         # Print one full line at a time. A partial line is kept in memory.
@@ -339,7 +350,7 @@ class CommandLogger:
 
 
 class Action:
-    def __init__(self):
+    def __init__(self, job: Job):
         """
         Actions get added to pipelines by calling the
         Pipeline.add_action function. Other Action
@@ -355,20 +366,23 @@ class Action:
         # First command in pipeline within outer pipeline: 1.1
         # Level is set during pipeline creation and must not be changed
         # subsequently except by RetryCommand.
+        self.job = job
         self.level = None
-        self.pipeline = None
+        self.pipeline: Optional[Pipeline] = None
         self.__parameters__ = {}
         self.__errors__ = []
-        self.job = None
         self.logger = logging.getLogger("dispatcher")
         self.__results__ = {}
-        self.timeout = Timeout(self.name, exception=self.timeout_exception)
+        self.timeout: Timeout = Timeout(
+            self.name, self, exception=self.timeout_exception
+        )
         # unless the strategy or the job parameters change this, do not retry
         self.max_retries = 1
-        self.diagnostics = []
         # list of protocol objects supported by this action, full list in job.protocols
         self.protocols = []
-        self.connection_timeout = Timeout(self.name, exception=self.timeout_exception)
+        self.connection_timeout = Timeout(
+            self.name, self, exception=self.timeout_exception
+        )
         self.character_delay = 0
         self.force_prompt = False
 
@@ -404,13 +418,6 @@ class Action:
         """
         self.job.context.update(value)
 
-    @classmethod
-    def select(cls, name):
-        for subclass in cls.__subclasses__():
-            if subclass.name == name:
-                return subclass
-        raise JobError('Cannot find action named "%s"' % name)
-
     @property
     def errors(self):
         if self.pipeline:
@@ -465,25 +472,22 @@ class Action:
             raise JobError("Unable to use repeat and failure_retry, use a repeat block")
         if "failure_retry" in self.parameters:
             self.max_retries = self.parameters["failure_retry"]
-        else:
-            if self.job:
-                if self.job.device and type(self.job.device).__name__ != "dict":
-                    if "constants" in self.job.device:
-                        max_retry = self.get_constant("failure_retry", "")
-                        if max_retry:
-                            self.max_retries = int(max_retry)
-                            # In case of a boot section, used boot_retry if it exists
-                            boot_retry = self.get_constant("boot_retry", "")
-                            if self.section == "boot" and boot_retry:
-                                self.max_retries = int(boot_retry)
+        elif "constants" in self.job.device:
+            device_max_retry = self.get_constant("failure_retry", "")
+            if device_max_retry:
+                device_max_retry = int(device_max_retry)
+                if device_max_retry > self.max_retries:
+                    self.max_retries = device_max_retry
+                # In case of a boot section, used boot_retry if it exists
+                boot_retry = self.get_constant("boot_retry", "")
+                if self.section == "boot" and boot_retry:
+                    self.max_retries = int(boot_retry)
         if "repeat" in self.parameters:
             self.max_retries = self.parameters["repeat"]
-        if self.job:
-            if self.job.device:
-                if "character_delays" in self.job.device:
-                    self.character_delay = self.job.device["character_delays"].get(
-                        self.section, 0
-                    )
+        if "character_delays" in self.job.device:
+            self.character_delay = self.job.device["character_delays"].get(
+                self.section, 0
+            )
 
     @parameters.setter
     def parameters(self, data):
@@ -514,6 +518,9 @@ class Action:
                 return self.parameters["deployment_data"][key]
         return self.job.device.get_constant(key, prefix=prefix)
 
+    def on_timeout(self) -> None:
+        ...
+
     def validate(self):
         """
         This method needs to validate the parameters to the action. For each
@@ -577,48 +584,58 @@ class Action:
         """
         if not isinstance(command_list, list):
             raise LAVABug("commands to parsed_command need to be a list")
-        log = ""
+        output = ""
         command_list = [str(s) for s in command_list]
         self.logger.debug("%s", " ".join(command_list))
         try:
-            log = subprocess.check_output(  # nosec - internal
-                command_list, stderr=subprocess.STDOUT, cwd=cwd
+            output = subprocess.check_output(  # nosec - internal
+                command_list,
+                stderr=subprocess.STDOUT,
+                cwd=cwd,
+                timeout=self.timeout.duration,
             )
-            log = log.decode("utf-8", errors="replace")
+            output = output.decode("utf-8", errors="replace")
             if allow_fail:
                 self.results = {"returncode": "0"}
-                self.results = {"output_len": len(log)}
+                self.results = {"output_len": len(output)}
                 self.logger.info(
                     "Parsed command exited zero with allow_fail set, returning %s bytes."
-                    % len(log)
+                    % len(output)
                 )
-        except subprocess.CalledProcessError as exc:
+        except (
+            FileNotFoundError,
+            subprocess.CalledProcessError,
+            subprocess.TimeoutExpired,
+        ) as exc:
             # the errors property doesn't support removing errors
             errors = []
-            retcode = exc.returncode
-            if exc.output:
+
+            # Extract some data about the exception
+            # Have sane default for some exceptions
+            returncode = getattr(exc, "returncode", 127)
+            if getattr(exc, "output", None):
                 output = exc.output.strip().decode("utf-8", errors="replace")
             else:
                 output = str(exc)
+
             errors.append(output)
-            self.results = {"returncode": "%s" % exc.returncode}
-            self.logger.info("Parsed command exited %s." % retcode)
+            self.results = {"returncode": "%s" % returncode}
+            self.logger.info("Parsed command exited %s." % returncode)
             base = (
                 "action: {0}\ncommand: {1}\nmessage: {2}\noutput: {3}\nreturn code: {4}"
             )
             msg = base.format(
                 self.name,
-                [i.strip() for i in exc.cmd],
+                [i.strip() for i in command_list],
                 str(exc),
                 "\n".join(errors),
-                retcode,
+                returncode,
             )
             self.results = {"output": output}
 
-            # the exception is raised due to a non-zero exc.returncode
+            # the exception is raised due to a non-zero returncode
             if allow_fail:
                 self.logger.info(msg)
-                log = exc.output.strip().decode("utf-8", errors="replace")
             else:
                 for error in errors:
                     self.errors = error
@@ -626,9 +643,44 @@ class Action:
                 # if not allow_fail, fail the command with the specified exception.
                 raise self.command_exception(exc) from exc
 
-        for line in log.split("\n"):
+        for line in output.split("\n"):
             self.logger.debug("output: %s", line)
-        return log
+        return output
+
+    _SUBPROCESS_SIGTERM_TIMEOUT = 3.0
+
+    def _run_pexpect_popen(
+        self,
+        pexpect_popen: PexpectPopenSpawn,
+        start: float,
+    ) -> int | None:
+        proc = pexpect_popen.proc
+
+        def terminate_or_kill_proc() -> None:
+            if proc.returncode is not None:
+                return
+
+            try:
+                proc.terminate()
+                proc.wait(self._SUBPROCESS_SIGTERM_TIMEOUT)
+            except subprocess.TimeoutExpired:
+                # Subprocess ignores SIGTERM. Kill it with SIGKILL.
+                proc.kill()
+                # The process context manager will wait killed process.
+
+        with contextlib.ExitStack() as exit_stack:
+            exit_stack.enter_context(proc)
+            exit_stack.callback(terminate_or_kill_proc)
+            try:
+                pexpect_popen.expect(pexpect.EOF)
+                # wait for the process and record the return value
+                return pexpect_popen.wait()
+            except pexpect.TIMEOUT:
+                self.logger.error(
+                    "Timed out after %s seconds", int(time.monotonic() - start)
+                )
+
+        return
 
     def run_cmd(self, command_list, allow_fail=False, error_msg=None, cwd=None):
         """
@@ -642,28 +694,22 @@ class Action:
         :param: cwd - the current working directory for this command
         :return: return code of the command
         """
-        # Build the command list (adding 'nice' at the front)
+        # Build the command list
         if isinstance(command_list, str):
-            command_list = shlex.split(command_list)
+            command_list = shlex_split(command_list)
         elif not isinstance(command_list, list):
             raise LAVABug("commands to run_cmd need to be a list or a string")
         command_list = [str(s) for s in command_list]
 
-        # Build the error message
-        log_error_msg = "Unable to run 'nice' '%s'" % "' '".join(command_list)
-        if error_msg is None:
-            error_msg = log_error_msg
-
         # Start the subprocess
-        self.logger.debug("Calling: 'nice' '%s'", "' '".join(command_list))
+        self.logger.debug("Calling: %r", command_list)
         start = time.monotonic()
 
         cmd_logger = CommandLogger(self.logger)
-        ret = None
+        ret: int | None = None
         try:
-            proc = pexpect.spawn(
-                "nice",
-                command_list,
+            proc = PexpectPopenSpawn(
+                cmd=command_list,
                 cwd=cwd,
                 encoding="utf-8",
                 codec_errors="replace",
@@ -671,16 +717,8 @@ class Action:
                 timeout=self.timeout.duration,
                 searchwindowsize=10,
             )
-            proc.expect(pexpect.EOF)
-            # wait for the process and record the return value
-            ret = proc.wait()
-        except pexpect.TIMEOUT:
-            self.logger.error(
-                "Timed out after %s seconds", int(time.monotonic() - start)
-            )
-            proc.terminate()
-            proc.wait()
-        except pexpect.ExceptionPexpect as exc:
+            ret = self._run_pexpect_popen(proc, start)
+        except (OSError, pexpect.ExceptionPexpect) as exc:
             self.logger.error("Unable to run: %s", exc)
 
         cmd_logger.flush(force=True)
@@ -691,7 +729,9 @@ class Action:
 
         # Check the return value
         if ret != 0 and not allow_fail:
-            self.logger.error("Unable to run 'nice' '%s'", command_list)
+            self.logger.error("Unable to run: %r", command_list)
+            if error_msg is None:
+                error_msg = f"Unable to run: {command_list!r}"
             raise self.command_exception(error_msg)
         return ret
 
@@ -827,7 +867,7 @@ class Action:
             connection.timeout = self.connection_timeout
         return connection
 
-    def cleanup(self, connection):
+    def cleanup(self, connection, max_end_time=None):
         """
         cleanup will *only* be called after run() if run() raises an exception.
         Use cleanup with any resources that may be left open by an interrupt or failed operation
@@ -841,7 +881,7 @@ class Action:
         instead of using cleanup().
         """
         if self.pipeline:
-            self.pipeline.cleanup(connection)
+            self.pipeline.cleanup(connection, max_end_time)
 
     def get_namespace_keys(self, action, parameters=None):
         """Return the keys for the given action"""
