@@ -2,25 +2,13 @@
 #
 # Author: Neil Williams <neil.williams@linaro.org>
 #
-# This file is part of LAVA Dispatcher.
-#
-# LAVA Dispatcher is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# LAVA Dispatcher is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along
-# with this program; if not, see <http://www.gnu.org/licenses>.
+# SPDX-License-Identifier: GPL-2.0-or-later
+from __future__ import annotations
 
 import contextlib
 import os
 import re
+from typing import TYPE_CHECKING
 
 from lava_common.constants import (
     BOOTLOADER_DEFAULT_CMD_TIMEOUT,
@@ -48,6 +36,9 @@ from lava_dispatcher.utils.messages import LinuxKernelMessages
 from lava_dispatcher.utils.network import dispatcher_ip
 from lava_dispatcher.utils.strings import substitute
 
+if TYPE_CHECKING:
+    from lava_dispatcher.job import Job
+
 
 class BootHasMixin:
     """Add the two methods to boot classes using it"""
@@ -60,7 +51,6 @@ class BootHasMixin:
 
 
 class LoginAction(Action):
-
     name = "login-action"
     description = "Real login action."
     summary = "Login after boot."
@@ -72,6 +62,10 @@ class LoginAction(Action):
         " actual prompt string more closely."
     )
 
+    def __init__(self, job: Job):
+        super().__init__(job)
+        self.force_prompt = True  # Kernel logs may overlap with login prompt on boot
+
     def check_kernel_messages(
         self, connection, max_end_time, fail_msg, auto_login=False
     ):
@@ -84,6 +78,13 @@ class LoginAction(Action):
         if isinstance(connection, SShSession):
             self.logger.debug("Skipping kernel messages")
             return
+        if self.parameters.get("ignore_kernel_messages", False):
+            self.logger.debug("Skipping kernel messages. Flag set to false")
+            if self.force_prompt:
+                connection.force_prompt_wait(max_end_time)
+            else:
+                connection.wait(max_end_time)
+            return
         self.logger.info("Parsing kernel messages")
         self.logger.debug(connection.prompt_str)
         parsed = LinuxKernelMessages.parse_failures(
@@ -105,21 +106,24 @@ class LoginAction(Action):
             self.results = {"fail": parsed}
             self.logger.warning("Kernel warnings or errors detected.")
 
-    def run(self, connection, max_end_time):
-        def check_prompt_characters(chk_prompt):
-            if not any(
-                [True for c in DISTINCTIVE_PROMPT_CHARACTERS if c in chk_prompt]
-            ):
-                self.logger.warning(self.check_prompt_characters_warning, chk_prompt)
+    def _check_prompt_characters(self, chk_prompt: str | type) -> None:
+        if not isinstance(chk_prompt, str):
+            return
 
+        if not any(c in chk_prompt for c in DISTINCTIVE_PROMPT_CHARACTERS):
+            self.logger.warning(self.check_prompt_characters_warning, chk_prompt)
+
+    def run(self, connection, max_end_time):
         connection = super().run(connection, max_end_time)
         if not connection:
             return connection
         prompts = self.parameters.get("prompts")
         for prompt in prompts:
-            check_prompt_characters(prompt)
+            self._check_prompt_characters(prompt)
 
-        connection.prompt_str = LinuxKernelMessages.get_init_prompts()
+        connection.prompt_str = []
+        if not self.parameters.get("ignore_kernel_messages", False):
+            connection.prompt_str = LinuxKernelMessages.get_init_prompts()
         connection.prompt_str.extend(prompts)
 
         # Needs to be added after the standard kernel message matches
@@ -237,8 +241,8 @@ class AutoLoginAction(RetryAction):
     )
     summary = "Auto-login after boot with support for kernel messages."
 
-    def __init__(self, booting=True):
-        super().__init__()
+    def __init__(self, job: Job, booting=True):
+        super().__init__(job)
         self.params = None
         self.booting = booting  # if a boot is expected, False for second UART or ssh.
 
@@ -294,7 +298,7 @@ class AutoLoginAction(RetryAction):
 
     def populate(self, parameters):
         self.pipeline = Pipeline(parent=self, job=self.job, parameters=parameters)
-        self.pipeline.add_action(LoginAction())
+        self.pipeline.add_action(LoginAction(self.job))
 
     def run(self, connection, max_end_time):
         # Prompts commonly include # - when logging such strings,
@@ -348,8 +352,8 @@ class BootloaderCommandOverlay(Action):
     description = "substitute job data into bootloader command list"
     summary = "replace placeholders with job data"
 
-    def __init__(self, method=None, commands=None):
-        super().__init__()
+    def __init__(self, job: Job, method=None, commands=None):
+        super().__init__(job)
         self.commands = commands
         self.method = method
         self.use_bootscript = False
@@ -397,8 +401,10 @@ class BootloaderCommandOverlay(Action):
         if "use_bootscript" in self.parameters:
             self.use_bootscript = self.parameters["use_bootscript"]
 
+        lava_mac = None
         if "lava_mac" in device_methods[self.method]["parameters"]:
             lava_mac = device_methods[self.method]["parameters"]["lava_mac"]
+        if lava_mac:
             if re.match("([0-9A-F]{2}[:-]){5}([0-9A-F]{2})", lava_mac, re.IGNORECASE):
                 self.lava_mac = lava_mac
             else:
@@ -444,6 +450,9 @@ class BootloaderCommandOverlay(Action):
             "{TEE}": self.get_namespace_data(
                 action="download-action", label="file", key="tee"
             ),
+            "{DTB_BASE_ADDR}": None,
+            "{DTB_BASE_RESIZE}": None,
+            "{APPLY_DTBO_COMMANDS}": None,
         }
         self.bootcommand = self.get_namespace_data(
             action="uboot-prepare-kernel", label="bootcommand", key="bootcommand"
@@ -463,6 +472,35 @@ class BootloaderCommandOverlay(Action):
             dtb_addr = self.job.device["parameters"][self.bootcommand]["dtb"]
             ramdisk_addr = self.job.device["parameters"][self.bootcommand]["ramdisk"]
             tee_addr = self.job.device["parameters"][self.bootcommand].get("tee")
+            dtbo_addr = self.job.device["parameters"][self.bootcommand].get("dtbo")
+            dtb_base_resize = self.job.device["parameters"].get("dtb_base_resize")
+            apply_dtbo_commands = self.job.device["actions"]["boot"]["methods"][
+                self.method
+            ]["parameters"].get("apply_dtbo_commands", [])
+            with contextlib.suppress(ValueError):
+                dtbo_pos = self.commands.index("{APPLY_DTBO_COMMANDS}")
+                index = 0
+                while True:
+                    file = self.get_namespace_data(
+                        action="download-action", label="file", key=f"dtbo{index}"
+                    )
+                    if file:
+                        for apply_dtbo_command in apply_dtbo_commands:
+                            dtbo_pos += 1
+                            self.commands.insert(
+                                dtbo_pos,
+                                apply_dtbo_command.replace(
+                                    "{DTBO}", f"{{DTBO{index}}}"
+                                ),
+                            )
+                            substitutions[f"{{DTBO{index}}}"] = file
+                    else:
+                        break
+                    index += 1
+                if index > 0:
+                    substitutions["{DTB_BASE_ADDR}"] = dtb_addr
+                    substitutions["{DTB_BASE_RESIZE}"] = str(dtb_base_resize)
+                    substitutions["{DTBO_ADDR}"] = dtbo_addr
 
             if (
                 not self.get_namespace_data(
@@ -775,8 +813,8 @@ class BootloaderInterruptAction(Action):
     summary = "interrupt bootloader to get an interactive shell"
     timeout_exception = InfrastructureError
 
-    def __init__(self, method=None):
-        super().__init__()
+    def __init__(self, job: Job, method=None):
+        super().__init__(job)
         self.params = {}
         self.method = method
         self.needs_interrupt = False
@@ -876,11 +914,14 @@ class BootloaderCommandsActionAltBank(Action):
     summary = "interactive bootloader altbank"
     timeout_exception = InfrastructureError
 
-    def __init__(self, expect_final=True, method=None):
-        super().__init__()
+    def __init__(self, job: Job, expect_final=True, method=None):
+        super().__init__(job)
         self.params = None
         self.timeout = Timeout(
-            self.name, BOOTLOADER_DEFAULT_CMD_TIMEOUT, exception=self.timeout_exception
+            self.name,
+            self,
+            duration=BOOTLOADER_DEFAULT_CMD_TIMEOUT,
+            exception=self.timeout_exception,
         )
         self.method = method
         self.expect_final = expect_final
@@ -939,11 +980,14 @@ class BootloaderCommandsAction(Action):
     summary = "interactive bootloader"
     timeout_exception = InfrastructureError
 
-    def __init__(self, expect_final=True, method=None):
-        super().__init__()
+    def __init__(self, job: Job, expect_final=True, method=None):
+        super().__init__(job)
         self.params = None
         self.timeout = Timeout(
-            self.name, BOOTLOADER_DEFAULT_CMD_TIMEOUT, exception=self.timeout_exception
+            self.name,
+            self,
+            duration=BOOTLOADER_DEFAULT_CMD_TIMEOUT,
+            exception=self.timeout_exception,
         )
         self.method = method
         self.expect_final = expect_final
@@ -984,7 +1028,7 @@ class BootloaderCommandsAction(Action):
                 connection.prompt_str = [connection.prompt_str]
             connection.prompt_str = connection.prompt_str + error_messages
 
-        for (index, line) in enumerate(commands):
+        for index, line in enumerate(commands):
             connection.sendline(line, delay=self.character_delay)
             if index + 1 == len(commands):
                 if not final_message or not self.expect_final:
@@ -1009,7 +1053,6 @@ class BootloaderCommandsAction(Action):
 
 
 class AdbOverlayUnpack(Action):
-
     name = "adb-overlay-unpack"
     summary = "unpack the overlay on the remote device"
     description = "unpack the overlay over adb"
