@@ -2,24 +2,13 @@
 #
 # Author: Michael Grzeschik <mgr@pengutronix.de>
 #
-# This file is part of LAVA Dispatcher.
-#
-# LAVA Dispatcher is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# LAVA Dispatcher is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along
-# with this program; if not, see <http://www.gnu.org/licenses>.
+# SPDX-License-Identifier: GPL-2.0-or-later
 
 # List just the subclasses supported for this base strategy
 # imported by the parser to populate the list of subclasses.
+from __future__ import annotations
+
+from typing import TYPE_CHECKING
 
 from lava_common.exceptions import ConfigurationError
 from lava_dispatcher.action import Action, Pipeline
@@ -37,6 +26,9 @@ from lava_dispatcher.logical import Boot, RetryAction
 from lava_dispatcher.power import ResetDevice
 from lava_dispatcher.shell import ExpectShellSession
 
+if TYPE_CHECKING:
+    from lava_dispatcher.job import Job
+
 
 class Barebox(Boot):
     """
@@ -48,11 +40,9 @@ class Barebox(Boot):
     hand this pexpect wrapper to subsequent actions as a shell connection.
     """
 
-    compatibility = 1
-
     @classmethod
-    def action(cls):
-        return BareboxAction()
+    def action(cls, job: Job) -> Action:
+        return BareboxAction(job)
 
     @classmethod
     def accepts(cls, device, parameters):
@@ -78,13 +68,12 @@ class BareboxAction(Action):
     def populate(self, parameters):
         self.pipeline = Pipeline(parent=self, job=self.job, parameters=parameters)
         # customize the device configuration for this job
-        self.pipeline.add_action(BootloaderCommandOverlay())
-        self.pipeline.add_action(ConnectDevice())
-        self.pipeline.add_action(BareboxRetry())
+        self.pipeline.add_action(BootloaderCommandOverlay(self.job))
+        self.pipeline.add_action(ConnectDevice(self.job))
+        self.pipeline.add_action(BareboxRetry(self.job))
 
 
 class BareboxRetry(BootHasMixin, RetryAction):
-
     name = "barebox-retry"
     description = "interactive barebox retry action"
     summary = "barebox commands with retry"
@@ -92,16 +81,16 @@ class BareboxRetry(BootHasMixin, RetryAction):
     def populate(self, parameters):
         self.pipeline = Pipeline(parent=self, job=self.job, parameters=parameters)
         # establish a new connection before trying the reset
-        self.pipeline.add_action(ResetDevice())
-        self.pipeline.add_action(BootloaderInterruptAction())
-        self.pipeline.add_action(BootloaderCommandsAction())
+        self.pipeline.add_action(ResetDevice(self.job))
+        self.pipeline.add_action(BootloaderInterruptAction(self.job))
+        self.pipeline.add_action(BootloaderCommandsAction(self.job))
         if self.has_prompts(parameters):
-            self.pipeline.add_action(AutoLoginAction())
+            self.pipeline.add_action(AutoLoginAction(self.job))
             if self.test_has_shell(parameters):
-                self.pipeline.add_action(ExpectShellSession())
+                self.pipeline.add_action(ExpectShellSession(self.job))
                 if "transfer_overlay" in parameters:
-                    self.pipeline.add_action(OverlayUnpack())
-                self.pipeline.add_action(ExportDeviceEnvironment())
+                    self.pipeline.add_action(OverlayUnpack(self.job))
+                self.pipeline.add_action(ExportDeviceEnvironment(self.job))
 
     def validate(self):
         super().validate()
