@@ -2,21 +2,10 @@
 #
 # Author: Remi Duraffort <remi.duraffort@linaro.org>
 #
-# This file is part of LAVA Dispatcher.
-#
-# LAVA Dispatcher is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# LAVA Dispatcher is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along
-# with this program; if not, see <http://www.gnu.org/licenses>.
+# SPDX-License-Identifier: GPL-2.0-or-later
+from __future__ import annotations
+
+from typing import TYPE_CHECKING
 
 from lava_dispatcher.action import Action, Pipeline
 from lava_dispatcher.actions.boot import (
@@ -28,13 +17,14 @@ from lava_dispatcher.connections.serial import ConnectDevice
 from lava_dispatcher.logical import Boot, RetryAction
 from lava_dispatcher.power import ResetDevice
 
+if TYPE_CHECKING:
+    from lava_dispatcher.job import Job
+
 
 class BootBootloader(Boot):
-    compatibility = 4
-
     @classmethod
-    def action(cls):
-        return BootBootloaderRetry()
+    def action(cls, job: Job) -> Action:
+        return BootBootloaderRetry(job)
 
     @classmethod
     def accepts(cls, device, parameters):
@@ -52,7 +42,6 @@ class BootBootloader(Boot):
 
 
 class BootBootloaderRetry(RetryAction):
-
     name = "boot-bootloader-retry"
     description = "boot to bootloader with retry"
     summary = "boot bootloader retry"
@@ -60,26 +49,25 @@ class BootBootloaderRetry(RetryAction):
     def populate(self, parameters):
         self.pipeline = Pipeline(parent=self, job=self.job, parameters=parameters)
         self.pipeline.add_action(
-            BootloaderCommandOverlay(method=parameters["bootloader"])
+            BootloaderCommandOverlay(self.job, method=parameters["bootloader"])
         )
-        self.pipeline.add_action(BootBootloaderAction())
+        self.pipeline.add_action(BootBootloaderAction(self.job))
 
 
 class BootBootloaderAction(Action):
-
     name = "boot-bootloader"
     description = "boot to bootloader"
     summary = "boot bootloader"
 
     def populate(self, parameters):
         self.pipeline = Pipeline(parent=self, job=self.job, parameters=parameters)
-        self.pipeline.add_action(ConnectDevice())
-        self.pipeline.add_action(ResetDevice())
+        self.pipeline.add_action(ConnectDevice(self.job))
+        self.pipeline.add_action(ResetDevice(self.job))
         self.pipeline.add_action(
-            BootloaderInterruptAction(method=parameters["bootloader"])
+            BootloaderInterruptAction(self.job, method=parameters["bootloader"])
         )
         self.pipeline.add_action(
             BootloaderCommandsAction(
-                expect_final=False, method=parameters["bootloader"]
+                self.job, expect_final=False, method=parameters["bootloader"]
             )
         )
