@@ -2,25 +2,13 @@
 #
 # Author: Tyler Baker <tyler.baker@linaro.org>
 #
-# This file is part of LAVA Dispatcher.
-#
-# LAVA Dispatcher is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# LAVA Dispatcher is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along
-# with this program; if not, see <http://www.gnu.org/licenses>.
+# SPDX-License-Identifier: GPL-2.0-or-later
+from __future__ import annotations
 
 import os
 import shutil
 import time
+from typing import TYPE_CHECKING
 
 from lava_common.exceptions import InfrastructureError
 from lava_dispatcher.action import Action, Pipeline
@@ -30,21 +18,19 @@ from lava_dispatcher.power import ResetDevice
 from lava_dispatcher.utils.filesystem import mkdtemp
 from lava_dispatcher.utils.udev import WaitDevicePathAction, WaitUSBSerialDeviceAction
 
+if TYPE_CHECKING:
+    from lava_dispatcher.job import Job
+
 
 class CMSIS(Boot):
-
-    compatibility = 4  # FIXME: change this to 5 and update test cases
-
     @classmethod
-    def action(cls):
-        return BootCMSISRetry()
+    def action(cls, job: Job) -> Action:
+        return BootCMSISRetry(job)
 
     @classmethod
     def accepts(cls, device, parameters):
         if "cmsis-dap" not in device["actions"]["boot"]["methods"]:
             return False, '"cmsis-dap" is not in the device configuration boot methods'
-        if "method" not in parameters:
-            return False, '"method" not in parameters'
         if parameters["method"] != "cmsis-dap":
             return False, '"method" was not "cmsis-dap"'
         if "board_id" not in device:
@@ -66,7 +52,6 @@ class CMSIS(Boot):
 
 
 class BootCMSISRetry(RetryAction):
-
     name = "boot-cmsis-retry"
     description = "boot cmsis usb image with retry"
     summary = "boot cmsis usb image with retry"
@@ -88,23 +73,22 @@ class BootCMSISRetry(RetryAction):
         usb_mass_device = method_params.get("usb_mass_device")
         resets_after_flash = method_params.get("resets_after_flash", True)
         if self.job.device.hard_reset_command:
-            self.pipeline.add_action(ResetDevice())
-            self.pipeline.add_action(WaitDevicePathAction(usb_mass_device))
-        self.pipeline.add_action(FlashCMSISAction())
+            self.pipeline.add_action(ResetDevice(self.job))
+            self.pipeline.add_action(WaitDevicePathAction(self.job, usb_mass_device))
+        self.pipeline.add_action(FlashCMSISAction(self.job))
         if resets_after_flash:
-            self.pipeline.add_action(WaitUSBSerialDeviceAction())
-        self.pipeline.add_action(ConnectDevice())
+            self.pipeline.add_action(WaitUSBSerialDeviceAction(self.job))
+        self.pipeline.add_action(ConnectDevice(self.job))
 
 
 class FlashCMSISAction(Action):
-
     name = "flash-cmsis"
     description = "flash cmsis to usb mass storage"
     summary = "flash cmsis to usb mass storage"
     command_exception = InfrastructureError
 
-    def __init__(self):
-        super().__init__()
+    def __init__(self, job: Job):
+        super().__init__(job)
         self.filelist = []
         self.usb_mass_device = None
 
