@@ -3,24 +3,11 @@
 # Author: Tomeu Vizoso <tomeu.vizoso@collabora.com>
 # Author: Guillaume Tucker <guillaume.tucker@collabora.com>
 #
-# This file is part of LAVA Dispatcher.
-#
-# LAVA Dispatcher is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# LAVA Dispatcher is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along
-# with this program; if not, see <http://www.gnu.org/licenses>.
-
+# SPDX-License-Identifier: GPL-2.0-or-later
+from __future__ import annotations
 
 import os.path
+from typing import TYPE_CHECKING
 
 from lava_common.constants import BOOTLOADER_DEFAULT_CMD_TIMEOUT
 from lava_common.exceptions import ConfigurationError, InfrastructureError
@@ -41,6 +28,9 @@ from lava_dispatcher.shell import ExpectShellSession
 from lava_dispatcher.utils.network import dispatcher_ip
 from lava_dispatcher.utils.strings import substitute
 
+if TYPE_CHECKING:
+    from lava_dispatcher.job import Job
+
 
 class Depthcharge(Boot):
     """
@@ -55,8 +45,8 @@ class Depthcharge(Boot):
     """
 
     @classmethod
-    def action(cls):
-        return DepthchargeAction()
+    def action(cls, job: Job) -> Action:
+        return DepthchargeAction(job)
 
     @classmethod
     def accepts(cls, device, parameters):
@@ -79,8 +69,8 @@ class DepthchargeCommandOverlay(BootloaderCommandOverlay):
 
     name = "depthcharge-overlay"
 
-    def __init__(self):
-        super().__init__()
+    def __init__(self, job: Job):
+        super().__init__(job)
         self.cmdline = None
 
     def validate(self):
@@ -90,7 +80,7 @@ class DepthchargeCommandOverlay(BootloaderCommandOverlay):
         method_params = method[commands_name]
         self.cmdline = method_params.get("cmdline")
         if self.cmdline is None:
-            self.errors = "No cmdline found in {}".format(commands_name)
+            self.errors = f"No cmdline found in {commands_name}"
 
     def run(self, connection, max_end_time):
         connection = super().run(connection, max_end_time)
@@ -180,29 +170,28 @@ class DepthchargeAction(Action):
 
     def populate(self, parameters):
         self.pipeline = Pipeline(parent=self, job=self.job, parameters=parameters)
-        self.pipeline.add_action(DepthchargeCommandOverlay())
-        self.pipeline.add_action(DepthchargeRetry())
+        self.pipeline.add_action(DepthchargeCommandOverlay(self.job))
+        self.pipeline.add_action(DepthchargeRetry(self.job))
 
 
 class DepthchargeRetry(BootHasMixin, RetryAction):
-
     name = "depthcharge-retry"
     description = "interactive depthcharge retry action"
     summary = "depthcharge commands with retry"
 
     def populate(self, parameters):
         self.pipeline = Pipeline(parent=self, job=self.job, parameters=parameters)
-        self.pipeline.add_action(ResetConnection())
-        self.pipeline.add_action(ResetDevice())
-        self.pipeline.add_action(DepthchargeStart())
-        self.pipeline.add_action(BootloaderCommandsAction())
+        self.pipeline.add_action(ResetConnection(self.job))
+        self.pipeline.add_action(ResetDevice(self.job))
+        self.pipeline.add_action(DepthchargeStart(self.job))
+        self.pipeline.add_action(BootloaderCommandsAction(self.job))
         if self.has_prompts(parameters):
-            self.pipeline.add_action(AutoLoginAction())
+            self.pipeline.add_action(AutoLoginAction(self.job))
             if self.test_has_shell(parameters):
-                self.pipeline.add_action(ExpectShellSession())
+                self.pipeline.add_action(ExpectShellSession(self.job))
                 if "transfer_overlay" in parameters:
-                    self.pipeline.add_action(OverlayUnpack())
-                self.pipeline.add_action(ExportDeviceEnvironment())
+                    self.pipeline.add_action(OverlayUnpack(self.job))
+                self.pipeline.add_action(ExportDeviceEnvironment(self.job))
 
 
 class DepthchargeStart(Action):
@@ -215,11 +204,14 @@ class DepthchargeStart(Action):
     summary = "Depthcharge start"
     timeout_exception = InfrastructureError
 
-    def __init__(self):
-        super().__init__()
+    def __init__(self, job: Job):
+        super().__init__(job)
         self.start_message = None
         self.timeout = Timeout(
-            self.name, BOOTLOADER_DEFAULT_CMD_TIMEOUT, exception=self.timeout_exception
+            self.name,
+            self,
+            duration=BOOTLOADER_DEFAULT_CMD_TIMEOUT,
+            exception=self.timeout_exception,
         )
 
     def validate(self):
