@@ -2,21 +2,10 @@
 #
 # Author: Tyler Baker <tyler.baker@linaro.org>
 #
-# This file is part of LAVA Dispatcher.
-#
-# LAVA Dispatcher is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# LAVA Dispatcher is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along
-# with this program; if not, see <http://www.gnu.org/licenses>.
+# SPDX-License-Identifier: GPL-2.0-or-later
+from __future__ import annotations
+
+from typing import TYPE_CHECKING
 
 from lava_common.exceptions import ConfigurationError, InfrastructureError
 from lava_dispatcher.action import Action, Pipeline
@@ -28,21 +17,19 @@ from lava_dispatcher.utils.shell import which
 from lava_dispatcher.utils.strings import substitute
 from lava_dispatcher.utils.udev import WaitDFUDeviceAction
 
+if TYPE_CHECKING:
+    from lava_dispatcher.job import Job
+
 
 class DFU(Boot):
-
-    compatibility = 4  # FIXME: change this to 5 and update test cases
-
     @classmethod
-    def action(cls):
-        return BootDFURetry()
+    def action(cls, job: Job) -> Action:
+        return BootDFURetry(job)
 
     @classmethod
     def accepts(cls, device, parameters):
         if "dfu" not in device["actions"]["boot"]["methods"]:
             return False, '"dfu" was not in the device configuration boot methods'
-        if "method" not in parameters:
-            return False, '"method" was in the parameters'
         if parameters["method"] != "dfu":
             return False, '"method" was not "dfu"'
         if "board_id" not in device:
@@ -51,27 +38,25 @@ class DFU(Boot):
 
 
 class BootDFURetry(RetryAction):
-
     name = "boot-dfu-retry"
     description = "boot dfu image using the command line interface"
     summary = "boot dfu image"
 
     def populate(self, parameters):
         dfu = self.job.device["actions"]["boot"]["methods"]["dfu"]
-        parameters = dfu["parameters"]
-
         self.pipeline = Pipeline(parent=self, job=self.job, parameters=parameters)
-        self.pipeline.add_action(ConnectDevice())
-        self.pipeline.add_action(ResetDevice())
+        self.pipeline.add_action(ConnectDevice(self.job))
+        self.pipeline.add_action(ResetDevice(self.job))
         if dfu.get("implementation") == "u-boot":
-            self.pipeline.add_action(BootloaderInterruptAction(method="u-boot"))
-            self.pipeline.add_action(EnterDFU())
-        self.pipeline.add_action(WaitDFUDeviceAction())
-        self.pipeline.add_action(FlashDFUAction())
+            self.pipeline.add_action(
+                BootloaderInterruptAction(self.job, method="u-boot")
+            )
+            self.pipeline.add_action(EnterDFU(self.job))
+        self.pipeline.add_action(WaitDFUDeviceAction(self.job))
+        self.pipeline.add_action(FlashDFUAction(self.job))
 
 
 class EnterDFU(Action):
-
     name = "enter-dfu"
     description = "enter software dfu mode"
     summary = "enter software dfu mode"
@@ -87,7 +72,7 @@ class EnterDFU(Action):
     def run(self, connection, max_end_time):
         connection = super().run(connection, max_end_time)
         parameters = self.job.device["actions"]["boot"]["methods"]["dfu"]["parameters"]
-        for (index, cmd) in enumerate(parameters["enter-commands"]):
+        for index, cmd in enumerate(parameters["enter-commands"]):
             connection.sendline(cmd)
             # Do not wait for the bootloader prompt for the last command.
             # This command does not return.
@@ -96,13 +81,12 @@ class EnterDFU(Action):
 
 
 class FlashDFUAction(Action):
-
     name = "flash-dfu"
     description = "use dfu to flash the images"
     summary = "use dfu to flash the images"
 
-    def __init__(self):
-        super().__init__()
+    def __init__(self, job: Job):
+        super().__init__(job)
         self.base_command = []
         self.exec_list = []
         self.board_id = "0000000000"
@@ -168,7 +152,7 @@ class FlashDFUAction(Action):
                 "dfu-download", prefix=implementation
             )
 
-        for (index, dfu_command) in enumerate(self.exec_list):
+        for index, dfu_command in enumerate(self.exec_list):
             # add --reset for the last command (if reset works)
             if index + 1 == len(self.exec_list) and reset_works:
                 dfu_command.extend(["--reset"])
