@@ -2,24 +2,12 @@
 #
 # Author: Remi Duraffort <remi.duraffort@linaro.org>
 #
-# This file is part of LAVA Dispatcher.
-#
-# LAVA Dispatcher is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# LAVA Dispatcher is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along
-# with this program; if not, see <http://www.gnu.org/licenses>.
+# SPDX-License-Identifier: GPL-2.0-or-later
+from __future__ import annotations
 
 import os
 import pathlib
+from typing import TYPE_CHECKING
 
 from lava_common.constants import LAVA_DOWNLOADS
 from lava_common.exceptions import JobError
@@ -30,13 +18,14 @@ from lava_dispatcher.logical import Boot, RetryAction
 from lava_dispatcher.shell import ExpectShellSession, ShellCommand, ShellSession
 from lava_dispatcher.utils.network import dispatcher_ip
 
+if TYPE_CHECKING:
+    from lava_dispatcher.job import Job
+
 
 class BootDocker(Boot):
-    compatibility = 4
-
     @classmethod
-    def action(cls):
-        return BootDockerAction()
+    def action(cls, job: Job) -> Action:
+        return BootDockerAction(job)
 
     @classmethod
     def accepts(cls, device, parameters):
@@ -50,28 +39,26 @@ class BootDocker(Boot):
 
 
 class BootDockerAction(BootHasMixin, RetryAction):
-
     name = "boot-docker"
     description = "boot docker image"
     summary = "boot docker image"
 
     def populate(self, parameters):
         self.pipeline = Pipeline(parent=self, job=self.job, parameters=parameters)
-        self.pipeline.add_action(CallDockerAction())
+        self.pipeline.add_action(CallDockerAction(self.job))
         if self.has_prompts(parameters):
             if self.test_has_shell(parameters):
-                self.pipeline.add_action(ExpectShellSession())
-                self.pipeline.add_action(ExportDeviceEnvironment())
+                self.pipeline.add_action(ExpectShellSession(self.job))
+                self.pipeline.add_action(ExportDeviceEnvironment(self.job))
 
 
 class CallDockerAction(Action):
-
     name = "docker-run"
     description = "call docker run on the image"
     summary = "call docker run"
 
-    def __init__(self):
-        super().__init__()
+    def __init__(self, job: Job):
+        super().__init__(job)
         self.cleanup_required = False
         self.remote = ""
         self.extra_options = ""
@@ -80,6 +67,9 @@ class CallDockerAction(Action):
     def validate(self):
         super().validate()
         self.container = "lava-%s-%s" % (self.job.job_id, self.level)
+        prefix = self.job.parameters.get("dispatcher", {}).get("prefix", "")
+        if prefix:
+            self.container = "lava-%s-%s-%s" % (prefix, self.job.job_id, self.level)
 
         options = self.job.device["actions"]["boot"]["methods"]["docker"]["options"]
 
