@@ -2,24 +2,16 @@
 #
 # Author: Stevan Radakovic <stevan.radakovic@linaro.org>
 #
-# This file is part of LAVA Dispatcher.
-#
-# LAVA Dispatcher is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# LAVA Dispatcher is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along
-# with this program; if not, see <http://www.gnu.org/licenses>.
+# SPDX-License-Identifier: GPL-2.0-or-later
+from __future__ import annotations
+
+from typing import TYPE_CHECKING
 
 from lava_dispatcher.action import Action
 
+if TYPE_CHECKING:
+    from lava_dispatcher.job import Job
+
 
 class ExportDeviceEnvironment(Action):
     """
@@ -30,8 +22,8 @@ class ExportDeviceEnvironment(Action):
     description = "Exports environment variables to the device"
     summary = "Exports environment variables action"
 
-    def __init__(self):
-        super().__init__()
+    def __init__(self, job: Job):
+        super().__init__(job)
         self.env = []
 
     def validate(self):
@@ -52,7 +44,6 @@ class ExportDeviceEnvironment(Action):
             )
 
     def run(self, connection, max_end_time):
-
         if not connection:
             return
 
