@@ -2,26 +2,15 @@
 #
 # Author: Dean Arnold <dean.arnold@arm.com>
 #
-# This file is part of LAVA Dispatcher.
-#
-# LAVA Dispatcher is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# LAVA Dispatcher is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along
-# with this program; if not, see <http://www.gnu.org/licenses>.
+# SPDX-License-Identifier: GPL-2.0-or-later
+from __future__ import annotations
 
 import os
 import re
 import shlex
 import time
+from pathlib import Path
+from typing import TYPE_CHECKING
 
 from lava_common.exceptions import JobError
 from lava_dispatcher.action import Action, Pipeline
@@ -30,13 +19,14 @@ from lava_dispatcher.logical import Boot, RetryAction
 from lava_dispatcher.power import ReadFeedback
 from lava_dispatcher.shell import ExpectShellSession, ShellCommand, ShellSession
 
+if TYPE_CHECKING:
+    from lava_dispatcher.job import Job
+
 
 class BootFVP(Boot):
-    compatibility = 4
-
     @classmethod
-    def action(cls):
-        return BootFVPAction()
+    def action(cls, job: Job) -> Action:
+        return BootFVPAction(job)
 
     @classmethod
     def accepts(cls, device, parameters):
@@ -50,49 +40,47 @@ class BootFVP(Boot):
 
 
 class BootFVPAction(BootHasMixin, RetryAction):
-
     name = "boot-fvp"
     description = "boot fvp"
     summary = "boot fvp"
 
     def populate(self, parameters):
         self.pipeline = Pipeline(parent=self, job=self.job, parameters=parameters)
-        self.pipeline.add_action(BootFVPMain())
+        self.pipeline.add_action(BootFVPMain(self.job))
         if self.has_prompts(parameters):
-            self.pipeline.add_action(ReadFeedback())
-            self.pipeline.add_action(AutoLoginAction())
+            self.pipeline.add_action(ReadFeedback(self.job))
+            self.pipeline.add_action(AutoLoginAction(self.job))
             if self.test_has_shell(parameters):
-                self.pipeline.add_action(ExpectShellSession())
+                self.pipeline.add_action(ExpectShellSession(self.job))
                 if "transfer_overlay" in parameters:
-                    self.pipeline.add_action(OverlayUnpack())
+                    self.pipeline.add_action(OverlayUnpack(self.job))
 
 
 class BootFVPMain(Action):
-
     name = "boot-fvp-main"
     description = "boot fvp"
     summary = "boot fvp"
 
     def populate(self, parameters):
         self.pipeline = Pipeline(parent=self, job=self.job, parameters=parameters)
-        self.pipeline.add_action(CheckFVPVersionAction())
-        self.pipeline.add_action(StartFVPAction())
+        self.pipeline.add_action(CheckFVPVersionAction(self.job))
+        self.pipeline.add_action(StartFVPAction(self.job))
         if parameters.get("use_telnet", True):
-            self.pipeline.add_action(GetFVPSerialAction())
-            self.pipeline.add_action(ReadFeedback())
+            self.pipeline.add_action(GetFVPSerialAction(self.job))
+            self.pipeline.add_action(ReadFeedback(self.job))
 
 
 class BaseFVPAction(Action):
-
     name = "base-fvp-action"
     description = "call docker run with fvp entry point"
     summary = "base fvp action"
 
-    def __init__(self):
-        super().__init__()
+    def __init__(self, job: Job):
+        super().__init__(job)
         self.extra_options = ""
         self.container = ""
         self.fvp_license = None
+        self.ubl_license = None
         self.docker_image = None
         self.local_docker_image = False
 
@@ -134,6 +122,10 @@ class BaseFVPAction(Action):
             self.extra_options += " --volume %s" % volume
         if "license_variable" in self.parameters:
             self.fvp_license = shlex.quote(self.parameters["license_variable"])
+        if options.get("ubl_license"):
+            self.ubl_license = shlex.quote(options["ubl_license"])
+        if "ubl_license" in self.parameters:
+            self.ubl_license = shlex.quote(self.parameters["ubl_license"])
 
     def construct_docker_fvp_command(self, docker_image, fvp_arguments):
         substitutions = {}
@@ -172,21 +164,94 @@ class BaseFVPAction(Action):
 
         substitutions["ARTIFACT_DIR"] = os.path.join("/", self.container)
         if not self.fvp_license:
-            self.logger.warning("'license_variable' not set, model may not function.")
+            if not self.ubl_license:
+                self.logger.warning(
+                    "'license_variable' or 'ubl_license' not set, model may not function."
+                )
         else:
             cmd += " -e %s" % self.fvp_license
         fvp_image = self.parameters.get("image")
         cmd += self.extra_options
-        cmd += " %s %s %s" % (docker_image, fvp_image, fvp_arguments)
+
+        # Build the script to start the model
+        script = Path(self.mkdtemp()) / "script.sh"
+        data = "#!/bin/sh\n"
+
+        if self.ubl_license:
+            license_type = "file"
+            license_pat = re.compile(r"^[^\W_]+-[^\W_]+-[^\W_]+-[^\W_]+-[^\W_]+$")
+            if license_pat.match(self.ubl_license):
+                license_type = "code"
+
+            data += f"""
+echo "Finding the armlm binary"
+ARMLM="$(find /opt/model -maxdepth 1 -type d | grep -v license_terms | tail -n 1)/bin/arm_license_management_utilities/armlm"
+if [ -e "$ARMLM" ]; then
+  echo "armlm at $ARMLM"
+
+  ubl_license="{self.ubl_license}"
+  license_type="{license_type}"
+
+  if [ "$license_type" = "file" ] && [ -e "$ubl_license" ]; then
+    echo "Trying to import UBL license"
+    if ! $ARMLM import -file "$ubl_license"; then
+      echo "Failed to import license file!"
+      exit 1
+    fi
+  elif [ "$license_type" = "code" ]; then
+    max_retries=5
+    attempt=0
+    backoff=1
+
+    while [ $attempt -lt $max_retries ]; do
+      echo "Trying to Activate UBL license"
+
+      if $ARMLM activate -code "$ubl_license"; then
+        echo "UBL Activation Successful"
+        break
+      else
+        echo "UBL activation failed. Retrying in $backoff seconds..."
+        sleep $backoff
+        backoff=$((backoff * 2))
+        attempt=$((attempt + 1))
+      fi
+    done
+
+    if [ $attempt -eq $max_retries ]; then
+      echo "UBL Activation failed after $max_retries attempts."
+      exit 1
+    fi
+  else
+    echo "Invalid UBL file/code!"
+    exit 1
+  fi
+else
+  echo "No armlm binary found, Won't import/activate UBL license"
+fi
+"""
+        # Substitute in the fvp arguments
         try:
-            cmd = cmd.format(**substitutions)
+            fvp_arguments = " \\\n".join(
+                line.format(**substitutions) for line in fvp_arguments
+            )
         except KeyError as exc:
             raise JobError(f"Key {exc} does not exist, may need escaping")
+
+        self.logger.debug("FVP model arguments:")
+        self.logger.debug("%s\n%s", fvp_image, fvp_arguments)
+
+        data += f"""echo "Starting the FVP model"
+{fvp_image} {fvp_arguments}"""
+        script.write_text(data, encoding="utf-8")
+
+        cmd += f" --volume {script}:/root/script.sh {docker_image} sh /root/script.sh"
         return cmd
 
     def cleanup(self, connection):
         super().cleanup(connection)
-        self.logger.debug("Stopping container %s", self.container)
+        self.logger.debug(
+            "Stopping container %s from action %s", self.container, self.name
+        )
         return_value = self.run_cmd(["docker", "stop", self.container], allow_fail=True)
         if return_value == 0:
             self.logger.debug("Stopped container %s", self.container)
@@ -197,8 +262,8 @@ class CheckFVPVersionAction(BaseFVPAction):
     description = "call docker run with fvp version entry point"
     summary = "check fvp version"
 
-    def __init__(self):
-        super().__init__()
+    def __init__(self, job: Job):
+        super().__init__(job)
         self.extra_options = ""
         self.container = ""
         self.fvp_version_string = ""
@@ -212,17 +277,33 @@ class CheckFVPVersionAction(BaseFVPAction):
     def run(self, connection, max_end_time):
         start = time.monotonic()
 
-        if not self.local_docker_image:
+        pull = not self.local_docker_image
+        if self.local_docker_image:
+            if self.run_cmd(
+                [
+                    "docker",
+                    "image",
+                    "inspect",
+                    "--format",
+                    f"Image {self.docker_image} exists locally",
+                    self.docker_image,
+                ],
+                allow_fail=True,
+            ):
+                self.logger.warning(
+                    "Unable to inspect docker image '%s'", self.docker_image
+                )
+                pull = True
+        if pull:
             self.logger.debug("Pulling image %s", self.docker_image)
             self.run_cmd(["docker", "pull", self.docker_image])
 
         fvp_image = self.parameters.get("image")
         fvp_arguments = self.parameters.get("version_args", "--version")
 
-        fvp_image = shlex.quote(fvp_image)
-        fvp_arguments = shlex.quote(fvp_arguments)
-        cmd = f"docker run --rm {self.docker_image} {fvp_image} {fvp_arguments}"
-        output = self.parsed_command(["sh", "-c", cmd])
+        output = self.parsed_command(
+            ["docker", "run", "--rm", self.docker_image, fvp_image, fvp_arguments]
+        )
         m = re.match(self.fvp_version_string, output)
         matched_version_string = m and m.group(0) or output
         result = {
@@ -243,8 +324,8 @@ class StartFVPAction(BaseFVPAction):
     description = "call docker run with fvp boot entry point"
     summary = "run fvp model"
 
-    def __init__(self):
-        super().__init__()
+    def __init__(self, job: Job):
+        super().__init__(job)
         self.extra_options = ""
         self.container = ""
         self.fvp_console_string = ""
@@ -267,11 +348,13 @@ class StartFVPAction(BaseFVPAction):
                 self.fvp_feedbacks.add(feedback)
 
     def run(self, connection, max_end_time):
-        fvp_arguments = " ".join(self.parameters.get("arguments"))
-
         # Build the command line
         # The docker image is safe to be included in the command line
-        cmd = self.construct_docker_fvp_command(self.docker_image, fvp_arguments)
+        cmd = self.construct_docker_fvp_command(
+            self.docker_image, self.parameters.get("arguments")
+        )
+        # Prefix by lava-outerr to add buffering to stdout and err
+        cmd = "lava-outerr " + cmd
 
         self.logger.debug("Boot command: %s", cmd)
         shell = ShellCommand(cmd, self.timeout, logger=self.logger)
@@ -335,17 +418,26 @@ class StartFVPAction(BaseFVPAction):
         # which will terminate the model.
         self.shell = shell
 
+        self.set_namespace_data(
+            action="shared", label="shared", key="connection", value=shell_connection
+        )
+
         self.shell_session = shell_connection
         return shell_connection
 
     def cleanup(self, connection):
+        start = time.monotonic()
+        if self.shell_session:
+            self.logger.debug("Listening to feedback from FVP binary for 60s.")
+            bytes_read = 1
+            while bytes_read and (time.monotonic() - start) < 60:
+                bytes_read = self.shell_session.listen_feedback(5)
+        super().cleanup(connection)
         if self.shell_session:
             self.logger.debug("Listening to feedback from FVP binary.")
-            while True:
+            bytes_read = 1
+            while bytes_read:
                 bytes_read = self.shell_session.listen_feedback(5)
-                if bytes_read == 0:
-                    break
-        super().cleanup(connection)
 
 
 class GetFVPSerialAction(Action):
@@ -377,7 +469,7 @@ class GetFVPSerialAction(Action):
 
             shell_connection = ShellSession(self.job, shell)
             shell_connection = super().run(shell_connection, max_end_time)
-            shell_connection.raw_connection.logfile.is_feedback = True
+            shell_connection.raw_connection.logfile_read.is_feedback = True
 
             feedback_name = feedback_dict.get("name")
             if not feedback_name:
@@ -390,7 +482,7 @@ class GetFVPSerialAction(Action):
                 parameters={"namespace": feedback_name},
             )
 
-        cmd = "docker exec --interactive --tty %s telnet localhost %s" % (
+        cmd = "lava-outerr docker exec --interactive --tty %s telnet localhost %s" % (
             container,
             serial_port,
         )
