@@ -2,24 +2,12 @@
 #
 # Author: Remi Duraffort <remi.duraffort@linaro.org>
 #
-# This file is part of LAVA Dispatcher.
-#
-# LAVA Dispatcher is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# LAVA Dispatcher is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along
-# with this program; if not, see <http://www.gnu.org/licenses>.
+# SPDX-License-Identifier: GPL-2.0-or-later
+from __future__ import annotations
 
 import os
 import time
+from typing import TYPE_CHECKING
 
 from lava_common.exceptions import JobError
 from lava_dispatcher.action import Action, Pipeline
@@ -31,22 +19,20 @@ from lava_dispatcher.utils.shell import which
 from lava_dispatcher.utils.strings import substitute
 from lava_dispatcher.utils.udev import WaitUSBSerialDeviceAction
 
+if TYPE_CHECKING:
+    from lava_dispatcher.job import Job
+
 
 class GDB(Boot):
-
-    compatibility = 4
-
     @classmethod
-    def action(cls):
-        return BootGDB()
+    def action(cls, job: Job) -> Action:
+        return BootGDB(job)
 
     @classmethod
     def accepts(cls, device, parameters):
         methods = device["actions"]["boot"]["methods"]
         if "gdb" not in methods:
             return False, '"gdb" is not in the device configuration boot methods'
-        if "method" not in parameters:
-            return False, '"method" not in parameters'
         if parameters["method"] != "gdb":
             return False, '"method" was not "gdb"'
         if "commands" not in parameters:
@@ -61,24 +47,22 @@ class GDB(Boot):
 
 
 class BootGDB(Action):
-
     name = "boot-gdb"
     description = "boot with gdb"
     summary = "boot with gdb"
 
     def populate(self, parameters):
         self.pipeline = Pipeline(parent=self, job=self.job, parameters=parameters)
-        self.pipeline.add_action(BootGDBRetry())
+        self.pipeline.add_action(BootGDBRetry(self.job))
 
 
 class BootGDBRetry(RetryAction):
-
     name = "boot-gdb-retry"
     description = "boot with gdb with retry and optional docker support"
     summary = "boot with gdb with retry"
 
-    def __init__(self):
-        super().__init__()
+    def __init__(self, job: Job):
+        super().__init__(job)
         self.gdb = None
         self.gdb_connection = None
         self.commands = []
@@ -130,9 +114,9 @@ class BootGDBRetry(RetryAction):
     def populate(self, parameters):
         self.pipeline = Pipeline(parent=self, job=self.job, parameters=parameters)
         if self.job.device.hard_reset_command:
-            self.pipeline.add_action(ResetDevice())
-        self.pipeline.add_action(WaitUSBSerialDeviceAction())
-        self.pipeline.add_action(ConnectDevice())
+            self.pipeline.add_action(ResetDevice(self.job))
+        self.pipeline.add_action(WaitUSBSerialDeviceAction(self.job))
+        self.pipeline.add_action(ConnectDevice(self.job))
 
     def run(self, connection, max_end_time):
         connection = super().run(connection, max_end_time)
