@@ -2,26 +2,14 @@
 #
 # Author: Matthew Hart <matthew.hart@linaro.org>
 #
-# This file is part of LAVA Dispatcher.
-#
-# LAVA Dispatcher is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# LAVA Dispatcher is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along
-# with this program; if not, see <http://www.gnu.org/licenses>.
+# SPDX-License-Identifier: GPL-2.0-or-later
 
 # List just the subclasses supported for this base strategy
 # imported by the parser to populate the list of subclasses.
+from __future__ import annotations
+
+from typing import TYPE_CHECKING
 
-from lava_common.exceptions import ConfigurationError
 from lava_dispatcher.action import Action, Pipeline
 from lava_dispatcher.actions.boot import (
     AutoLoginAction,
@@ -40,27 +28,22 @@ from lava_dispatcher.logical import Boot, RetryAction
 from lava_dispatcher.power import PowerOff, ResetDevice
 from lava_dispatcher.shell import ExpectShellSession
 
+if TYPE_CHECKING:
+    from typing import Optional
+
+    from lava_dispatcher.job import Job
+
 
 class GrubSequence(Boot):
-
-    compatibility = 3
-
     @classmethod
-    def action(cls):
-        return GrubSequenceAction()
+    def action(cls, job: Job) -> Action:
+        return GrubSequenceAction(job)
 
     @classmethod
     def accepts(cls, device, parameters):
-        if "method" not in parameters:
-            raise ConfigurationError("method not specified in boot parameters")
         if parameters["method"] not in ["grub", "grub-efi"]:
             return False, '"method" was not "grub" or "grub-efi"'
-        if "actions" not in device:
-            raise ConfigurationError("Invalid device configuration")
-        if "boot" not in device["actions"]:
-            return False, '"boot" was not in the device configuration actions'
-        if "methods" not in device["actions"]["boot"]:
-            raise ConfigurationError("Device misconfiguration")
+
         params = device["actions"]["boot"]["methods"]
         if "grub" not in params:
             return False, '"grub" was not in the device configuration boot methods'
@@ -72,25 +55,15 @@ class GrubSequence(Boot):
 
 
 class Grub(Boot):
-
-    compatibility = 3
-
     @classmethod
-    def action(cls):
-        return GrubMainAction()
+    def action(cls, job: Job) -> Action:
+        return GrubMainAction(job)
 
     @classmethod
     def accepts(cls, device, parameters):
-        if "method" not in parameters:
-            raise ConfigurationError("method not specified in boot parameters")
         if parameters["method"] not in ["grub", "grub-efi"]:
             return False, '"method" was not "grub" or "grub-efi"'
-        if "actions" not in device:
-            raise ConfigurationError("Invalid device configuration")
-        if "boot" not in device["actions"]:
-            return False, '"boot" was not in the device configuration actions'
-        if "methods" not in device["actions"]["boot"]:
-            raise ConfigurationError("Device misconfiguration")
+
         params = device["actions"]["boot"]["methods"]
         if "grub" in params and "sequence" in params["grub"]:
             return False, '"sequence" was in "grub" parameters'
@@ -103,9 +76,11 @@ class Grub(Boot):
             )
 
 
-def _grub_sequence_map(sequence):
+def _grub_sequence_map(
+    sequence: str,
+) -> tuple[Optional[type[Action]], Optional[str]]:
     """Maps grub sequence with corresponding class."""
-    sequence_map = {
+    sequence_map: dict[str, tuple[type[Action], Optional[str]]] = {
         "wait-fastboot-interrupt": (WaitFastBootInterrupt, "grub"),
         "auto-login": (AutoLoginAction, None),
         "shell-session": (ExpectShellSession, None),
@@ -115,13 +90,12 @@ def _grub_sequence_map(sequence):
 
 
 class GrubSequenceAction(BootHasMixin, RetryAction):
-
     name = "grub-sequence-action"
     description = "grub boot sequence"
     summary = "run grub boot using specified sequence of actions"
 
-    def __init__(self):
-        super().__init__()
+    def __init__(self, job: Job):
+        super().__init__(job)
         self.expect_shell = False
 
     def validate(self):
@@ -140,41 +114,44 @@ class GrubSequenceAction(BootHasMixin, RetryAction):
             "sequence", []
         )
         for sequence in sequences:
-            mapped = _grub_sequence_map(sequence)
-            if mapped[1]:
-                self.pipeline.add_action(mapped[0](itype=mapped[1]))
-            elif mapped[0]:
-                self.pipeline.add_action(mapped[0]())
+            action_type, itype = _grub_sequence_map(sequence)
+            if (
+                itype is not None
+                and action_type is not None
+                and issubclass(action_type, WaitFastBootInterrupt)
+            ):
+                self.pipeline.add_action(action_type(self.job, itype=itype))
+            elif action_type is not None:
+                self.pipeline.add_action(action_type(self.job))
         if self.has_prompts(parameters):
-            self.pipeline.add_action(AutoLoginAction())
+            self.pipeline.add_action(AutoLoginAction(self.job))
             if self.test_has_shell(parameters):
-                self.pipeline.add_action(ExpectShellSession())
+                self.pipeline.add_action(ExpectShellSession(self.job))
                 if "transfer_overlay" in parameters:
-                    self.pipeline.add_action(OverlayUnpack())
-                self.pipeline.add_action(ExportDeviceEnvironment())
+                    self.pipeline.add_action(OverlayUnpack(self.job))
+                self.pipeline.add_action(ExportDeviceEnvironment(self.job))
         else:
             if self.has_boot_finished(parameters):
                 self.logger.debug("Doing a boot without a shell (installer)")
-                self.pipeline.add_action(InstallerWait())
-                self.pipeline.add_action(PowerOff())
+                self.pipeline.add_action(InstallerWait(self.job))
+                self.pipeline.add_action(PowerOff(self.job))
 
 
 class GrubMainAction(BootHasMixin, RetryAction):
-
     name = "grub-main-action"
     description = "main grub boot action"
     summary = "run grub boot from power to system"
 
-    def __init__(self):
-        super().__init__()
+    def __init__(self, job: Job):
+        super().__init__(job)
         self.expect_shell = True
 
     def populate(self, parameters):
         self.expect_shell = parameters.get("expect_shell", True)
         self.pipeline = Pipeline(parent=self, job=self.job, parameters=parameters)
-        self.pipeline.add_action(BootloaderSecondaryMedia())
-        self.pipeline.add_action(BootloaderCommandOverlay())
-        self.pipeline.add_action(ConnectDevice())
+        self.pipeline.add_action(BootloaderSecondaryMedia(self.job))
+        self.pipeline.add_action(BootloaderCommandOverlay(self.job))
+        self.pipeline.add_action(ConnectDevice(self.job))
         # FIXME: reset_device is a hikey hack due to fastboot/OTG issues
         # remove as part of LAVA-940 - convert to use fastboot-sequence
         reset_device = (
@@ -184,36 +161,35 @@ class GrubMainAction(BootHasMixin, RetryAction):
         )
         if parameters["method"] == "grub-efi" and reset_device:
             # added unless the device specifies not to reset the device in grub.
-            self.pipeline.add_action(ResetDevice())
+            self.pipeline.add_action(ResetDevice(self.job))
         elif parameters["method"] == "grub":
-            self.pipeline.add_action(ResetDevice())
+            self.pipeline.add_action(ResetDevice(self.job))
         if parameters["method"] == "grub-efi":
-            self.pipeline.add_action(UEFIMenuInterrupt())
-            self.pipeline.add_action(GrubMenuSelector())
-        self.pipeline.add_action(BootloaderInterruptAction())
-        self.pipeline.add_action(BootloaderCommandsAction())
+            self.pipeline.add_action(UEFIMenuInterrupt(self.job))
+            self.pipeline.add_action(GrubMenuSelector(self.job))
+        self.pipeline.add_action(BootloaderInterruptAction(self.job))
+        self.pipeline.add_action(BootloaderCommandsAction(self.job))
         if self.has_prompts(parameters):
-            self.pipeline.add_action(AutoLoginAction())
+            self.pipeline.add_action(AutoLoginAction(self.job))
             if self.test_has_shell(parameters):
-                self.pipeline.add_action(ExpectShellSession())
+                self.pipeline.add_action(ExpectShellSession(self.job))
                 if "transfer_overlay" in parameters:
-                    self.pipeline.add_action(OverlayUnpack())
-                self.pipeline.add_action(ExportDeviceEnvironment())
+                    self.pipeline.add_action(OverlayUnpack(self.job))
+                self.pipeline.add_action(ExportDeviceEnvironment(self.job))
         else:
             if self.has_boot_finished(parameters):
                 self.logger.debug("Doing a boot without a shell (installer)")
-                self.pipeline.add_action(InstallerWait())
-                self.pipeline.add_action(PowerOff())
+                self.pipeline.add_action(InstallerWait(self.job))
+                self.pipeline.add_action(PowerOff(self.job))
 
 
 class GrubMenuSelector(UefiMenuSelector):
-
     name = "grub-efi-menu-selector"
     description = "select specified grub-efi menu items"
     summary = "select grub options in the efi menu"
 
-    def __init__(self):
-        super().__init__()
+    def __init__(self, job: Job):
+        super().__init__(job)
         self.selector.prompt = "Start:"
         self.commands = []
         self.boot_message = None
@@ -253,8 +229,8 @@ class InstallerWait(Action):
     description = "installer wait"
     summary = "wait for task to finish match arbitrary string"
 
-    def __init__(self):
-        super().__init__()
+    def __init__(self, job: Job):
+        super().__init__(job)
         self.type = "grub"
 
     def validate(self):
