@@ -2,24 +2,11 @@
 #
 # Author: Neil Williams <neil.williams@linaro.org>
 #
-# This file is part of LAVA Dispatcher.
-#
-# LAVA Dispatcher is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# LAVA Dispatcher is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along
-# with this program; if not, see <http://www.gnu.org/licenses>.
-
+# SPDX-License-Identifier: GPL-2.0-or-later
+from __future__ import annotations
 
 import os
+from typing import TYPE_CHECKING
 
 from lava_common.constants import INSTALLER_QUIET_MSG
 from lava_common.exceptions import ConfigurationError, JobError
@@ -31,39 +18,42 @@ from lava_dispatcher.shell import ExpectShellSession, ShellCommand, ShellSession
 from lava_dispatcher.utils.shell import which
 from lava_dispatcher.utils.strings import substitute
 
+if TYPE_CHECKING:
+    from lava_dispatcher.job import Job
+
 
 class BootIsoInstaller(Boot):
-
-    compatibility = 3
-
     @classmethod
-    def action(cls):
-        return BootIsoInstallerAction()
+    def action(cls, job: Job) -> Action:
+        return BootIsoInstallerAction(job)
 
     @classmethod
     def accepts(cls, device, parameters):
-        if "media" in parameters and parameters["media"] == "img":
-            if "method" in parameters and parameters["method"] == "qemu-iso":
-                return True, "accepted"
-        return False, '"media" was not in parameters or "media" was not "img"'
+        if parameters["method"] != "qemu-iso":
+            return False, '"method" was not "qemu-iso"'
+        if "media" not in parameters:
+            return False, '"media" was not in parameters'
+        if parameters["media"] != "img":
+            return False, '"media" was not "img"'
+
+        return True, "accepted"
 
 
 class BootIsoInstallerAction(RetryAction):
-
     name = "boot-installer-iso"
     description = "boot installer with preseed"
     summary = "boot installer iso image"
 
     def populate(self, parameters):
         self.pipeline = Pipeline(parent=self, job=self.job, parameters=parameters)
-        self.pipeline.add_action(IsoCommandLine())
-        self.pipeline.add_action(MonitorInstallerSession())
-        self.pipeline.add_action(IsoRebootAction())
+        self.pipeline.add_action(IsoCommandLine(self.job))
+        self.pipeline.add_action(MonitorInstallerSession(self.job))
+        self.pipeline.add_action(IsoRebootAction(self.job))
         # Add AutoLoginAction unconditionally as this action does nothing if
         # the configuration does not contain 'auto_login'
-        self.pipeline.add_action(AutoLoginAction())
-        self.pipeline.add_action(ExpectShellSession())
-        self.pipeline.add_action(ExportDeviceEnvironment())
+        self.pipeline.add_action(AutoLoginAction(self.job))
+        self.pipeline.add_action(ExpectShellSession(self.job))
+        self.pipeline.add_action(ExportDeviceEnvironment(self.job))
 
 
 class IsoCommandLine(Action):
@@ -142,13 +132,12 @@ class MonitorInstallerSession(Action):
     all that is needed is a prompt.
     """
 
-    compatibility = 3
     name = "monitor-installer-connection"
     description = "Monitor installer operation"
     summary = "Watch for error strings or end of install"
 
-    def __init__(self):
-        super().__init__()
+    def __init__(self, job: Job):
+        super().__init__(job)
         self.force_prompt = True
 
     def validate(self):
@@ -165,13 +154,12 @@ class MonitorInstallerSession(Action):
 
 
 class IsoRebootAction(Action):
-
     name = "reboot-into-installed"
     description = "reboot and login to the new system"
     summary = "reboot into installed image"
 
-    def __init__(self):
-        super().__init__()
+    def __init__(self, job: Job):
+        super().__init__(job)
         self.sub_command = None
 
     def validate(self):
