@@ -2,22 +2,10 @@
 #
 # Author: Neil Williams <neil.williams@linaro.org>
 #
-# This file is part of LAVA Dispatcher.
-#
-# LAVA Dispatcher is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# LAVA Dispatcher is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along
-# with this program; if not, see <http://www.gnu.org/licenses>.
+# SPDX-License-Identifier: GPL-2.0-or-later
+from __future__ import annotations
 
+from typing import TYPE_CHECKING
 
 from lava_common.constants import DISPATCHER_DOWNLOAD_DIR
 from lava_common.exceptions import JobError
@@ -28,6 +16,9 @@ from lava_dispatcher.logical import Boot, RetryAction
 from lava_dispatcher.shell import ExpectShellSession
 from lava_dispatcher.utils.network import dispatcher_ip
 
+if TYPE_CHECKING:
+    from lava_dispatcher.job import Job
+
 
 class BootKExec(Boot):
     """
@@ -35,18 +26,16 @@ class BootKExec(Boot):
     prepares the arguments to run kexec,
     """
 
-    compatibility = 1
-
     @classmethod
-    def action(cls):
-        return BootKexecAction()
+    def action(cls, job: Job) -> Action:
+        return BootKexecAction(job)
 
     @classmethod
     def accepts(cls, device, parameters):
-        if "method" in parameters:
-            if parameters["method"] == "kexec":
-                return True, "accepted"
-        return False, '"method" was not in parameters, or "method" was not "kexec"'
+        if parameters["method"] != "kexec":
+            return False, '"method" was not "kexec"'
+
+        return True, "accepted"
 
 
 class BootKexecAction(RetryAction):
@@ -60,14 +49,14 @@ class BootKexecAction(RetryAction):
 
     def populate(self, parameters):
         self.pipeline = Pipeline(parent=self, job=self.job, parameters=parameters)
-        self.pipeline.add_action(KexecAction())
+        self.pipeline.add_action(KexecAction(self.job))
         # Add AutoLoginAction unconditionally as this action does nothing if
         # the configuration does not contain 'auto_login'
-        self.pipeline.add_action(AutoLoginAction())
-        self.pipeline.add_action(ExpectShellSession())
+        self.pipeline.add_action(AutoLoginAction(self.job))
+        self.pipeline.add_action(ExpectShellSession(self.job))
         if "transfer_overlay" in parameters:
-            self.pipeline.add_action(OverlayUnpack())
-        self.pipeline.add_action(ExportDeviceEnvironment())
+            self.pipeline.add_action(OverlayUnpack(self.job))
+        self.pipeline.add_action(ExportDeviceEnvironment(self.job))
 
 
 class KexecAction(Action):
@@ -81,8 +70,8 @@ class KexecAction(Action):
     summary = "attempt to kexec new kernel"
     description = "call kexec with specified arguments"
 
-    def __init__(self):
-        super().__init__()
+    def __init__(self, job: Job):
+        super().__init__(job)
         self.command = ""
         self.load_command = ""
         self.deploy_commands = []
