@@ -2,21 +2,10 @@
 #
 # Author: Dean Birch <dean.birch@arm.com>
 #
-# This file is part of LAVA Dispatcher.
-#
-# LAVA Dispatcher is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# LAVA Dispatcher is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along
-# with this program; if not, see <http://www.gnu.org/licenses>.
+# SPDX-License-Identifier: GPL-2.0-or-later
+from __future__ import annotations
+
+from typing import TYPE_CHECKING
 
 from lava_dispatcher.action import Pipeline
 from lava_dispatcher.actions.boot import (
@@ -31,21 +20,20 @@ from lava_dispatcher.power import ResetDevice
 from lava_dispatcher.shell import ExpectShellSession
 from lava_dispatcher.utils.udev import WaitUSBSerialDeviceAction
 
+if TYPE_CHECKING:
+    from lava_dispatcher.action import Action
+    from lava_dispatcher.job import Job
+
 
 class Musca(Boot):
-
-    compatibility = 1
-
     @classmethod
-    def action(cls):
-        return MuscaBoot()
+    def action(cls, job: Job) -> Action:
+        return MuscaBoot(job)
 
     @classmethod
     def accepts(cls, device, parameters):
         if "musca" not in device["actions"]["boot"]["methods"]:
             return False, '"musca" was not in device configuration boot methods'
-        if "method" not in parameters:
-            return False, '"method" was not in parameters'
         if parameters["method"] != "musca":
             return False, '"method" was not "musca"'
         if "board_id" not in device:
@@ -54,20 +42,19 @@ class Musca(Boot):
 
 
 class MuscaBoot(BootHasMixin, RetryAction):
-
     name = "musca-boot"
     description = "power device and trigger software to run"
     summary = "power device and trigger software to run"
 
     def populate(self, parameters):
         self.pipeline = Pipeline(parent=self, job=self.job, parameters=parameters)
-        self.pipeline.add_action(ResetDevice())
-        self.pipeline.add_action(WaitUSBSerialDeviceAction())
-        self.pipeline.add_action(ConnectDevice())
+        self.pipeline.add_action(ResetDevice(self.job))
+        self.pipeline.add_action(WaitUSBSerialDeviceAction(self.job))
+        self.pipeline.add_action(ConnectDevice(self.job))
         if self.has_prompts(parameters):
-            self.pipeline.add_action(AutoLoginAction())
+            self.pipeline.add_action(AutoLoginAction(self.job))
         if self.test_has_shell(parameters):
-            self.pipeline.add_action(ExpectShellSession())
+            self.pipeline.add_action(ExpectShellSession(self.job))
             if "transfer_overlay" in parameters:
-                self.pipeline.add_action(OverlayUnpack())
-            self.pipeline.add_action(ExportDeviceEnvironment())
+                self.pipeline.add_action(OverlayUnpack(self.job))
+            self.pipeline.add_action(ExportDeviceEnvironment(self.job))
