@@ -2,23 +2,11 @@
 #
 # Author: Vincent Wan <vincent.wan@linaro.org>
 #
-# This file is part of LAVA Dispatcher.
-#
-# LAVA Dispatcher is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# LAVA Dispatcher is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along
-# with this program; if not, see <http://www.gnu.org/licenses>.
+# SPDX-License-Identifier: GPL-2.0-or-later
+from __future__ import annotations
 
 import os
+from typing import TYPE_CHECKING
 
 from lava_common.utils import binary_version
 from lava_dispatcher.action import Action, Pipeline
@@ -29,21 +17,19 @@ from lava_dispatcher.utils.shell import which
 from lava_dispatcher.utils.strings import substitute
 from lava_dispatcher.utils.udev import WaitDeviceBoardID
 
+if TYPE_CHECKING:
+    from lava_dispatcher.job import Job
+
 
 class OpenOCD(Boot):
-
-    compatibility = 4  # FIXME: change this to 5 and update test cases
-
     @classmethod
-    def action(cls):
-        return BootOpenOCDRetry()
+    def action(cls, job: Job) -> Action:
+        return BootOpenOCDRetry(job)
 
     @classmethod
     def accepts(cls, device, parameters):
         if "openocd" not in device["actions"]["boot"]["methods"]:
             return False, '"openocd" was not in the device configuration boot methods'
-        if "method" not in parameters:
-            return False, '"method" was not in parameters'
         if parameters["method"] != "openocd":
             return False, '"method" was not "openocd"'
         if "board_id" not in device:
@@ -52,7 +38,6 @@ class OpenOCD(Boot):
 
 
 class BootOpenOCDRetry(RetryAction):
-
     name = "boot-openocd-image"
     description = "boot openocd image using the command line interface"
     summary = "boot openocd image"
@@ -60,20 +45,21 @@ class BootOpenOCDRetry(RetryAction):
     def populate(self, parameters):
         self.pipeline = Pipeline(parent=self, job=self.job, parameters=parameters)
         if self.job.device.hard_reset_command:
-            self.pipeline.add_action(ResetDevice())
-            self.pipeline.add_action(WaitDeviceBoardID(self.job.device.get("board_id")))
-        self.pipeline.add_action(ConnectDevice())
-        self.pipeline.add_action(FlashOpenOCDAction())
+            self.pipeline.add_action(ResetDevice(self.job))
+            self.pipeline.add_action(
+                WaitDeviceBoardID(self.job, self.job.device.get("board_id"))
+            )
+        self.pipeline.add_action(ConnectDevice(self.job))
+        self.pipeline.add_action(FlashOpenOCDAction(self.job))
 
 
 class FlashOpenOCDAction(Action):
-
     name = "flash-openocd"
     description = "use openocd to flash the image"
     summary = "use openocd to flash the image"
 
-    def __init__(self):
-        super().__init__()
+    def __init__(self, job: Job):
+        super().__init__(job)
         self.base_command = []
         self.exec_list = []
 
