@@ -2,21 +2,10 @@
 #
 # Author: Tyler Baker <tyler.baker@linaro.org>
 #
-# This file is part of LAVA Dispatcher.
-#
-# LAVA Dispatcher is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# LAVA Dispatcher is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along
-# with this program; if not, see <http://www.gnu.org/licenses>.
+# SPDX-License-Identifier: GPL-2.0-or-later
+from __future__ import annotations
+
+from typing import TYPE_CHECKING
 
 from lava_common.utils import binary_version
 from lava_dispatcher.action import Action, JobError, Pipeline
@@ -27,21 +16,19 @@ from lava_dispatcher.utils.shell import which
 from lava_dispatcher.utils.strings import substitute
 from lava_dispatcher.utils.udev import WaitDeviceBoardID
 
+if TYPE_CHECKING:
+    from lava_dispatcher.job import Job
+
 
 class PyOCD(Boot):
-
-    compatibility = 4  # FIXME: change this to 5 and update test cases
-
     @classmethod
-    def action(cls):
-        return BootPyOCD()
+    def action(cls, job: Job) -> Action:
+        return BootPyOCD(job)
 
     @classmethod
     def accepts(cls, device, parameters):
         if "pyocd" not in device["actions"]["boot"]["methods"]:
             return False, '"pyocd" was not in the device configuration boot methods'
-        if "method" not in parameters:
-            return False, '"method" was not in parameters'
         if parameters["method"] != "pyocd":
             return False, '"method" was not "pyocd"'
         if "board_id" not in device:
@@ -50,18 +37,16 @@ class PyOCD(Boot):
 
 
 class BootPyOCD(RetryAction):
-
     name = "boot-pyocd-image"
     description = "boot pyocd image with retry"
     summary = "boot pyocd image with retry"
 
     def populate(self, parameters):
         self.pipeline = Pipeline(parent=self, job=self.job, parameters=parameters)
-        self.pipeline.add_action(BootPyOCDRetry())
+        self.pipeline.add_action(BootPyOCDRetry(self.job))
 
 
 class BootPyOCDRetry(RetryAction):
-
     name = "boot-pyocd-image"
     description = "boot pyocd image using the command line interface"
     summary = "boot pyocd image"
@@ -72,23 +57,24 @@ class BootPyOCDRetry(RetryAction):
             "parameters"
         ]
         if self.job.device.hard_reset_command:
-            self.pipeline.add_action(ResetDevice())
-            self.pipeline.add_action(WaitDeviceBoardID(self.job.device.get("board_id")))
+            self.pipeline.add_action(ResetDevice(self.job))
+            self.pipeline.add_action(
+                WaitDeviceBoardID(self.job, self.job.device.get("board_id"))
+            )
         if method_params.get("connect_before_flash", False):
-            self.pipeline.add_action(ConnectDevice())
-        self.pipeline.add_action(FlashPyOCDAction())
+            self.pipeline.add_action(ConnectDevice(self.job))
+        self.pipeline.add_action(FlashPyOCDAction(self.job))
         if not method_params.get("connect_before_flash", False):
-            self.pipeline.add_action(ConnectDevice())
+            self.pipeline.add_action(ConnectDevice(self.job))
 
 
 class FlashPyOCDAction(Action):
-
     name = "flash-pyocd"
     description = "flash pyocd to boot the image"
     summary = "flash pyocd to boot the image"
 
-    def __init__(self):
-        super().__init__()
+    def __init__(self, job: Job):
+        super().__init__(job)
         self.base_command = []
         self.exec_list = []
 
