@@ -2,25 +2,13 @@
 #
 # Author: Neil Williams <neil.williams@linaro.org>
 #
-# This file is part of LAVA Dispatcher.
-#
-# LAVA Dispatcher is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# LAVA Dispatcher is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along
-# with this program; if not, see <http://www.gnu.org/licenses>.
+# SPDX-License-Identifier: GPL-2.0-or-later
+from __future__ import annotations
 
 import os
 import shlex
 import subprocess
+from typing import TYPE_CHECKING
 
 from lava_common.constants import DISPATCHER_DOWNLOAD_DIR, SYS_CLASS_KVM
 from lava_common.exceptions import JobError
@@ -36,6 +24,9 @@ from lava_dispatcher.utils.network import dispatcher_ip
 from lava_dispatcher.utils.shell import which
 from lava_dispatcher.utils.strings import substitute
 
+if TYPE_CHECKING:
+    from lava_dispatcher.job import Job
+
 
 class BootQEMU(Boot):
     """
@@ -47,11 +38,9 @@ class BootQEMU(Boot):
     hand this pexpect wrapper to subsequent actions as a shell connection.
     """
 
-    compatibility = 4
-
     @classmethod
-    def action(cls):
-        return BootQEMUImageAction()
+    def action(cls, job: Job) -> Action:
+        return BootQEMUImageAction(job)
 
     @classmethod
     def accepts(cls, device, parameters):
@@ -61,44 +50,39 @@ class BootQEMU(Boot):
                 False,
                 '"qemu" or "qemu-nfs" was not in the device configuration boot methods',
             )
-        if "method" not in parameters:
-            return False, '"method" was not in parameters'
         if parameters["method"] not in ["qemu", "qemu-nfs", "monitor"]:
             return False, '"method" was not "qemu" or "qemu-nfs"'
         return True, "accepted"
 
 
 class BootQEMUImageAction(BootHasMixin, RetryAction):
-
     name = "boot-image-retry"
     description = "boot image with retry"
     summary = "boot with retry"
 
     def populate(self, parameters):
         self.pipeline = Pipeline(parent=self, job=self.job, parameters=parameters)
-        self.pipeline.add_action(BootQemuRetry())
+        self.pipeline.add_action(BootQemuRetry(self.job))
         if self.has_prompts(parameters):
-            self.pipeline.add_action(AutoLoginAction())
+            self.pipeline.add_action(AutoLoginAction(self.job))
             if self.test_has_shell(parameters):
-                self.pipeline.add_action(ExpectShellSession())
+                self.pipeline.add_action(ExpectShellSession(self.job))
                 if "transfer_overlay" in parameters:
-                    self.pipeline.add_action(OverlayUnpack())
-                self.pipeline.add_action(ExportDeviceEnvironment())
+                    self.pipeline.add_action(OverlayUnpack(self.job))
+                self.pipeline.add_action(ExportDeviceEnvironment(self.job))
 
 
 class BootQemuRetry(RetryAction):
-
     name = "boot-qemu-image"
     description = "boot image using QEMU command line"
     summary = "boot QEMU image"
 
     def populate(self, parameters):
         self.pipeline = Pipeline(parent=self, job=self.job, parameters=parameters)
-        self.pipeline.add_action(CallQemuAction())
+        self.pipeline.add_action(CallQemuAction(self.job))
 
 
 class CallQemuAction(Action):
-
     name = "execute-qemu"
     description = "call qemu to boot the image"
     summary = "execute qemu to boot the image"
@@ -106,8 +90,8 @@ class CallQemuAction(Action):
     session_class = QemuSession
     shell_class = ShellCommand
 
-    def __init__(self):
-        super().__init__()
+    def __init__(self, job: Job):
+        super().__init__(job)
         self.base_sub_command = []
         self.docker = None
         self.sub_command = []
@@ -155,18 +139,21 @@ class CallQemuAction(Action):
     def get_raw_version(self, architecture):
         if "docker" in self.parameters:
             docker = DockerRun.from_parameters(self.parameters["docker"], self.job)
-            docker.run(
+            ver_strs = docker.run(
                 *shlex.split(
                     "qemu-system-%s --version" % self.get_qemu_arch(architecture)
                 ),
-                action=self
+                action=self,
+                capture=True,
+            )
+        else:
+            ver_strs = subprocess.check_output(
+                (f"qemu-system-{architecture}", "--version"),
+                encoding="utf-8",
+                errors="replace",
             )
-            return True
-        ver_strs = subprocess.check_output(
-            ("qemu-system-%s" % architecture, "--version")
-        )
         # line is QEMU emulator version xxxx
-        ver_str = ver_strs.split()[3].decode("utf-8", errors="replace")
+        ver_str = ver_strs.split()[3]
         arch_str = (
             subprocess.check_output(("uname", "-m"))
             .strip()
@@ -367,10 +354,11 @@ class CallQemuAction(Action):
             self.docker = docker = DockerRun.from_parameters(
                 self.parameters["docker"], self.job
             )
-            docker.name(
-                "lava-docker-qemu-%s-%s-" % (self.job.job_id, self.level),
-                random_suffix=True,
-            )
+            if not self.parameters["docker"].get("container_name"):
+                docker.name(
+                    "lava-docker-qemu-%s-%s-" % (self.job.job_id, self.level),
+                    random_suffix=True,
+                )
             docker.interactive()
             docker.tty()
             if "QEMU_AUDIO_DRV" in os.environ:
