@@ -2,47 +2,34 @@
 #
 # Author: Neil Williams <neil.williams@linaro.org>
 #
-# This file is part of LAVA.
-#
-# LAVA is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# LAVA is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along
-# with this program; if not, see <http://www.gnu.org/licenses>.
+# SPDX-License-Identifier: GPL-2.0-or-later
+from __future__ import annotations
 
+from typing import TYPE_CHECKING
 
 from lava_common.exceptions import InfrastructureError
 from lava_dispatcher.action import Action, Pipeline
 from lava_dispatcher.logical import Boot
 from lava_dispatcher.power import PowerOff, PowerOn
 
+if TYPE_CHECKING:
+    from lava_dispatcher.job import Job
+
 
 class RecoveryBoot(Boot):
-
-    compatibility = 4
-
     @classmethod
-    def action(cls):
-        return RecoveryBootAction()
+    def action(cls, job: Job) -> Action:
+        return RecoveryBootAction(job)
 
     @classmethod
     def accepts(cls, device, parameters):
-        if "method" in parameters:
-            if parameters["method"] == "recovery":
-                return True, "accepted"
-        return False, 'boot "method" was not "recovery"'
+        if parameters["method"] != "recovery":
+            return False, 'boot "method" was not "recovery"'
+
+        return True, "accepted"
 
 
 class RecoveryBootAction(Action):
-
     name = "recovery-boot"
     description = "handle entering and leaving recovery mode"
     summary = "boot into or out of recovery mode"
@@ -59,25 +46,28 @@ class RecoveryBootAction(Action):
         self.pipeline = Pipeline(parent=self, job=self.job, parameters=parameters)
         if parameters["commands"] == "recovery":
             # only switch into recovery mode with power off.
-            self.pipeline.add_action(PowerOff())
-            self.pipeline.add_action(SwitchRecoveryCommand(mode="recovery_mode"))
-            self.pipeline.add_action(PowerOn())
+            self.pipeline.add_action(PowerOff(self.job))
+            self.pipeline.add_action(
+                SwitchRecoveryCommand(self.job, mode="recovery_mode")
+            )
+            self.pipeline.add_action(PowerOn(self.job))
         elif parameters["commands"] == "exit":
-            self.pipeline.add_action(PowerOff())
-            self.pipeline.add_action(SwitchRecoveryCommand(mode="recovery_exit"))
-            self.pipeline.add_action(PowerOn())
+            self.pipeline.add_action(PowerOff(self.job))
+            self.pipeline.add_action(
+                SwitchRecoveryCommand(self.job, mode="recovery_exit")
+            )
+            self.pipeline.add_action(PowerOn(self.job))
         else:
             self.errors = "Invalid recovery command"
 
 
 class SwitchRecoveryCommand(Action):
-
     name = "switch-recovery"
     description = "call commands to switch device into and out of recovery"
     summary = "execute recovery mode commands"
 
-    def __init__(self, mode):
-        super().__init__()
+    def __init__(self, job: Job, mode):
+        super().__init__(job)
         self.recovery = []
         self.mode = mode
 
