@@ -2,21 +2,10 @@
 #
 # Author: Dean Birch <dean.birch@linaro.org>
 #
-# This file is part of LAVA Dispatcher.
-#
-# LAVA Dispatcher is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# LAVA Dispatcher is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along
-# with this program; if not, see <http://www.gnu.org/licenses>.
+# SPDX-License-Identifier: GPL-2.0-or-later
+from __future__ import annotations
+
+from typing import TYPE_CHECKING
 
 from lava_common.constants import UEFI_LINE_SEPARATOR
 from lava_dispatcher.action import Pipeline
@@ -34,14 +23,15 @@ from lava_dispatcher.menus.menus import MenuConnect, MenuInterrupt
 from lava_dispatcher.power import ResetDevice
 from lava_dispatcher.shell import ExpectShellSession
 
+if TYPE_CHECKING:
+    from lava_dispatcher.action import Action
+    from lava_dispatcher.job import Job
+
 
 class UefiShell(Boot):
-
-    compatibility = 3
-
     @classmethod
-    def action(cls):
-        return UefiShellAction()
+    def action(cls, job: Job) -> Action:
+        return UefiShellAction(job)
 
     @classmethod
     def accepts(cls, device, parameters):
@@ -68,13 +58,12 @@ class UefiShell(Boot):
 
 
 class UefiShellAction(BootHasMixin, RetryAction):
-
     name = "uefi-shell-main-action"
     description = "UEFI shell boot action"
     summary = "run UEFI shell to system"
 
-    def __init__(self):
-        super().__init__()
+    def __init__(self, job: Job):
+        super().__init__(job)
         self.shell_menu = []
 
     def _skip_menu(self, parameters):
@@ -95,23 +84,23 @@ class UefiShellAction(BootHasMixin, RetryAction):
 
     def populate(self, parameters):
         self.pipeline = Pipeline(parent=self, job=self.job, parameters=parameters)
-        self.pipeline.add_action(BootloaderCommandOverlay())
-        self.pipeline.add_action(MenuConnect())
-        self.pipeline.add_action(ResetDevice())
+        self.pipeline.add_action(BootloaderCommandOverlay(self.job))
+        self.pipeline.add_action(MenuConnect(self.job))
+        self.pipeline.add_action(ResetDevice(self.job))
         # Newer firmware often needs no menu interaction, just press to drop to shell
         if not self._skip_menu(parameters):
             # Some older firmware, UEFI Shell has to be selected from a menu.
-            self.pipeline.add_action(UefiShellMenuInterrupt())
-            self.pipeline.add_action(UefiShellMenuSelector())
-        self.pipeline.add_action(UefiShellInterrupt())
-        self.pipeline.add_action(UefiBootloaderCommandsAction())
+            self.pipeline.add_action(UefiShellMenuInterrupt(self.job))
+            self.pipeline.add_action(UefiShellMenuSelector(self.job))
+        self.pipeline.add_action(UefiShellInterrupt(self.job))
+        self.pipeline.add_action(UefiBootloaderCommandsAction(self.job))
         if self.has_prompts(parameters):
-            self.pipeline.add_action(AutoLoginAction())
+            self.pipeline.add_action(AutoLoginAction(self.job))
             if self.test_has_shell(parameters):
-                self.pipeline.add_action(ExpectShellSession())
+                self.pipeline.add_action(ExpectShellSession(self.job))
                 if "transfer_overlay" in parameters:
-                    self.pipeline.add_action(OverlayUnpack())
-                self.pipeline.add_action(ExportDeviceEnvironment())
+                    self.pipeline.add_action(OverlayUnpack(self.job))
+                self.pipeline.add_action(ExportDeviceEnvironment(self.job))
 
     def run(self, connection, max_end_time):
         connection = super().run(connection, max_end_time)
@@ -133,13 +122,12 @@ class UefiShellAction(BootHasMixin, RetryAction):
 
 
 class UefiShellMenuInterrupt(UEFIMenuInterrupt):
-
     name = "uefi-shell-menu-interrupt"
     description = "interrupt default boot and to menu"
     summary = "interrupt default boot and to menu"
 
-    def __init__(self):
-        super().__init__()
+    def __init__(self, job: Job):
+        super().__init__(job)
         # Take parameters from the uefi method, not uefi menu.
         self.method = "uefi"
 
@@ -185,8 +173,8 @@ class UefiShellMenuSelector(UefiMenuSelector):
     description = "select uefi menu items to drop to a uefi shell"
     summary = "use uefi menu to drop to shell"
 
-    def __init__(self):
-        super().__init__()
+    def __init__(self, job: Job):
+        super().__init__(job)
         # Take parameters from the uefi method, not uefi menu.
         self.method_name = "uefi"
         # Default menu command name: drop to shell
