@@ -2,22 +2,10 @@
 #
 # Author: Neil Williams <neil.williams@linaro.org>
 #
-# This file is part of LAVA Dispatcher.
-#
-# LAVA Dispatcher is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# LAVA Dispatcher is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along
-# with this program; if not, see <http://www.gnu.org/licenses>.
+# SPDX-License-Identifier: GPL-2.0-or-later
+from __future__ import annotations
 
+from typing import TYPE_CHECKING
 
 from lava_common.constants import (
     DEFAULT_UEFI_LABEL_CLASS,
@@ -40,6 +28,9 @@ from lava_dispatcher.protocols.lxc import LxcProtocol
 from lava_dispatcher.utils.network import dispatcher_ip
 from lava_dispatcher.utils.strings import substitute
 
+if TYPE_CHECKING:
+    from lava_dispatcher.job import Job
+
 
 class UefiMenu(Boot):
     """
@@ -49,8 +40,8 @@ class UefiMenu(Boot):
     """
 
     @classmethod
-    def action(cls):
-        return UefiMenuAction()
+    def action(cls, job: Job) -> Action:
+        return UefiMenuAction(job)
 
     @classmethod
     def accepts(cls, device, parameters):
@@ -69,14 +60,13 @@ class UefiMenu(Boot):
 
 
 class UEFIMenuInterrupt(MenuInterrupt):
-
     name = "uefi-menu-interrupt"
     description = "interrupt for uefi menu"
     summary = "interrupt for uefi menu"
     timeout_exception = InfrastructureError
 
-    def __init__(self):
-        super().__init__()
+    def __init__(self, job: Job):
+        super().__init__(job)
         self.params = None
         self.method = "uefi-menu"
 
@@ -102,13 +92,12 @@ class UEFIMenuInterrupt(MenuInterrupt):
 
 
 class UefiMenuSelector(SelectorMenuAction):
-
     name = "uefi-menu-selector"
     description = "select specified uefi menu items"
     summary = "select options in the uefi menu"
 
-    def __init__(self):
-        super().__init__()
+    def __init__(self, job: Job):
+        super().__init__(job)
         self.selector.prompt = "Start:"
         self.method_name = "uefi-menu"
         self.commands = []
@@ -219,13 +208,12 @@ class UefiMenuSelector(SelectorMenuAction):
 
 
 class UefiSubstituteCommands(Action):
-
     name = "uefi-commands"
     description = "set job-specific variables into the uefi menu commands"
     summary = "substitute job values into uefi commands"
 
-    def __init__(self):
-        super().__init__()
+    def __init__(self, job: Job):
+        super().__init__(job)
         self.items = None
 
     def validate(self):
@@ -293,13 +281,12 @@ class UefiSubstituteCommands(Action):
 
 
 class UefiMenuAction(RetryAction):
-
     name = "uefi-menu-action"
     description = "interrupt and select uefi menu items"
     summary = "interact with uefi menu"
 
-    def __init__(self):
-        super().__init__()
+    def __init__(self, job: Job):
+        super().__init__(job)
         self.method = "uefi-menu"
 
     def validate(self):
@@ -316,18 +303,18 @@ class UefiMenuAction(RetryAction):
     def populate(self, parameters):
         self.pipeline = Pipeline(parent=self, job=self.job, parameters=parameters)
         if "commands" in parameters and "fastboot" in parameters["commands"]:
-            self.pipeline.add_action(UefiSubstituteCommands())
-            self.pipeline.add_action(UEFIMenuInterrupt())
-            self.pipeline.add_action(UefiMenuSelector())
-            self.pipeline.add_action(MenuReset())
-            self.pipeline.add_action(AutoLoginAction())
-            self.pipeline.add_action(ExportDeviceEnvironment())
+            self.pipeline.add_action(UefiSubstituteCommands(self.job))
+            self.pipeline.add_action(UEFIMenuInterrupt(self.job))
+            self.pipeline.add_action(UefiMenuSelector(self.job))
+            self.pipeline.add_action(MenuReset(self.job))
+            self.pipeline.add_action(AutoLoginAction(self.job))
+            self.pipeline.add_action(ExportDeviceEnvironment(self.job))
         else:
-            self.pipeline.add_action(UefiSubstituteCommands())
-            self.pipeline.add_action(MenuConnect())
-            self.pipeline.add_action(ResetDevice())
-            self.pipeline.add_action(UEFIMenuInterrupt())
-            self.pipeline.add_action(UefiMenuSelector())
-            self.pipeline.add_action(MenuReset())
-            self.pipeline.add_action(AutoLoginAction())
-            self.pipeline.add_action(ExportDeviceEnvironment())
+            self.pipeline.add_action(UefiSubstituteCommands(self.job))
+            self.pipeline.add_action(MenuConnect(self.job))
+            self.pipeline.add_action(ResetDevice(self.job))
+            self.pipeline.add_action(UEFIMenuInterrupt(self.job))
+            self.pipeline.add_action(UefiMenuSelector(self.job))
+            self.pipeline.add_action(MenuReset(self.job))
+            self.pipeline.add_action(AutoLoginAction(self.job))
+            self.pipeline.add_action(ExportDeviceEnvironment(self.job))
