@@ -1,32 +1,21 @@
-# Copyright 2019-2022 NXP
+# Copyright 2019-2024 NXP
 #
 # Author: Thomas Mahe <thomas.mahe@nxp.com>
 #         Franck Lenormand <franck.lenormand@nxp.com>
 #         Gopalakrishnan RAJINE ANAND <gopalakrishnan.rajineanand@nxp.com>
+#         Larry Shen <larry.shen@nxp.com>
 #
-# This file is part of LAVA Dispatcher.
-#
-# LAVA Dispatcher is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# LAVA Dispatcher is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along
-# with this program; if not, see <http://www.gnu.org/licenses>.
+# SPDX-License-Identifier: GPL-2.0-or-later
 
 # List just the subclasses supported for this base strategy
 # imported by the parser to populate the list of subclasses.
+from __future__ import annotations
 
 import re
 import time
+from typing import TYPE_CHECKING
 
-from lava_common.exceptions import ConfigurationError, JobError
+from lava_common.exceptions import ConfigurationError, InfrastructureError, JobError
 from lava_dispatcher.action import Action, Pipeline
 from lava_dispatcher.actions.boot import BootloaderCommandOverlay
 from lava_dispatcher.actions.boot.bootloader import BootBootloaderAction
@@ -36,6 +25,9 @@ from lava_dispatcher.power import PowerOff, ResetDevice
 from lava_dispatcher.utils.strings import safe_dict_format
 from lava_dispatcher.utils.uuu import OptionalContainerUuuAction
 
+if TYPE_CHECKING:
+    from lava_dispatcher.job import Job
+
 
 class CheckSerialDownloadMode(OptionalContainerUuuAction):
     name = "check-serial-availability"
@@ -66,15 +58,21 @@ class CheckSerialDownloadMode(OptionalContainerUuuAction):
         usb_otg_path = self.job.device["actions"]["boot"]["methods"]["uuu"]["options"][
             "usb_otg_path"
         ]
-
+        path_args = " -m ".join(usb_otg_path)
         cmd = "{} --preserve-status 10 {} -m {} {}".format(
-            self.linux_timeout, self.uuu, usb_otg_path, boot
+            self.linux_timeout, self.uuu, path_args, boot
         )
 
         self.maybe_copy_to_container(boot)
 
         # Board available if cmd terminates correctly
-        return self.run_uuu(cmd.split(" "), allow_fail=True) == 0
+        ret = self.run_uuu(cmd.split(" "), allow_fail=True)
+        if ret == 0:
+            return True
+        elif ret == 143:
+            return False
+        else:
+            raise self.command_exception(f"Fail UUUBootAction on cmd : {cmd}")
 
     def run(self, connection, max_end_time):
         connection = super().run(connection, max_end_time)
@@ -108,11 +106,9 @@ class UUUBoot(Boot):
 
     """
 
-    compatibility = 1
-
     @classmethod
-    def action(cls):
-        return UUUBootRetryAction()
+    def action(cls, job: Job) -> Action:
+        return UUUBootRetryAction(job)
 
     @classmethod
     def accepts(cls, device, parameters):
@@ -156,17 +152,21 @@ class BootBootloaderCorruptBootMediaAction(Action):
             "commands": SD_ERASE_CMDS,
             "prompts": ["=>"],
         }
-        self.pipeline = Pipeline(parent=self, job=self.job, parameters=u_boot_params)
+        self.pipeline = Pipeline(
+            parent=self, job=self.job, parameters={**parameters, **u_boot_params}
+        )
         if power_off:
-            self.pipeline.add_action(PowerOff())
-        self.pipeline.add_action(ConnectDevice())
+            self.pipeline.add_action(PowerOff(self.job))
+        self.pipeline.add_action(ConnectDevice(self.job))
         self.pipeline.add_action(
             BootloaderCommandOverlay(
-                method=u_boot_params["bootloader"], commands=u_boot_params["commands"]
+                self.job,
+                method=u_boot_params["bootloader"],
+                commands=u_boot_params["commands"],
             )
         )
-        self.pipeline.add_action(BootBootloaderAction())
-        self.pipeline.add_action(DisconnectDevice())
+        self.pipeline.add_action(BootBootloaderAction(self.job))
+        self.pipeline.add_action(DisconnectDevice(self.job))
 
     def run(self, connection, max_end_time):
         otg_available = self.get_namespace_data(
@@ -178,13 +178,79 @@ class BootBootloaderCorruptBootMediaAction(Action):
             return super().run(connection, max_end_time)
 
 
+class CheckBootloaderValidAction(Action):
+    name = "check-bootloader-valid"
+    description = "Store in 'board_valid_check' namespace_data"
+    summary = "Store in 'board_valid_check' namespace_data"
+
+    def populate(self, parameters):
+        self.parameters = parameters
+
+        u_boot_params = {
+            "method": "bootloader",
+            "bootloader": "u-boot",
+            "commands": [],
+            "prompts": [
+                self.job.device["actions"]["boot"]["methods"]["u-boot"]["parameters"][
+                    "bootloader_prompt"
+                ]
+            ],
+        }
+        u_boot_params = {**parameters, **u_boot_params}
+        u_boot_params.setdefault("timeouts", {}).setdefault(
+            "bootloader-interrupt", {"seconds": 15}
+        )
+        self.pipeline = Pipeline(parent=self, job=self.job, parameters=u_boot_params)
+        self.pipeline.add_action(ConnectDevice(self.job))
+        self.pipeline.add_action(
+            BootloaderCommandOverlay(
+                self.job,
+                method=u_boot_params["bootloader"],
+                commands=u_boot_params["commands"],
+            )
+        )
+        self.pipeline.add_action(BootBootloaderAction(self.job))
+        self.pipeline.add_action(DisconnectDevice(self.job))
+
+    def run(self, connection, max_end_time):
+        try:
+            con = super().run(connection, max_end_time)
+            self.set_namespace_data(
+                action="boot", label="uuu", key="bootloader_valid_check", value=True
+            )
+            return con
+        except InfrastructureError:
+            self.logger.info("Start uuu flash due to no valid bootloader")
+            self.set_namespace_data(
+                action="boot", label="uuu", key="bootloader_valid_check", value=False
+            )
+            return connection
+
+
 class UUUBootRetryAction(RetryAction):
     """
-    Wraps the Retry Action to allow for actions which precede
-    the reset, e.g. Connect.
+    Wraps the retryaction to allow for bootloader valid check
+    if `skip_uuu_if_bootloader_valid` specified
     """
 
     name = "uuu-boot-retry"
+    description = "wrap action to allow bootloader valid check"
+    summary = "check valid bootloader before uuu"
+
+    def populate(self, parameters):
+        self.pipeline = Pipeline(parent=self, job=self.job, parameters=parameters)
+        if parameters.get("skip_uuu_if_bootloader_valid"):
+            self.pipeline.add_action(CheckBootloaderValidAction(self.job))
+        self.pipeline.add_action(UUUAction(self.job))
+
+
+class UUUAction(Action):
+    """
+    Wraps the action to allow for actions which precede
+    the reset, e.g. Connect.
+    """
+
+    name = "uuu"
     description = "Boot the board using uboot and perform uuu commands"
     summary = "Pass uuu commands"
 
@@ -195,18 +261,47 @@ class UUUBootRetryAction(RetryAction):
         ] = self.eval_otg_path()
         self.job.device["actions"]["boot"]["methods"]["uuu"]["options"][
             "has_bcu_commands"
-        ] = self.has_bcu_commands(parameters)
-        if self.has_bcu_commands(parameters):
+        ] = self.has_protocol("bcu", parameters)
+        if self.has_protocol("bcu", parameters):
             self.job.device["actions"]["boot"]["methods"]["uuu"]["options"][
                 "bcu_board_id"
             ] = self.eval_bcu_board_id()
         self.pipeline = Pipeline(parent=self, job=self.job, parameters=parameters)
-        self.pipeline.add_action(ResetDevice())
-        self.pipeline.add_action(CheckSerialDownloadMode())
-        self.pipeline.add_action(BootBootloaderCorruptBootMediaAction())
-        self.pipeline.add_action(ResetDevice())
-        self.pipeline.add_action(UUUBootAction(), parameters=parameters)
-        self.pipeline.add_action(ConnectDevice())
+        self.pipeline.add_action(DisconnectDevice(self.job))
+        self.pipeline.add_action(ResetDevice(self.job))
+
+        # Serial availability check is skipped if
+        #  bcu reset usb is the first command
+        #  or bcu is the only protocol used in uuu commands block
+        if self.first_command_is_bcu_reset_usb(parameters) or self.has_only_protocol(
+            "bcu", parameters
+        ):
+            self.logger.info(
+                """\
+First command is 'bcu reset usb' or commands blocks contain only bcu commands only.
+Following actions will be skipped :
+    - CheckSerialDownloadMode
+    - BootBootloaderCorruptBootMediaAction
+    - ResetDevice"""
+            )
+        else:
+            self.pipeline.add_action(CheckSerialDownloadMode(self.job))
+            self.pipeline.add_action(BootBootloaderCorruptBootMediaAction(self.job))
+            self.pipeline.add_action(ResetDevice(self.job))
+
+        self.pipeline.add_action(ConnectDevice(self.job))
+        self.pipeline.add_action(UUUBootAction(self.job), parameters=parameters)
+        self.pipeline.add_action(DisconnectDevice(self.job))
+
+    def run(self, connection, max_end_time):
+        bootloader_valid = self.get_namespace_data(
+            action="boot", label="uuu", key="bootloader_valid_check"
+        )
+        if bootloader_valid:
+            self.logger.info("Skip uuu flash due to valid bootloader")
+            return connection
+        else:
+            return super().run(connection, max_end_time)
 
     def eval_otg_path(self):
         uuu_options = self.job.device["actions"]["boot"]["methods"]["uuu"]["options"]
@@ -214,14 +309,25 @@ class UUUBootRetryAction(RetryAction):
         # Match example : "2:112"
         otg_path_fsm = re.compile(r"\d+:\d+")
 
-        if otg_path_fsm.fullmatch(uuu_options.get("usb_otg_path")):
-            return uuu_options.get("usb_otg_path")
+        # from uuu_otg_path device parameter
+        raw_option = uuu_options.get("usb_otg_path")
+        if raw_option:
+            if isinstance(raw_option, str):
+                otg_paths = [raw_option]
+            else:
+                # uuu_otg_path accept only List[str] or str
+                otg_paths = raw_option
+
+            # All path are matching with uuu path format
+            if None not in map(lambda p: otg_path_fsm.fullmatch(p), otg_paths):
+                return otg_paths
 
         if uuu_options.get("usb_otg_path_command") is None:
             raise JobError(
                 "uuu_usb_otg_path '{}' does not match with uuu path pattern and 'uuu_usb_otg_path_command' not "
                 "defined in device".format(uuu_options.get("usb_otg_path"))
             )
+        # Retrieve usb_otg_path from command execution
         else:
             usb_otg_path_command = uuu_options.get("usb_otg_path_command")
             cmd = usb_otg_path_command
@@ -229,26 +335,50 @@ class UUUBootRetryAction(RetryAction):
                 "Retrieving 'usb_otg_path' using command : '%s'", " ".join(cmd)
             )
 
-            uuu_path_expected = self.parsed_command(cmd).strip()
+            command_output = self.parsed_command(cmd)
+            uuu_otg_paths = list(map(lambda p: p.strip(), command_output.splitlines()))
 
-            if otg_path_fsm.fullmatch(uuu_path_expected):
-                self.logger.info("uuu_otg_path matched : %s", uuu_path_expected)
-                return uuu_path_expected
+            # All path are matching with uuu path format
+            if None not in map(lambda p: otg_path_fsm.fullmatch(p), uuu_otg_paths):
+                self.logger.info("uuu_otg_path matched : %s", uuu_otg_paths)
+                return uuu_otg_paths
 
-            raise JobError(
-                "Unable to parse uuu_usb_otg_path from command '{}'".format(cmd)
-            )
+            raise JobError(f"Unable to parse uuu_usb_otg_path from command '{cmd}'")
 
-    def has_bcu_commands(self, parameters):
+    def _get_protocols_from_commands(self, parameters):
         """
-        Checks if the bcu is present in the commands list. Returns true if bcu is present
+        Return typing.Set of protocols defined in commands block
+        eg: {"uuu", "bcu"}
         """
-        cmds = parameters["commands"]
-        for commands in cmds:
-            for protocol, cmd in commands.items():
-                if protocol == "bcu":
-                    return True
-        return False
+        return {proto for cmd in parameters.get("commands", []) for proto in cmd.keys()}
+
+    def has_protocol(self, protocol, parameters):
+        """
+        Return: True if given protocol is present in commands block
+                False otherwise
+        """
+        protocols = self._get_protocols_from_commands(parameters)
+        return protocol in protocols
+
+    def has_only_protocol(self, protocol, parameters):
+        """
+        Return: True if given protocol is the only one present in commands block
+                False otherwise
+        """
+        protocols = self._get_protocols_from_commands(parameters)
+        return protocol in protocols and len(protocols) == 1
+
+    def first_command_is_bcu_reset_usb(self, parameters):
+        """
+        Return: True if "bcu: reset usb" is the first command listed in commands block
+                False otherwise
+        """
+        # First command
+        ((proto, command),) = parameters.get("commands")[0].items()
+        if proto != "bcu":
+            return False
+
+        return ["reset", "usb"] == list(map(str.strip, command.split(" ")))
 
     def eval_bcu_board_id(self):
         uuu_options = self.job.device["actions"]["boot"]["methods"]["uuu"]["options"]
@@ -281,9 +411,7 @@ class UUUBootRetryAction(RetryAction):
             self.logger.info("Matched bcu_board_id : %s", bcu_id_expected)
             return bcu_id_expected
 
-        raise JobError(
-            "Unable to parse bcu_id from command '{}'".format(bcu_board_id_command)
-        )
+        raise JobError(f"Unable to parse bcu_id from command '{bcu_board_id_command}'")
 
 
 class UUUBootAction(OptionalContainerUuuAction):
@@ -298,6 +426,7 @@ class UUUBootAction(OptionalContainerUuuAction):
             "has_bcu_commands"
         ]:
             self.bcu = self.which("bcu")
+        self.cleanup_required = False
 
     def validate(self):
         super().validate()
@@ -311,12 +440,12 @@ class UUUBootAction(OptionalContainerUuuAction):
             "usb_otg_path"
         ]
         uuu_cmds = self.parameters["commands"]
-        bcu_board_name = self.job.device["actions"]["boot"]["methods"]["uuu"][
+        self.bcu_board_name = self.job.device["actions"]["boot"]["methods"]["uuu"][
             "options"
         ]["bcu_board_name"]
-        bcu_board_id = self.job.device["actions"]["boot"]["methods"]["uuu"]["options"][
-            "bcu_board_id"
-        ]
+        self.bcu_board_id = self.job.device["actions"]["boot"]["methods"]["uuu"][
+            "options"
+        ]["bcu_board_id"]
         images_name = self.get_namespace_data(
             action="uuu-deploy", label="uuu-images", key="images_names"
         )
@@ -335,7 +464,7 @@ class UUUBootAction(OptionalContainerUuuAction):
         for dico in uuu_cmds:
             for protocol, cmd in dico.items():
                 cmd = safe_dict_format(cmd, templates)
-                uuu_cmd_list.append("{}: {}".format(protocol, cmd))
+                uuu_cmd_list.append(f"{protocol}: {cmd}")
 
         uuu_cmds = uuu_cmd_list
 
@@ -343,11 +472,16 @@ class UUUBootAction(OptionalContainerUuuAction):
             raise JobError("USB path of the device not set for uuu")
         for cmd in uuu_cmds:
             if "bcu:" in cmd:
+                self.cleanup_required = True
                 cmd = cmd.replace("bcu: ", "")
                 exec_cmd = "{} {} -board={} -id={}".format(
-                    self.bcu, cmd, bcu_board_name, bcu_board_id
+                    self.bcu, cmd, self.bcu_board_name, self.bcu_board_id
+                )
+                self.run_bcu(
+                    exec_cmd.split(" "),
+                    allow_fail=False,
+                    error_msg=f"Fail UUUBootAction on cmd : {cmd}",
                 )
-                self.run_cmd(exec_cmd.split(" "))
                 continue
             if "uuu:" in cmd:
                 # uuu can be used in 2 different ways, by using built-in scripts in a single command,
@@ -357,13 +491,28 @@ class UUUBootAction(OptionalContainerUuuAction):
                 # So it is more convenient for LAVA job writers to use uuu built-in script with a tuple (protocol, command), like uuu: -b sd_all
                 # In this last case, we remove the 'uuu: ' here
                 cmd = cmd.replace("uuu: ", "")
-            exec_cmd = "{} -m {} {}".format(self.uuu, usb_otg_path, cmd)
+
+            path_args = " -m ".join(usb_otg_path)
+            exec_cmd = f"{self.uuu} -m {path_args} {cmd}"
 
             time.sleep(1)
             self.run_uuu(
                 exec_cmd.split(" "),
                 allow_fail=False,
-                error_msg="Fail UUUBootAction on cmd : {}".format(cmd),
+                error_msg=f"Fail UUUBootAction on cmd : {cmd}",
             )
 
         return connection
+
+    def cleanup(self, connection):
+        super().cleanup(connection)
+        if self.cleanup_required:
+            exec_cmd = "{} deinit -board={} -id={}".format(
+                self.bcu, self.bcu_board_name, self.bcu_board_id
+            )
+            self.run_bcu(
+                exec_cmd.split(" "),
+                allow_fail=False,
+                error_msg="Fail UUUBootAction on cmd : deinit",
+            )
+            self.cleanup_required = False
