@@ -2,29 +2,19 @@
 #
 # Author: Remi Duraffort <remi.duraffort@linaro.org>
 #
-# This file is part of LAVA Dispatcher.
-#
-# LAVA Dispatcher is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# LAVA Dispatcher is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along
-# with this program; if not, see <http://www.gnu.org/licenses>.
+# SPDX-License-Identifier: GPL-2.0-or-later
+from __future__ import annotations
+
+from typing import TYPE_CHECKING
 
 from lava_common.exceptions import ConfigurationError, InfrastructureError
 from lava_dispatcher.action import Action
 
+if TYPE_CHECKING:
+    from lava_dispatcher.job import Job
+
 
 class CommandAction(Action):
-
-    compatibility = 1
     name = "user-command"
     description = "execute one of the commands listed by the admin"
     summary = "execute commands"
@@ -40,8 +30,8 @@ class CommandAction(Action):
         "recovery_exit",
     ]
 
-    def __init__(self):
-        super().__init__()
+    def __init__(self, job: Job):
+        super().__init__(job)
         self.section = "command"
         self.cmd = None
         self.ran = False
