@@ -2,25 +2,13 @@
 #
 # Author: Neil Williams <neil.williams@linaro.org>
 #
-# This file is part of LAVA Dispatcher.
-#
-# LAVA Dispatcher is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# LAVA Dispatcher is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along
-# with this program; if not, see <http://www.gnu.org/licenses>.
+# SPDX-License-Identifier: GPL-2.0-or-later
+from __future__ import annotations
 
 import os
 import shutil
 from functools import partial
+from typing import TYPE_CHECKING
 
 import guestfs
 
@@ -51,16 +39,18 @@ from lava_dispatcher.utils.network import dispatcher_ip
 from lava_dispatcher.utils.shell import which
 from lava_dispatcher.utils.strings import substitute
 
+if TYPE_CHECKING:
+    from lava_dispatcher.job import Job
+
 
 class ApplyOverlayGuest(Action):
-
     name = "apply-overlay-guest"
     description = "prepare a qcow2 drive containing the overlay"
     summary = "build a guest filesystem with the overlay"
     timeout_exception = InfrastructureError
 
-    def __init__(self):
-        super().__init__()
+    def __init__(self, job: Job):
+        super().__init__(job)
         self.guest_filename = "lava-guest.qcow2"
 
     def validate(self):
@@ -117,14 +107,13 @@ class ApplyOverlayGuest(Action):
 
 
 class ApplyOverlayImage(Action):
-
     name = "apply-overlay-image"
     description = "apply overlay via guestfs to the test image"
     summary = "apply overlay to test image"
     timeout_exception = InfrastructureError
 
-    def __init__(self, image_key="image", use_root_partition=True):
-        super().__init__()
+    def __init__(self, job: Job, image_key="image", use_root_partition=True):
+        super().__init__(job)
         self.image_key = image_key
         self.use_root_partition = use_root_partition
 
@@ -173,15 +162,14 @@ class ApplyOverlayImage(Action):
 
 
 class ApplyOverlaySparseImage(Action):
-
     name = "apply-overlay-sparse-image"
     description = "apply overlay to sparse image"
     summary = "apply overlay to sparse image"
     command_exception = InfrastructureError
     timeout_exception = InfrastructureError
 
-    def __init__(self, image_key):
-        super().__init__()
+    def __init__(self, job: Job, image_key):
+        super().__init__(job)
         self.image_key = image_key  # the sparse image key in the parameters
 
     def validate(self):
@@ -236,26 +224,28 @@ class PrepareOverlayTftp(Action):
     def populate(self, parameters):
         self.pipeline = Pipeline(parent=self, job=self.job, parameters=parameters)
         self.pipeline.add_action(
-            ExtractNfsRootfs()
+            ExtractNfsRootfs(self.job)
         )  # idempotent, checks for nfsrootfs parameter
-        self.pipeline.add_action(OverlayAction())  # idempotent, includes testdef
         self.pipeline.add_action(
-            ExtractRamdisk()
+            OverlayAction(self.job)
+        )  # idempotent, includes testdef
+        self.pipeline.add_action(
+            ExtractRamdisk(self.job)
         )  # idempotent, checks for a ramdisk parameter
         self.pipeline.add_action(
-            ExtractModules()
+            ExtractModules(self.job)
         )  # idempotent, checks for a modules parameter
-        self.pipeline.add_action(ApplyOverlayTftp())
+        self.pipeline.add_action(ApplyOverlayTftp(self.job))
         if "kernel" in parameters and "type" in parameters["kernel"]:
-            self.pipeline.add_action(PrepareKernelAction())
+            self.pipeline.add_action(PrepareKernelAction(self.job))
         self.pipeline.add_action(
-            ConfigurePreseedFile()
+            ConfigurePreseedFile(self.job)
         )  # idempotent, checks for a preseed parameter
         self.pipeline.add_action(
-            CompressRamdisk()
+            CompressRamdisk(self.job)
         )  # idempotent, checks for a ramdisk parameter
         if "depthcharge" in self.job.device["actions"]["boot"]["methods"]:
-            self.pipeline.add_action(PrepareKernelAction())
+            self.pipeline.add_action(PrepareKernelAction(self.job))
 
     def run(self, connection, max_end_time):
         connection = super().run(connection, max_end_time)
@@ -411,8 +401,8 @@ class ExtractRootfs(Action):
     summary = "unpack rootfs, ready to apply lava overlay"
     timeout_exception = InfrastructureError
 
-    def __init__(self):
-        super().__init__()
+    def __init__(self, job: Job):
+        super().__init__(job)
         self.param_key = "rootfs"
         self.file_key = "root"
         self.extra_compression = ["xz"]
@@ -445,8 +435,8 @@ class ExtractNfsRootfs(ExtractRootfs):
     summary = "unpack nfsrootfs, ready to apply lava overlay"
     timeout_exception = InfrastructureError
 
-    def __init__(self):
-        super().__init__()
+    def __init__(self, job: Job):
+        super().__init__(job)
         self.param_key = "nfsrootfs"
         self.file_key = "nfsroot"
 
@@ -559,8 +549,8 @@ class ExtractRamdisk(Action):
     summary = "extract the ramdisk"
     timeout_exception = InfrastructureError
 
-    def __init__(self):
-        super().__init__()
+    def __init__(self, job: Job):
+        super().__init__(job)
         self.skip = False
 
     def validate(self):
@@ -636,8 +626,8 @@ class CompressRamdisk(Action):
     summary = "compress ramdisk with overlay"
     timeout_exception = InfrastructureError
 
-    def __init__(self):
-        super().__init__()
+    def __init__(self, job: Job):
+        super().__init__(job)
         self.mkimage_arch = None
         self.add_header = None
         self.skip = False
@@ -755,14 +745,13 @@ class CompressRamdisk(Action):
 
 
 class ApplyLxcOverlay(Action):
-
     name = "apply-lxc-overlay"
     description = "apply the overlay to the container by copying"
     summary = "apply overlay on the container"
     timeout_exception = InfrastructureError
 
-    def __init__(self):
-        super().__init__()
+    def __init__(self, job: Job):
+        super().__init__(job)
         self.lava_test_dir = os.path.realpath(
             "%s/../../lava_test_shell" % os.path.dirname(__file__)
         )
@@ -818,7 +807,6 @@ class ApplyLxcOverlay(Action):
 
 
 class ConfigurePreseedFile(Action):
-
     name = "configure-preseed-file"
     description = "add commands to automated installers, to copy the lava test overlay to the installed system"
     summary = "add commands to installer config"
@@ -855,7 +843,6 @@ class ConfigurePreseedFile(Action):
 
 
 class AppendOverlays(Action):
-
     name = "append-overlays"
     description = "append overlays to an image"
     summary = "append overlays to an image"
@@ -864,8 +851,8 @@ class AppendOverlays(Action):
     IMAGE_FORMATS = ["cpio.newc", "ext4", "tar"]
     OVERLAY_FORMATS = ["file", "tar"]
 
-    def __init__(self, key, params):
-        super().__init__()
+    def __init__(self, job: Job, key, params):
+        super().__init__(job)
         self.key = key
         self.params = params
 
@@ -999,7 +986,10 @@ class AppendOverlays(Action):
         try:
             guest.launch()
             if partition is not None:
-                device = guest.list_partitions()[partition]
+                try:
+                    device = guest.list_partitions()[partition]
+                except (IndexError, TypeError) as exc:
+                    raise JobError("Invalid partition number '%s'" % partition) from exc
             else:
                 device = guest.list_devices()[0]
             guest.mount(device, "/")
@@ -1015,7 +1005,10 @@ class AppendOverlays(Action):
                 overlay_image = self.get_namespace_data(
                     action="compress-overlay", label="output", key="file"
                 )
-                path = "/"
+                lava_test_results_dir = self.get_namespace_data(
+                    action="test", label="results", key="lava_test_results_dir"
+                )
+                path = os.path.dirname(lava_test_results_dir or "/")
                 compress = "gzip"
             else:
                 overlay_image = self.get_namespace_data(
