@@ -2,24 +2,12 @@
 #
 # Author: Remi Duraffort <remi.duraffort@linaro.org>
 #
-# This file is part of LAVA Dispatcher.
-#
-# LAVA Dispatcher is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# LAVA Dispatcher is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along
-# with this program; if not, see <http://www.gnu.org/licenses>.
+# SPDX-License-Identifier: GPL-2.0-or-later
+from __future__ import annotations
 
 import re
 import subprocess  # nosec - internal
+from typing import TYPE_CHECKING
 
 from lava_common.exceptions import InfrastructureError
 from lava_common.schemas import docker_image_format_pattern
@@ -29,15 +17,17 @@ from lava_dispatcher.actions.deploy.overlay import OverlayAction
 from lava_dispatcher.logical import Deployment
 from lava_dispatcher.utils.shell import which
 
+if TYPE_CHECKING:
+    from lava_dispatcher.job import Job
+
 
 class DockerAction(Action):
-
     name = "deploy-docker"
     description = "deploy docker images"
     summary = "deploy docker"
 
-    def __init__(self):
-        super().__init__()
+    def __init__(self, job: Job):
+        super().__init__(job)
         self.remote = []
 
     def validate(self):
@@ -85,21 +75,31 @@ class DockerAction(Action):
     def populate(self, parameters):
         self.pipeline = Pipeline(parent=self, job=self.job, parameters=parameters)
         if self.test_needs_deployment(parameters):
-            self.pipeline.add_action(DeployDeviceEnvironment())
+            self.pipeline.add_action(DeployDeviceEnvironment(self.job))
         if self.test_needs_overlay(parameters):
-            self.pipeline.add_action(OverlayAction())
+            self.pipeline.add_action(OverlayAction(self.job))
 
     def run(self, connection, max_end_time):
         # Pull the image
+        pull = not self.local
         if self.local:
             cmd = (
                 ["docker"]
                 + self.remote
-                + ["image", "inspect", "--format", "image exists", self.image_name]
+                + [
+                    "image",
+                    "inspect",
+                    "--format",
+                    f"Image {self.image_name} exists locally\nImage Id: {{{{.Id}}}}",
+                    self.image_name,
+                ]
             )
-            error_msg = "Unable to inspect docker image '%s'" % self.image_name
-            self.run_cmd(cmd, error_msg=error_msg)
-        else:
+            if self.run_cmd(cmd, allow_fail=True):
+                self.logger.warning(
+                    "Unable to inspect docker image '%s'", self.image_name
+                )
+                pull = True
+        if pull:
             self.run_cmd(
                 ["docker"] + self.remote + ["pull", self.image_name],
                 error_msg="Unable to pull docker image '%s'" % self.image_name,
@@ -109,12 +109,11 @@ class DockerAction(Action):
 
 
 class Docker(Deployment):
-    compatibility = 4
     name = "docker"
 
     @classmethod
-    def action(cls):
-        return DockerAction()
+    def action(cls, job: Job) -> Action:
+        return DockerAction(job)
 
     @classmethod
     def accepts(cls, device, parameters):
