@@ -4,34 +4,20 @@
 #         Remi Duraffort <remi.duraffort@linaro.org>
 #         Antonio Terceiro <antonio.terceiro@linaro.org>
 #
-# This file is part of LAVA Dispatcher.
-#
-# LAVA Dispatcher is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# LAVA Dispatcher is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along
-# with this program; if not, see <http://www.gnu.org/licenses>.
+# SPDX-License-Identifier: GPL-2.0-or-later
 
 # This class is used for all downloads, including images and individual files for tftp.
 # python2 only
+from __future__ import annotations
 
 import contextlib
-import errno
 import hashlib
 import math
 import os
 import pathlib
-import shutil
 import subprocess  # nosec - verified.
 import time
+from typing import TYPE_CHECKING
 from urllib.parse import quote_plus, urlparse
 
 import requests
@@ -59,6 +45,10 @@ from lava_dispatcher.utils.filesystem import (
     lava_lxc_home,
 )
 from lava_dispatcher.utils.network import requests_retry
+from lava_dispatcher.utils.strings import substitute_address_with_static_info
+
+if TYPE_CHECKING:
+    from lava_dispatcher.job import Job
 
 
 class DownloaderAction(RetryAction):
@@ -71,8 +61,8 @@ class DownloaderAction(RetryAction):
     description = "download with retry"
     summary = "download-retry"
 
-    def __init__(self, key, path, params, uniquify=True):
-        super().__init__()
+    def __init__(self, job: Job, key, path, params, uniquify=True):
+        super().__init__(job)
         self.max_retries = 3
         self.key = key  # the key in the parameters of what to download
         self.path = path  # where to download
@@ -92,35 +82,43 @@ class DownloaderAction(RetryAction):
         url = urlparse(url)
         if url.scheme == "scp":
             action = ScpDownloadAction(
-                self.key, self.path, url, self.uniquify, params=self.params
+                self.job, self.key, self.path, url, self.uniquify, params=self.params
             )
         elif url.scheme in ["http", "https"]:
             action = HttpDownloadAction(
-                self.key, self.path, url, self.uniquify, params=self.params
+                self.job, self.key, self.path, url, self.uniquify, params=self.params
             )
         elif url.scheme == "file":
             action = FileDownloadAction(
-                self.key, self.path, url, self.uniquify, params=self.params
+                self.job, self.key, self.path, url, self.uniquify, params=self.params
             )
         elif url.scheme == "lxc":
-            action = LxcDownloadAction(self.key, self.path, url)
+            action = LxcDownloadAction(self.job, self.key, self.path, url)
         elif url.scheme == "downloads":
-            action = PreDownloadedAction(self.key, url, self.path, params=self.params)
+            action = PreDownloadedAction(
+                self.job, self.key, url, self.path, params=self.params
+            )
         else:
             raise JobError("Unsupported url protocol scheme: %s" % url.scheme)
         self.pipeline.add_action(action)
         overlays = self.params.get("overlays", [])
         for overlay in overlays:
             if overlay == "lava":
-                # Special case, don't download an overlay, just defined where to find overlays
+                # Special case, don't download an overlay,
+                # just defined where to find overlays
                 continue
             self.pipeline.add_action(
                 DownloaderAction(
-                    "%s.%s" % (self.key, overlay), self.path, params=overlays[overlay]
+                    self.job,
+                    "%s.%s" % (self.key, overlay),
+                    self.path,
+                    params=overlays[overlay],
                 )
             )
         if overlays:
-            self.pipeline.add_action(AppendOverlays(self.key, params=self.params))
+            self.pipeline.add_action(
+                AppendOverlays(self.job, self.key, params=self.params)
+            )
 
 
 class DownloadHandler(Action):
@@ -147,8 +145,8 @@ class DownloadHandler(Action):
         "zstd": "unzstd",
     }
 
-    def __init__(self, key, path, url, uniquify=True, params=None):
-        super().__init__()
+    def __init__(self, job: Job, key, path, url, uniquify=True, params=None):
+        super().__init__(job)
         self.url = url
         self.key = key
         self.size = -1
@@ -164,12 +162,15 @@ class DownloadHandler(Action):
         raise LAVABug("'reader' function unimplemented")
 
     def cleanup(self, connection):
-        if os.path.exists(self.path):
-            self.logger.debug("Cleaning up download directory: %s", self.path)
-            shutil.rmtree(self.path)
-        self.set_namespace_data(
-            action="download-action", label=self.key, key="file", value=""
-        )
+        if self.fname is not None and os.path.exists(self.fname):
+            self.logger.debug("Cleaning up downloaded image: %s", self.fname)
+            try:
+                os.remove(self.fname)
+                self.set_namespace_data(
+                    action="download-action", label=self.key, key="file", value=""
+                )
+            except OSError as exc:
+                self.logger.debug(str(exc))
         super().cleanup(connection)
 
     def _compression(self):
@@ -197,6 +198,10 @@ class DownloadHandler(Action):
         if len(pathlib.Path(self.key).parts) != 1:
             raise JobError("Invalid key %r" % self.key)
 
+        self.params["url"] = substitute_address_with_static_info(
+            self.params["url"], self.job.device.get("static_info", [])
+        )
+
         self.url = urlparse(self.params["url"])
         compression = self.params.get("compression")
         archive = self.params.get("archive")
@@ -258,25 +263,32 @@ class DownloadHandler(Action):
         raise JobError("%s for '%s' does not match." % (algorithm, self.url.geturl()))
 
     def run(self, connection, max_end_time):
-        def progress_unknown_total(downloaded_sz, last_val):
+        def progress_unknown_total(downloaded_sz, last_val, last_update):
             """Compute progress when the size is unknown"""
-            condition = downloaded_sz >= last_val + 25 * 1024 * 1024
+            condition = (
+                downloaded_sz >= last_val + 25 * 1024 * 1024
+                and time.monotonic() - last_update >= 0.1
+                or time.monotonic() - last_update >= 1
+            )
             return (
                 condition,
                 downloaded_sz,
-                "progress %dMB" % (int(downloaded_sz / (1024 * 1024)))
+                "progress %d MB" % (int(downloaded_sz / (1024 * 1024)))
                 if condition
                 else "",
             )
 
-        def progress_known_total(downloaded_sz, last_val):
+        def progress_known_total(downloaded_sz, last_val, last_update):
             """Compute progress when the size is known"""
             percent = math.floor(downloaded_sz / float(self.size) * 100)
-            condition = percent >= last_val + 5
+            condition = (
+                percent >= last_val + 5 and time.monotonic() - last_update >= 0.1
+            )
             return (
                 condition,
                 percent,
-                "progress %3d%% (%dMB)" % (percent, int(downloaded_sz / (1024 * 1024)))
+                "progress %3d %% (%d MB)"
+                % (percent, int(downloaded_sz / (1024 * 1024)))
                 if condition
                 else "",
             )
@@ -290,12 +302,9 @@ class DownloadHandler(Action):
         # Create a fresh directory if the old one has been removed by a previous cleanup
         # (when retrying inside a RetryAction)
         try:
-            os.makedirs(self.path, 0o755)
+            os.makedirs(self.path, 0o755, exist_ok=True)
         except OSError as exc:
-            if exc.errno != errno.EEXIST:
-                raise InfrastructureError(
-                    "Unable to create %s: %s" % (self.path, str(exc))
-                )
+            raise InfrastructureError(f"Unable to create {self.path}: {exc}")
 
         compression = self._compression()
         if self.key == "ramdisk":
@@ -329,7 +338,7 @@ class DownloadHandler(Action):
             progress = progress_unknown_total
         else:
             self.logger.debug(
-                "total size: %d (%dMB)", self.size, int(self.size / (1024 * 1024))
+                "total size: %d (%d MB)", self.size, int(self.size / (1024 * 1024))
             )
             last_value = -5
             progress = progress_known_total
@@ -349,11 +358,16 @@ class DownloadHandler(Action):
         elif not self.params.get("compression", False):
             self.logger.debug("No compression specified")
 
-        def update_progress():
-            nonlocal downloaded_size, last_value, md5, sha256, sha512
+        last_update = time.monotonic()  # time for rate limiting the progress output
+
+        def update_progress(buff):
+            nonlocal downloaded_size, last_update, last_value, md5, sha256, sha512
             downloaded_size += len(buff)
-            (printing, new_value, msg) = progress(downloaded_size, last_value)
+            (printing, new_value, msg) = progress(
+                downloaded_size, last_value, last_update
+            )
             if printing:
+                last_update = time.monotonic()
                 last_value = new_value
                 self.logger.debug(msg)
             md5.update(buff)
@@ -363,44 +377,25 @@ class DownloadHandler(Action):
         if compression and decompress_command:
             try:
                 with open(self.fname, "wb") as dwnld_file:
-                    proc = subprocess.Popen(  # nosec - internal.
-                        [decompress_command],
-                        stdin=subprocess.PIPE,
-                        stdout=dwnld_file,
-                        stderr=subprocess.PIPE,
+                    self.run_download_decompression_subprocess(
+                        dwnld_file,
+                        update_progress,
+                        decompress_command,
                     )
             except OSError as exc:
-                msg = "Unable to open %s: %s" % (self.fname, exc.strerror)
+                msg = f"Unable to open {self.fname}: {exc.strerror}"
                 self.logger.error(msg)
                 raise InfrastructureError(msg)
-
-            with proc.stdin as pipe:
-                for buff in self.reader():
-                    update_progress()
-                    try:
-                        pipe.write(buff)
-                    except BrokenPipeError as exc:
-                        error_message = (
-                            str(exc) + ": " + proc.stderr.read().decode("utf-8").strip()
-                        )
-                        self.logger.exception(error_message)
-                        msg = (
-                            "Make sure the 'compression' is corresponding "
-                            "to the image file type."
-                        )
-                        self.logger.error(msg)
-                        raise JobError(error_message)
-            proc.wait()
         else:
             with open(self.fname, "wb") as dwnld_file:
                 for buff in self.reader():
-                    update_progress()
+                    update_progress(buff)
                     dwnld_file.write(buff)
 
         # Log the download speed
         ending = time.monotonic()
         self.logger.info(
-            "%dMB downloaded in %0.2fs (%0.2fMB/s)",
+            "%d MB downloaded in %0.2f s (%0.2f MB/s)",
             downloaded_size / (1024 * 1024),
             round(ending - beginning, 2),
             round(downloaded_size / (1024 * 1024 * (ending - beginning)), 2),
@@ -514,6 +509,41 @@ class DownloadHandler(Action):
         }
         return connection
 
+    def run_download_decompression_subprocess(
+        self, dwnld_file, update_progress, decompress_command
+    ) -> None:
+        with subprocess.Popen(
+            [decompress_command],
+            stdin=subprocess.PIPE,
+            stdout=dwnld_file,
+            stderr=subprocess.PIPE,
+        ) as proc:
+            for buff in self.reader():
+                update_progress(buff)
+                try:
+                    proc.stdin.write(buff)
+                except BrokenPipeError as exc:
+                    error_message = (
+                        str(exc) + ": " + proc.stderr.read().decode("utf-8").strip()
+                    )
+                    self.logger.exception(error_message)
+                    msg = (
+                        "Make sure the 'compression' is corresponding "
+                        "to the image file type."
+                    )
+                    self.logger.error(msg)
+                    raise JobError(error_message)
+
+            proc.stdin.close()
+            compression_exitcode = proc.wait()
+            if compression_exitcode != 0:
+                exit_error_msg = (
+                    "Decompression subprocess exited with non-zero code: "
+                    + proc.stderr.read().decode("utf-8").strip()
+                )
+                self.logger.error(exit_error_msg)
+                raise JobError(exit_error_msg)
+
 
 class FileDownloadAction(DownloadHandler):
     """
@@ -563,7 +593,12 @@ class HttpDownloadAction(DownloadHandler):
             http_cache = self.job.parameters["dispatcher"].get(
                 "http_url_format_string", ""
             )
-            if http_cache:
+
+            use_cache = True
+            if self.params:
+                use_cache = self.params.get("use_cache", True)
+
+            if http_cache and use_cache:
                 self.logger.info("Using caching service: '%s'", http_cache)
                 try:
                     self.url = urlparse(http_cache % quote_plus(self.url.geturl()))
@@ -634,8 +669,7 @@ class HttpDownloadAction(DownloadHandler):
                 raise InfrastructureError(
                     "Unable to download '%s'" % (self.url.geturl())
                 )
-            for buff in res.iter_content(HTTP_DOWNLOAD_CHUNK_SIZE):
-                yield buff
+            yield from res.iter_content(HTTP_DOWNLOAD_CHUNK_SIZE)
         except requests.RequestException as exc:
             raise InfrastructureError(
                 "Unable to download '%s': %s" % (self.url.geturl(), str(exc))
@@ -695,8 +729,8 @@ class LxcDownloadAction(Action):
     description = "Map to the correct lxc path"
     summary = "lxc download"
 
-    def __init__(self, key, path, url):
-        super().__init__()
+    def __init__(self, job: Job, key, path, url):
+        super().__init__(job)
         self.key = key
         self.path = path
         self.url = url
@@ -747,8 +781,8 @@ class PreDownloadedAction(Action):
     description = "Map to the correct downloaded path"
     summary = "pre downloaded"
 
-    def __init__(self, key, url, path, params=None):
-        super().__init__()
+    def __init__(self, job: Job, key, url, path, params=None):
+        super().__init__(job)
         self.key = key
         self.url = url
         self.path = path
@@ -835,8 +869,8 @@ class QCowConversionAction(Action):
     description = "convert qcow image using qemu-img"
     summary = "qcow conversion"
 
-    def __init__(self, key):
-        super().__init__()
+    def __init__(self, job: Job, key):
+        super().__init__(job)
         self.key = key
 
     def run(self, connection, max_end_time):
@@ -876,12 +910,11 @@ class Download(Deployment):
     Downloads the relevant parts, copies to LXC if available.
     """
 
-    compatibility = 1
     name = "download"
 
     @classmethod
-    def action(cls):
-        return DownloadAction()
+    def action(cls, job: Job) -> Action:
+        return DownloadAction(job)
 
     @classmethod
     def accepts(cls, device, parameters):
@@ -893,13 +926,12 @@ class Download(Deployment):
 
 
 class DownloadAction(Action):
-
     name = "download-deploy"
     description = "download files and copy to LXC if available"
     summary = "download deployment"
 
-    def __init__(self):
-        super().__init__()
+    def __init__(self, job: Job):
+        super().__init__(job)
         self.download_dir = None
 
     def validate(self):
@@ -919,25 +951,28 @@ class DownloadAction(Action):
         #
         # NOTE: Add more power on strategies, if required for specific devices.
         if self.job.device.get("fastboot_via_uboot", False):
-            self.pipeline.add_action(ConnectDevice())
-            self.pipeline.add_action(UBootEnterFastbootAction())
+            self.pipeline.add_action(ConnectDevice(self.job))
+            self.pipeline.add_action(UBootEnterFastbootAction(self.job))
         elif self.job.device.hard_reset_command:
             self.force_prompt = True
-            self.pipeline.add_action(ConnectDevice())
-            self.pipeline.add_action(ResetDevice())
+            self.pipeline.add_action(ConnectDevice(self.job))
+            self.pipeline.add_action(ResetDevice(self.job))
         else:
-            self.pipeline.add_action(EnterFastbootAction())
+            self.pipeline.add_action(EnterFastbootAction(self.job))
 
         self.download_dir = self.mkdtemp()
         for image in sorted(parameters["images"].keys()):
             self.pipeline.add_action(
                 DownloaderAction(
-                    image, self.download_dir, params=parameters["images"][image]
+                    self.job,
+                    image,
+                    self.download_dir,
+                    params=parameters["images"][image],
                 )
             )
         if self.test_needs_overlay(parameters):
-            self.pipeline.add_action(OverlayAction())
-        self.pipeline.add_action(CopyToLxcAction())
+            self.pipeline.add_action(OverlayAction(self.job))
+        self.pipeline.add_action(CopyToLxcAction(self.job))
 
 
 class CopyToLxcAction(Action):
@@ -949,8 +984,8 @@ class CopyToLxcAction(Action):
     description = "copy files to lxc"
     summary = "copy to lxc"
 
-    def __init__(self):
-        super().__init__()
+    def __init__(self, job: Job):
+        super().__init__(job)
         self.retries = 3
         self.sleep = 10
 
