@@ -1,24 +1,13 @@
-# -*- coding: utf-8 -*-
 # Copyright (C) 2020 Linaro Limited
 #
 # Author: Antonio Terceiro <antonio.terceiro@linaro.org>
 #
-# This file is part of LAVA.
-#
-# LAVA is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# LAVA is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, see <http://www.gnu.org/licenses>.
+# SPDX-License-Identifier: GPL-2.0-or-later
+from __future__ import annotations
 
 from pathlib import Path
+from shutil import copy
+from typing import TYPE_CHECKING
 
 from lava_common.constants import LAVA_DOWNLOADS
 from lava_dispatcher.action import Action, Pipeline
@@ -27,6 +16,9 @@ from lava_dispatcher.actions.deploy.overlay import OverlayAction
 from lava_dispatcher.logical import Deployment
 from lava_dispatcher.utils.docker import DockerRun
 
+if TYPE_CHECKING:
+    from lava_dispatcher.job import Job
+
 
 class Downloads(Deployment):
     """
@@ -34,12 +26,11 @@ class Downloads(Deployment):
     Just downloads files, and that's it.
     """
 
-    compatibility = 1
     name = "downloads"
 
     @classmethod
-    def action(cls):
-        return DownloadsAction()
+    def action(cls, job: Job) -> Action:
+        return DownloadsAction(job)
 
     @classmethod
     def accepts(cls, device, parameters):
@@ -59,13 +50,14 @@ class DownloadsAction(DownloadAction):
         self.pipeline = Pipeline(parent=self, job=self.job, parameters=parameters)
 
         if self.test_needs_overlay(parameters):
-            self.pipeline.add_action(OverlayAction())
+            self.pipeline.add_action(OverlayAction(self.job))
 
         namespace = parameters["namespace"]
         download_dir = Path(self.job.tmp_dir) / "downloads" / namespace
         for image in sorted(parameters["images"].keys()):
             self.pipeline.add_action(
                 DownloaderAction(
+                    self.job,
                     image,
                     download_dir,
                     params=parameters["images"][image],
@@ -76,7 +68,7 @@ class DownloadsAction(DownloadAction):
         postprocess = parameters.get("postprocess")
         if postprocess:
             if postprocess.get("docker"):
-                self.pipeline.add_action(PostprocessWithDocker(download_dir))
+                self.pipeline.add_action(PostprocessWithDocker(self.job, download_dir))
 
 
 class PostprocessWithDocker(Action):
@@ -88,10 +80,10 @@ class PostprocessWithDocker(Action):
     description = "Postprocess downloaded images with Docker"
     summary = "download-postprocess"
 
-    def __init__(self, path):
-        super().__init__()
-        self.path = Path(path)
-        self.steps = []
+    def __init__(self, job: Job, path: Path):
+        super().__init__(job)
+        self.path = path
+        self.steps: list[str] = []
 
     def populate(self, parameters):
         self.docker_parameters = parameters.get("postprocess", {}).get("docker", {})
@@ -105,8 +97,6 @@ class PostprocessWithDocker(Action):
         return res
 
     def run(self, connection, max_end_time):
-        job_id = self.job.job_id
-
         script = ["#!/bin/sh", "exec 2>&1", "set -ex"]
 
         # Export data generated during run of the Pipeline like NFS settings
@@ -123,12 +113,24 @@ class PostprocessWithDocker(Action):
         scriptfile.write_text(script)
         scriptfile.chmod(0o755)
 
+        # make overlay available in the downloads directory
+        # Note: overlay filename is not constant
+        overlay_full_path = self.get_namespace_data(
+            action="compress-overlay", label="output", key="file"
+        )
+        if overlay_full_path:
+            copy(overlay_full_path, self.path)
+
         docker = DockerRun.from_parameters(self.docker_parameters, self.job)
         docker.add_device("/dev/kvm", skip_missing=True)
         docker.bind_mount(self.path, LAVA_DOWNLOADS)
 
         docker.workdir(LAVA_DOWNLOADS)
 
-        docker.run(f"{LAVA_DOWNLOADS}/postprocess.sh", action=self)
+        docker.run(
+            f"{LAVA_DOWNLOADS}/postprocess.sh",
+            action=self,
+            error_msg="Post-processing of downloads failed",
+        )
 
         return connection
