@@ -2,31 +2,22 @@
 #
 # Author: Stevan Radakovic <stevan.radakovic@linaro.org>
 #
-# This file is part of LAVA Dispatcher.
-#
-# LAVA Dispatcher is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# LAVA Dispatcher is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along
-# with this program; if not, see <http://www.gnu.org/licenses>.
+# SPDX-License-Identifier: GPL-2.0-or-later
+from __future__ import annotations
 
 import contextlib
 import os
+from typing import TYPE_CHECKING
 
 import yaml
 
-from lava_common.compat import yaml_safe_load
 from lava_common.constants import LINE_SEPARATOR
+from lava_common.yaml import yaml_safe_load
 from lava_dispatcher.action import Action
 
+if TYPE_CHECKING:
+    from lava_dispatcher.job import Job
+
 
 class DeployDeviceEnvironment(Action):
     """
@@ -37,8 +28,8 @@ class DeployDeviceEnvironment(Action):
     description = "deploy device environment"
     summary = "deploy device environment"
 
-    def __init__(self):
-        super().__init__()
+    def __init__(self, job: Job):
+        super().__init__(job)
         self.env = ""
 
     def validate(self):
