@@ -2,23 +2,11 @@
 #
 # Author: Neil Williams <neil.williams@linaro.org>
 #
-# This file is part of LAVA Dispatcher.
-#
-# LAVA Dispatcher is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# LAVA Dispatcher is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along
-# with this program; if not, see <http://www.gnu.org/licenses>.
+# SPDX-License-Identifier: GPL-2.0-or-later
+from __future__ import annotations
 
 import os
+from typing import TYPE_CHECKING
 
 from lava_dispatcher.action import Action, Pipeline
 from lava_dispatcher.actions.deploy.apply_overlay import (
@@ -34,9 +22,11 @@ from lava_dispatcher.actions.deploy.overlay import OverlayAction
 from lava_dispatcher.logical import Deployment
 from lava_dispatcher.utils.compression import untar_file
 
+if TYPE_CHECKING:
+    from lava_dispatcher.job import Job
+
 
 class DeployImagesAction(Action):  # FIXME: Rename to DeployPosixImages
-
     name = "deployimages"
     description = "deploy images using guestfs"
     summary = "deploy images"
@@ -46,15 +36,16 @@ class DeployImagesAction(Action):  # FIXME: Rename to DeployPosixImages
         path = self.mkdtemp()
 
         if self.test_needs_overlay(parameters):
-            self.pipeline.add_action(OverlayAction())  # idempotent, includes testdef
-            self.pipeline.add_action(ApplyOverlayGuest())
+            # idempotent, includes testdef
+            self.pipeline.add_action(OverlayAction(self.job))
+            self.pipeline.add_action(ApplyOverlayGuest(self.job))
         if self.test_needs_deployment(parameters):
-            self.pipeline.add_action(DeployDeviceEnvironment())
+            self.pipeline.add_action(DeployDeviceEnvironment(self.job))
 
         if "uefi" in parameters:
             uefi_path = self.mkdtemp()
             self.pipeline.add_action(
-                DownloaderAction("uefi", uefi_path, params=parameters["uefi"])
+                DownloaderAction(self.job, "uefi", uefi_path, params=parameters["uefi"])
             )
             # uefi option of QEMU needs a directory, not the filename
             self.set_namespace_data(
@@ -67,10 +58,12 @@ class DeployImagesAction(Action):  # FIXME: Rename to DeployPosixImages
             # alternatively use the -bios option and standard image args
         for image in parameters["images"].keys():
             self.pipeline.add_action(
-                DownloaderAction(image, path, params=parameters["images"][image])
+                DownloaderAction(
+                    self.job, image, path, params=parameters["images"][image]
+                )
             )
             if parameters["images"][image].get("format", "") == "qcow2":
-                self.pipeline.add_action(QCowConversionAction(image))
+                self.pipeline.add_action(QCowConversionAction(self.job, image))
 
 
 class DeployQemuNfs(Deployment):
@@ -79,12 +72,11 @@ class DeployQemuNfs(Deployment):
     Does not use GuestFS, adds overlay to the NFS
     """
 
-    compatibility = 5
     name = "qemu-nfs"
 
     @classmethod
-    def action(cls):
-        return DeployQemuNfsAction()
+    def action(cls, job: Job) -> Action:
+        return DeployQemuNfsAction(job)
 
     @classmethod
     def accepts(cls, device, parameters):
@@ -107,7 +99,6 @@ class DeployQemuNfs(Deployment):
 
 
 class DeployQemuNfsAction(Action):
-
     name = "deploy-qemu-nfs"
     description = "deploy qemu with NFS"
     summary = "deploy NFS for QEMU"
@@ -118,7 +109,7 @@ class DeployQemuNfsAction(Action):
         if "uefi" in parameters:
             uefi_path = self.mkdtemp()
             self.pipeline.add_action(
-                DownloaderAction("uefi", uefi_path, params=parameters["uefi"])
+                DownloaderAction(self.job, "uefi", uefi_path, params=parameters["uefi"])
             )
             # uefi option of QEMU needs a directory, not the filename
             self.set_namespace_data(
@@ -131,26 +122,27 @@ class DeployQemuNfsAction(Action):
             # alternatively use the -bios option and standard image args
         for image in parameters["images"].keys():
             self.pipeline.add_action(
-                DownloaderAction(image, path, params=parameters["images"][image])
+                DownloaderAction(
+                    self.job, image, path, params=parameters["images"][image]
+                )
             )
             if parameters["images"][image].get("format", "") == "qcow2":
-                self.pipeline.add_action(QCowConversionAction(image))
-        self.pipeline.add_action(ExtractNfsAction())
+                self.pipeline.add_action(QCowConversionAction(self.job, image))
+        self.pipeline.add_action(ExtractNfsAction(self.job))
         if self.test_needs_overlay(parameters):
-            self.pipeline.add_action(OverlayAction())
-            self.pipeline.add_action(ApplyOverlayTftp())
+            self.pipeline.add_action(OverlayAction(self.job))
+            self.pipeline.add_action(ApplyOverlayTftp(self.job))
         if self.test_needs_deployment(parameters):
-            self.pipeline.add_action(DeployDeviceEnvironment())
+            self.pipeline.add_action(DeployDeviceEnvironment(self.job))
 
 
 class ExtractNfsAction(Action):
-
     name = "qemu-nfs-deploy"
     description = "deploy nfsrootfs for QEMU"
     summary = "NFS deployment for QEMU"
 
-    def __init__(self):
-        super().__init__()
+    def __init__(self, job: Job):
+        super().__init__(job)
         self.param_key = "nfsrootfs"
         self.file_key = "nfsroot"
         self.extra_compression = ["xz"]
@@ -224,12 +216,11 @@ class DeployImages(Deployment):
         test-definitions-action
     """
 
-    compatibility = 4
     name = "images"
 
     @classmethod
-    def action(cls):
-        return DeployImagesAction()
+    def action(cls, job: Job) -> Action:
+        return DeployImagesAction(job)
 
     @classmethod
     def accepts(cls, device, parameters):
