@@ -2,23 +2,11 @@
 #
 # Author: Senthil Kumaran S <senthil.kumaran@linaro.org>
 #
-# This file is part of LAVA Dispatcher.
-#
-# LAVA Dispatcher is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# LAVA Dispatcher is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along
-# with this program; if not, see <http://www.gnu.org/licenses>.
+# SPDX-License-Identifier: GPL-2.0-or-later
+from __future__ import annotations
 
 import os
+from typing import TYPE_CHECKING
 
 from lava_common.constants import (
     LXC_DEFAULT_PACKAGES,
@@ -34,10 +22,13 @@ from lava_dispatcher.actions.deploy.environment import DeployDeviceEnvironment
 from lava_dispatcher.actions.deploy.overlay import OverlayAction
 from lava_dispatcher.logical import Deployment
 from lava_dispatcher.protocols.lxc import LxcProtocol
+from lava_dispatcher.utils.containers import DeviceContainerMappingMixin
 from lava_dispatcher.utils.filesystem import lxc_path
 from lava_dispatcher.utils.shell import which
 from lava_dispatcher.utils.udev import allow_fs_label
-from lava_dispatcher_host.action import DeviceContainerMappingMixin
+
+if TYPE_CHECKING:
+    from lava_dispatcher.job import Job
 
 
 class Lxc(Deployment):
@@ -46,12 +37,11 @@ class Lxc(Deployment):
     Downloads the relevant parts, copies to the locations using lxc.
     """
 
-    compatibility = 1
     name = "lxc"
 
     @classmethod
-    def action(cls):
-        return LxcAction()
+    def action(cls, job: Job) -> Action:
+        return LxcAction(job)
 
     @classmethod
     def accepts(cls, device, parameters):
@@ -65,13 +55,12 @@ class Lxc(Deployment):
 
 
 class LxcAction(Action):
-
     name = "lxc-deploy"
     description = "download files and deploy using lxc"
     summary = "lxc deployment"
 
-    def __init__(self):
-        super().__init__()
+    def __init__(self, job: Job):
+        super().__init__(job)
         self.lxc_data = {}
 
     def validate(self):
@@ -94,18 +83,18 @@ class LxcAction(Action):
 
     def populate(self, parameters):
         self.pipeline = Pipeline(parent=self, job=self.job, parameters=parameters)
-        self.pipeline.add_action(LxcCreateAction())
-        self.pipeline.add_action(LxcCreateUdevRuleAction())
+        self.pipeline.add_action(LxcCreateAction(self.job))
+        self.pipeline.add_action(LxcCreateUdevRuleAction(self.job))
         if "packages" in parameters:
-            self.pipeline.add_action(LxcStartAction())
-            self.pipeline.add_action(LxcAptUpdateAction())
-            self.pipeline.add_action(LxcAptInstallAction())
-            self.pipeline.add_action(LxcStopAction())
+            self.pipeline.add_action(LxcStartAction(self.job))
+            self.pipeline.add_action(LxcAptUpdateAction(self.job))
+            self.pipeline.add_action(LxcAptInstallAction(self.job))
+            self.pipeline.add_action(LxcStopAction(self.job))
         if self.test_needs_deployment(parameters):
-            self.pipeline.add_action(DeployDeviceEnvironment())
+            self.pipeline.add_action(DeployDeviceEnvironment(self.job))
         if self.test_needs_overlay(parameters):
-            self.pipeline.add_action(OverlayAction())
-            self.pipeline.add_action(ApplyLxcOverlay())
+            self.pipeline.add_action(OverlayAction(self.job))
+            self.pipeline.add_action(ApplyLxcOverlay(self.job))
 
 
 class LxcCreateAction(Action):
@@ -117,8 +106,8 @@ class LxcCreateAction(Action):
     description = "create lxc action"
     summary = "create lxc"
 
-    def __init__(self):
-        super().__init__()
+    def __init__(self, job: Job):
+        super().__init__(job)
         self.retries = 10
         self.sleep = 10
         self.lxc_data = {}
@@ -219,7 +208,7 @@ class LxcCreateAction(Action):
         return connection
 
 
-class LxcCreateUdevRuleAction(Action, DeviceContainerMappingMixin):
+class LxcCreateUdevRuleAction(DeviceContainerMappingMixin):
     """
     Creates Lxc related udev rules for this container.
     """
@@ -228,8 +217,8 @@ class LxcCreateUdevRuleAction(Action, DeviceContainerMappingMixin):
     description = "create lxc udev rule action"
     summary = "create lxc udev rule"
 
-    def __init__(self):
-        super().__init__()
+    def __init__(self, job: Job):
+        super().__init__(job)
         self.retries = 10
         self.sleep = 10
 
@@ -292,8 +281,8 @@ class LxcAptUpdateAction(Action):
     description = "lxc apt update action"
     summary = "lxc apt update"
 
-    def __init__(self):
-        super().__init__()
+    def __init__(self, job: Job):
+        super().__init__(job)
         self.retries = 10
         self.sleep = 10
 
@@ -317,8 +306,8 @@ class LxcAptInstallAction(Action):
     description = "lxc apt install packages action"
     summary = "lxc apt install"
 
-    def __init__(self):
-        super().__init__()
+    def __init__(self, job: Job):
+        super().__init__(job)
         self.retries = 10
         self.sleep = 10
 
