@@ -2,21 +2,8 @@
 #
 # Author: Neil Williams <neil.williams@linaro.org>
 #
-# This file is part of LAVA Dispatcher.
-#
-# LAVA Dispatcher is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# LAVA Dispatcher is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along
-# with this program; if not, see <http://www.gnu.org/licenses>.
+# SPDX-License-Identifier: GPL-2.0-or-later
+from __future__ import annotations
 
 import glob
 import os
@@ -24,8 +11,9 @@ import shlex
 import shutil
 import stat
 import tarfile
+from typing import TYPE_CHECKING
 
-from lava_common.exceptions import InfrastructureError, LAVABug
+from lava_common.exceptions import InfrastructureError, JobError, LAVABug
 from lava_dispatcher.action import Action, Pipeline
 from lava_dispatcher.actions.deploy.testdef import TestDefinitionAction
 from lava_dispatcher.logical import Deployment
@@ -35,15 +23,18 @@ from lava_dispatcher.utils.contextmanager import chdir
 from lava_dispatcher.utils.filesystem import check_ssh_identity_file
 from lava_dispatcher.utils.network import dispatcher_ip, rpcinfo_nfs
 from lava_dispatcher.utils.shell import which
+from lava_dispatcher.utils.strings import substitute_address_with_static_info
+
+if TYPE_CHECKING:
+    from lava_dispatcher.job import Job
 
 
 class Overlay(Deployment):
-    compatibility = 4
     name = "overlay"
 
     @classmethod
-    def action(cls):
-        return OverlayAction()
+    def action(cls, job: Job) -> Action:
+        return OverlayAction(job)
 
     @classmethod
     def accepts(cls, device, parameters):
@@ -79,8 +70,8 @@ class CreateOverlay(Action):
     description = "add lava scripts during deployment for test shell use"
     summary = "overlay the lava support scripts"
 
-    def __init__(self):
-        super().__init__()
+    def __init__(self, job: Job):
+        super().__init__(job)
         self.lava_test_dir = os.path.realpath(
             "%s/../../lava_test_shell" % os.path.dirname(__file__)
         )
@@ -102,7 +93,10 @@ class CreateOverlay(Action):
         )
 
         lava_test_results_dir = self.get_constant("lava_test_results_dir", "posix")
-        lava_test_results_dir = lava_test_results_dir % self.job.job_id
+        try:
+            lava_test_results_dir = lava_test_results_dir % self.job.job_id
+        except ValueError:
+            raise JobError("Invalid lava_test_results_dir, should include '%s'")
         self.set_namespace_data(
             action="test",
             label="results",
@@ -150,12 +144,22 @@ class CreateOverlay(Action):
             "ssh" in data for data in self.job.device["actions"]["deploy"]["methods"]
         ):
             # only devices supporting ssh deployments add this action.
-            self.pipeline.add_action(SshAuthorize())
-        self.pipeline.add_action(VlandOverlayAction())
-        self.pipeline.add_action(MultinodeOverlayAction())
-        self.pipeline.add_action(TestDefinitionAction())
-        self.pipeline.add_action(CompressOverlay())
-        self.pipeline.add_action(PersistentNFSOverlay())  # idempotent
+            self.pipeline.add_action(SshAuthorize(self.job))
+        self.pipeline.add_action(VlandOverlayAction(self.job))
+        self.pipeline.add_action(MultinodeOverlayAction(self.job))
+        self.pipeline.add_action(TestDefinitionAction(self.job))
+        # Skip compress-overlay for actions that mount overlay directory
+        # that located on lava worker directly.
+        skip_overlay_compression = False
+        # Skip compress-overlay for 'to: docker' deploy action.
+        if parameters.get("to") == "docker":
+            skip_overlay_compression = True
+        # Skip compress-overlay for docker test shell based test action.
+        if {"docker", "definitions"}.issubset(parameters):
+            skip_overlay_compression = True
+        if not skip_overlay_compression:
+            self.pipeline.add_action(CompressOverlay(self.job))
+        self.pipeline.add_action(PersistentNFSOverlay(self.job))  # idempotent
 
     def _export_data(self, fout, data, prefix):
         if isinstance(data, dict):
@@ -201,7 +205,7 @@ class CreateOverlay(Action):
                 os.makedirs(path, 0o755)
                 self.logger.debug("makedir: %s", path)
         for fname in self.scripts_to_copy:
-            with open(fname, "r") as fin:
+            with open(fname) as fin:
                 foutname = os.path.basename(fname)
                 output_file = "%s/bin/%s" % (lava_path, foutname)
                 if "distro" in fname:
@@ -324,13 +328,12 @@ class OverlayAction(CreateOverlay):
 
 
 class MultinodeOverlayAction(OverlayAction):
-
     name = "lava-multinode-overlay"
     description = "add lava scripts during deployment for multinode test shell use"
     summary = "overlay the lava multinode scripts"
 
-    def __init__(self):
-        super().__init__()
+    def __init__(self, job: Job):
+        super().__init__(job)
         # Multinode-only
         self.lava_multi_node_test_dir = os.path.realpath(
             "%s/../../lava_test_shell/multi_node" % os.path.dirname(__file__)
@@ -396,7 +399,7 @@ class MultinodeOverlayAction(OverlayAction):
         self.logger.debug("scripts to copy %s", scripts_to_copy)
 
         for fname in scripts_to_copy:
-            with open(fname, "r") as fin:
+            with open(fname) as fin:
                 foutname = os.path.basename(fname)
                 output_file = "%s/bin/%s" % (lava_path, foutname)
                 self.logger.debug("Creating %s", output_file)
@@ -446,8 +449,8 @@ class VlandOverlayAction(OverlayAction):
     description = "Populate specific vland scripts for tests to lookup vlan data."
     summary = "Add files detailing vlan configuration."
 
-    def __init__(self):
-        super().__init__()
+    def __init__(self, job: Job):
+        super().__init__(job)
         # vland-only
         self.lava_vland_test_dir = os.path.realpath(
             "%s/../../lava_test_shell/vland" % os.path.dirname(__file__)
@@ -543,7 +546,7 @@ class VlandOverlayAction(OverlayAction):
         self.logger.debug({"lava_path": lava_path, "scripts": scripts_to_copy})
 
         for fname in scripts_to_copy:
-            with open(fname, "r") as fin:
+            with open(fname) as fin:
                 foutname = os.path.basename(fname)
                 output_file = "%s/bin/%s" % (lava_path, foutname)
                 self.logger.debug("Creating %s", output_file)
@@ -644,8 +647,8 @@ class SshAuthorize(Action):
     description = "include public key in overlay and authorize root user"
     summary = "add public key to authorized_keys"
 
-    def __init__(self):
-        super().__init__()
+    def __init__(self, job: Job):
+        super().__init__(job)
         self.active = False
         self.identity_file = None
 
@@ -741,6 +744,11 @@ class PersistentNFSOverlay(Action):
                 % self.parameters["persistent_nfs"]["address"]
             )
             return
+
+        persist["address"] = substitute_address_with_static_info(
+            persist["address"], self.job.device.get("static_info", [])
+        )
+
         nfs_server, dirname = persist["address"].split(":")
         which("rpcinfo")
         self.errors = rpcinfo_nfs(nfs_server)
