@@ -2,29 +2,20 @@
 #
 # Author: Matthew Hart <matthew.hart@linaro.org>
 #
-# This file is part of LAVA Dispatcher.
-#
-# LAVA Dispatcher is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# LAVA Dispatcher is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along
-# with this program; if not, see <http://www.gnu.org/licenses>.
+# SPDX-License-Identifier: GPL-2.0-or-later
+from __future__ import annotations
 
 import os
+from typing import TYPE_CHECKING
 
 from lava_common.exceptions import InfrastructureError
 from lava_dispatcher.action import Action, Pipeline
 from lava_dispatcher.utils.shell import which
 from lava_dispatcher.utils.strings import map_kernel_uboot
 
+if TYPE_CHECKING:
+    from lava_dispatcher.job import Job
+
 
 class PrepareKernelAction(Action):
     """
@@ -40,9 +31,9 @@ class PrepareKernelAction(Action):
         # the logic here can be upgraded in future if needed with more parameters to the deploy.
         methods = self.job.device["actions"]["boot"]["methods"]
         if "u-boot" in methods:
-            self.pipeline.add_action(UBootPrepareKernelAction())
+            self.pipeline.add_action(UBootPrepareKernelAction(self.job))
         elif "depthcharge" in methods:
-            self.pipeline.add_action(PrepareFITAction())
+            self.pipeline.add_action(PrepareFITAction(self.job))
 
 
 class UBootPrepareKernelAction(Action):
@@ -54,8 +45,8 @@ class UBootPrepareKernelAction(Action):
     description = "convert kernel to uimage"
     summary = "prepare/convert kernel"
 
-    def __init__(self):
-        super().__init__()
+    def __init__(self, job: Job):
+        super().__init__(job)
         self.bootcommand = None
         self.params = None
         self.kernel_type = None
@@ -189,8 +180,8 @@ class PrepareFITAction(Action):
     description = "package kernel, dtb and ramdisk into an FIT image"
     summary = "generate depthcharge FIT image"
 
-    def __init__(self):
-        super().__init__()
+    def __init__(self, job: Job):
+        super().__init__(job)
         self.deploy_params = None
         self.device_params = None
 
