@@ -2,26 +2,14 @@
 #
 # Author: Neil Williams <neil.williams@linaro.org>
 #
-# This file is part of LAVA Dispatcher.
-#
-# LAVA Dispatcher is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# LAVA Dispatcher is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along
-# with this program; if not, see <http://www.gnu.org/licenses>.
+# SPDX-License-Identifier: GPL-2.0-or-later
 
 # List just the subclasses supported for this base strategy
 # imported by the parser to populate the list of subclasses.
+from __future__ import annotations
 
 import os
+from typing import TYPE_CHECKING
 
 from lava_common.constants import DD_PROMPTS, DISPATCHER_DOWNLOAD_DIR
 from lava_dispatcher.action import Action, JobError, Pipeline, Timeout
@@ -33,6 +21,9 @@ from lava_dispatcher.logical import Deployment
 from lava_dispatcher.utils.network import dispatcher_ip
 from lava_dispatcher.utils.strings import substitute
 
+if TYPE_CHECKING:
+    from lava_dispatcher.job import Job
+
 
 class Removable(Deployment):
     """
@@ -47,12 +38,11 @@ class Removable(Deployment):
 
     """
 
-    compatibility = 1
     name = "removable"
 
     @classmethod
-    def action(cls):
-        return MassStorage()
+    def action(cls, job: Job) -> Action:
+        return MassStorage(job)
 
     @classmethod
     def accepts(cls, device, parameters):
@@ -92,9 +82,11 @@ class DDAction(Action):
     description = "deploy image to drive"
     summary = "write image to drive"
 
-    def __init__(self):
-        super().__init__()
-        self.timeout = Timeout(self.name, 600, exception=self.timeout_exception)
+    def __init__(self, job: Job):
+        super().__init__(job)
+        self.timeout = Timeout(
+            self.name, self, duration=600, exception=self.timeout_exception
+        )
         self.boot_params = None
         self.tool_prompts = None
         self.tool_flags = None
@@ -224,7 +216,7 @@ class DDAction(Action):
             tool_cmd = [writer_params["tool"], tool_options]
         else:
             tool_cmd = [
-                "dd of='{}' bs=4M".format(device_path)
+                f"dd of='{device_path}' bs=4M"
             ]  # busybox dd does not support other flags
         if self.tool_flags:
             tool_cmd.append(self.tool_flags)
@@ -259,13 +251,12 @@ class DDAction(Action):
 
 
 class MassStorage(Action):
-
     name = "storage-deploy"
     description = "Deploy image to mass storage"
     summary = "write image to storage"
 
-    def __init__(self):
-        super().__init__()
+    def __init__(self, job: Job):
+        super().__init__(job)
         self.suffix = None
         self.image_path = None
 
@@ -297,12 +288,14 @@ class MassStorage(Action):
         self.image_path = self.mkdtemp()
         self.pipeline = Pipeline(parent=self, job=self.job, parameters=parameters)
         if self.test_needs_overlay(parameters):
-            self.pipeline.add_action(OverlayAction())  # idempotent, includes testdef
+            # idempotent, includes testdef
+            self.pipeline.add_action(OverlayAction(self.job))
         uniquify = parameters.get("uniquify", True)
         if "images" in parameters:
             for k in sorted(parameters["images"].keys()):
                 self.pipeline.add_action(
                     DownloaderAction(
+                        self.job,
                         k,
                         path=self.image_path,
                         uniquify=uniquify,
@@ -311,11 +304,12 @@ class MassStorage(Action):
                 )
                 if parameters["images"][k].get("apply-overlay", False):
                     if self.test_needs_overlay(parameters):
-                        self.pipeline.add_action(ApplyOverlayImage())
-            self.pipeline.add_action(DDAction())
+                        self.pipeline.add_action(ApplyOverlayImage(self.job))
+            self.pipeline.add_action(DDAction(self.job))
         elif "image" in parameters:
             self.pipeline.add_action(
                 DownloaderAction(
+                    self.job,
                     "image",
                     path=self.image_path,
                     uniquify=uniquify,
@@ -323,9 +317,9 @@ class MassStorage(Action):
                 )
             )
             if self.test_needs_overlay(parameters):
-                self.pipeline.add_action(ApplyOverlayImage())
-            self.pipeline.add_action(DDAction())
+                self.pipeline.add_action(ApplyOverlayImage(self.job))
+            self.pipeline.add_action(DDAction(self.job))
 
         # FIXME: could support tarballs too
         if self.test_needs_deployment(parameters):
-            self.pipeline.add_action(DeployDeviceEnvironment())
+            self.pipeline.add_action(DeployDeviceEnvironment(self.job))
