@@ -2,24 +2,11 @@
 #
 # Author: Neil Williams <neil.williams@linaro.org>
 #
-# This file is part of LAVA Dispatcher.
-#
-# LAVA Dispatcher is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# LAVA Dispatcher is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along
-# with this program; if not, see <http://www.gnu.org/licenses>.
-
+# SPDX-License-Identifier: GPL-2.0-or-later
+from __future__ import annotations
 
 import os
+from typing import TYPE_CHECKING
 
 from lava_dispatcher.action import Action, Pipeline
 from lava_dispatcher.actions.deploy.apply_overlay import ExtractModules, ExtractRootfs
@@ -29,6 +16,9 @@ from lava_dispatcher.actions.deploy.overlay import OverlayAction
 from lava_dispatcher.logical import Deployment
 from lava_dispatcher.protocols.multinode import MultinodeProtocol
 
+if TYPE_CHECKING:
+    from lava_dispatcher.job import Job
+
 # Deploy SSH can mean a few options:
 # for a primary connection, the device might need to be powered_on
 # for a secondary connection, the device must be deployed
@@ -43,12 +33,11 @@ class Ssh(Deployment):
     typically the overlay.
     """
 
-    compatibility = 1
     name = "ssh"
 
     @classmethod
-    def action(cls):
-        return ScpOverlay()
+    def action(cls, job: Job) -> Action:
+        return ScpOverlay(job)
 
     @classmethod
     def accepts(cls, device, parameters):
@@ -68,8 +57,8 @@ class ScpOverlay(Action):
     description = "prepare overlay and scp to device"
     summary = "copy overlay to device"
 
-    def __init__(self):
-        super().__init__()
+    def __init__(self, job: Job):
+        super().__init__(job)
         self.items = []
 
     def validate(self):
@@ -95,11 +84,13 @@ class ScpOverlay(Action):
             value=tar_flags,
             parameters=parameters,
         )
-        self.pipeline.add_action(OverlayAction())
+        self.pipeline.add_action(OverlayAction(self.job))
         for item in self.items:
             if item in parameters:
                 self.pipeline.add_action(
-                    DownloaderAction(item, path=self.mkdtemp()), params=parameters[item]
+                    DownloaderAction(
+                        self.job, item, path=self.mkdtemp(), params=parameters[item]
+                    ),
                 )
                 self.set_namespace_data(
                     action=self.name,
@@ -109,9 +100,9 @@ class ScpOverlay(Action):
                     parameters=parameters,
                 )
         # we might not have anything to download, just the overlay to push
-        self.pipeline.add_action(PrepareOverlayScp())
+        self.pipeline.add_action(PrepareOverlayScp(self.job))
         # prepare the device environment settings in common data for enabling in the boot step
-        self.pipeline.add_action(DeployDeviceEnvironment())
+        self.pipeline.add_action(DeployDeviceEnvironment(self.job))
 
 
 class PrepareOverlayScp(Action):
@@ -124,8 +115,8 @@ class PrepareOverlayScp(Action):
     description = "copy the overlay over an existing ssh connection"
     summary = "scp the overlay to the remote device"
 
-    def __init__(self):
-        super().__init__()
+    def __init__(self, job: Job):
+        super().__init__(job)
         self.host_keys = []
 
     def validate(self):
@@ -163,10 +154,10 @@ class PrepareOverlayScp(Action):
     def populate(self, parameters):
         self.pipeline = Pipeline(parent=self, job=self.job, parameters=parameters)
         self.pipeline.add_action(
-            ExtractRootfs()
+            ExtractRootfs(self.job)
         )  # idempotent, checks for nfsrootfs parameter
         self.pipeline.add_action(
-            ExtractModules()
+            ExtractModules(self.job)
         )  # idempotent, checks for a modules parameter
 
     def run(self, connection, max_end_time):
