@@ -2,35 +2,26 @@
 #
 # Author: Neil Williams <neil.williams@linaro.org>
 #
-# This file is part of LAVA Dispatcher.
-#
-# LAVA Dispatcher is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# LAVA Dispatcher is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along
-# with this program; if not, see <http://www.gnu.org/licenses>.
+# SPDX-License-Identifier: GPL-2.0-or-later
+from __future__ import annotations
 
 import hashlib
 import os
 import re
 import shutil
+from typing import TYPE_CHECKING
 
-from lava_common.compat import yaml_safe_dump, yaml_safe_load
 from lava_common.constants import DEFAULT_TESTDEF_NAME_CLASS, DISPATCHER_DOWNLOAD_DIR
 from lava_common.decorators import nottest
 from lava_common.exceptions import InfrastructureError, JobError, LAVABug, TestError
+from lava_common.yaml import yaml_safe_dump, yaml_safe_load
 from lava_dispatcher.action import Action, Pipeline
 from lava_dispatcher.utils.compression import untar_file
 from lava_dispatcher.utils.vcs import GitHelper
 
+if TYPE_CHECKING:
+    from lava_dispatcher.job import Job
+
 
 @nottest
 def identify_test_definitions(test_info, namespace):
@@ -73,13 +64,12 @@ def get_test_action_namespaces(parameters=None):
 
 
 class RepoAction(Action):
-
     name = "repo-action"
     description = "apply tests to the test image"
     summary = "repo base class"
 
-    def __init__(self):
-        super().__init__()
+    def __init__(self, job: Job):
+        super().__init__(job)
         self.vcs = None
         self.runner = None
         self.uuid = None
@@ -317,7 +307,7 @@ class GitRepoAction(RepoAction):
         yaml_file = os.path.join(runner_path, self.parameters["path"])
         self.logger.debug("Tests stored (tmp) in %s", yaml_file)
         try:
-            with open(yaml_file, "r") as test_file:
+            with open(yaml_file) as test_file:
                 testdef = yaml_safe_load(test_file)
         except OSError as exc:
             raise JobError(
@@ -332,7 +322,6 @@ class GitRepoAction(RepoAction):
 
 
 class InlineRepoAction(RepoAction):
-
     priority = 1
     name = "inline-repo-action"
     description = "apply inline test definition to the test image"
@@ -383,14 +372,13 @@ class InlineRepoAction(RepoAction):
 
 
 class UrlRepoAction(RepoAction):
-
     priority = 1
     name = "url-repo-action"
     description = "apply a single test file to the test image"
     summary = "download file test"
 
-    def __init__(self):
-        super().__init__()
+    def __init__(self, job: Job):
+        super().__init__(job)
         self.tmpdir = None  # FIXME: needs to be a /mntpoint/lava-%hostname/ directory.
         self.testdef = None
 
@@ -417,7 +405,9 @@ class UrlRepoAction(RepoAction):
         self.action_key = "url_repo"
         self.internal_pipeline = Pipeline(parent=self, job=self.job, parameters=params)
         self.internal_pipeline.add_action(
-            DownloaderAction(self.action_key, self.download_dir, params=params)
+            DownloaderAction(
+                self.job, self.action_key, self.download_dir, params=params
+            )
         )
 
     def run(self, connection, max_end_time):
@@ -444,9 +434,9 @@ class UrlRepoAction(RepoAction):
         self.logger.debug("Tests stored (tmp) in %s", yaml_file)
 
         try:
-            with open(yaml_file, "r") as test_file:
+            with open(yaml_file) as test_file:
                 testdef = yaml_safe_load(test_file)
-        except IOError as exc:
+        except OSError as exc:
             raise JobError(
                 "Unable to open test definition '%s': %s"
                 % (self.parameters["path"], str(exc))
@@ -459,12 +449,11 @@ class UrlRepoAction(RepoAction):
 
 @nottest
 class TestDefinitionAction(Action):
-
     name = "test-definition"
     description = "load test definitions into image"
     summary = "loading test definitions"
 
-    def __init__(self):
+    def __init__(self, job: Job):
         """
         The TestDefinitionAction installs each test definition into
         the overlay. It does not execute the scripts in the test
@@ -474,7 +463,7 @@ class TestDefinitionAction(Action):
         In addition, a TestOverlayAction is added to the pipeline
         to handle parts of the overlay which are test definition dependent.
         """
-        super().__init__()
+        super().__init__(job)
         self.test_list = None
         self.stages = 0
         self.run_levels = {}
@@ -503,7 +492,7 @@ class TestDefinitionAction(Action):
             for testdef in testdefs:
                 # namespace support allows only running the install steps for the relevant
                 # deployment as the next deployment could be a different OS.
-                handler = RepoAction.select(testdef["from"])()
+                handler = RepoAction.select(testdef["from"])(self.job)
 
                 # set the full set of job YAML parameters for this handler as handler parameters.
                 handler.job = self.job
@@ -522,21 +511,21 @@ class TestDefinitionAction(Action):
                 self.run_levels[testdef["name"]] = self.stages
 
                 # copy details into the overlay, one per handler but the same class each time.
-                overlay = TestOverlayAction()
+                overlay = TestOverlayAction(self.job)
                 overlay.job = self.job
                 overlay.parameters = testdef
                 overlay.parameters["test_name"] = handler.parameters["test_name"]
                 overlay.test_uuid = handler.uuid
 
                 # add install handler - uses job parameters
-                installer = TestInstallAction()
+                installer = TestInstallAction(self.job)
                 installer.job = self.job
                 installer.parameters = testdef
                 installer.parameters["test_name"] = handler.parameters["test_name"]
                 installer.test_uuid = handler.uuid
 
                 # add runsh handler - uses job parameters
-                runsh = TestRunnerAction()
+                runsh = TestRunnerAction(self.job)
                 runsh.job = self.job
                 runsh.parameters = testdef
                 runsh.parameters["test_name"] = handler.parameters["test_name"]
@@ -593,7 +582,7 @@ class TestDefinitionAction(Action):
         for testdefs in self.test_list:
             for testdef in testdefs:
                 try:
-                    RepoAction.select(testdef["from"])()
+                    RepoAction.select(testdef["from"])(self.job)
                 except JobError as exc:
                     self.errors = str(exc)
 
@@ -647,12 +636,11 @@ class TestDefinitionAction(Action):
 
 @nottest
 class TestOverlayAction(Action):
-
     name = "test-overlay"
     description = "overlay test support files onto image"
     summary = "applying LAVA test overlay"
 
-    def __init__(self):
+    def __init__(self, job: Job):
         """
         TestOverlayAction is a simple helper to do the same routine boilerplate
         for every RepoAction, tweaking the data for the specific parameters of
@@ -663,7 +651,7 @@ class TestOverlayAction(Action):
         so the overlay has access to the same parameters as the handler and is
         always executed immediately after the relevant handler.
         """
-        super().__init__()
+        super().__init__(job)
         self.test_uuid = None  # Match the overlay to the handler
 
     def validate(self):
@@ -682,13 +670,13 @@ class TestOverlayAction(Action):
         if "params" in testdef:
             raise_if_not_dict(testdef, "params")
             for def_param_name, def_param_value in list(testdef["params"].items()):
-                if not def_param_value:
+                if def_param_value is None:
                     def_param_value = ""
                 ret_val.append("%s='%s'\n" % (def_param_name, def_param_value))
         if "parameters" in testdef:
             raise_if_not_dict(testdef, "parameters")
             for def_param_name, def_param_value in list(testdef["parameters"].items()):
-                if not def_param_value:
+                if def_param_value is None:
                     def_param_value = ""
                 ret_val.append("%s='%s'\n" % (def_param_name, def_param_value))
         ret_val.append("######\n")
@@ -698,7 +686,7 @@ class TestOverlayAction(Action):
             raise_if_not_dict(self.parameters, "parameters")
             # turn a string into a local variable.
             for param_name, param_value in list(self.parameters["parameters"].items()):
-                if not param_value:
+                if param_value is None:
                     param_value = ""
                 ret_val.append("%s='%s'\n" % (param_name, param_value))
                 self.logger.debug("%s='%s'", param_name, param_value)
@@ -706,7 +694,7 @@ class TestOverlayAction(Action):
             raise_if_not_dict(self.parameters, "params")
             # turn a string into a local variable.
             for param_name, param_value in list(self.parameters["params"].items()):
-                if not param_value:
+                if param_value is None:
                     param_value = ""
                 ret_val.append("%s='%s'\n" % (param_name, param_value))
                 self.logger.debug("%s='%s'", param_name, param_value)
@@ -722,7 +710,7 @@ class TestOverlayAction(Action):
         # now read the YAML to create a testdef dict to retrieve metadata
         yaml_file = os.path.join(runner_path, self.parameters["path"])
         try:
-            with open(yaml_file, "r") as test_file:
+            with open(yaml_file) as test_file:
                 testdef = yaml_safe_load(test_file)
         except OSError as exc:
             raise JobError(
@@ -763,12 +751,11 @@ class TestOverlayAction(Action):
 
 @nottest
 class TestInstallAction(TestOverlayAction):
-
     name = "test-install-overlay"
     description = "overlay dependency installation support files onto image"
     summary = "applying LAVA test install scripts"
 
-    def __init__(self):
+    def __init__(self, job: Job):
         """
         This Action will need a run check that the file does not exist
         and then it will create it.
@@ -779,7 +766,7 @@ class TestInstallAction(TestOverlayAction):
         run the pipeline at the start of the TestOverlayAction
         run step.
         """
-        super().__init__()
+        super().__init__(job)
         self.test_uuid = None  # Match the overlay to the handler
         self.skip_list = [
             "keys",
@@ -874,7 +861,7 @@ class TestInstallAction(TestOverlayAction):
             else:
                 raise TestError("Unrecognised git-repos block.")
             if commit_id is None:
-                raise JobError("Unable to clone %s" % str((repo)))
+                raise JobError("Unable to clone %s" % str(repo))
 
     def run(self, connection, max_end_time):
         connection = super().run(connection, max_end_time)
@@ -885,7 +872,7 @@ class TestInstallAction(TestOverlayAction):
         # now read the YAML to create a testdef dict to retrieve metadata
         yaml_file = os.path.join(runner_path, self.parameters["path"])
         try:
-            with open(yaml_file, "r") as test_file:
+            with open(yaml_file) as test_file:
                 testdef = yaml_safe_load(test_file)
         except OSError as exc:
             raise JobError(
@@ -958,7 +945,6 @@ class TestInstallAction(TestOverlayAction):
 
 
 class TestRunnerAction(TestOverlayAction):
-
     # This name is used to tally the submitted definitions
     # to the definitions which actually reported results.
     # avoid changing the self.name of this class.
@@ -966,8 +952,8 @@ class TestRunnerAction(TestOverlayAction):
     description = "overlay run script onto image"
     summary = "applying LAVA test run script"
 
-    def __init__(self):
-        super().__init__()
+    def __init__(self, job: Job):
+        super().__init__(job)
         self.testdef_levels = (
             {}
         )  # allow looking up the testname from the level of this action
@@ -984,7 +970,7 @@ class TestRunnerAction(TestOverlayAction):
             self.errors = "Test definition names need to be unique."
         # convert from testdef_index {0: 'smoke-tests', 1: 'singlenode-advanced'}
         # to self.testdef_levels {'1.3.4.1': '0_smoke-tests', ...}
-        for (count, name) in enumerate(testdef_index):
+        for count, name in enumerate(testdef_index):
             if self.parameters["name"] == name:
                 self.testdef_levels[self.level] = "%s_%s" % (count, name)
         if not self.testdef_levels:
@@ -1009,7 +995,7 @@ class TestRunnerAction(TestOverlayAction):
         # now read the YAML to create a testdef dict to retrieve metadata
         yaml_file = os.path.join(runner_path, self.parameters["path"])
         try:
-            with open(yaml_file, "r") as test_file:
+            with open(yaml_file) as test_file:
                 testdef = yaml_safe_load(test_file)
         except OSError as exc:
             raise JobError(
