@@ -2,27 +2,15 @@
 #
 # Author: Neil Williams <neil.williams@linaro.org>
 #
-# This file is part of LAVA Dispatcher.
-#
-# LAVA Dispatcher is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# LAVA Dispatcher is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along
-# with this program; if not, see <http://www.gnu.org/licenses>.
+# SPDX-License-Identifier: GPL-2.0-or-later
 
 # List just the subclasses supported for this base strategy
 # imported by the parser to populate the list of subclasses.
+from __future__ import annotations
 
 import os
 import tempfile
+from typing import TYPE_CHECKING
 
 from lava_common.constants import TFTP_SIZE_LIMIT
 from lava_common.exceptions import JobError
@@ -35,6 +23,9 @@ from lava_dispatcher.logical import Deployment
 from lava_dispatcher.utils import filesystem
 from lava_dispatcher.utils.shell import which
 
+if TYPE_CHECKING:
+    from lava_dispatcher.job import Job
+
 
 class Tftp(Deployment):
     """
@@ -44,12 +35,11 @@ class Tftp(Deployment):
     rootfs deployments would format the device and create a single partition for the rootfs.
     """
 
-    compatibility = 1
     name = "tftp"
 
     @classmethod
-    def action(cls):
-        return TftpAction()
+    def action(cls, job: Job) -> Action:
+        return TftpAction(job)
 
     @classmethod
     def accepts(cls, device, parameters):
@@ -63,13 +53,12 @@ class Tftp(Deployment):
 
 
 class TftpAction(Action):
-
     name = "tftp-deploy"
     description = "download files and deploy using tftp"
     summary = "tftp deployment"
 
-    def __init__(self):
-        super().__init__()
+    def __init__(self, job: Job):
+        super().__init__(job)
         self.tftp_dir = None
 
     def validate(self):
@@ -111,25 +100,39 @@ class TftpAction(Action):
             "modules",
             "preseed",
             "tee",
+            "dtbo",
         ]:
             if key in parameters:
-                self.pipeline.add_action(
-                    DownloaderAction(key, path=self.tftp_dir, params=parameters[key])
-                )
-                if key == "ramdisk":
-                    self.set_namespace_data(
-                        action=self.name,
-                        label="tftp",
-                        key="ramdisk",
-                        value=True,
-                        parameters=parameters,
+                if key == "dtbo":
+                    for index, parameter in enumerate(parameters[key]):
+                        self.pipeline.add_action(
+                            DownloaderAction(
+                                self.job,
+                                f"{key}{index}",
+                                path=self.tftp_dir,
+                                params=parameter,
+                            )
+                        )
+                else:
+                    self.pipeline.add_action(
+                        DownloaderAction(
+                            self.job, key, path=self.tftp_dir, params=parameters[key]
+                        )
                     )
+                    if key == "ramdisk":
+                        self.set_namespace_data(
+                            action=self.name,
+                            label="tftp",
+                            key="ramdisk",
+                            value=True,
+                            parameters=parameters,
+                        )
 
         # TftpAction is a deployment, so once the files are in place, just do the overlay
-        self.pipeline.add_action(PrepareOverlayTftp())
-        self.pipeline.add_action(LxcCreateUdevRuleAction())
+        self.pipeline.add_action(PrepareOverlayTftp(self.job))
+        self.pipeline.add_action(LxcCreateUdevRuleAction(self.job))
         if self.test_needs_deployment(parameters):
-            self.pipeline.add_action(DeployDeviceEnvironment())
+            self.pipeline.add_action(DeployDeviceEnvironment(self.job))
 
     def run(self, connection, max_end_time):
         # Extract the 3 last path elements. See action.mkdtemp()
@@ -143,7 +146,7 @@ class TftpAction(Action):
             "tftp_size_limit", TFTP_SIZE_LIMIT
         )
         self.logger.debug("Checking files for TFTP limit of %s bytes.", tftp_size_limit)
-        for (action, key) in [
+        for action, key in [
             ("compress-ramdisk", "ramdisk"),
             ("download-action", "kernel"),
             ("download-action", "dtb"),
