@@ -2,27 +2,15 @@
 #
 # Author: Dean Arnold <dean.arnold@linaro.org>
 #
-# This file is part of LAVA Dispatcher.
-#
-# LAVA Dispatcher is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# LAVA Dispatcher is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along
-# with this program; if not, see <http://www.gnu.org/licenses>.
+# SPDX-License-Identifier: GPL-2.0-or-later
 
 # List just the subclasses supported for this base strategy
 # imported by the parser to populate the list of subclasses.
+from __future__ import annotations
 
 import os
 import shutil
+from typing import TYPE_CHECKING
 
 from lava_common.constants import VEXPRESS_AUTORUN_INTERRUPT_CHARACTER
 from lava_common.exceptions import InfrastructureError, LAVABug
@@ -39,6 +27,9 @@ from lava_dispatcher.utils.filesystem import (
 )
 from lava_dispatcher.utils.udev import WaitUSBMassStorageDeviceAction
 
+if TYPE_CHECKING:
+    from lava_dispatcher.job import Job
+
 
 class VExpressMsd(Deployment):
     """
@@ -47,12 +38,11 @@ class VExpressMsd(Deployment):
     to target device
     """
 
-    compatibility = 1
     name = "vemsd"
 
     @classmethod
-    def action(cls):
-        return VExpressMsdRetry()
+    def action(cls, job: Job) -> Action:
+        return VExpressMsdRetry(job)
 
     @classmethod
     def uses_deployment_data(cls):
@@ -71,14 +61,13 @@ class VExpressMsd(Deployment):
 
 
 class VExpressMsdRetry(RetryAction):
-
     name = "vexpress-fw-deploy-retry"
     description = "deploy vexpress board recovery image with retry"
     summary = "VExpress FW deployment with retry"
 
     def populate(self, parameters):
         self.pipeline = Pipeline(parent=self, job=self.job, parameters=parameters)
-        self.pipeline.add_action(VExpressMsdAction())
+        self.pipeline.add_action(VExpressMsdAction(self.job))
 
 
 class VExpressMsdAction(Action):
@@ -101,24 +90,27 @@ class VExpressMsdAction(Action):
         self.pipeline = Pipeline(parent=self, job=self.job, parameters=parameters)
         self.pipeline.add_action(
             DownloaderAction(
-                "recovery_image", path=download_dir, params=parameters["recovery_image"]
+                self.job,
+                "recovery_image",
+                path=download_dir,
+                params=parameters["recovery_image"],
             )
         )
-        self.pipeline.add_action(LxcCreateUdevRuleAction())
+        self.pipeline.add_action(LxcCreateUdevRuleAction(self.job))
         self.force_prompt = True
-        self.pipeline.add_action(ConnectDevice())
-        self.pipeline.add_action(ResetDevice())
-        self.pipeline.add_action(ExtractVExpressRecoveryImage())
-        self.pipeline.add_action(EnterVExpressMCC())
-        self.pipeline.add_action(EnableVExpressMassStorage())
-        self.pipeline.add_action(WaitUSBMassStorageDeviceAction())
-        self.pipeline.add_action(MountVExpressMassStorageDevice())
-        self.pipeline.add_action(DeployVExpressRecoveryImage())
-        self.pipeline.add_action(UnmountVExpressMassStorageDevice())
+        self.pipeline.add_action(ConnectDevice(self.job))
+        self.pipeline.add_action(ResetDevice(self.job))
+        self.pipeline.add_action(ExtractVExpressRecoveryImage(self.job))
+        self.pipeline.add_action(EnterVExpressMCC(self.job))
+        self.pipeline.add_action(EnableVExpressMassStorage(self.job))
+        self.pipeline.add_action(WaitUSBMassStorageDeviceAction(self.job))
+        self.pipeline.add_action(MountVExpressMassStorageDevice(self.job))
+        self.pipeline.add_action(DeployVExpressRecoveryImage(self.job))
+        self.pipeline.add_action(UnmountVExpressMassStorageDevice(self.job))
         if self.job.device["actions"]["deploy"]["methods"]["vemsd"]["parameters"].get(
             "flash_prompt", False
         ):
-            self.pipeline.add_action(VExpressFlashErase())
+            self.pipeline.add_action(VExpressFlashErase(self.job))
 
 
 class ExtractVExpressRecoveryImage(Action):
@@ -130,8 +122,8 @@ class ExtractVExpressRecoveryImage(Action):
     description = "unpack versatile express recovery image"
     summary = "unpack versatile express recovery image ready for deployment"
 
-    def __init__(self):
-        super().__init__()
+    def __init__(self, job: Job):
+        super().__init__(job)
         self.param_key = "recovery_image"
         self.file_key = "recovery_image"
         self.compression = None
@@ -200,8 +192,8 @@ class EnterVExpressMCC(Action):
     description = "enter Versatile Express MCC"
     summary = "enter Versatile Express MCC, interrupting autorun if needed"
 
-    def __init__(self):
-        super().__init__()
+    def __init__(self, job: Job):
+        super().__init__(job)
         self.device_params = None
         self.interrupt_char = None
         self.mcc_prompt = None
@@ -264,8 +256,8 @@ class EnableVExpressMassStorage(Action):
     description = "enable vexpress usb msd"
     summary = "enable vexpress usb mass storage device"
 
-    def __init__(self):
-        super().__init__()
+    def __init__(self, job: Job):
+        super().__init__(job)
         self.mcc_prompt = None
         self.mcc_cmd = None
 
@@ -305,8 +297,8 @@ class MountDeviceMassStorageDevice(Action):
     command_exception = InfrastructureError
     timeout_exception = InfrastructureError
 
-    def __init__(self):
-        super().__init__()
+    def __init__(self, job: Job):
+        super().__init__(job)
         self.device_name = None
         self.disk_identifier = None
         self.disk_identifier_type = None  # uuid, id, label
@@ -369,8 +361,8 @@ class MountVExpressMassStorageDevice(MountDeviceMassStorageDevice):
     description = "mount vexpress usb msd"
     summary = "mount vexpress usb mass storage device on the dispatcher"
 
-    def __init__(self):
-        super().__init__()
+    def __init__(self, job: Job):
+        super().__init__(job)
         self.device_name = "Versatile Express"
         self.disk_identifier = None
         self.disk_identifier_type = "label"
@@ -440,8 +432,8 @@ class UnmountVExpressMassStorageDevice(Action):
     command_exception = InfrastructureError
     timeout_exception = InfrastructureError
 
-    def __init__(self):
-        super().__init__()
+    def __init__(self, job: Job):
+        super().__init__(job)
         self.namespace_label = "vexpress-fw"
         self.namespace_action = "mount-vexpress-usbmsd"
 
@@ -471,8 +463,8 @@ class VExpressFlashErase(Action):
     description = "erase vexpress flash"
     summary = "erase vexpress flash using the commands set by the user"
 
-    def __init__(self):
-        super().__init__()
+    def __init__(self, job: Job):
+        super().__init__(job)
         self.mcc_prompt = None
         self.flash_prompt = None
         self.flash_enter_cmd = None
