@@ -2,25 +2,13 @@
 #
 # Author: Antonio Terceiro <antonio.terceiro@linaro.org>
 #
-# This file is part of LAVA Dispatcher.
-#
-# LAVA Dispatcher is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# LAVA Dispatcher is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along
-# with this program; if not, see <http://www.gnu.org/licenses>.
+# SPDX-License-Identifier: GPL-2.0-or-later
+from __future__ import annotations
 
 import os
 import pathlib
 import shlex
+from typing import TYPE_CHECKING
 
 from lava_common.constants import LAVA_DOWNLOADS
 from lava_common.exceptions import LAVATimeoutError
@@ -31,9 +19,12 @@ from lava_dispatcher.actions.test.shell import TestShellAction
 from lava_dispatcher.logical import LavaTest
 from lava_dispatcher.power import ReadFeedback
 from lava_dispatcher.shell import ShellCommand, ShellSession
+from lava_dispatcher.utils.containers import DeviceContainerMappingMixin
 from lava_dispatcher.utils.docker import DockerRun
 from lava_dispatcher.utils.udev import get_udev_devices
-from lava_dispatcher_host.action import DeviceContainerMappingMixin
+
+if TYPE_CHECKING:
+    from lava_dispatcher.job import Job
 
 
 class DockerTest(LavaTest):
@@ -44,8 +35,8 @@ class DockerTest(LavaTest):
     priority = 10
 
     @classmethod
-    def action(cls, parameters):
-        return DockerTestAction()
+    def action(cls, job: Job, parameters) -> Action:
+        return DockerTestAction(job)
 
     @classmethod
     def accepts(cls, device, parameters):
@@ -67,7 +58,7 @@ class DockerTest(LavaTest):
         return True
 
 
-class GetBoardId:
+class GetBoardId(Action):
     @property
     def device_info(self):
         return self.job.device.get("device_info")
@@ -79,7 +70,7 @@ class GetBoardId:
         return device_info[0].get("board_id")
 
 
-class DockerTestAction(Action, GetBoardId):
+class DockerTestAction(GetBoardId):
     name = "lava-docker-test"
     description = "Runs tests in a docker container"
     summary = "Runs tests in a docker container, with the DUT available via adb/fastboot over USB"
@@ -87,16 +78,16 @@ class DockerTestAction(Action, GetBoardId):
 
     def populate(self, parameters):
         self.pipeline = Pipeline(parent=self, job=self.job, parameters=parameters)
-        self.pipeline.add_action(DockerTestSetEnvironment())
-        self.pipeline.add_action(CreateOverlay())
+        self.pipeline.add_action(DockerTestSetEnvironment(self.job))
+        self.pipeline.add_action(CreateOverlay(self.job))
         if "role" in parameters:
-            self.pipeline.add_action(MultinodeDockerTestShell())
+            self.pipeline.add_action(MultinodeDockerTestShell(self.job))
         else:
-            self.pipeline.add_action(DockerTestShell())
-        self.pipeline.add_action(ReadFeedback())
+            self.pipeline.add_action(DockerTestShell(self.job))
+        self.pipeline.add_action(ReadFeedback(self.job))
 
 
-class DockerTestSetEnvironment(Action, GetBoardId):
+class DockerTestSetEnvironment(GetBoardId):
     name = "lava-docker-test-set-environment"
     description = "Adds necessary environments variables for docker-test-shell"
     summary = description
@@ -169,9 +160,16 @@ class DockerTestShell(TestShellAction, GetBoardId, DeviceContainerMappingMixin):
         ).strip("/")
 
         container = "lava-docker-test-shell-%s-%s" % (self.job.job_id, self.level)
+        prefix = self.job.parameters.get("dispatcher", {}).get("prefix", "")
+        if prefix:
+            container = "lava-docker-test-shell-%s-%s-%s" % (
+                prefix,
+                self.job.job_id,
+                self.level,
+            )
 
         docker = DockerRun.from_parameters(self.parameters["docker"], self.job)
-        docker.prepare()
+        docker.prepare(action=self)
         docker.bind_mount(os.path.join(location, overlay), "/" + overlay)
 
         docker_method_conf = (
