@@ -2,24 +2,12 @@
 #
 # Author: Neil Williams <neil.williams@linaro.org>
 #
-# This file is part of LAVA Dispatcher.
-#
-# LAVA Dispatcher is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# LAVA Dispatcher is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along
-# with this program; if not, see <http://www.gnu.org/licenses>.
+# SPDX-License-Identifier: GPL-2.0-or-later
+from __future__ import annotations
 
 import json
 import re
+from typing import TYPE_CHECKING
 
 from lava_common.exceptions import MultinodeProtocolTimeoutError, TestError
 from lava_common.timeout import Timeout
@@ -29,6 +17,9 @@ from lava_dispatcher.actions.test.shell import TestShell, TestShellAction
 from lava_dispatcher.logical import LavaTest
 from lava_dispatcher.protocols.multinode import MultinodeProtocol
 
+if TYPE_CHECKING:
+    from lava_dispatcher.action import Action
+    from lava_dispatcher.job import Job
 
 # TODO: This is a workaround allowing to run multinode jobs with "monitors"
 # and "interactive" test actions - simple scenarios, without cross-device
@@ -39,12 +30,6 @@ from lava_dispatcher.protocols.multinode import MultinodeProtocol
 # from which individual test action implementations would inherit. In the
 # meantime, this is a small and localized workaround enabling multinode
 # for those test actions ahead of heavy refactors above.
-def get_subaction(parameters):
-    if "monitors" in parameters:
-        return TestMonitor.action(parameters)
-    if "interactive" in parameters:
-        return TestInteractive.action(parameters)
-    return MultinodeTestAction()
 
 
 def get_subaction_class(parameters):
@@ -64,8 +49,12 @@ class MultinodeTestShell(LavaTest):
     priority = 2
 
     @classmethod
-    def action(cls, parameters):
-        return get_subaction(parameters)
+    def action(cls, job: Job, parameters) -> Action:
+        if "monitors" in parameters:
+            return TestMonitor.action(job, parameters)
+        if "interactive" in parameters:
+            return TestInteractive.action(job, parameters)
+        return MultinodeTestAction(job)
 
     @classmethod
     def accepts(cls, device, parameters):
@@ -98,11 +87,10 @@ class MultinodeTestShell(LavaTest):
 
 
 class MultinodeMixin:
-
     timeout_exception = TestError
 
-    def __init__(self):
-        super().__init__()
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
         self.multinode_dict = {"multinode": r"<LAVA_MULTI_NODE> <LAVA_(\S+) ([^>]+)>"}
 
     def validate(self):
@@ -137,11 +125,11 @@ class MultinodeMixin:
         ]
         self.signal_director = self.SignalDirector(self.protocols[0])
 
-    def check_patterns(self, event, test_connection, check_char):
+    def check_patterns(self, event, test_connection):
         """
         Calls the parent check_patterns first, then checks for subclass pattern.
         """
-        ret = super().check_patterns(event, test_connection, check_char)
+        ret = super().check_patterns(event, test_connection)
         if event == "multinode":
             name, params = test_connection.match.groups()
             self.logger.debug("Received Multi_Node API <LAVA_%s>" % name)
@@ -270,7 +258,6 @@ class MultinodeMixin:
 
 
 class MultinodeTestAction(MultinodeMixin, TestShellAction):
-
     name = "multinode-test"
     description = "Executing lava-test-runner"
     summary = "Multinode Lava Test Shell"
