@@ -2,30 +2,17 @@
 #
 # Author: Neil Williams <neil.williams@linaro.org>
 #
-# This file is part of LAVA Dispatcher.
-#
-# LAVA Dispatcher is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# LAVA Dispatcher is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along
-# with this program; if not, see <http://www.gnu.org/licenses>.
+# SPDX-License-Identifier: GPL-2.0-or-later
+from __future__ import annotations
 
 import decimal
 import logging
 import re
 import time
+from typing import TYPE_CHECKING
 
 import pexpect
 
-from lava_common.compat import yaml_safe_dump
 from lava_common.decorators import nottest
 from lava_common.exceptions import (
     ConnectionClosedError,
@@ -33,10 +20,14 @@ from lava_common.exceptions import (
     LAVATimeoutError,
     TestError,
 )
+from lava_common.yaml import yaml_safe_dump
 from lava_dispatcher.action import Action, Pipeline
 from lava_dispatcher.connection import SignalMatch
 from lava_dispatcher.logical import LavaTest, RetryAction
 
+if TYPE_CHECKING:
+    from lava_dispatcher.job import Job
+
 
 def handle_testcase(params):
     data = {}
@@ -57,8 +48,8 @@ class TestShell(LavaTest):
     """
 
     @classmethod
-    def action(cls, parameters):
-        return TestShellRetry()
+    def action(cls, job: Job, parameters) -> Action:
+        return TestShellRetry(job)
 
     @classmethod
     def accepts(cls, device, parameters):
@@ -82,14 +73,13 @@ class TestShell(LavaTest):
 
 @nottest
 class TestShellRetry(RetryAction):
-
     name = "lava-test-retry"
     description = "Retry wrapper for lava-test-shell"
     summary = "Retry support for Lava Test Shell"
 
     def populate(self, parameters):
         self.pipeline = Pipeline(parent=self, job=self.job, parameters=parameters)
-        self.pipeline.add_action(TestShellAction())
+        self.pipeline.add_action(TestShellAction(self.job))
 
 
 # FIXME: move to utils and call inside the overlay
@@ -163,8 +153,8 @@ class TestShellAction(Action):
     summary = "Lava Test Shell"
     timeout_exception = LAVATimeoutError
 
-    def __init__(self):
-        super().__init__()
+    def __init__(self, job: Job):
+        super().__init__(job)
         self.signal_director = self.SignalDirector(None)  # no default protocol
         self.patterns = {}
         self.signal_match = SignalMatch()
@@ -343,9 +333,7 @@ class TestShellAction(Action):
                 # Because of the feedbacks, we use a small value for the
                 # timeout.  This allows to grab feedback regularly.
                 last_check = time.monotonic()
-                while self._keep_running(
-                    test_connection, test_connection.timeout, connection.check_char
-                ):
+                while self._keep_running(test_connection, test_connection.timeout):
                     # Only grab the feedbacks every test_connection.timeout
                     if (
                         feedbacks
@@ -396,8 +384,11 @@ class TestShellAction(Action):
         return True
 
     def signal_start_run(self, params):
-        self.signal_director.test_uuid = params[1]
-        self.definition = params[0]
+        try:
+            self.signal_director.test_uuid = params[1]
+            self.definition = params[0]
+        except IndexError:
+            raise TestError("Invalid signal")
         uuid = params[1]
         self.start = time.monotonic()
         self.logger.info("Starting test lava.%s (%s)", self.definition, uuid)
@@ -408,7 +399,7 @@ class TestShellAction(Action):
         uuid_list = self.get_namespace_data(
             action="repo-action", label="repo-action", key="uuid-list"
         )
-        for (key, value) in enumerate(testdef_index):
+        for key, value in enumerate(testdef_index):
             if self.definition == "%s_%s" % (key, value):
                 pattern_dict = self.get_namespace_data(
                     action="test", label=uuid_list[key], key="testdef_pattern"
@@ -437,8 +428,11 @@ class TestShellAction(Action):
             self.current_run.update({"commit_id": testdef_commit})
 
     def signal_end_run(self, params):
-        self.definition = params[0]
-        uuid = params[1]
+        try:
+            self.definition = params[0]
+            uuid = params[1]
+        except IndexError:
+            raise TestError("Invalid signal")
         # remove the pattern for this run from pattern_dict
         self._reset_patterns()
         # catch error in ENDRUN being handled without STARTRUN
@@ -615,7 +609,7 @@ class TestShellAction(Action):
             self.report[res["test_case_id"]] = res["result"]
         return True
 
-    def check_patterns(self, event, test_connection, check_char):
+    def check_patterns(self, event, test_connection):
         """
         Defines the base set of pattern responses.
         Stores the results of testcases inside the TestAction
@@ -680,18 +674,15 @@ class TestShellAction(Action):
             ret_val = self.pattern_test_case_result(test_connection)
         return ret_val
 
-    def _keep_running(self, test_connection, timeout, check_char):
+    def _keep_running(self, test_connection, timeout):
         if "test_case_results" in self.patterns:
             self.logger.info(
                 "Test case result pattern: %r" % self.patterns["test_case_results"]
             )
         retval = test_connection.expect(list(self.patterns.values()), timeout=timeout)
-        return self.check_patterns(
-            list(self.patterns.keys())[retval], test_connection, check_char
-        )
+        return self.check_patterns(list(self.patterns.keys())[retval], test_connection)
 
     class SignalDirector:
-
         # FIXME: create proxy handlers
         def __init__(self, protocol=None):
             """
