@@ -2,27 +2,16 @@
 #
 # Author: Neil Williams <neil.williams@linaro.org>
 #
-# This file is part of LAVA Dispatcher.
-#
-# LAVA Dispatcher is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# LAVA Dispatcher is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along
-# with this program; if not, see <http://www.gnu.org/licenses>.
+# SPDX-License-Identifier: GPL-2.0-or-later
 
+import contextlib
 import decimal
 import logging
 import os
 import signal
 
+from pexpect.exceptions import ExceptionPexpect
+
 from lava_common.exceptions import LAVABug, TestError
 from lava_common.timeout import Timeout
 from lava_dispatcher.action import InternalObject
@@ -96,13 +85,7 @@ class Connection:
     name = "Connection"
 
     def __init__(self, job, raw_connection):
-        self.device = job.device
-        self.job = job
-        # provide access to the context data of the running job
-        self.data = self.job.context
         self.raw_connection = raw_connection
-        self.results = {}
-        self.match = None
         self.connected = True
         self.check_char = "#"
         self.tags = []
@@ -170,8 +153,9 @@ class Connection:
 
         self.connected = False
         if self.raw_connection:
-            self.raw_connection.close(force=True)
-        self.raw_connection = None
+            with contextlib.suppress(ExceptionPexpect):
+                self.raw_connection.close(force=True)
+                self.raw_connection = None
 
     def finalise(self):
         # logger = logging.getLogger('dispatcher')
@@ -212,7 +196,7 @@ class Protocol:
 
     def __init__(self, parameters, job_id):
         self.logger = logging.getLogger("dispatcher")
-        self.poll_timeout = Timeout(self.name)
+        self.poll_timeout = Timeout(self.name, None)
         self.__errors__ = []
         self.parameters = parameters
         self.configured = False
