@@ -2,21 +2,7 @@
 #
 # Author: Senthil Kumaran S <senthil.kumaran@linaro.org>
 #
-# This file is part of LAVA Dispatcher.
-#
-# LAVA Dispatcher is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# LAVA Dispatcher is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along
-# with this program; if not, see <http://www.gnu.org/licenses>.
+# SPDX-License-Identifier: GPL-2.0-or-later
 
 from lava_dispatcher.action import Action, JobError
 from lava_dispatcher.shell import ShellCommand, ShellSession
@@ -62,7 +48,7 @@ class ConnectAdb(Action):
         self.run_command(adb_cmd)
         self.logger.debug("%s: Waiting for device", adb_serial_number)
 
-        cmd = "adb -s {0} shell".format(adb_serial_number)
+        cmd = f"adb -s {adb_serial_number} shell"
         self.logger.info("%s Connecting to device using '%s'", self.name, cmd)
         # ShellCommand executes the connection command
         shell = self.shell_class(
