@@ -2,21 +2,10 @@
 #
 # Author: Neil Williams <neil.williams@linaro.org>
 #
-# This file is part of LAVA Dispatcher.
-#
-# LAVA Dispatcher is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# LAVA Dispatcher is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along
-# with this program; if not, see <http://www.gnu.org/licenses>.
+# SPDX-License-Identifier: GPL-2.0-or-later
+from __future__ import annotations
+
+from typing import TYPE_CHECKING
 
 from lava_common.exceptions import InfrastructureError, JobError
 from lava_dispatcher.action import Action, Pipeline
@@ -25,6 +14,9 @@ from lava_dispatcher.logical import RetryAction
 from lava_dispatcher.shell import ShellCommand, ShellSession
 from lava_dispatcher.utils.shell import which
 
+if TYPE_CHECKING:
+    from lava_dispatcher.job import Job
+
 
 class ConnectDevice(Action):
     """
@@ -43,8 +35,8 @@ class ConnectDevice(Action):
     # runs the command to initiate the connection
     shell_class = ShellCommand
 
-    def __init__(self):
-        super().__init__()
+    def __init__(self, job: Job):
+        super().__init__(job)
         self.command = ""
         self.hardware = None
         self.primary = True
@@ -212,8 +204,8 @@ class ConnectShell(ConnectDevice):
     # runs the command to initiate the connection
     shell_class = ShellCommand
 
-    def __init__(self, name=None):
-        super().__init__()
+    def __init__(self, job: Job, name=None):
+        super().__init__(job)
         self.name = "connect-shell"
         self.primary = False
         self.hardware = name
@@ -266,8 +258,8 @@ class DisconnectDevice(ConnectDevice):
     Breaks the serial connection made by ConnectDevice.
     """
 
-    def __init__(self):
-        super().__init__()
+    def __init__(self, job: Job):
+        super().__init__(job)
         self.name = "disconnect-device"
         self.description = "disconnect from console"
         self.summary = self.description
@@ -338,5 +330,5 @@ class ResetConnection(RetryAction):
 
     def populate(self, parameters):
         self.pipeline = Pipeline(parent=self, job=self.job, parameters=parameters)
-        self.pipeline.add_action(DisconnectDevice())
-        self.pipeline.add_action(ConnectDevice())
+        self.pipeline.add_action(DisconnectDevice(self.job))
+        self.pipeline.add_action(ConnectDevice(self.job))
