@@ -2,22 +2,10 @@
 #
 # Author: Neil Williams <neil.williams@linaro.org>
 #
-# This file is part of LAVA Dispatcher.
-#
-# LAVA Dispatcher is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# LAVA Dispatcher is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along
-# with this program; if not, see <http://www.gnu.org/licenses>.
+# SPDX-License-Identifier: GPL-2.0-or-later
+from __future__ import annotations
 
+from typing import TYPE_CHECKING
 
 from lava_common.exceptions import InfrastructureError, JobError
 from lava_dispatcher.action import Action
@@ -25,6 +13,9 @@ from lava_dispatcher.shell import ShellCommand, ShellSession
 from lava_dispatcher.utils.filesystem import check_ssh_identity_file
 from lava_dispatcher.utils.shell import which
 
+if TYPE_CHECKING:
+    from lava_dispatcher.job import Job
+
 
 class SShSession(ShellSession):
     """Extends a ShellSession to include the ability to copy files using scp
@@ -64,8 +55,8 @@ class ConnectSsh(Action):
     summary = "make an ssh connection to a device"
     timeout_exception = InfrastructureError
 
-    def __init__(self):
-        super().__init__()
+    def __init__(self, job: Job):
+        super().__init__(job)
         self.command = None
         self.host = None
         self.ssh_port = ["-p", "22"]
@@ -146,6 +137,9 @@ class ConnectSsh(Action):
             self.command.extend(self.ssh_port)
 
     def run(self, connection, max_end_time):
+        connection = self.get_namespace_data(
+            action="shared", label="shared", key="connection", deepcopy=False
+        )
         if connection:
             self.logger.debug("Already connected")
             return connection
