@@ -2,21 +2,8 @@
 #
 # Author: Neil Williams <neil.williams@linaro.org>
 #
-# This file is part of LAVA Dispatcher.
-#
-# LAVA Dispatcher is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# LAVA Dispatcher is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along
-# with this program; if not, see <http://www.gnu.org/licenses>.
+# SPDX-License-Identifier: GPL-2.0-or-later
+from __future__ import annotations
 
 import datetime
 import errno
@@ -25,18 +12,24 @@ import shutil
 import tempfile
 import time
 import traceback
-
-import pytz
+from typing import TYPE_CHECKING
 
 from lava_common.constants import CLEANUP_TIMEOUT, DISPATCHER_DOWNLOAD_DIR
 from lava_common.exceptions import JobError, LAVABug, LAVAError
 from lava_common.version import __version__
-from lava_dispatcher.diagnostics import DiagnoseNetwork
 from lava_dispatcher.logical import PipelineContext
 from lava_dispatcher.protocols.multinode import (  # pylint: disable=unused-import
     MultinodeProtocol,
 )
 
+if TYPE_CHECKING:
+    from logging import Logger
+    from typing import Any
+
+    from lava_common.timeout import Timeout
+
+    from .device import Device
+
 
 class Job:
     """
@@ -54,19 +47,23 @@ class Job:
     device for this job - one job, one device.
     """
 
-    def __init__(self, job_id, parameters, logger):
+    def __init__(
+        self,
+        job_id: int,
+        parameters: dict[str, Any],
+        logger: Logger,
+        device: Device,
+        timeout: Timeout,
+    ):
         self.job_id = job_id
         self.logger = logger
-        self.device = None
+        self.device = device
         self.parameters = parameters
         self.__context__ = PipelineContext()
         self.pipeline = None
         self.connection = None
-        self.triggers = []  # actions can add trigger strings to the run a diagnostic
-        self.diagnostics = [DiagnoseNetwork]
-        self.timeout = None
+        self.timeout = timeout
         self.protocols = []
-        self.compatibility = 2
         # Was the job cleaned
         self.cleaned = False
         # override in use
@@ -82,20 +79,8 @@ class Job:
     def context(self, data):
         self.__context__.pipeline_data.update(data)
 
-    def diagnose(self, trigger):
-        """
-        Looks up the class to execute to diagnose the problem described by the
-         specified trigger.
-        """
-        trigger_tuples = [(cls.trigger(), cls) for cls in self.diagnostics]
-        for diagnostic in trigger_tuples:
-            if trigger is diagnostic[0]:
-                return diagnostic[1]()
-        return None
-
     def describe(self):
         return {
-            "compatibility": self.compatibility,
             "pipeline": self.pipeline.describe(),
         }
 
@@ -142,7 +127,7 @@ class Job:
         LAVAError) if it fails.
         """
         self.logger.info(
-            "Start time: %s (UTC)", pytz.utc.localize(datetime.datetime.utcnow())
+            "Start time: %s (UTC)", datetime.datetime.now(datetime.timezone.utc)
         )
         for protocol in self.protocols:
             try:
@@ -255,10 +240,11 @@ class Job:
             self.cleanup(self.connection)
 
     def cleanup(self, connection):
+        self.timeout.name = "job-cleanup"
         with self.timeout(None, time.monotonic() + CLEANUP_TIMEOUT) as max_end_time:
-            return self._cleanup(connection)
+            return self._cleanup(connection, max_end_time)
 
-    def _cleanup(self, connection):
+    def _cleanup(self, connection, max_end_time):
         if self.cleaned:
             self.logger.info("Cleanup already called, skipping")
             return
@@ -267,10 +253,10 @@ class Job:
         # connection and poweroff the device (the cleanup action will do that
         # for us)
         self.logger.info("Cleaning after the job")
-        self.pipeline.cleanup(connection)
+        self.pipeline.cleanup(connection, max_end_time)
 
         for tmp_dir in self.base_overrides.values():
-            self.logger.info("Override tmp directory removed at %s", tmp_dir)
+            self.logger.info("Removing override tmp directory at %s", tmp_dir)
             try:
                 shutil.rmtree(tmp_dir)
             except OSError as exc:
@@ -280,7 +266,7 @@ class Job:
                     )
 
         if self.tmp_dir is not None:
-            self.logger.info("Root tmp directory removed at %s", self.tmp_dir)
+            self.logger.info("Removing root tmp directory at %s", self.tmp_dir)
             try:
                 shutil.rmtree(self.tmp_dir)
             except OSError as exc:
