@@ -2,23 +2,11 @@
 #
 # Author: Neil Williams <neil.williams@linaro.org>
 #
-# This file is part of LAVA Dispatcher.
-#
-# LAVA Dispatcher is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# LAVA Dispatcher is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along
-# with this program; if not, see <http://www.gnu.org/licenses>.
+# SPDX-License-Identifier: GPL-2.0-or-later
+from __future__ import annotations
 
 import time
+from typing import TYPE_CHECKING
 
 from lava_common.exceptions import (
     ConfigurationError,
@@ -30,6 +18,11 @@ from lava_common.exceptions import (
 from lava_dispatcher.action import Action
 from lava_dispatcher.utils.strings import seconds_to_str
 
+if TYPE_CHECKING:
+    from typing import Optional
+
+    from lava_dispatcher.job import Job
+
 
 class RetryAction(Action):
     """
@@ -39,8 +32,8 @@ class RetryAction(Action):
     Only the top level Boot and Test actions support 'repeat' as this is set in the job.
     """
 
-    def __init__(self):
-        super().__init__()
+    def __init__(self, job: Job):
+        super().__init__(job)
         self.sleep = 1
 
     def validate(self):
@@ -60,7 +53,8 @@ class RetryAction(Action):
         parent_end_time = max_end_time
 
         retries = 0
-        has_failed_exc = None
+        has_failed_exc: Optional[Exception] = None
+        has_parent_timed_out = False
         self.call_protocols()
         while retries < self.max_retries:
             retries += 1
@@ -90,6 +84,7 @@ class RetryAction(Action):
 
                 # Stop retrying if parent timed out
                 if time.monotonic() >= parent_end_time:
+                    has_parent_timed_out = True
                     break
 
                 # Wait some time before retrying
@@ -108,40 +103,21 @@ class RetryAction(Action):
         # If we are repeating, check that all repeat were a success.
         if has_failed_exc:
             # tried and failed
-            # Use the same exception class as the last failed
-            # run exception.
-            retry_fail_exc = type(has_failed_exc)(
+            exception_text = (
                 f"{retries} retries out of "
                 f"{self.max_retries} failed for {self.name}"
             )
+            if has_parent_timed_out:
+                exception_text = (
+                    f"No time left for remaining {self.max_retries - retries} retries. "
+                ) + exception_text
+            # Use the same exception class as the last failed
+            # run exception.
+            retry_fail_exc = type(has_failed_exc)(exception_text)
             raise retry_fail_exc from has_failed_exc
         return connection
 
 
-class DiagnosticAction(Action):
-    """
-    Base class for actions which are run only if a failure is detected.
-    Diagnostics have no level and are not intended to be added to Pipeline.
-    """
-
-    section = "diagnostic"
-    name = "diagnose"
-    description = "action-specific diagnostics in case of failure"
-    summary = "diagnose action failure"
-
-    @classmethod
-    def trigger(cls):
-        raise NotImplementedError("Define in the subclass: %s" % cls)
-
-    def run(self, connection, max_end_time):
-        """
-        Log the requested diagnostic.
-        Raises NotImplementedError if subclass has omitted a trigger classmethod.
-        """
-        self.logger.debug("%s diagnostic triggered.", self.trigger())
-        return connection
-
-
 class Deployment:
     """
     Deployment is a strategy class which aggregates Actions
@@ -155,7 +131,6 @@ class Deployment:
 
     priority = 0
     section = "deploy"
-    compatibility = 0
 
     @property
     def parameters(self):
@@ -181,7 +156,7 @@ class Deployment:
     @classmethod
     def accepts(cls, device, parameters):
         """
-        Returns True if this deployment strategy can be used the the
+        Returns True if this deployment strategy can be used by the
         given device and details of an image in the parameters.
 
         Must be implemented by subclasses.
@@ -248,7 +223,6 @@ class Boot:
 
     priority = 0
     section = "boot"
-    compatibility = 0
 
     @classmethod
     def boot_check(cls, device, parameters):
@@ -272,7 +246,7 @@ class Boot:
     @classmethod
     def accepts(cls, device, parameters):
         """
-        Returns True if this deployment strategy can be used the the
+        Returns True if this deployment strategy can be used by the
         given device and details of an image in the parameters.
 
         Must be implemented by subclasses.
@@ -317,7 +291,6 @@ class LavaTest:
 
     priority = 1
     section = "test"
-    compatibility = 1  # used directly
 
     @classmethod
     def accepts(cls, device, parameters):
