@@ -2,29 +2,17 @@
 #
 # Author: Neil Williams <neil.williams@linaro.org>
 #
-# This file is part of LAVA Dispatcher.
-#
-# LAVA Dispatcher is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# LAVA Dispatcher is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along
-# with this program; if not, see <http://www.gnu.org/licenses>.
+# SPDX-License-Identifier: GPL-2.0-or-later
 
 # Bring in the strategy subclass lists, ignore pylint warnings.
 # pylint: disable=unused-import
+from __future__ import annotations
+
 import lava_dispatcher.actions.boot.strategies
 import lava_dispatcher.actions.deploy.strategies
 import lava_dispatcher.actions.test.strategies
 import lava_dispatcher.protocols.strategies
-from lava_common.compat import yaml_safe_load
+from lava_common.yaml import yaml_safe_load
 from lava_dispatcher.action import JobError, Pipeline, Timeout
 from lava_dispatcher.actions.commands import CommandAction
 from lava_dispatcher.connection import Protocol
@@ -44,11 +32,11 @@ def parse_action(job_data, name, device, pipeline, test_info, test_count):
 
     if name == "boot":
         cls = Boot.select(device, parameters)
-        action = cls.action()
+        action = cls.action(pipeline.job)
     elif name == "test":
         parameters["stage"] = test_count
         cls = LavaTest.select(device, parameters)
-        action = cls.action(parameters)
+        action = cls.action(pipeline.job, parameters)
     elif name == "deploy":
         cls = Deployment.select(device, parameters)
         ns = parameters["namespace"]
@@ -66,11 +54,10 @@ def parse_action(job_data, name, device, pipeline, test_info, test_count):
                 {"deployment_data": get_deployment_data(parameters.get("os", ""))}
             )
         cls = Deployment.select(device, parameters)
-        action = cls.action()
+        action = cls.action(pipeline.job)
 
     action.section = cls.section
     pipeline.add_action(action, parameters)
-    pipeline.job.compatibility = max(cls.compatibility, pipeline.job.compatibility)
     return cls
 
 
@@ -95,16 +82,22 @@ class JobParser:
 
     # FIXME: needs a Schema and a check routine
 
-    def _timeouts(self, data, job):
-        if "job" in data.get("timeouts", {}):
-            duration = Timeout.parse(data["timeouts"]["job"])
-            job.timeout = Timeout("job", duration)
+    @classmethod
+    def _parse_job_timeout(cls, data: dict) -> Timeout:
+        timeouts_dict = data["timeouts"]
+        duration = Timeout.parse(timeouts_dict["job"])
+        return Timeout("job", None, duration=duration)
 
     def parse(self, content, device, job_id, logger, dispatcher_config, env_dut=None):
         data = yaml_safe_load(content)
-        job = Job(job_id, data, logger)
+        job = Job(
+            job_id=job_id,
+            parameters=data,
+            logger=logger,
+            device=device,
+            timeout=self._parse_job_timeout(data),
+        )
         test_counts = {}
-        job.device = device
         job.parameters["env_dut"] = env_dut
         # Load the dispatcher config
         job.parameters["dispatcher"] = {}
@@ -120,7 +113,6 @@ class JobParser:
             for item in sorted(level_tuple, key=lambda level_tuple: level_tuple[1])
         ]
         pipeline = Pipeline(job=job)
-        self._timeouts(data, job)
 
         # deploy and boot classes can populate the pipeline differently depending
         # on the test action type they are linked with (via namespacing).
@@ -164,7 +156,7 @@ class JobParser:
                     if name == "test" and action.needs_overlay(action_data["test"]):
                         test_counts[namespace] += 1
                 elif name == "command":
-                    action = CommandAction()
+                    action = CommandAction(job)
                     action.parameters = action_data[name]
                     pipeline.add_action(action)
 
@@ -172,19 +164,9 @@ class JobParser:
                     raise JobError("Unknown action name '%s'" % name)
 
         # there's always going to need to be a finalize_process action
-        finalize = FinalizeAction()
+        finalize = FinalizeAction(job)
         pipeline.add_action(finalize)
-        finalize.populate(None)
+
         job.pipeline = pipeline
-        if "compatibility" in data:
-            try:
-                job_c = int(job.compatibility)
-                data_c = int(data["compatibility"])
-            except ValueError as exc:
-                raise JobError("invalid compatibility value: %s" % exc)
-            if job_c < data_c:
-                raise JobError(
-                    "Dispatcher unable to meet job compatibility requirement. %d > %d"
-                    % (job_c, data_c)
-                )
+
         return job
