@@ -2,21 +2,7 @@
 #
 # Author: Jan-Simon Moeller <jsmoeller@linuxfoundation.org>
 #
-# This file is part of LAVA Dispatcher.
-#
-# LAVA Dispatcher is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# LAVA Dispatcher is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along
-# with this program; if not, see <http://www.gnu.org/licenses>.
+# SPDX-License-Identifier: GPL-2.0-or-later
 
 
 import logging
@@ -41,7 +27,7 @@ class XnbdProtocol(Protocol):
     def __init__(self, parameters, job_id):
         super().__init__(parameters, job_id)
         # timeout in utils.constants, default 10000
-        self.system_timeout = Timeout("system", XNBD_SYSTEM_TIMEOUT)
+        self.system_timeout = Timeout("system", None, duration=XNBD_SYSTEM_TIMEOUT)
         self.logger = logging.getLogger("dispatcher")
         self.parameters = parameters
         self.port = None
