@@ -2,26 +2,15 @@
 #
 # Author: Neil Williams <neil.williams@linaro.org>
 #
-# This file is part of LAVA Dispatcher.
-#
-# LAVA Dispatcher is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# LAVA Dispatcher is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along
-# with this program; if not, see <http://www.gnu.org/licenses>.
+# SPDX-License-Identifier: GPL-2.0-or-later
+from __future__ import annotations
 
 import contextlib
 import logging
-import sre_constants
 import time
+from re import error as re_error
+from re import split as re_split
+from typing import TYPE_CHECKING
 
 import pexpect
 
@@ -38,6 +27,9 @@ from lava_dispatcher.action import Action
 from lava_dispatcher.connection import Connection
 from lava_dispatcher.utils.strings import seconds_to_str
 
+if TYPE_CHECKING:
+    from lava_dispatcher.job import Job
+
 
 class ShellLogger:
     """
@@ -45,33 +37,33 @@ class ShellLogger:
     using the logfile support built into pexpect.
     """
 
-    def __init__(self, logger):
+    def __init__(self, logger, is_input: bool = False):
         self.line = ""
         self.logger = logger
         self.is_feedback = False
+        self.is_input = is_input
 
     def write(self, new_line):
         replacements = {
-            "\n\n": "\n",  # double lines to single
-            "\r": "",
-            '"': '\\"',  # escape double quotes for YAML syntax
             "\x1b": "",  # remove escape control characters
         }
-        for key, value in replacements.items():
-            new_line = new_line.replace(key, value)
         lines = self.line + new_line
 
         # Print one full line at a time. A partial line is kept in memory.
         if "\n" in lines:
             last_ret = lines.rindex("\n")
             self.line = lines[last_ret + 1 :]
-            lines = lines[:last_ret]
-            for line in lines.split("\n"):
+            lines = lines[: last_ret + 1]
+            for line in re_split("\r\r\n|\r\n|\n", lines)[:-1]:
+                for key, value in replacements.items():
+                    line = line.replace(key, value)
                 if self.is_feedback:
                     if self.namespace:
                         self.logger.feedback(line, namespace=self.namespace)
                     else:
                         self.logger.feedback(line)
+                elif self.is_input:
+                    self.logger.input(line)
                 else:
                     self.logger.target(line)
         else:
@@ -107,24 +99,30 @@ class ShellCommand(pexpect.spawn):
                     "ShellCommand was passed an invalid window size of %s bytes."
                     % window
                 )
+        searchwindowsize = 2 * window
         if not lava_timeout or not isinstance(lava_timeout, Timeout):
             raise LAVABug("ShellCommand needs a timeout set by the calling Action")
         if not logger:
             raise LAVABug("ShellCommand needs a logger")
-        pexpect.spawn.__init__(
-            self,
+        super().__init__(
             command,
             timeout=lava_timeout.duration,
             cwd=cwd,
-            logfile=ShellLogger(logger),
+            logfile=None,  # Split logging
             encoding="utf-8",
             # Data before searchwindowsize point is preserved, but not searched.
-            searchwindowsize=4000,  # pattern match in twice the default window size
+            searchwindowsize=searchwindowsize,  # pattern match in twice the window size
             maxread=window,  # limit the size of the buffer. 1 to turn off buffering
             codec_errors="replace",
         )
+        self.logfile_read = ShellLogger(logger)
+        self.logfile_send = ShellLogger(logger, is_input=True)
         self.name = "ShellCommand"
         self.logger = logger
+        # delayafterterminate allow for some spare time for a process to terminate
+        # If the system is loaded
+        # See https://github.com/pexpect/pexpect/issues/462
+        self.delayafterterminate = 1.0
         # set a default newline character, but allow actions to override as necessary
         self.linesep = LINE_SEPARATOR
         self.lava_timeout = lava_timeout
@@ -142,12 +140,12 @@ class ShellCommand(pexpect.spawn):
         if delay > 0:
             self.logger.debug("Sending with %s millisecond of delay", delay)
             send_char = True
-        self.logger.input(s + self.linesep)
+        self.logger.debug("Sending line: %r", s)
         self.send(s, delay, send_char)
         self.send(self.linesep, delay)
 
     def sendcontrol(self, char):
-        self.logger.input(char)
+        self.logger.debug("Sending character: %r", char)
         return super().sendcontrol(char)
 
     def send(self, string, delay=0, send_char=True):
@@ -173,8 +171,8 @@ class ShellCommand(pexpect.spawn):
         """
         try:
             proc = super().expect(*args, **kw)
-        except sre_constants.error as exc:
-            msg = "Invalid regular expression '%s': %s" % (exc.pattern, exc.msg)
+        except re_error as exc:
+            msg = f"Invalid regular expression '{exc.pattern}': {exc.msg}"
             raise TestError(msg)
         except pexpect.TIMEOUT:
             raise TestError("ShellCommand command timed out.")
@@ -185,19 +183,13 @@ class ShellCommand(pexpect.spawn):
             raise ConnectionClosedError("Connection closed")
         return proc
 
-    def empty_buffer(self):
-        """Make sure there is nothing in the pexpect buffer."""
-        index = 0
-        while index == 0:
-            index = self.expect([".+", pexpect.EOF, pexpect.TIMEOUT], timeout=1)
-
     def flush(self):
         """Will be called by pexpect itself when closing the connection"""
-        self.logfile.flush(force=True)
+        self.logfile_read.flush(force=True)
+        self.logfile_send.flush(force=True)
 
 
 class ShellSession(Connection):
-
     name = "ShellSession"
 
     def __init__(self, job, shell_command):
@@ -290,7 +282,7 @@ class ShellSession(Connection):
 
     def wait(self, max_end_time=None, max_searchwindowsize=False):
         """
-        Simple wait without sendling blank lines as that causes the menu
+        Simple wait without sending blank lines as that causes the menu
         to advance without data which can cause blank entries and can cause
         the menu to exit to an unrecognised prompt.
         """
@@ -325,14 +317,14 @@ class ShellSession(Connection):
         if timeout < 0:
             raise LAVABug("Invalid timeout value passed to listen_feedback()")
         try:
-            self.raw_connection.logfile.is_feedback = True
-            self.raw_connection.logfile.namespace = namespace
+            self.raw_connection.logfile_read.is_feedback = True
+            self.raw_connection.logfile_read.namespace = namespace
             index = self.raw_connection.expect(
                 [".+", pexpect.EOF, pexpect.TIMEOUT], timeout=timeout
             )
         finally:
-            self.raw_connection.logfile.is_feedback = False
-            self.raw_connection.logfile.namespace = None
+            self.raw_connection.logfile_read.is_feedback = False
+            self.raw_connection.logfile_read.namespace = None
 
         if index == 0:
             return len(self.raw_connection.after)
@@ -346,13 +338,12 @@ class ExpectShellSession(Action):
     all that is needed is a prompt.
     """
 
-    compatibility = 2
     name = "expect-shell-connection"
     description = "Wait for a shell"
     summary = "Expect a shell prompt"
 
-    def __init__(self):
-        super().__init__()
+    def __init__(self, job: Job):
+        super().__init__(job)
         self.force_prompt = True
 
     def validate(self):
