@@ -2,21 +2,7 @@
 #
 # Author: Matthew Hart <matthew.hart@linaro.org>
 #
-# This file is part of LAVA Dispatcher.
-#
-# LAVA Dispatcher is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# LAVA Dispatcher is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along
-# with this program; if not, see <http://www.gnu.org/licenses>.
+# SPDX-License-Identifier: GPL-2.0-or-later
 
 
 # ramdisk, always cpio, comp: gz,xz
@@ -34,12 +20,18 @@ from lava_dispatcher.utils.contextmanager import chdir
 from lava_dispatcher.utils.shell import which
 
 # https://www.kernel.org/doc/Documentation/xz.txt
-compress_command_map = {"xz": ["xz", "--check=crc32"], "gz": ["gzip"], "bz2": ["bzip2"]}
+compress_command_map = {
+    "xz": ["xz", "--check=crc32"],
+    "gz": ["gzip"],
+    "bz2": ["bzip2"],
+    "zstd": ["zstd"],
+}
 decompress_command_map = {
     "xz": ["unxz"],
     "gz": ["gunzip"],
     "bz2": ["bunzip2"],
     "zip": ["unzip"],
+    "zstd": ["unzstd"],
 }
 
 
