@@ -2,37 +2,68 @@
 #
 # Author: Antonio Terceiro <antonio.terceiro@linaro.org>
 #
-# This file is part of LAVA Dispatcher.
-#
-# LAVA Dispatcher is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# LAVA Dispatcher is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along
-# with this program; if not, see <http://www.gnu.org/licenses>.
+# SPDX-License-Identifier: GPL-2.0-or-later
 
 
 import os
 import uuid
+from subprocess import CalledProcessError
+from typing import TYPE_CHECKING
 
-from lava_common.exceptions import InfrastructureError
+from lava_common.device_mappings import add_device_container_mapping
+from lava_common.exceptions import InfrastructureError, LAVABug
 from lava_dispatcher.action import Action, InternalObject
+from lava_dispatcher.utils.decorator import retry
 from lava_dispatcher.utils.docker import DockerContainer, DockerRun
 from lava_dispatcher.utils.filesystem import copy_to_lxc
 from lava_dispatcher.utils.lxc import is_lxc_requested, lxc_cmd_prefix
 from lava_dispatcher.utils.udev import get_udev_devices
-from lava_dispatcher_host.action import DeviceContainerMappingMixin
+
+if TYPE_CHECKING:
+    from lava_dispatcher.job import Job
 
 
-class OptionalContainerAction(Action, DeviceContainerMappingMixin):
+class DeviceContainerMappingMixin(Action):
+    """
+    This mixing should be included by action classes that add device/container
+    mappings.
+    """
+
+    def __init__(self, job: "Job"):
+        super().__init__(job)
+        self.containers: list[DockerContainer] = []
+
+    def add_device_container_mappings(self, container, container_type):
+        device_info = self.job.device.get("device_info", [])
+        static_info = self.job.device.get("static_info", [])
+        job_id = self.job.job_id
+        job_prefix = self.job.parameters["dispatcher"].get("prefix", "")
+        devices = []
+        for origdevice in device_info + static_info:
+            device = origdevice.copy()
+            if "board_id" in device:
+                device["serial_number"] = device["board_id"]
+                del device["board_id"]
+            devices.append(device)
+        for device in devices:
+            add_device_container_mapping(
+                job_prefix + job_id, device, container, container_type=container_type
+            )
+            self.logger.info(
+                f"Added mapping for {device} to {container_type} container {container}"
+            )
+
+    def trigger_share_device_with_container(self, device):
+        """
+        Trigger udev to let lava-dispatcher-host into sharing the device with
+        container.
+        """
+        self.run_cmd(["udevadm", "trigger", "--action=add", device], allow_fail=True)
+
+
+class OptionalContainerAction(DeviceContainerMappingMixin):
     command_exception = InfrastructureError
+    timeout_exception = InfrastructureError
 
     def validate(self):
         super().validate()
@@ -73,6 +104,14 @@ class OptionalContainerAction(Action, DeviceContainerMappingMixin):
     def get_output_maybe_in_container(self, cmd, **kwargs):
         return self.driver.get_output(cmd)
 
+    def cleanup(self, connection):
+        super().cleanup(connection)
+
+        if isinstance(self.driver, DockerDriver):
+            for container in self.containers:
+                self.logger.debug(f"Destroying docker container {container.__name__}")
+                container.destroy()
+
 
 class NullDriver(InternalObject):
     is_container = False
@@ -150,36 +189,60 @@ class DockerDriver(NullDriver):
         docker = self.build(DockerContainer)
         name = self.get_container_name()
         docker.name(name)
+        self.action.containers.append(docker)
         docker.start(self.action)
         try:
-            self.__map_devices__(name)
+            self.__map_devices__(name, docker)
             docker.run(cmd, self.action)
         finally:
-            docker.stop()
+            docker.stop(self.action)
 
     def get_output(self, cmd):
         # FIXME duplicates most of run()
         docker = self.build(DockerContainer)
         name = self.get_container_name()
         docker.name(name)
+        self.action.containers.append(docker)
         docker.start(self.action)
         try:
-            self.__map_devices__(name)
+            self.__map_devices__(name, docker)
             return docker.get_output(cmd, self.action)
         finally:
-            docker.stop()
+            docker.stop(self.action)
 
     def maybe_copy_to_container(self, src):
         if src not in self.copied_files:
             self.copied_files.append(src)
         return src
 
-    def __map_devices__(self, container_name):
+    @retry(exception=LAVABug, retries=3, delay=1)
+    def _retry_trigger_share_device_with_container(
+        self, action: Action, dev: str, docker: DockerContainer
+    ) -> None:
+        """
+        Re-trigger device sharing if device doesn't appear in docker container after 60s
+        waiting. 60s is the default udev event processing timeout.
+        """
+        action.trigger_share_device_with_container(dev)
+        action.logger.debug(
+            f"Waiting for device '{dev}' to appear in docker container {docker.__name__} ..."
+        )
+        try:
+            docker.wait_file(dev, 60)
+        except CalledProcessError:
+            raise LAVABug(
+                f"Failed to share device '{dev}' to docker container {docker.__name__}"
+            )
+        action.logger.info(
+            f"Shared device '{dev}' to docker container {docker.__name__}"
+        )
+
+    def __map_devices__(self, container_name, docker):
         action = self.action
         action.add_device_container_mappings(container_name, "docker")
         for dev in self.__get_device_nodes__():
             if not os.path.islink(dev):
-                action.trigger_share_device_with_container(dev)
+                self._retry_trigger_share_device_with_container(action, dev, docker)
 
     def __get_device_nodes__(self):
         device_info = self.action.job.device.get("device_info", {})
@@ -195,4 +258,5 @@ class DockerDriver(NullDriver):
 
     def validate(self):
         docker = DockerRun.from_parameters(self.params, self.action.job)
+        docker.add_docker_options(*self.docker_options)
         docker.prepare(self.action)
