@@ -2,26 +2,14 @@
 #
 # Author: Antonio Terceiro <antonio.terceiro@linaro.org>
 #
-# This file is part of LAVA Dispatcher.
-#
-# LAVA Dispatcher is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# LAVA Dispatcher is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, see <http://www.gnu.org/licenses>.
+# SPDX-License-Identifier: GPL-2.0-or-later
 
 import logging
 import random
 import subprocess
 import time
 from pathlib import Path
+from typing import Optional
 
 from lava_common.exceptions import InfrastructureError
 
@@ -40,6 +28,7 @@ class DockerRun:
         self.__environment__ = []
         self.__interactive__ = False
         self.__tty__ = False
+        self.__init = True
         self.__docker_options__ = []
         self.__docker_run_options__ = []
 
@@ -62,7 +51,7 @@ class DockerRun:
         suffix = ""
         if random_suffix:
             CHARS = "01234567890abcdefghijklmnopqrtsuwxyz"
-            suffix = "".join((random.SystemRandom().choice(CHARS) for i in range(10)))
+            suffix = "".join(random.SystemRandom().choice(CHARS) for i in range(10))
         self.__name__ = name + suffix
 
     def network(self, network):
@@ -77,6 +66,9 @@ class DockerRun:
     def workdir(self, workdir):
         self.__workdir__ = workdir
 
+    def init(self, init):
+        self.__init = init
+
     def add_device(self, device, skip_missing=False):
         if not Path(device).exists() and skip_missing:
             return
@@ -123,7 +115,9 @@ class DockerRun:
         return cmd
 
     def start_options(self):
-        cmd = ["--rm", "--init"]
+        cmd = ["--rm"]
+        if self.__init:
+            cmd.append("--init")
         if self.__name__:
             cmd.append(f"--name={self.__name__}")
         if self.__network__:
@@ -143,34 +137,37 @@ class DockerRun:
             cmd.append(f"--env={variable}={value}")
         return cmd
 
-    def run(self, *args, action=None):
+    def run(self, *args, action, capture=False, error_msg=None):
         self.prepare(action)
         cmd = self.cmdline(*args)
-        self.run_cmd(cmd, action=action)
-
-    def run_cmd(self, cmd, action=None):
-        if action:
-            runner = action.run_cmd
-        else:
-            runner = subprocess.check_call
-        logger = logging.getLogger("dispatcher")
-        logger.debug("cmd: %s", cmd)
-        runner(cmd)
-
-    def prepare(self, action=None):
-        if self.__local__:
-            self.run_cmd(
-                [
-                    "docker",
-                    "image",
-                    "inspect",
-                    f"--format=Image {self.image} exists locally",
-                    self.image,
-                ],
-                action=action,
+        if capture:
+            return subprocess.check_output(cmd, stderr=subprocess.STDOUT).decode(
+                "utf-8", errors="replace"
             )
         else:
-            self.run_cmd(["docker", "pull", self.image], action=action)
+            return action.run_cmd(cmd, error_msg=error_msg)
+
+    def prepare(self, action):
+        pull = not self.__local__
+        if self.__local__:
+            if action.run_cmd(
+                [
+                    "docker",
+                    *self.__docker_options__,
+                    "image",
+                    "inspect",
+                    "--format",
+                    f"Image {self.image} exists locally",
+                    self.image,
+                ],
+                allow_fail=True,
+            ):
+                action.logger.warning(
+                    "Unable to inspect docker image '%s'" % self.image
+                )
+                pull = True
+        if pull:
+            action.run_cmd(["docker", *self.__docker_options__, "pull", self.image])
         self.__check_image_arch__()
 
     def wait(self, shell=None):
@@ -182,7 +179,13 @@ class DockerRun:
                 if shell and not shell.isalive():
                     raise InfrastructureError("Docker container unexpectedly exited")
                 subprocess.check_call(
-                    ["docker", "inspect", "--format=.", self.__name__],
+                    [
+                        "docker",
+                        *self.__docker_options__,
+                        "inspect",
+                        "--format=.",
+                        self.__name__,
+                    ],
                     stdout=subprocess.DEVNULL,
                     stderr=subprocess.DEVNULL,
                 )
@@ -191,39 +194,64 @@ class DockerRun:
                 time.sleep(delay)
                 delay = delay * 2  # exponential backoff
 
-    def wait_file(self, filename):
+    def wait_file(self, filename: str, timeout: Optional[int] = None) -> None:
         delay = 1
+        start = time.monotonic()
         while True:
             try:
                 subprocess.check_call(
-                    ["docker", "exec", self.__name__, "test", "-e", filename],
+                    [
+                        "docker",
+                        *self.__docker_options__,
+                        "exec",
+                        self.__name__,
+                        "test",
+                        "-e",
+                        filename,
+                    ],
                     stdout=subprocess.DEVNULL,
                     stderr=subprocess.DEVNULL,
                 )
                 return
             except subprocess.CalledProcessError:
+                if timeout is not None and time.monotonic() - start > timeout:
+                    raise
                 time.sleep(delay)
                 delay = delay * 2  # exponential backoff
 
     def destroy(self):
         if self.__name__:
             subprocess.call(
-                ["docker", "rm", "-f", self.__name__],
+                ["docker", *self.__docker_options__, "rm", "-f", self.__name__],
                 stdout=subprocess.DEVNULL,
                 stderr=subprocess.DEVNULL,
             )
 
     def __check_image_arch__(self):
         host = subprocess.check_output(["arch"], text=True).strip()
-        container = subprocess.check_output(
-            ["docker", "inspect", "--format", "{{.Architecture}}", self.image],
-            text=True,
-        ).strip()
+        try:
+            container = subprocess.check_output(
+                [
+                    "docker",
+                    *self.__docker_options__,
+                    "inspect",
+                    "--format",
+                    "{{.Architecture}}",
+                    self.image,
+                ],
+                text=True,
+            ).strip()
+        except FileNotFoundError:
+            raise InfrastructureError("'docker' command not available on the worker")
         # amd64 = x86_64
         if host == "amd64":
             host = "x86_64"
         if container == "amd64":
             container = "x86_64"
+        if host == "arm64":
+            host = "aarch64"
+        if container == "arm64":
+            container = "aarch64"
         if host != container:
             logger = logging.getLogger("dispatcher")
             logger.warning(
@@ -234,40 +262,36 @@ class DockerRun:
 class DockerContainer(DockerRun):
     __started__ = False
 
-    def run(self, args, action=None):
+    def run(self, args, action):
         self.start(action)
         cmd = ["docker", *self.__docker_options__, "exec"]
         cmd += self.interaction_options()
         cmd.append(self.__name__)
         cmd += args
-        self.run_cmd(cmd, action)
+        action.run_cmd(cmd)
 
-    def get_output(self, args, action=None):
-        if action:
-            runner = action.parsed_command
-        else:
-            runner = self.check_output
+    def get_output(self, args, action):
         self.start(action)
         cmd = ["docker", *self.__docker_options__, "exec"]
         cmd += self.interaction_options()
         cmd.append(self.__name__)
         cmd += args
-        return runner(cmd)
+        return action.parsed_command(cmd)
 
     def check_output(self, cmd):
         return subprocess.check_output(cmd).decode("utf-8")
 
-    def start(self, action=None):
+    def start(self, action):
         if self.__started__:
             return
 
-        cmd = ["docker", "run", "--detach"]
+        cmd = ["docker", *self.__docker_options__, "run", "--detach"]
         cmd += self.start_options()
         cmd.append(self.image)
         cmd += ["sleep", "infinity"]
-        self.run_cmd(cmd, action)
+        action.run_cmd(cmd)
         self.wait()
         self.__started__ = True
 
-    def stop(self, action=None):
-        self.run_cmd(["docker", "stop", self.__name__])
+    def stop(self, action):
+        action.run_cmd(["docker", *self.__docker_options__, "stop", self.__name__])
