@@ -2,23 +2,10 @@
 #
 # Author: Antonio Terceiro <antonio.terceiro@linaro.org>
 #
-# This file is part of LAVA Dispatcher.
-#
-# LAVA Dispatcher is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# LAVA Dispatcher is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along
-# with this program; if not, see <http://www.gnu.org/licenses>.
+# SPDX-License-Identifier: GPL-2.0-or-later
 
 
+from lava_dispatcher.power import PowerOff
 from lava_dispatcher.utils.containers import OptionalContainerAction
 
 
@@ -34,3 +21,8 @@ class OptionalContainerFastbootAction(OptionalContainerAction):
 
     def get_fastboot_output(self, cmd, **kwargs):
         return self.get_output_maybe_in_container(self.get_fastboot_cmd(cmd), **kwargs)
+
+    def on_timeout(self):
+        self.logger.error("fastboot timing out, power-off the DuT")
+        power_off = PowerOff(self.job)
+        power_off.run(None, self.timeout.duration)
