@@ -2,39 +2,29 @@
 #
 # Author: Senthil Kumaran S <senthil.kumaran@linaro.org>
 #
-# This file is part of LAVA Dispatcher.
-#
-# LAVA Dispatcher is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# LAVA Dispatcher is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along
-# with this program; if not, see <http://www.gnu.org/licenses>.
+# SPDX-License-Identifier: GPL-2.0-or-later
+from __future__ import annotations
 
 import time
+from typing import TYPE_CHECKING
 
 import pyudev
 
 from lava_common.exceptions import InfrastructureError, LAVABug
 from lava_dispatcher.action import Action
 
+if TYPE_CHECKING:
+    from lava_dispatcher.job import Job
+
 
 class WaitUSBSerialDeviceAction(Action):
-
     name = "wait-usb-serial"
     description = "wait for USB serial device"
     summary = "wait for USB serial device"
     timeout_exception = InfrastructureError
 
-    def __init__(self):
-        super().__init__()
+    def __init__(self, job: Job):
+        super().__init__(job)
         self.serial_device = {}
         self.usb_sleep = 0
 
@@ -73,14 +63,13 @@ class WaitUSBSerialDeviceAction(Action):
 
 
 class WaitDFUDeviceAction(Action):
-
     name = "wait-dfu-device"
     description = "wait for DFU device"
     summary = "wait for DFU device"
     timeout_exception = InfrastructureError
 
-    def __init__(self):
-        super().__init__()
+    def __init__(self, job: Job):
+        super().__init__(job)
         self.dfu_device = {}
 
     def validate(self):
@@ -110,14 +99,13 @@ class WaitDFUDeviceAction(Action):
 
 
 class WaitUSBMassStorageDeviceAction(Action):
-
     name = "wait-usb-mass-storage-device"
     description = "wait for USB mass storage device"
     summary = "wait for USB mass storage device"
     timeout_exception = InfrastructureError
 
-    def __init__(self):
-        super().__init__()
+    def __init__(self, job: Job):
+        super().__init__(job)
         self.ms_device = {}
 
     def validate(self):
@@ -137,14 +125,13 @@ class WaitUSBMassStorageDeviceAction(Action):
 
 
 class WaitDevicePathAction(Action):
-
     name = "wait-device-path"
     description = "wait for udev device path"
     summary = "wait for udev device path"
     timeout_exception = InfrastructureError
 
-    def __init__(self, path=None):
-        super().__init__()
+    def __init__(self, job: Job, path=None):
+        super().__init__(job)
         self.devicepath = path
 
     def validate(self):
@@ -160,14 +147,13 @@ class WaitDevicePathAction(Action):
 
 
 class WaitDeviceBoardID(Action):
-
     name = "wait-device-boardid"
     description = "wait for udev device with board ID"
     summary = "wait for udev device with board ID"
     timeout_exception = InfrastructureError
 
-    def __init__(self, board_id=None):
-        super().__init__()
+    def __init__(self, job: Job, board_id=None):
+        super().__init__(job)
         self.udev_device = None
         if not board_id:
             self.board_id = self.job.device.get("board_id")
@@ -206,7 +192,7 @@ def _dict_compare(d1, d2):
     d1_keys = set(d1.keys())
     d2_keys = set(d2.keys())
     intersect_keys = d1_keys.intersection(d2_keys)
-    return set(o for o in intersect_keys if d1[o] == d2[o])
+    return {o for o in intersect_keys if d1[o] == d2[o]}
 
 
 def match(device, match_dict, devicepath):
@@ -319,64 +305,65 @@ def get_udev_devices(job=None, logger=None, device_info=None, required=False):
         # check if device is already connected
         # try with all parameters such as board id, usb_vendor_id and
         # usb_product_id
-        for device in context.list_devices():
+        for udev_device in context.list_devices():
+            udev_device_properties = udev_device.properties
             if board_id and usb_vendor_id and usb_product_id:
                 if (
-                    (device.get("ID_SERIAL_SHORT") == board_id)
-                    and (device.get("ID_VENDOR_ID") == usb_vendor_id)
-                    and (device.get("ID_MODEL_ID") == usb_product_id)
+                    (udev_device_properties.get("ID_SERIAL_SHORT") == board_id)
+                    and (udev_device_properties.get("ID_VENDOR_ID") == usb_vendor_id)
+                    and (udev_device_properties.get("ID_MODEL_ID") == usb_product_id)
                 ):
-                    device_paths.add(device.device_node)
+                    device_paths.add(udev_device.device_node)
                     added.add(board_id)
-                    for child in device.children:
+                    for child in udev_device.children:
                         if child.device_node:
                             device_paths.add(child.device_node)
-                    for link in device.device_links:
+                    for link in udev_device.device_links:
                         device_paths.add(link)
             elif board_id and usb_vendor_id and not usb_product_id:
                 # try with parameters such as board id, usb_vendor_id
-                if (device.get("ID_SERIAL_SHORT") == board_id) and (
-                    device.get("ID_VENDOR_ID") == usb_vendor_id
+                if (udev_device_properties.get("ID_SERIAL_SHORT") == board_id) and (
+                    udev_device_properties.get("ID_VENDOR_ID") == usb_vendor_id
                 ):
-                    device_paths.add(device.device_node)
+                    device_paths.add(udev_device.device_node)
                     added.add(board_id)
-                    for child in device.children:
+                    for child in udev_device.children:
                         if child.device_node:
                             device_paths.add(child.device_node)
-                    for link in device.device_links:
+                    for link in udev_device.device_links:
                         device_paths.add(link)
             elif board_id and not usb_vendor_id and not usb_product_id:
                 # try with board id alone
-                if device.get("ID_SERIAL_SHORT") == board_id:
-                    device_paths.add(device.device_node)
+                if udev_device_properties.get("ID_SERIAL_SHORT") == board_id:
+                    device_paths.add(udev_device.device_node)
                     added.add(board_id)
-                    for child in device.children:
+                    for child in udev_device.children:
                         if child.device_node:
                             device_paths.add(child.device_node)
-                    for link in device.device_links:
+                    for link in udev_device.device_links:
                         device_paths.add(link)
             elif usb_vendor_id and usb_product_id:
                 # try with vendor and product id
                 if (
-                    device.get("ID_VENDOR_ID") == usb_vendor_id
-                    and device.get("ID_MODEL_ID") == usb_product_id
+                    udev_device_properties.get("ID_VENDOR_ID") == usb_vendor_id
+                    and udev_device_properties.get("ID_MODEL_ID") == usb_product_id
                 ):
-                    device_paths.add(device.device_node)
+                    device_paths.add(udev_device.device_node)
                     added.add(usb_product_id)
-                    for child in device.children:
+                    for child in udev_device.children:
                         if child.device_node:
                             device_paths.add(child.device_node)
-                    for link in device.device_links:
+                    for link in udev_device.device_links:
                         device_paths.add(link)
             elif usb_fs_label:
                 # Just restrict by filesystem label.
-                if device.get("ID_FS_LABEL") == usb_fs_label:
-                    device_paths.add(device.device_node)
+                if udev_device_properties.get("ID_FS_LABEL") == usb_fs_label:
+                    device_paths.add(udev_device.device_node)
                     added.add(usb_fs_label)
-                    for child in device.children:
+                    for child in udev_device.children:
                         if child.device_node:
                             device_paths.add(child.device_node)
-                    for link in device.device_links:
+                    for link in udev_device.device_links:
                         device_paths.add(link)
     if device_info and required:
         for static_device in device_info:
