@@ -1,24 +1,11 @@
 #!/usr/bin/python3
-# -*- coding: utf-8 -*-
 #
 # Copyright (C) 2020-present Linaro Limited
 #
 # Author: Remi Duraffort <remi.duraffort@linaro.org>
 #
-# This file is part of LAVA.
-#
-# LAVA is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# LAVA is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, see <http://www.gnu.org/licenses>.
+# SPDX-License-Identifier: GPL-2.0-or-later
+from __future__ import annotations
 
 import asyncio
 import contextlib
@@ -28,35 +15,38 @@ import logging
 import logging.handlers
 import os
 import shutil
-import signal
 import sqlite3
 import subprocess
 import sys
 import time
 import traceback
+from collections.abc import Iterator
+from concurrent.futures import ThreadPoolExecutor
 from dataclasses import dataclass
+from functools import partial
+from json import loads as json_loads
 from pathlib import Path
-from typing import Any, Dict, Iterator, List, Optional, Union
+from signal import Signals
+from typing import Any
 
 import aiohttp
-import requests
+import sentry_sdk
 import yaml
 
-from lava_common.compat import yaml_safe_load
-from lava_common.constants import DISPATCHER_DOWNLOAD_DIR, WORKER_DIR
+from lava_common.constants import DISPATCHER_DOWNLOAD_DIR
+from lava_common.constants import WORKER_DIR as _WORKER_DIR_STR
 from lava_common.exceptions import LAVABug
 from lava_common.version import __version__
-from lava_common.worker import get_parser
+from lava_common.worker import get_parser, init_sentry_sdk
+from lava_common.yaml import yaml_safe_load
 
 ###########
 # Constants
 ###########
 FINISH_MAX_DURATION = 120
-JOBS_CHECK_INTERVAL = 5
 
 TIMEOUT = 60 * 10  # http timeout to 10 minutes
-WORKER_DIR = Path(WORKER_DIR)
-HEADERS = {"User-Agent": f"lava-worker {__version__}"}
+WORKER_DIR = Path(_WORKER_DIR_STR)
 
 
 #########
@@ -67,8 +57,6 @@ logging.Formatter.convert = time.gmtime
 LOG = logging.getLogger("lava-worker")
 FORMAT = "%(asctime)-15s %(levelname)7s %(message)s"
 
-SESSION = requests.Session()
-
 ping_interval = 20
 debug = False
 tmp_dir = WORKER_DIR / "tmp"
@@ -83,10 +71,14 @@ STALE_CONFIG = {
 URL_JOBS = "/scheduler/internal/v1/jobs/"
 URL_WORKERS = "/scheduler/internal/v1/workers/"
 
+THREAD_EXECUTOR = ThreadPoolExecutor(max_workers=8)
+JOB_ASYNC_TASKS: set[asyncio.Task[None]] = set()
+
+
 ###########
 # Helpers #
 ###########
-def create_environ(env: str) -> Dict[str, str]:
+def create_environ(env: str) -> dict[str, str]:
     """
     Generate the env variables for the job.
     """
@@ -104,7 +96,7 @@ def create_environ(env: str) -> Dict[str, str]:
     return environ
 
 
-def get_prefix(cfg):
+def get_prefix(cfg) -> str:
     if isinstance(cfg, dict):
         return cfg.get("prefix", "")
     return ""
@@ -115,31 +107,44 @@ class Response:
     status_code: int
     text: str
 
-    def json(self):
-        raise NotImplementedError()
+    def json(self) -> Any:
+        return json_loads(self.text)
 
 
-def requests_get(
-    url: str, token: str, params: Dict[str, str] = None
-) -> Union[requests.Response, Response]:
-
+async def aiohttp_get(
+    session: aiohttp.ClientSession,
+    url: str,
+    token: str | None,
+    params: dict[str, str] | None = None,
+) -> Response:
     if params is None:
         params = {}
+    headers: dict[str, str] = {}
+    if token is not None:
+        headers["LAVA-Token"] = token
 
     try:
-        headers = {**HEADERS, "LAVA-Token": token}
-        return SESSION.get(url, params=params, headers=headers, timeout=TIMEOUT)
-    except requests.RequestException as exc:
+        async with session.get(
+            url, params=params, headers=headers, timeout=TIMEOUT
+        ) as request:
+            return Response(request.status, await request.text())
+    except aiohttp.ClientError as exc:
         return Response(503, str(exc))
 
 
-def requests_post(
-    url: str, token: Union[str, None], data: Dict[str, str]
-) -> Union[requests.Response, Response]:
+async def aiohttp_post(
+    session: aiohttp.ClientSession, url: str, token: str | None, data: dict[str, str]
+) -> Response:
+    headers: dict[str, str] = {}
+    if token is not None:
+        headers["LAVA-Token"] = token
+
     try:
-        headers = {**HEADERS, "LAVA-Token": token}
-        return SESSION.post(url, data=data, headers=headers, timeout=TIMEOUT)
-    except requests.RequestException as exc:
+        async with session.post(
+            url, data=data, headers=headers, timeout=TIMEOUT
+        ) as request:
+            return Response(request.status, await request.text())
+    except aiohttp.ClientError as exc:
         return Response(503, str(exc))
 
 
@@ -156,7 +161,7 @@ def start_job(
     env_str: str,
     env_dut: str,
     job_log_interval: int,
-) -> Optional[int]:
+) -> int | None:
     """
     Start the lava-run process and return the pid
     """
@@ -183,8 +188,6 @@ def start_job(
             err_file = (base_dir / "stderr").open("w")
         env = create_environ(env_str)
         args = [
-            # Run lava-run under nice so every sub-commands will be niced
-            "nice",
             "lava-run",
             f"--device={base_dir / 'device.yaml'}",
             f"--dispatcher={base_dir / 'dispatcher.yaml'}",
@@ -219,6 +222,22 @@ def start_job(
         return None
 
 
+async def cleanup_job(prefix: str, job_id: int) -> None:
+    loop = asyncio.get_running_loop()
+    for directory, pattern in STALE_CONFIG.items():
+        dir_name = pattern.format(prefix=prefix, job_id=job_id)
+        dir_path = directory / dir_name
+        if not dir_path.exists():
+            continue
+        LOG.debug("[%d] Removing %s", job_id, dir_path)
+        await loop.run_in_executor(
+            THREAD_EXECUTOR,
+            partial(shutil.rmtree, str(dir_path), ignore_errors=True),
+        )
+
+    LOG.debug("[%d] Finished cleanup", job_id)
+
+
 #########
 # Classes
 #########
@@ -227,18 +246,18 @@ class Job:
 
     RUNNING, CANCELING, FINISHED = range(3)
 
-    def __init__(self, row):
-        self.job_id = row["id"]
-        self.pid = row["pid"]
-        self.status = row["status"]
-        self.prefix = row["prefix"]
-        self.last_update = row["last_update"]
-        self.token = row["token"]
+    def __init__(self, row: sqlite3.Row):
+        self.job_id: int = row["id"]
+        self.pid: int = row["pid"]
+        self.status: int = row["status"]
+        self.prefix: str = row["prefix"]
+        self.last_update: int = row["last_update"]
+        self.token: str = row["token"]
         # Create the base directory
         self.base_dir = tmp_dir / "{prefix}{job_id}".format(
             prefix=self.prefix, job_id=str(self.job_id)
         )
-        self.base_dir.mkdir(mode=0o755, exist_ok=True)
+        self.base_dir.mkdir(mode=0o755, exist_ok=True, parents=True)
 
     def errors(self) -> str:
         with contextlib.suppress(OSError, UnicodeDecodeError):
@@ -249,10 +268,12 @@ class Job:
 
     def description(self) -> str:
         with contextlib.suppress(OSError):
-            return (self.base_dir / "description.yaml").read_bytes()
+            return (self.base_dir / "description.yaml").read_text(
+                encoding="utf-8", errors="backslashreplace"
+            )
         return ""
 
-    def result(self) -> Dict[str, Any]:
+    def result(self) -> dict[str, Any]:
         with contextlib.suppress(OSError, yaml.YAMLError):
             data = yaml_safe_load((self.base_dir / "result.yaml").read_bytes())
             if isinstance(data, dict):
@@ -263,17 +284,17 @@ class Job:
         # If the pid is 0, just skip because lava-run was not started
         if self.pid == 0:
             return
-        os.kill(self.pid, signal.SIGKILL)
+        os.kill(self.pid, Signals.SIGKILL)
 
     def terminate(self) -> None:
         # If the pid is 0, just skip because lava-run was not started
         if self.pid == 0:
             return
-        os.kill(self.pid, signal.SIGTERM)
+        os.kill(self.pid, Signals.SIGTERM)
 
     def is_running(self) -> bool:
         with contextlib.suppress(OSError):
-            with open("/proc/%d/cmdline" % self.pid, "r") as fd:
+            with open("/proc/%d/cmdline" % self.pid) as fd:
                 return "lava-run" in fd.read()
         return False
 
@@ -299,9 +320,11 @@ class JobsDB:
             )
             self.conn.commit()
 
+        self.lock = asyncio.Lock()
+
     def create(
         self, job_id: int, pid: int, status: int, dispatcher_cfg: str, token: str
-    ) -> Optional[Job]:
+    ) -> Job | None:
         """
         When pid is 0, the pid is unknown
         """
@@ -325,13 +348,17 @@ class JobsDB:
             return self.get(job_id)
         return None
 
-    def get(self, job_id: int) -> Optional[Job]:
+    def get(self, job_id: int) -> Job | None:
         row = self.conn.execute(
             "SELECT * FROM jobs WHERE id=?", (str(job_id),)
         ).fetchone()
         return None if row is None else Job(row)
 
-    def update(self, job_id: int, status) -> Optional[Job]:
+    def get_from_pid(self, pid: int) -> Job | None:
+        row = self.conn.execute("SELECT * FROM jobs WHERE pid=?", (pid,)).fetchone()
+        return None if row is None else Job(row)
+
+    def update(self, job_id: int, status) -> Job | None:
         with contextlib.suppress(sqlite3.Error):
             self.conn.execute(
                 "UPDATE jobs SET status=?, last_update=? WHERE id=?",
@@ -346,7 +373,7 @@ class JobsDB:
             self.conn.execute("DELETE FROM jobs WHERE id=?", (str(job_id),))
             self.conn.commit()
 
-    def all_ids(self) -> List[int]:
+    def all_ids(self) -> list[int]:
         jobs = self.conn.execute("SELECT * FROM jobs")
         return [job["id"] for job in jobs]
 
@@ -384,7 +411,7 @@ def setup_logger(log_file: str, level: str) -> None:
     """
     # Configure the log handler
     if log_file == "-":
-        handler = logging.StreamHandler(sys.stdout)
+        handler: logging.Handler = logging.StreamHandler(sys.stdout)
     else:
         handler = logging.handlers.WatchedFileHandler(log_file)
     handler.setFormatter(logging.Formatter(FORMAT))
@@ -417,7 +444,45 @@ def cancel(url: str, jobs: JobsDB, job_id: int, token: str) -> None:
             jobs.update(job_id, Job.CANCELING)
 
 
-def check(url: str, jobs: JobsDB) -> None:
+async def finish_job(
+    session: aiohttp.ClientSession, url: str, job: Job, jobs: JobsDB
+) -> None:
+    LOG.info("[%d] FINISHED => server", job.job_id)
+    result = job.result()
+    # Default error values
+    if result.get("result") == "pass":
+        default_error_type = ""
+    else:
+        default_error_type = LAVABug.error_type
+    data = {
+        "state": "FINISHED",
+        "result": result.get("result", "fail"),
+        "error_type": result.get("error_type", default_error_type),
+        "errors": job.errors(),
+        "description": job.description(),
+    }
+
+    ret = await aiohttp_post(
+        session, f"{url}{URL_JOBS}{job.job_id}/", job.token, data=data
+    )
+    if ret.status_code != 200:
+        LOG.error("[%d] -> server error: code %d", job.job_id, ret.status_code)
+        LOG.debug("[%d] --> %s", job.job_id, ret.text)
+        if ret.status_code != 404:  # If the job is not present on lava-server delete it
+            return
+
+    # Remove stale resources
+    prefix = "" if job is None else job.prefix
+    cleanup_task = asyncio.get_running_loop().create_task(
+        cleanup_job(prefix, job.job_id)
+    )
+    JOB_ASYNC_TASKS.add(cleanup_task)
+    cleanup_task.add_done_callback(JOB_ASYNC_TASKS.discard)
+
+    jobs.delete(job.job_id)
+
+
+async def check(session: aiohttp.ClientSession, url: str, jobs: JobsDB) -> None:
     # Loop on running jobs
     for job in jobs.running():
         if not job.is_running():
@@ -453,55 +518,26 @@ def check(url: str, jobs: JobsDB) -> None:
 
     # Loop on finished jobs
     for job in jobs.finished():
-        LOG.info("[%d] FINISHED => server", job.job_id)
-        result = job.result()
-        # Default error values
-        if result.get("result") == "pass":
-            default_error_type = ""
-        else:
-            default_error_type = LAVABug.error_type
-        data = {
-            "state": "FINISHED",
-            "result": result.get("result", "fail"),
-            "error_type": result.get("error_type", default_error_type),
-            "errors": job.errors(),
-            "description": job.description(),
-        }
-
-        ret = requests_post(f"{url}{URL_JOBS}{job.job_id}/", job.token, data=data)
-        if ret.status_code != 200:
-            LOG.error("[%d] -> server error: code %d", job.job_id, ret.status_code)
-            LOG.debug("[%d] --> %s", job.job_id, ret.text)
-            return
-
-        # Remove stale resources
-        prefix = "" if job is None else job.prefix
-        for directory in STALE_CONFIG:
-            pattern = STALE_CONFIG[directory]
-            dir_name = pattern.format(prefix=prefix, job_id=job.job_id)
-            dir_path = directory / dir_name
-            if not dir_path.exists():
-                continue
-            LOG.debug("[%d] Removing %s", job.job_id, dir_path)
-            shutil.rmtree(str(dir_path), ignore_errors=True)
-
-        jobs.delete(job.job_id)
+        await finish_job(session, url, job, jobs)
 
 
 class ServerUnavailable(Exception):
-    pass
+    def __init__(self, message):
+        super().__init__(message)
+        sentry_sdk.capture_exception(self)
 
 
 class VersionMismatch(Exception):
     pass
 
 
-def ping(url: str, token: str, name: str) -> Dict[str, List]:
-    LOG.info("PING => server")
-    ret = requests_get(
-        f"{url}{URL_WORKERS}{name}/", token, params={"version": __version__}
+async def ping(
+    session: aiohttp.ClientSession, url: str, token: str, name: str
+) -> dict[str, list]:
+    LOG.debug("PING => server")
+    ret = await aiohttp_get(
+        session, f"{url}{URL_WORKERS}{name}/", token, params={"version": __version__}
     )
-
     if ret.status_code != 200:
         LOG.error("-> server error: code %d", ret.status_code)
         LOG.debug("--> %s", ret.text)
@@ -518,7 +554,13 @@ def ping(url: str, token: str, name: str) -> Dict[str, List]:
         return {}
 
 
-def register(url: str, name: str, username: str = None, password: str = None) -> str:
+async def register(
+    session: aiohttp.ClientSession,
+    url: str,
+    name: str,
+    username: str | None = None,
+    password: str | None = None,
+) -> str:
     data = {"name": name}
     if username is not None and password is not None:
         data["username"] = username
@@ -526,24 +568,34 @@ def register(url: str, name: str, username: str = None, password: str = None) ->
 
     while True:
         LOG.debug("[INIT] Auto register as %r", name)
-        ret = requests_post(f"{url}{URL_WORKERS}", None, data=data)
+        ret = await aiohttp_post(session, f"{url}{URL_WORKERS}", None, data=data)
         if ret.status_code == 200:
             return ret.json()["token"]
         LOG.error("[INIT] -> server error: code %d", ret.status_code)
         LOG.debug("[INIT] --> %s", ret.text)
-        time.sleep(5)
+        await asyncio.sleep(5)
 
 
-def running(
-    url: str, jobs: JobsDB, job_id: int, token: str, job_log_interval: int
+async def running(
+    session: aiohttp.ClientSession,
+    url: str,
+    jobs: JobsDB,
+    job_id: int,
+    token: str,
+    job_log_interval: int,
 ) -> None:
     job = jobs.get(job_id)
     if job is None:
-        start(url, jobs, job_id, token, job_log_interval)
+        await start(session, url, jobs, job_id, token, job_log_interval)
 
 
-def start(
-    url: str, jobs: JobsDB, job_id: int, token: str, job_log_interval: int
+async def start(
+    session: aiohttp.ClientSession,
+    url: str,
+    jobs: JobsDB,
+    job_id: int,
+    token: str,
+    job_log_interval: int,
 ) -> None:
     LOG.info("[%d] server => START", job_id)
     # Was the job already started?
@@ -551,7 +603,7 @@ def start(
 
     # Start the job
     if job is None:
-        ret = requests_get(f"{url}{URL_JOBS}{job_id}/", token)
+        ret = await aiohttp_get(session, f"{url}{URL_JOBS}{job_id}/", token)
         if ret.status_code != 200:
             LOG.error("[%d] -> server error: code %d", job_id, ret.status_code)
             LOG.debug("[%d] --> %s", job_id, ret.text)
@@ -569,11 +621,11 @@ def start(
             return
 
         LOG.info("[%d] Starting job", job_id)
-        LOG.debug("[%d]         : %s", job_id, yaml_safe_load(definition))
-        LOG.debug("[%d] device  : %s", job_id, yaml_safe_load(device))
-        LOG.debug("[%d] dispatch: %s", job_id, yaml_safe_load(dispatcher))
-        LOG.debug("[%d] env     : %s", job_id, yaml_safe_load(env))
-        LOG.debug("[%d] env-dut : %s", job_id, yaml_safe_load(env_dut))
+        LOG.debug("[%d]         : %r", job_id, definition)
+        LOG.debug("[%d] device  : %r", job_id, device)
+        LOG.debug("[%d] dispatch: %r", job_id, dispatcher)
+        LOG.debug("[%d] env     : %r", job_id, env)
+        LOG.debug("[%d] env-dut : %r", job_id, env_dut)
 
         # Start the job, grab the pid and create it in the dabatase
         pid = start_job(
@@ -599,7 +651,9 @@ def start(
 
     # Update the server state
     LOG.info("[%d] RUNNING => server", job_id)
-    ret = requests_post(f"{url}{URL_JOBS}{job_id}/", token, data={"state": "RUNNING"})
+    ret = await aiohttp_post(
+        session, f"{url}{URL_JOBS}{job_id}/", token, data={"state": "RUNNING"}
+    )
     if ret.status_code != 200:
         LOG.error("[%d] -> server error: code %d", job_id, ret.status_code)
         LOG.debug("[%d] --> %s", job_id, ret.text)
@@ -608,27 +662,25 @@ def start(
 ###############
 # Entrypoints #
 ###############
-def handle(options, jobs: JobsDB) -> float:
-    begin: float = time.monotonic()
-
+async def handle(options, session: aiohttp.ClientSession, jobs: JobsDB) -> None:
     name: str = options.name
     token: str = options.token
     url: str = options.url
     job_log_interval: int = options.job_log_interval
 
     try:
-        data = ping(url, token, name)
+        data = await ping(session, url, token, name)
     except ServerUnavailable:
         LOG.error("-> server unavailable")
-        return max(1 - (time.monotonic() - begin), 0)
+        return
     except VersionMismatch as exc:
         if options.exit_on_version_mismatch:
             raise exc
-        return max(ping_interval - (time.monotonic() - begin), 0)
+        return
 
     # running jobs
     for job in data.get("running", []):
-        running(url, jobs, job["id"], job["token"], job_log_interval)
+        await running(session, url, jobs, job["id"], job["token"], job_log_interval)
 
     # cancel jobs
     for job in data.get("cancel", []):
@@ -636,52 +688,113 @@ def handle(options, jobs: JobsDB) -> float:
 
     # start jobs
     for job in data.get("start", []):
-        start(url, jobs, job["id"], job["token"], job_log_interval)
+        await start(session, url, jobs, job["id"], job["token"], job_log_interval)
 
     # Check job status
     # TODO: store the token and reuse it
-    check(url, jobs)
-
-    # Compute the sleep duration
-    return max(ping_interval - (time.monotonic() - begin), 0)
+    await check(session, url, jobs)
 
 
-async def main_loop(options, jobs: JobsDB, event: asyncio.Event) -> None:
+async def main_loop(
+    options, session: aiohttp.ClientSession, jobs: JobsDB, event: asyncio.Event
+) -> None:
+    loop = asyncio.get_running_loop()
     while True:
-        timeout = handle(options, jobs)
-        with contextlib.suppress(asyncio.TimeoutError):
-            await asyncio.wait_for(event.wait(), timeout=timeout)
+        timer_handle = loop.call_later(ping_interval, event.set)
+        try:
+            async with jobs.lock:
+                await handle(options, session, jobs)
+            await event.wait()
+        finally:
+            timer_handle.cancel()
             event.clear()
 
 
-async def listen_for_events(options, event: asyncio.Event) -> None:
+async def sigchild_handler_async(
+    session: aiohttp.ClientSession, url: str, jobs: JobsDB
+) -> None:
+    async with jobs.lock:
+        jobs_to_finish: list[Job] = []
+        with contextlib.suppress(ChildProcessError):
+            while (waitpid_tuple := os.waitpid(-1, os.WNOHANG)) != (0, 0):
+                pid, exit_code = waitpid_tuple
+                LOG.debug(
+                    "Collected PID %d with exit code %d from SIGCHLD", pid, exit_code
+                )
+
+                job = jobs.get_from_pid(pid)
+                if job is not None:
+                    jobs_to_finish.append(job)
+                else:
+                    LOG.debug("Unknown PID collected %d from SIGCHLD", pid)
+
+        results = await asyncio.gather(
+            *(finish_job(session, url, job, jobs) for job in jobs_to_finish),
+            # If a job finish fails it will remain in database
+            # and later can be finished by the main loop
+            return_exceptions=True,
+        )
+
+        for r in results:
+            if isinstance(r, BaseException):
+                LOG.exception("Exception raised during SIGCHLD job finish", exc_info=r)
+
+
+def sigchld_handler(session: aiohttp.ClientSession, url: str, jobs: JobsDB) -> None:
+    handler_task = asyncio.get_running_loop().create_task(
+        sigchild_handler_async(session, url, jobs)
+    )
+    JOB_ASYNC_TASKS.add(handler_task)
+    handler_task.add_done_callback(JOB_ASYNC_TASKS.discard)
+
+
+async def listen_for_events(
+    options, session: aiohttp.ClientSession, event: asyncio.Event
+) -> None:
+    retry_interval = 1
     while True:
-        with contextlib.suppress(aiohttp.ClientError):
-            async with aiohttp.ClientSession(headers=HEADERS) as session:
-                async with session.ws_connect(f"{options.ws_url}", heartbeat=30) as ws:
-                    async for msg in ws:
-                        if msg.type != aiohttp.WSMsgType.TEXT:
-                            continue
-                        try:
-                            data = json.loads(msg.data)
-                            (topic, _, dt, username, data) = data
-                            data = json.loads(data)
-                        except ValueError:
-                            LOG.warning("[EVENT] Invalid message: %s", msg)
-                            continue
-                        if not topic.endswith(".testjob"):
-                            continue
-                        if data.get("worker") != options.name:
-                            continue
-                        if data.get("state") in ["Scheduled", "Canceling"]:
-                            LOG.info("[EVENT] Worker mentioned")
-                            event.set()
-        await asyncio.sleep(1)
+        try:
+            LOG.info("[EVENT] Connecting to websocket")
+            async with session.ws_connect(
+                f"{options.ws_url}",
+                headers={"LAVA-Token": options.token, "LAVA-Host": options.name},
+                heartbeat=30,
+            ) as ws:
+                retry_interval = 1
+                async for msg in ws:
+                    if msg.type != aiohttp.WSMsgType.TEXT:
+                        continue
+                    try:
+                        data = json.loads(msg.data)
+                        (topic, _, dt, username, data) = data
+                        data = json.loads(data)
+                    except ValueError:
+                        LOG.warning("[EVENT] Invalid message: %s", msg)
+                        continue
+                    if not topic.endswith(".testjob"):
+                        continue
+                    if data.get("worker") != options.name:
+                        continue
+                    if data.get("state") in ["Scheduled", "Canceling"]:
+                        LOG.info("[EVENT] Worker mentioned")
+                        event.set()
+        except (aiohttp.ClientError, asyncio.TimeoutError):
+            retry_interval = min(60, retry_interval * 2)
+        await asyncio.sleep(retry_interval)
+
+
+def ask_exit(signame: str, group: asyncio.Future[Any]) -> None:
+    LOG.info(f"[EXIT] Received signal {signame}")
+    # await cancelled group throws asyncio.CancelledError. The
+    # exception is handled in the main().
+    group.cancel()
 
 
 async def main() -> int:
     # Parse command line
     options = get_parser().parse_args()
+    if options.sentry_dsn:
+        init_sentry_sdk(options.sentry_dsn)
     if options.token_file is None:
         options.token_file = Path(options.worker_dir) / "token"
     options.url = options.url.rstrip("/")
@@ -712,64 +825,88 @@ async def main() -> int:
     if worker_dir != WORKER_DIR:
         global tmp_dir
         tmp_dir = worker_dir / "tmp"
+        # Update stale config dictionary.
+        STALE_CONFIG[tmp_dir] = "{prefix}{job_id}"
 
-    try:
-        if options.username is not None:
-            LOG.info("[INIT] Token  : '<auto register with %s>'", options.username)
-            password = getpass.getpass()
-            options.token = register(
-                options.url, options.name, options.username, password
-            )
-            options.token_file.write_text(options.token, encoding="utf-8")
-            options.token_file.chmod(0o600)
+    loop = asyncio.get_running_loop()
+    loop.set_default_executor(THREAD_EXECUTOR)
 
-        elif options.token is not None:
-            LOG.info("[INIT] Token  : '<command line>'")
-            options.token_file.write_text(options.token, encoding="utf-8")
-            options.token_file.chmod(0o600)
-            options.token_file = str(worker_dir / "token")
-        elif options.token_file.exists():
-            LOG.info("[INIT] Token  : file %r", str(options.token_file))
-            options.token = options.token_file.read_text(encoding="utf-8").rstrip("\n")
-        else:
-            LOG.info("[INIT] Token  : '<auto register>'")
-            options.token = register(options.url, options.name)
-            options.token_file.write_text(options.token, encoding="utf-8")
-            options.token_file.chmod(0o600)
-
-        jobs = JobsDB(str(worker_dir / "db.sqlite3"))
-
-        event = asyncio.Event()
-        await asyncio.gather(
-            main_loop(options, jobs, event), listen_for_events(options, event)
-        )
-        return 0
-    except asyncio.CancelledError:
-        LOG.info("[EXIT] Canceled")
-        if options.wait_jobs:
-            LOG.info("[EXIT] Wait for jobs to finish")
-            while True:
-                check(options.url, jobs)
-                all_ids = jobs.all_ids()
-                LOG.info(
-                    "[EXIT] => %d jobs ([%s])",
-                    len(all_ids),
-                    ", ".join(str(i) for i in all_ids),
+    async with aiohttp.ClientSession(
+        headers={
+            "User-Agent": f"lava-worker {__version__}",
+        }
+    ) as session:
+        try:
+            if options.username is not None:
+                LOG.info("[INIT] Token  : '<auto register with %s>'", options.username)
+                password = getpass.getpass()
+                options.token = await register(
+                    options.url, options.name, options.username, password
                 )
-                if not all_ids:
-                    break
-                time.sleep(ping_interval)
-        return 1
-    except VersionMismatch as exc:
-        LOG.info("[EXIT] %s" % exc)
-        return 0
-    except Exception as exc:
-        LOG.error("[EXIT] %s", exc)
-        LOG.exception(exc)
-        return 1
+                options.token_file.write_text(options.token, encoding="utf-8")
+                options.token_file.chmod(0o600)
+            elif options.token is not None:
+                LOG.info("[INIT] Token  : '<command line>'")
+                options.token_file.write_text(options.token, encoding="utf-8")
+                options.token_file.chmod(0o600)
+            elif options.token_file.exists():
+                LOG.info("[INIT] Token  : file %r", str(options.token_file))
+                options.token = options.token_file.read_text(encoding="utf-8").rstrip(
+                    "\n"
+                )
+            else:
+                LOG.info("[INIT] Token  : '<auto register>'")
+                options.token = await register(session, options.url, options.name)
+                options.token_file.write_text(options.token, encoding="utf-8")
+                options.token_file.chmod(0o600)
+
+            jobs = JobsDB(str(worker_dir / "db.sqlite3"))
+
+            event = asyncio.Event()
+            group = asyncio.gather(
+                main_loop(options, session, jobs, event),
+                listen_for_events(options, session, event),
+            )
+
+            LOG.debug(f"LAVA worker pid is {os.getpid()}")
+            for sig in (Signals.SIGINT, Signals.SIGTERM):
+                loop.add_signal_handler(
+                    sig,
+                    partial(ask_exit, sig.name, group),
+                )
+
+            loop.add_signal_handler(
+                Signals.SIGCHLD, partial(sigchld_handler, session, options.url, jobs)
+            )
+
+            await group
+            return 0
+        except asyncio.CancelledError:
+            LOG.info("[EXIT] Canceled")
+            if options.wait_jobs:
+                LOG.info("[EXIT] Wait for jobs to finish")
+                while True:
+                    await check(session, options.url, jobs)
+                    all_ids = jobs.all_ids()
+                    LOG.info(
+                        "[EXIT] => %d jobs ([%s])",
+                        len(all_ids),
+                        ", ".join(str(i) for i in all_ids),
+                    )
+                    if not all_ids:
+                        break
+                    await asyncio.sleep(ping_interval)
+            return 1
+        except VersionMismatch as exc:
+            LOG.info("[EXIT] %s" % exc)
+            return 0
+        except Exception as exc:
+            LOG.error("[EXIT] %s", exc)
+            LOG.exception(exc)
+            return 1
 
 
-def run():
+def run() -> None:
     try:
         sys.exit(asyncio.run(main()))
     except KeyboardInterrupt:
