@@ -1,19 +1,7 @@
 # Copyright (C) 2019 Linaro Limited
 #
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-2.0-or-later
 
-import glob
 import logging
 import logging.handlers
 import os
@@ -23,8 +11,8 @@ from pathlib import Path
 
 import pyudev
 
-from lava_common.compat import yaml_dump, yaml_load
 from lava_common.constants import DISPATCHER_DOWNLOAD_DIR as JOBS_DIR
+from lava_common.device_mappings import find_mapping
 from lava_common.exceptions import InfrastructureError
 from lava_dispatcher_host.docker_devices import Device, DeviceFilter
 
@@ -35,43 +23,6 @@ logger.addHandler(logging.handlers.SysLogHandler(address="/dev/log"))
 logger.setLevel(logging.INFO)
 
 
-def get_mapping_path(job_id):
-    return os.path.join(JOBS_DIR, job_id, "usbmap.yaml")
-
-
-def add_device_container_mapping(job_id, device_info, container, container_type="lxc"):
-    validate_device_info(device_info)
-    item = {
-        "device_info": device_info,
-        "container": container,
-        "container_type": container_type,
-        "job_id": job_id,
-    }
-    mapping_path = get_mapping_path(job_id)
-    data = load_mapping_data(mapping_path)
-
-    # remove old mappings for the same device_info
-    newdata = [old for old in data if old["device_info"] != item["device_info"]]
-    newdata.append(item)
-
-    os.makedirs(os.path.dirname(mapping_path), exist_ok=True)
-    with open(mapping_path, "w") as f:
-        f.write(yaml_dump(newdata))
-
-
-def remove_device_container_mappings(job_id):
-    os.unlink(get_mapping_path(job_id))
-
-
-def validate_device_info(device_info):
-    if not device_info:
-        raise ValueError("Addind mapping for empty device info: %r" % device_info)
-    if not any(device_info.values()):
-        raise ValueError(
-            "Addind mapping for device info with empty keys: %r" % device_info
-        )
-
-
 def share_device_with_container(options):
     data, job_id = find_mapping(options)
     if not data:
@@ -81,7 +32,7 @@ def share_device_with_container(options):
     if not device.startswith("/dev/"):
         device = "/dev/" + device
     if not os.path.exists(device):
-        logger.warning("Can't share {device}: file not found".format(device=device))
+        logger.warning(f"Can't share {device}: file not found")
         return
 
     container_type = data["container_type"]
@@ -93,40 +44,6 @@ def share_device_with_container(options):
         raise InfrastructureError('Unsupported container type: "%s"' % container_type)
 
 
-def find_mapping(options):
-    for mapping in glob.glob(get_mapping_path("*")):
-        data = load_mapping_data(mapping)
-        for item in data:
-            if match_mapping(item["device_info"], options):
-                job_id = str(Path(mapping).parent.name)
-                return item, job_id
-    return None, None
-
-
-def load_mapping_data(filename):
-    try:
-        with open(filename) as f:
-            data = yaml_load(f) or []
-        if isinstance(data, dict):
-            data = [data]
-        return data
-    except FileNotFoundError:
-        return []
-
-
-def match_mapping(device_info, options):
-    matched = False
-    for k, v in device_info.items():
-        if v:
-            if k in options and getattr(options, k) == v:
-                matched = True
-            else:
-                return False
-        else:
-            matched = True
-    return matched
-
-
 def log_sharing_device(device, container_type, container):
     logger.info(f"Sharing {device} with {container_type} container {container}")
 
