@@ -1,17 +1,6 @@
 # Copyright (C) 2019 Linaro Limited
 #
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-2.0-or-later
 
 import argparse
 import os
@@ -21,11 +10,11 @@ import sys
 import time
 
 from lava_common.constants import UDEV_RULE_FILENAME
-from lava_dispatcher_host import (
+from lava_common.device_mappings import (
     add_device_container_mapping,
     remove_device_container_mappings,
-    share_device_with_container,
 )
+from lava_dispatcher_host import share_device_with_container
 from lava_dispatcher_host.server import Client
 from lava_dispatcher_host.udev import get_udev_rules
 
