@@ -1,23 +1,9 @@
-# -*- coding: utf-8 -*-
 #
 # Copyright (C) 2020-present Linaro Limited
 #
 # Author: Antonio Terceiro <antonio.terceiro@linaro.org>
 #
-# This file is part of LAVA.
-#
-# LAVA is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# LAVA is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, see <http://www.gnu.org/licenses>.
+# SPDX-License-Identifier: GPL-2.0-or-later
 
 import logging
 import logging.handlers
@@ -34,11 +20,13 @@ import sys
 import time
 
 import requests
+import sentry_sdk
 from requests.adapters import HTTPAdapter
 from requests.packages.urllib3.util.retry import Retry
 
 from lava_common.constants import DISPATCHER_DOWNLOAD_DIR
-from lava_common.worker import get_parser
+from lava_common.version import __version__
+from lava_common.worker import get_parser, init_sentry_sdk
 
 #########
 # Globals
@@ -48,7 +36,7 @@ logging.Formatter.convert = time.gmtime
 LOG = logging.getLogger("lava-worker")
 FORMAT = "%(asctime)-15s %(levelname)7s %(message)s"
 
-PAT = re.compile(r"\d+\.\d+\.(\d+\.){0,1}\d{4}\.g[\d\w]+")
+RELEASE_PAT = re.compile(r"^\d+\.\d+(\.\d+){0,1}$")
 
 
 ###########
@@ -62,7 +50,32 @@ def log_output(line):
         LOG.info("> " + line)
 
 
-def filter_options(options):
+def get_options(image: str) -> list:
+    """
+    Return a list of available worker option names.
+    Do not handle the CalledProcessError exception here. This will be
+    propagated to the caller that will fail correctly.
+    """
+    option_ns = subprocess.check_output(
+        [
+            "docker",
+            "run",
+            "--rm",
+            image,
+            "python3",
+            "-c",
+            "from lava_common.worker import get_parser; print(get_parser().parse_args(['--url', 'dummy-url']))",
+        ],
+        stderr=subprocess.DEVNULL,
+    )
+
+    option_names = re.findall(r"(\w+)=", option_ns.decode("utf-8"))
+
+    return option_names
+
+
+def filter_options(options, image):
+    image_available_options = get_options(image)
     ret = ["--worker-dir", options.worker_dir, "--url", options.url]
     if options.ws_url:
         ret.extend(["--ws-url", options.ws_url])
@@ -77,13 +90,19 @@ def filter_options(options):
         ret.extend(["--token", options.token])
     if options.token_file:
         ret.extend(["--token-file", options.token_file])
+
+    if options.sentry_dsn and "sentry_dns" in image_available_options:
+        ret.extend(["--sentry-dsn", options.sentry_dsn])
+
+    if options.level and "level" in image_available_options:
+        ret.extend(["--level", options.level])
     return ret
 
 
-def has_image(image):
+def has_image(image, manifest=False):
     try:
         subprocess.check_output(
-            ["docker", "image", "inspect", image],
+            ["docker", "manifest" if manifest else "image", "inspect", image],
             stderr=subprocess.DEVNULL,
         )
         return True
@@ -96,6 +115,7 @@ def get_image(image):
         return True
 
     try:
+        LOG.debug("Pulling image %s", image)
         subprocess.check_output(["docker", "pull", image], stderr=subprocess.STDOUT)
         return True
     except subprocess.CalledProcessError as exc:
@@ -103,7 +123,7 @@ def get_image(image):
     return False
 
 
-def build_customized_image(image, build_dir):
+def build_customized_image(image, build_dir, use_cache=False):
     dockerfile = build_dir / "Dockerfile"
     if not dockerfile.exists():
         LOG.warning("Dockerfile (%s) not found", dockerfile)
@@ -113,7 +133,7 @@ def build_customized_image(image, build_dir):
     # all the FROM commands defined in the original Dockerfile will be ignored
     # and Dockerfile.lava using the image passed to the function as base image
     # will be generated for building.
-    with open(dockerfile, "r") as f:
+    with open(dockerfile) as f:
         instructions = f.readlines()
     lava_dockerfile = build_dir / "Dockerfile.lava"
     with open(lava_dockerfile, "w") as f:
@@ -124,42 +144,65 @@ def build_customized_image(image, build_dir):
                 f.write(instruction)
 
     tag = f"{image}.customized"
+    build_cmd = [
+        "docker",
+        "build",
+        "--force-rm",
+        "-f",
+        "Dockerfile.lava",
+        "-t",
+        tag,
+    ]
+    if not use_cache:
+        build_cmd.append("--no-cache")
+    build_cmd.append(".")
     try:
         p = subprocess.Popen(
-            ["docker", "build", "--force-rm", "-f", "Dockerfile.lava", "-t", tag, "."],
+            build_cmd,
             cwd=build_dir,
             stdout=subprocess.PIPE,
             stderr=subprocess.STDOUT,
         )
         for line in p.stdout:
             log_output(line)
+        p.communicate()
+        rc = p.returncode
+        if rc != 0:
+            sys.exit(rc)
         return tag
     except subprocess.CalledProcessError:
         sys.exit(1)
 
 
-class Terminate(RuntimeError):
+class Terminate(KeyboardInterrupt):
     @classmethod
     def trigger(cls, *args):
         raise cls()
 
 
-def run(version, options):
-    if PAT.match(version):
-        # development
-        if platform.machine() == "x86_64":
-            image = f"hub.lavasoftware.org/lava/lava/amd64/lava-dispatcher:{version}"
-        elif platform.machine() == "aarch64":
-            image = f"hub.lavasoftware.org/lava/lava/aarch64/lava-dispatcher:{version}"
-        else:
-            print("ERROR: unsupported architecture '{platform.machine()}'")
-            sys.exit(1)
-    else:
+def start(version, options):
+    if RELEASE_PAT.match(version):
         # released version
         image = f"lavasoftware/lava-dispatcher:{version}"
+    else:
+        # Replace "+" by "." which is not accepted by docker in a tag
+        version = version.replace("+", ".")
+        # development
+        if platform.machine() == "x86_64":
+            suffix = f"/lava/lava/amd64/lava-dispatcher:{version}"
+        elif platform.machine() == "aarch64":
+            suffix = f"/lava/lava/aarch64/lava-dispatcher:{version}"
+        else:
+            LOG.error("Unsupported architecture '%s'", platform.machine())
+            sys.exit(1)
+
+        if has_image(f"registry.gitlab.com{suffix}", manifest=True):
+            image = f"registry.gitlab.com{suffix}"
+        else:
+            image = f"hub.lavasoftware.org{suffix}"
 
     LOG.info("Using image %s", image)
-    rand = "".join((random.choice(string.hexdigits) for c in range(5)))
+    rand = "".join(random.choice(string.hexdigits) for c in range(5))
     docker_name = f"lava-worker-{version}-{rand}"
     LOG.info("Docker name %s", docker_name)
     service = [
@@ -174,27 +217,31 @@ def run(version, options):
     ]
 
     mounts = []
-    mounts.append((DISPATCHER_DOWNLOAD_DIR, None))
+    mounts.append((DISPATCHER_DOWNLOAD_DIR, None, None))
 
     tftp_dir = pathlib.Path("/srv/tftp")
     if tftp_dir.exists():
-        mounts.append((str(tftp_dir), None))
+        mounts.append((str(tftp_dir), None, None))
 
     worker_dir = options.worker_dir.absolute()
     worker_dir.mkdir(parents=True, exist_ok=True)
-    mounts.append((worker_dir, None))
-    mounts.append(("/run/udev", None))
-    mounts.append(("/dev", None))
-    mounts.append(("/var/run/docker.sock", None))
-    mounts.append(("/boot", "readonly=true"))
-    mounts.append(("/lib/modules", "readonly=true"))
-    for path, opts in mounts:
-        m = f"--mount=type=bind,source={path},destination={path}"
+    mounts.append((worker_dir, None, None))
+    mounts.append(("/run/udev", None, None))
+    mounts.append(("/dev", None, None))
+    mounts.append(("/var/run/docker.sock", None, None))
+    mounts.append(("/boot", None, "readonly=true"))
+    mounts.append(("/lib/modules", None, "readonly=true"))
+    for d in ["/usr/share/lava-docker-worker", "/usr/local/share/lava-docker-worker"]:
+        if os.path.exists(d):
+            mounts.append((d, None, None))
+    if options.mount:
+        mounts += options.mount
+    for src, dst, opts in mounts:
+        m = f"--mount=type=bind,source={src},destination={dst or src}"
         if opts:
             m += "," + opts
         service.append(m)
 
-    # TODO handle ctrl-c/SIGINT
     # TODO dev move: provide default values for all options, including
     # TODO           translate localhost -> 172.17.0.1
     # TODO dev move: build and use docker image from local tree
@@ -209,16 +256,15 @@ def run(version, options):
 
     if options.build_dir.exists():
         LOG.info("Building custom image in %s", options.build_dir)
-        image = build_customized_image(image, options.build_dir)
+        image = build_customized_image(image, options.build_dir, options.use_cache)
     service.append(image)
 
     try:
-        signal.signal(signal.SIGTERM, Terminate.trigger)
         container = subprocess.Popen(
             service
             + ["lava-worker", "--exit-on-version-mismatch", "--wait-jobs"]
             + ["--log-file", "-", "--name", options.name]
-            + filter_options(options),
+            + filter_options(options, image),
             stdout=subprocess.PIPE,
             stderr=subprocess.STDOUT,
             preexec_fn=os.setpgrp,
@@ -231,7 +277,7 @@ def run(version, options):
     except FileNotFoundError as exc:
         LOG.error("'%s' not found", exc.filename)
         time.sleep(5)
-    except subprocess.CalledProcessError as failure:
+    except subprocess.CalledProcessError:
         LOG.info("Failed to start the worker")
         time.sleep(5)
     except (KeyboardInterrupt, Terminate):
@@ -253,7 +299,11 @@ def get_server_version(options):
     http = requests.Session()
     http.mount("http://", adapter)
     http.mount("https://", adapter)
-    server_version = http.get(server_version_url, timeout=10).json()
+    server_version = http.get(
+        server_version_url,
+        headers={"User-Agent": f"lava-docker-worker {__version__}"},
+        timeout=10,
+    ).json()
     return server_version["version"]
 
 
@@ -289,6 +339,8 @@ def setup_logger(log_file: str, level: str) -> None:
 def main():
     # Parse command line
     options = get_parser(docker_worker=True).parse_args()
+    if options.sentry_dsn:
+        init_sentry_sdk(options.sentry_dsn)
     options.build_dir = options.build_dir.resolve()
 
     if options.devel:
@@ -308,20 +360,32 @@ def main():
     LOG.info("[INIT] Server : %r", options.url)
 
     LOG.info("[INIT] Starting main loop")
+    signal.signal(signal.SIGTERM, Terminate.trigger)
     while True:
         LOG.info("Get server version")
         try:
             server_version = get_server_version(options)
-        except requests.RequestException:
-            LOG.warning("-> Unable to get server version")
+        except requests.RequestException as exc:
+            LOG.warning(f"-> Unable to get server version: {exc}")
+            sentry_sdk.capture_exception(exc)
+            time.sleep(5)
             continue
         LOG.info("=> %s", server_version)
         try:
-            run(server_version, options)
+            start(server_version, options)
         except Exception as exc:
             LOG.exception(exc)
+            sentry_sdk.capture_exception(exc)
             time.sleep(5)
 
 
+def run():
+    try:
+        main()
+    except KeyboardInterrupt:
+        LOG.info("[EXIT] Received Ctrl+C")
+        sys.exit(1)
+
+
 if __name__ == "__main__":
-    main()
+    run()
