@@ -1,23 +1,9 @@
-# -*- coding: utf-8 -*-
 #
 # Copyright (C) 2021 Linaro Limited
 #
 # Author: Antonio Terceiro <antonio.terceiro@linaro.org>
 #
-# This file is part of LAVA.
-#
-# LAVA is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# LAVA Dispatcher is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, see <http://www.gnu.org/licenses>.
+# SPDX-License-Identifier: GPL-2.0-or-later
 
 import asyncio
 import json
@@ -43,6 +29,16 @@ class CommandHandler:
         share_device_with_container(command.options)
 
 
+def encode_result(result, exception=None):
+    data = {
+        "result": result,
+    }
+    if exception:
+        data["message"] = repr(exception)
+
+    return bytes(json.dumps(data), "utf-8") + b"\n"
+
+
 class ServerWrapper:
     def __init__(self, socket=SOCKET):
         self.socket = socket
@@ -51,7 +47,7 @@ class ServerWrapper:
         logger.info(f"Exiting due to {signal}")
 
     async def start(self):
-        logger.info(f"Starting")
+        logger.info("Starting")
         loop = asyncio.get_running_loop()
 
         sd_sockets = os.getenv("LISTEN_FDS")
@@ -78,18 +74,26 @@ class ServerWrapper:
 
         try:
             command = ShareCommand(**json.loads(request))
-        except (TypeError, json.decoder.JSONDecodeError):
-            result = b'{"result": "INVALID_REQUEST"}\n'
+        except (TypeError, json.decoder.JSONDecodeError) as ex:
+            logger.warning(repr(ex))
+            result = encode_result("INVALID_REQUEST", ex)
 
         if not result:
-            handler = CommandHandler()
-            handler.handle(command)
-            result = b'{"result": "OK"}\n'
+            try:
+                handler = CommandHandler()
+                handler.handle(command)
+                result = encode_result("OK")
+            except Exception as ex:
+                logger.warning(repr(ex))
+                result = encode_result("FAILED", ex)
 
-        writer.write(result)
-        await writer.drain()
-        writer.close()
-        await writer.wait_closed()
+        try:
+            writer.write(result)
+            await writer.drain()
+            writer.close()
+            await writer.wait_closed()
+        except ConnectionResetError as ex:
+            logger.warning(repr(ex))
 
 
 class Client:
