@@ -1,38 +1,34 @@
-# -*- coding: utf-8 -*-
 # Copyright (C) 2019 Linaro Limited
 #
 # Author: Stevan Radaković <stevan.radakovic@linaro.org>
 #
-# This file is part of LAVA.
-#
-# LAVA is free software: you can redistribute it and/or modify
-# it under the terms of the GNU Affero General Public License version 3
-# as published by the Free Software Foundation
-#
-# LAVA is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU Affero General Public License
-# along with LAVA.  If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-2.0-or-later
 
 import csv
 import io
-import pathlib
 
+import junit_xml
+import tap
 import voluptuous
 import yaml
 from django.conf import settings
 from django.db import transaction
 from django.http import Http404
-from django.http.response import HttpResponse
-from jinja2.sandbox import SandboxedEnvironment as JinjaSandboxEnv
+from django.http.response import FileResponse, HttpResponse
 from rest_framework import status, viewsets
-from rest_framework.exceptions import ParseError, PermissionDenied, ValidationError
+from rest_framework.authtoken.views import ObtainAuthToken
+from rest_framework.decorators import action
+from rest_framework.exceptions import (
+    AuthenticationFailed,
+    NotFound,
+    ParseError,
+    PermissionDenied,
+    ValidationError,
+)
 from rest_framework.parsers import FormParser, JSONParser, MultiPartParser
 from rest_framework.permissions import (
     AllowAny,
+    BasePermission,
     DjangoModelPermissions,
     DjangoModelPermissionsOrAnonReadOnly,
     IsAuthenticatedOrReadOnly,
@@ -42,12 +38,10 @@ from rest_framework.utils import formatting
 from rest_framework_extensions.mixins import NestedViewSetMixin
 
 from lava_common import schemas
-from lava_common.compat import yaml_dump, yaml_safe_load
 from lava_common.schemas.test import testdef
 from lava_common.version import __version__
+from lava_common.yaml import yaml_safe_dump, yaml_safe_load
 from lava_rest_app import filters
-from lava_rest_app.base import views as base_views
-from lava_rest_app.base.pasers import PlainTextParser
 from lava_results_app.models import TestCase, TestSuite
 from lava_results_app.utils import (
     export_testcase,
@@ -55,6 +49,8 @@ from lava_results_app.utils import (
     testcase_export_fields,
 )
 from lava_scheduler_app.dbutils import testjob_submission
+from lava_scheduler_app.environment import DEVICES_JINJA_ENV
+from lava_scheduler_app.logutils import logs_instance
 from lava_scheduler_app.models import (
     Alias,
     Device,
@@ -64,23 +60,57 @@ from lava_scheduler_app.models import (
     GroupDeviceTypePermission,
     Tag,
     TestJob,
+    Worker,
 )
 from lava_scheduler_app.schema import SubmissionException
-from lava_scheduler_app.views import __set_device_health__
+from lava_scheduler_app.views import __set_device_health__, __set_worker_health__
 from lava_server.files import File
+from linaro_django_xmlrpc.models import AuthToken
 
 from . import serializers
-
-try:
-    from rest_framework.decorators import action, detail_route
-except ImportError:
-    from rest_framework.decorators import action
-
-    def detail_route(methods, suffix, url_path=None):
-        return action(detail=True, methods=methods, suffix=suffix)
+from .parsers import PlainTextParser
 
 
-class TestJobViewSet(base_views.TestJobViewSet):
+def safe_str2int(in_value):
+    out_value = in_value
+    if isinstance(in_value, str) and in_value.isnumeric():
+        out_value = int(in_value)
+        if out_value >= 0:
+            return out_value
+    return out_value
+
+
+class LavaObtainAuthToken(ObtainAuthToken):
+    def post(self, request, *args, **kwargs):
+        serializer = self.serializer_class(
+            data=request.data, context={"request": request}
+        )
+        serializer.is_valid(raise_exception=True)
+        user = serializer.validated_data["user"]
+        tokens = AuthToken.objects.filter(user=user)
+        token = None
+        if tokens.exists():
+            token = tokens[0]  # return 1st available token
+        else:
+            token, _ = AuthToken.objects.get_or_create(
+                user=user, description="Created by REST API call"
+            )
+        if not token:
+            # this shouldn't happen
+            raise AuthenticationFailed()
+        return Response({"token": token.secret})
+
+
+class IsSuperUser(BasePermission):
+    """
+    Allows access only to superusers.
+    """
+
+    def has_permission(self, request, view):
+        return bool(request.user and request.user.is_superuser)
+
+
+class TestJobViewSet(viewsets.ModelViewSet):
     """
     List TestJobs visible to the current user.
 
@@ -88,11 +118,13 @@ class TestJobViewSet(base_views.TestJobViewSet):
 
     * `/jobs/`
 
-    You can validate the given job definition against the schema validator via POST request on:
+    You can validate the given job definition against the schema validator
+    via POST request on:
 
     * `/jobs/validate/`
 
-    You can validate the given test definition against the schema validator via POST request on:
+    You can validate the given test definition against the schema validator
+    via POST request on:
 
     * `/jobs/validate_testdef/`
 
@@ -116,16 +148,212 @@ class TestJobViewSet(base_views.TestJobViewSet):
     * `/jobs/<job_id>/yaml/`
     """
 
-    def suites(self, request, **kwargs):
-        raise NotImplementedError()
+    queryset = TestJob.objects
+    serializer_class = serializers.TestJobSerializer
+    filterset_fields = (
+        "submitter",
+        "viewing_groups",
+        "description",
+        "health_check",
+        "requested_device_type",
+        "tags",
+        "actual_device",
+        "submit_time",
+        "start_time",
+        "end_time",
+        "state",
+        "health",
+        "priority",
+        "definition",
+        "original_definition",
+        "multinode_definition",
+        "failure_tags",
+        "failure_comment",
+    )
+    ordering_fields = ("id", "start_time", "end_time", "submit_time")
+    ordering = ("-id",)
+    filterset_class = filters.TestJobFilter
 
-    def tests(self, request, **kwargs):
-        raise NotImplementedError()
+    def get_queryset(self):
+        return (
+            self.queryset.select_related("submitter")
+            .prefetch_related("tags", "failure_tags", "viewing_groups")
+            .visible_by_user(self.request.user)
+        )
 
-    @detail_route(methods=["get"], suffix="csv")
+    def get_permissions(self):
+        if self.action in ["update", "destroy", "partial_update"]:
+            self.permission_classes = [IsSuperUser]
+        return super().get_permissions()
+
+    @action(detail=True, suffix="junit")
+    def junit(self, request, **kwargs):
+        suites = []
+        classname_prefix = request.query_params.get("classname_prefix", "")
+        if classname_prefix != "":
+            classname_prefix = str(classname_prefix) + "_"
+        for suite in self.get_object().testsuite_set.all().order_by("id"):
+            cases = []
+            for case in suite.testcase_set.all().order_by("id"):
+                # Grab the duration
+                md = case.action_metadata
+                duration = None
+                if md is not None:
+                    duration = md.get("duration")
+                    if duration is not None:
+                        duration = float(duration)
+
+                # Build the test case junit object
+                tc = junit_xml.TestCase(
+                    case.name,
+                    elapsed_sec=duration,
+                    classname="%s%s" % (classname_prefix, case.suite.name),
+                    timestamp=case.logged.isoformat(),
+                )
+                if case.result == TestCase.RESULT_FAIL:
+                    logs = None
+                    # TODO: is this of any use? (yaml inside xml!)
+                    if (
+                        case.start_log_line is not None
+                        and case.end_log_line is not None
+                    ):
+                        logs = logs_instance.read(
+                            self.get_object(), case.start_log_line, case.end_log_line
+                        )
+                    tc.add_failure_info("failed", output=logs)
+                elif case.result == TestCase.RESULT_SKIP:
+                    tc.add_skipped_info("skipped")
+                cases.append(tc)
+            suites.append(
+                junit_xml.TestSuite(
+                    suite.name,
+                    test_cases=cases,
+                    timestamp=suite.get_end_datetime().isoformat(),
+                )
+            )
+
+        data = junit_xml.to_xml_report_string(suites, encoding="utf-8")
+        response = HttpResponse(data, content_type="application/xml")
+        response["Content-Disposition"] = (
+            "attachment; filename=job_%d.xml" % self.get_object().id
+        )
+        return response
+
+    @action(detail=True, suffix="logs")
+    def logs(self, request, **kwargs):
+        start = safe_str2int(request.query_params.get("start", 0))
+        end = safe_str2int(request.query_params.get("end", None))
+        try:
+            if start == 0 and end is None:
+                job = self.get_object()
+                data = logs_instance.open(job)
+                size = logs_instance.size(job)
+                response = FileResponse(data, content_type="application/yaml")
+                response["Content-Length"] = size
+            else:
+                data = logs_instance.read(self.get_object(), start, end)
+                response = HttpResponse(data, content_type="application/yaml")
+            if not data:
+                raise NotFound()
+            response["Content-Disposition"] = (
+                "attachment; filename=job_%d.yaml" % self.get_object().id
+            )
+            return response
+        except FileNotFoundError:
+            raise NotFound()
+
+    @action(detail=True, suffix="tap13")
+    def tap13(self, request, **kwargs):
+        stream = io.StringIO()
+        count = TestCase.objects.filter(suite__job=self.get_object()).count()
+        tracker = tap.tracker.Tracker(plan=count, streaming=True, stream=stream)
+
+        # Loop on all test cases
+        for suite in self.get_object().testsuite_set.all().order_by("id"):
+            for case in suite.testcase_set.all().order_by("id"):
+                if case.result == TestCase.RESULT_FAIL:
+                    if (
+                        case.start_log_line is not None
+                        and case.end_log_line is not None
+                    ):
+                        logs = logs_instance.read(
+                            self.get_object(), case.start_log_line, case.end_log_line
+                        )
+                        logs = "\n ".join(logs.split("\n"))
+                        tracker.add_not_ok(
+                            suite.name, case.name, diagnostics=" ---\n " + logs + "..."
+                        )
+                    else:
+                        tracker.add_not_ok(suite.name, case.name)
+                elif case.result == TestCase.RESULT_SKIP:
+                    tracker.add_skip(suite.name, case.name, "test skipped")
+                elif case.result == TestCase.RESULT_UNKNOWN:
+                    tracker.add_not_ok(suite.name, case.name, "TODO unknown result")
+                else:
+                    tracker.add_ok(suite.name, case.name)
+
+        # Send back the stream
+        stream.seek(0)
+        response = HttpResponse(stream, content_type="application/yaml")
+        response["Content-Disposition"] = (
+            "attachment; filename=job_%d.yaml" % self.get_object().id
+        )
+        return response
+
+    def create(self, request, **kwargs):
+        serializer = serializers.TestJobSerializer(
+            data=request.data, context={"request": request}
+        )
+
+        serializer.is_valid(raise_exception=True)
+        definition = serializer.validated_data["definition"]
+
+        try:
+            job = testjob_submission(definition, self.request.user)
+        except SubmissionException as exc:
+            return Response(
+                {"message": "Problem with submitted job data: %s" % exc},
+                status=status.HTTP_400_BAD_REQUEST,
+            )
+        except (ValueError, KeyError, yaml.YAMLError) as exc:
+            return Response(
+                {"message": "job submission failed: %s." % exc},
+                status=status.HTTP_400_BAD_REQUEST,
+            )
+        except (Device.DoesNotExist, DeviceType.DoesNotExist):
+            return Response(
+                {"message": "Specified device or device type not found."},
+                status=status.HTTP_400_BAD_REQUEST,
+            )
+        except DevicesUnavailableException as exc:
+            return Response(
+                {"message": "Devices unavailable: %s" % exc},
+                status=status.HTTP_400_BAD_REQUEST,
+            )
+
+        if isinstance(job, list):
+            job_ids = [j.sub_id for j in job]
+        else:
+            job_ids = [job.id]
+
+        return Response(
+            {"message": "job(s) successfully submitted", "job_ids": job_ids},
+            status=status.HTTP_201_CREATED,
+        )
+
+    @action(detail=True, suffix="csv")
     def csv(self, request, **kwargs):
         limit = request.query_params.get("limit", None)
         offset = request.query_params.get("offset", None)
+        if limit is not None:
+            limit = int(limit)
+        if offset is not None:
+            offset = int(offset)
+
+        job = self.get_object()
+        testcases = TestCase.objects.filter(suite__job_id=job).order_by("id")[offset:][
+            :limit
+        ]
 
         output = io.StringIO()
         writer = csv.DictWriter(
@@ -135,38 +363,46 @@ class TestJobViewSet(base_views.TestJobViewSet):
             fieldnames=testcase_export_fields(),
         )
         writer.writeheader()
-        for test_suite in self.get_object().testsuite_set.all():
-            for row in test_suite.testcase_set.all():
-                writer.writerow(export_testcase(row))
+        for row in testcases:
+            writer.writerow(export_testcase(row))
 
         response = HttpResponse(output.getvalue(), content_type="application/csv")
-        response["Content-Disposition"] = (
-            "attachment; filename=job_%d.csv" % self.get_object().id
-        )
+        response["Content-Disposition"] = f"attachment; filename=job_{job.id}.csv"
         return response
 
-    @detail_route(methods=["get"], suffix="yaml")
+    @action(detail=True, suffix="yaml")
     def yaml(self, request, **kwargs):
         limit = request.query_params.get("limit", None)
         offset = request.query_params.get("offset", None)
+        if limit is not None:
+            limit = int(limit)
+        if offset is not None:
+            offset = int(offset)
+
+        job = self.get_object()
+        testcases = TestCase.objects.filter(suite__job_id=job).order_by("id")[offset:][
+            :limit
+        ]
 
         yaml_list = []
-        for test_suite in self.get_object().testsuite_set.all():
-            for test_case in test_suite.testcase_set.all():
-                yaml_list.append(export_testcase(test_case))
+        for test_case in testcases:
+            yaml_list.append(export_testcase(test_case))
 
-        response = HttpResponse(yaml_dump(yaml_list), content_type="application/yaml")
-        response["Content-Disposition"] = (
-            "attachment; filename=job_%d.yaml" % self.get_object().id
+        response = HttpResponse(
+            yaml_safe_dump(yaml_list), content_type="application/yaml"
         )
+        response["Content-Disposition"] = f"attachment; filename=job_{job.id}.yaml"
         return response
 
-    @detail_route(methods=["get"], suffix="metadata")
+    @action(detail=True, suffix="metadata")
     def metadata(self, request, **kwargs):
         return Response({"metadata": self.get_object().get_metadata_dict()})
 
     @action(
-        methods=["post"], detail=False, suffix="validate", permission_classes=[AllowAny]
+        methods=("post",),
+        detail=False,
+        suffix="validate",
+        permission_classes=(AllowAny,),
     )
     def validate(self, request, **kwargs):
         definition = request.data.get("definition", None)
@@ -188,10 +424,10 @@ class TestJobViewSet(base_views.TestJobViewSet):
             )
 
     @action(
-        methods=["post"],
+        methods=("post",),
         detail=False,
         suffix="validate-testdef",
-        permission_classes=[AllowAny],
+        permission_classes=(AllowAny,),
     )
     def validate_testdef(self, request, **kwargs):
         definition = request.data.get("definition", None)
@@ -210,7 +446,7 @@ class TestJobViewSet(base_views.TestJobViewSet):
                 status=status.HTTP_200_OK,
             )
 
-    @action(methods=["post"], detail=True, suffix="resubmit")
+    @action(methods=("post",), detail=True, suffix="resubmit")
     def resubmit(self, request, **kwargs):
         if self.get_object().is_multinode:
             definition = self.get_object().multinode_definition
@@ -250,7 +486,7 @@ class TestJobViewSet(base_views.TestJobViewSet):
             status=status.HTTP_201_CREATED,
         )
 
-    @detail_route(methods=["get"], suffix="cancel")
+    @action(detail=True, suffix="cancel")
     def cancel(self, request, **kwargs):
         # django-rest-framework will allow anyone to call this method.
         # Permissions on who can cancel the job are handled by LAVA internally.
@@ -264,6 +500,32 @@ class TestJobViewSet(base_views.TestJobViewSet):
             {"message": "Job cancel signal sent."}, status=status.HTTP_200_OK
         )
 
+    @action(methods=("post",), detail=False, suffix="batch-cancel")
+    def batch_cancel(self, request, **kwargs):
+        job_ids = request.data.get("job_ids", None)
+        job_ids_canceled = []
+        invalid_job_ids = []
+        for job_id in job_ids:
+            with transaction.atomic():
+                try:
+                    job = TestJob.get_restricted_job(
+                        job_id, request.user, for_update=True
+                    )
+                    job.cancel(request.user)
+                    job_ids_canceled.append(job_id)
+                except (PermissionDenied, Http404):
+                    # No view or cancel permissions
+                    # 404 is thrown by get_restricted_job when job is non existing
+                    invalid_job_ids.append(job_id)
+        return Response(
+            {
+                "message": "Job cancel signal sent for job ids.",
+                "job_ids": job_ids_canceled,
+                "invalid_job_ids": invalid_job_ids,
+            },
+            status=status.HTTP_200_OK,
+        )
+
 
 class TestSuiteViewSet(NestedViewSetMixin, viewsets.ReadOnlyModelViewSet):
     """
@@ -278,7 +540,17 @@ class TestSuiteViewSet(NestedViewSetMixin, viewsets.ReadOnlyModelViewSet):
     serializer_class = serializers.TestSuiteSerializer
     filterset_class = filters.TestSuiteFilter
 
-    @detail_route(methods=["get"], suffix="csv")
+    def get_queryset(self):
+        job_id = self.kwargs["parent_lookup_job_id"]
+
+        job = TestJob.objects.select_related("submitter").get(id=job_id)
+
+        if not job.can_view(self.request.user):
+            raise PermissionDenied
+
+        return super().get_queryset()
+
+    @action(detail=True, suffix="csv")
     def csv(self, request, **kwargs):
         limit = request.query_params.get("limit", None)
         offset = request.query_params.get("offset", None)
@@ -300,7 +572,7 @@ class TestSuiteViewSet(NestedViewSetMixin, viewsets.ReadOnlyModelViewSet):
         )
         return response
 
-    @detail_route(methods=["get"], suffix="yaml")
+    @action(detail=True, suffix="yaml")
     def yaml(self, request, **kwargs):
         limit = request.query_params.get("limit", None)
         offset = request.query_params.get("offset", None)
@@ -309,7 +581,9 @@ class TestSuiteViewSet(NestedViewSetMixin, viewsets.ReadOnlyModelViewSet):
         for test_case in get_testcases_with_limit(self.get_object(), limit, offset):
             yaml_list.append(export_testcase(test_case))
 
-        response = HttpResponse(yaml_dump(yaml_list), content_type="application/yaml")
+        response = HttpResponse(
+            yaml_safe_dump(yaml_list), content_type="application/yaml"
+        )
         response["Content-Disposition"] = (
             "attachment; filename=suite_%s.yaml" % self.get_object().name
         )
@@ -321,9 +595,49 @@ class TestCaseViewSet(NestedViewSetMixin, viewsets.ReadOnlyModelViewSet):
     serializer_class = serializers.TestCaseSerializer
     filterset_class = filters.TestCaseFilter
 
+    def get_queryset(self):
+        job_id = self.kwargs["parent_lookup_suite__job_id"]
 
-class DeviceTypeViewSet(base_views.DeviceTypeViewSet):
-    @detail_route(methods=["get", "post"], suffix="health-check")
+        job = TestJob.objects.select_related("submitter").get(id=job_id)
+
+        if not job.can_view(self.request.user):
+            raise PermissionDenied
+
+        return super().get_queryset()
+
+
+class DeviceTypeViewSet(viewsets.ModelViewSet):
+    queryset = DeviceType.objects
+    serializer_class = serializers.DeviceTypeSerializer
+    filterset_fields = (
+        "name",
+        "architecture",
+        "processor",
+        "cpu_model",
+        "aliases",
+        "bits",
+        "cores",
+        "core_count",
+        "description",
+        "health_frequency",
+        "disable_health_check",
+        "health_denominator",
+        "display",
+    )
+    filterset_class = filters.DeviceTypeFilter
+    lookup_value_regex = r"[^\0/]+"
+
+    def get_queryset(self):
+        return DeviceType.objects.visible_by_user(self.request.user).prefetch_related(
+            "cores", "aliases"
+        )
+
+    def get_permissions(self):
+        if self.action in ["create", "update", "destroy", "partial_update"]:
+            self.permission_classes = [IsSuperUser]
+        return super().get_permissions()
+
+    @action(detail=True, methods=("get", "post"), suffix="health-check")
     def health_check(self, request, **kwargs):
         if request.method == "GET":
             if not self.get_object().can_view(request.user):
@@ -372,7 +686,7 @@ class DeviceTypeViewSet(base_views.DeviceTypeViewSet):
                     "Unable to write health check configuration: %s" % exc.strerror
                 )
 
-    @detail_route(methods=["get", "post"], suffix="template")
+    @action(detail=True, methods=("get", "post"), suffix="template")
     def template(self, request, **kwargs):
         if request.method == "GET":
             if not self.get_object().can_view(request.user):
@@ -417,21 +731,65 @@ class DeviceTypeViewSet(base_views.DeviceTypeViewSet):
                 )
 
 
-class DeviceViewSet(base_views.DeviceViewSet, viewsets.ModelViewSet):
-    parser_classes = [JSONParser, FormParser, MultiPartParser, PlainTextParser]
+class DeviceViewSet(viewsets.ModelViewSet):
+    queryset = Device.objects
+    serializer_class = serializers.DeviceSerializer
+    filterset_fields = (
+        "hostname",
+        "device_type",
+        "device_version",
+        "physical_owner",
+        "physical_group",
+        "description",
+        "tags",
+        "state",
+        "health",
+        "worker_host",
+        "is_synced",
+    )
+    ordering_fields = (
+        "hostname",
+        "device_type",
+        "device_version",
+        "physical_owner",
+        "physical_group",
+        "description",
+        "tags",
+        "state",
+        "health",
+        "worker_host",
+        "is_synced",
+    )
+    filterset_class = filters.DeviceFilter
+    parser_classes = (JSONParser, FormParser, MultiPartParser)
 
-    lookup_value_regex = r"[\_\w0-9.-]+"
+    lookup_value_regex = r"[^\0/]+"
     serializer_class = serializers.DeviceSerializer
     filterset_class = filters.DeviceFilter
     permission_classes = [DjangoModelPermissionsOrAnonReadOnly]
 
+    def get_queryset(self):
+        query = (
+            Device.objects.select_related("physical_owner", "physical_group")
+            .prefetch_related("tags")
+            .visible_by_user(self.request.user)
+        )
+        if not self.request.query_params.get("all", False):
+            query = query.exclude(health=Device.HEALTH_RETIRED)
+        return query
+
+    def get_permissions(self):
+        if self.action in ["create", "update", "destroy", "partial_update"]:
+            self.permission_classes = [IsSuperUser]
+        return super().get_permissions()
+
     def get_serializer_class(self):
         if self.action == "dictionary":
             return serializers.DictionarySerializer
         else:
             return serializers.DeviceSerializer
 
-    @detail_route(methods=["post"], suffix="set_health")
+    @action(detail=True, methods=("post",), suffix="set_health")
     def set_health(self, request, **kwargs):
         if not request.user.has_perm("lava_scheduler_app.change_device"):
             raise PermissionDenied(
@@ -450,7 +808,7 @@ class DeviceViewSet(base_views.DeviceViewSet, viewsets.ModelViewSet):
             data = {"message": response.content}
             return Response(data, status=response.status_code)
 
-    @detail_route(methods=["get", "post"], suffix="dictionary")
+    @action(detail=True, methods=("get", "post"), suffix="dictionary")
     def dictionary(self, request, **kwargs):
         if request.method == "GET":
             job_ctx = None
@@ -505,7 +863,11 @@ class DeviceViewSet(base_views.DeviceViewSet, viewsets.ModelViewSet):
                 return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)
 
     @action(
-        methods=["post"], detail=False, suffix="validate", permission_classes=[AllowAny]
+        methods=("post",),
+        detail=False,
+        suffix="validate",
+        permission_classes=(DjangoModelPermissions,),
+        parser_classes=(PlainTextParser,),
     )
     def validate(self, request, **kwargs):
         """
@@ -517,9 +879,7 @@ class DeviceViewSet(base_views.DeviceViewSet, viewsets.ModelViewSet):
             raise ValidationError({"device": "Device dictionary is required."})
 
         try:
-            template = JinjaSandboxEnv(
-                loader=File("device").loader(), autoescape=False, trim_blocks=True
-            ).from_string(devicedict)
+            template = DEVICES_JINJA_ENV.from_string(devicedict)
             yaml_safe_load(template.render())
             return Response(
                 {"message": "Device dictionary valid."}, status=status.HTTP_200_OK
@@ -531,12 +891,19 @@ class DeviceViewSet(base_views.DeviceViewSet, viewsets.ModelViewSet):
             )
 
 
-class WorkerViewSet(base_views.WorkerViewSet, viewsets.ModelViewSet):
-    lookup_value_regex = r"[\_\w0-9.-]+"
+class WorkerViewSet(viewsets.ModelViewSet):
+    queryset = Worker.objects
+    serializer_class = serializers.WorkerSerializer
+    filterset_fields = "__all__"
+    ordering_fields = "__all__"
+    lookup_value_regex = r"[^\0/]+"
     serializer_class = serializers.WorkerSerializer
     filterset_class = filters.WorkerFilter
     permission_classes = [DjangoModelPermissionsOrAnonReadOnly]
 
+    def get_queryset(self):
+        return self.queryset.visible_by_user(self.request.user)
+
     def get_serializer_class(self):
         if self.action == "env":
             return serializers.EnvironmentSerializer
@@ -557,23 +924,20 @@ class WorkerViewSet(base_views.WorkerViewSet, viewsets.ModelViewSet):
         response["Content-Disposition"] = "attachment; filename=%s" % filename
         return response
 
-    def _set_file(self, request, path, content):
-        filename = path.name
+    def _set_file(self, request, kind, content):
         try:
-            path.parent.mkdir(mode=0o755, parents=True, exist_ok=True)
-            path.write_text(content, encoding="utf-8")
+            File(kind, self.get_object().hostname).write(content)
+
             return Response(
                 {"message": "content successfully updated"}, status=status.HTTP_200_OK
             )
         except OSError as e:
             raise ParseError(
-                "Error updating '%s' for worker %s: %s"
-                % (filename, self.get_object().hostname, str(e))
+                f"Error updating '{kind}' for worker {self.get_object().hostname}: {e}"
             )
 
-    @detail_route(methods=["get", "post"], suffix="env")
+    @action(detail=True, methods=("get", "post"), suffix="env")
     def env(self, request, **kwargs):
-        filename = "env.yaml"
         if request.method == "GET":
             return self._get_file(request, "env", "env.yaml")
 
@@ -582,18 +946,14 @@ class WorkerViewSet(base_views.WorkerViewSet, viewsets.ModelViewSet):
                 raise PermissionDenied(
                     "Insufficient permissions. Please contact system administrator."
                 )
-            path = (
-                pathlib.Path(settings.DISPATCHER_CONFIG_PATH)
-                / self.get_object().hostname
-                / filename
-            )
+
             serializer = serializers.EnvironmentSerializer(data=request.data)
             if serializer.is_valid():
-                return self._set_file(request, path, serializer.validated_data["env"])
+                return self._set_file(request, "env", serializer.validated_data["env"])
             else:
                 return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)
 
-    @detail_route(methods=["get", "post"], suffix="config")
+    @action(detail=True, methods=("get", "post"), suffix="config")
     def config(self, request, **kwargs):
         if request.method == "GET":
             return self._get_file(request, "dispatcher", "dispatcher.yaml")
@@ -603,19 +963,34 @@ class WorkerViewSet(base_views.WorkerViewSet, viewsets.ModelViewSet):
                 raise PermissionDenied(
                     "Insufficient permissions. Please contact system administrator."
                 )
-            path = (
-                pathlib.Path(settings.DISPATCHER_CONFIG_PATH)
-                / self.get_object().hostname
-                / "dispatcher.yaml"
-            )
+
             serializer = serializers.ConfigSerializer(data=request.data)
             if serializer.is_valid():
                 return self._set_file(
-                    request, path, serializer.validated_data["config"]
+                    request, "dispatcher", serializer.validated_data["config"]
                 )
             else:
                 return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)
 
+    @action(detail=True, methods=("post",), suffix="set_health")
+    def set_health(self, request, **kwargs):
+        if not request.user.has_perm("lava_scheduler_app.change_worker"):
+            raise PermissionDenied(
+                "Insufficient permissions. Please contact system administrator."
+            )
+        worker = self.get_object()
+        reason = request.data.get("reason", None)
+        health = request.data.get("health", None)
+        if health is not None:
+            health = health.title()
+        response = __set_worker_health__(worker, request.user, health, reason)
+        if response is None:
+            data = {"message": "OK"}
+            return Response(data, status=status.HTTP_202_ACCEPTED)
+        else:
+            data = {"message": response.content}
+            return Response(data, status=response.status_code)
+
 
 class AliasViewSet(viewsets.ModelViewSet):
     queryset = Alias.objects
@@ -650,7 +1025,7 @@ class GroupDeviceTypePermissionViewSet(viewsets.ModelViewSet):
     filterset_fields = "__all__"
     filterset_class = filters.GroupDeviceTypePermissionFilter
     ordering_fields = "__all__"
-    permission_classes = [base_views.IsSuperUser]
+    permission_classes = [IsSuperUser]
 
     def get_queryset(self):
         return self.queryset.all()
@@ -662,7 +1037,7 @@ class GroupDevicePermissionViewSet(viewsets.ModelViewSet):
     filterset_fields = "__all__"
     filterset_class = filters.GroupDevicePermissionFilter
     ordering_fields = "__all__"
-    permission_classes = [base_views.IsSuperUser]
+    permission_classes = [IsSuperUser]
 
     def get_queryset(self):
         return self.queryset.all()
@@ -696,7 +1071,7 @@ class SystemViewSet(viewsets.ViewSet):
     def list(self, request, **kwargs):
         return Response()
 
-    @action(detail=False, methods=["get"], suffix="master_config")
+    @action(detail=False, suffix="master_config")
     def master_config(self, request, **kwargs):
         """
         Description
@@ -725,7 +1100,7 @@ class SystemViewSet(viewsets.ViewSet):
         }
         return Response(data=ret_dict)
 
-    @action(detail=False, methods=["get"], suffix="version")
+    @action(detail=False, suffix="version")
     def version(self, request, **kwargs):
         """
         Description
@@ -744,7 +1119,7 @@ class SystemViewSet(viewsets.ViewSet):
         """
         return Response(data={"version": __version__})
 
-    @action(detail=False, methods=["get"], suffix="whoami")
+    @action(detail=False, suffix="whoami")
     def whoami(self, request, **kwargs):
         """
         Description
