@@ -1,21 +1,8 @@
-# -*- coding: utf-8 -*-
 # Copyright (C) 2015-2018 Linaro Limited
 #
 # Author: Stevan Radakovic <stevan.radakovic@linaro.org>
 #
-# This file is part of LAVA.
-#
-# LAVA is free software: you can redistribute it and/or modify
-# it under the terms of the GNU Affero General Public License version 3
-# as published by the Free Software Foundation
-#
-# LAVA is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU Affero General Public License
-# along with LAVA.  If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-2.0-or-later
 
 """
 Administration interface of the LAVA Results application.
@@ -43,6 +30,7 @@ class TestCaseAdmin(admin.ModelAdmin):
     list_select_related = ("suite", "suite__job")
     ordering = ("-suite__job__pk", "suite__name", "name")
     raw_id_fields = ("suite",)
+    show_full_result_count = False
 
     def job_pk(self, testcase):
         return testcase.suite.job.pk
@@ -60,6 +48,7 @@ class TestCaseAdmin(admin.ModelAdmin):
 class TestSetAdmin(admin.ModelAdmin):
     list_display = ("suite", "name")
     raw_id_fields = ("suite",)
+    show_full_result_count = False
 
     def has_add_permission(self, request):
         return False
@@ -73,6 +62,7 @@ class TestSuiteAdmin(admin.ModelAdmin):
     list_select_related = ("job",)
     ordering = ("-job__pk", "name")
     raw_id_fields = ("job",)
+    show_full_result_count = False
 
     def job_pk(self, testsuite):
         return testsuite.job.pk
