@@ -1,21 +1,8 @@
-# -*- coding: utf-8 -*-
 # Copyright (C) 2015-2018 Linaro Limited
 #
 # Author: Stevan Radakovic <stevan.radakovic@linaro.org>
 #
-# This file is part of LAVA.
-#
-# LAVA is free software: you can redistribute it and/or modify
-# it under the terms of the GNU Affero General Public License version 3
-# as published by the Free Software Foundation
-#
-# LAVA is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU Affero General Public License
-# along with LAVA.  If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-2.0-or-later
 
 import csv
 import io
@@ -23,7 +10,7 @@ import xmlrpc.client
 
 from django.core.exceptions import FieldDoesNotExist
 
-from lava_common.compat import yaml_dump
+from lava_common.yaml import yaml_safe_dump
 from lava_results_app.dbutils import export_testsuite, testsuite_export_fields
 from lava_results_app.models import (
     InvalidContentTypeError,
@@ -316,7 +303,7 @@ class ResultsAPI(ExposedAPI):
         except TestJob.DoesNotExist:
             raise xmlrpc.client.Fault(404, "Specified job not found.")
 
-        return yaml_dump(yaml_list)
+        return yaml_safe_dump(yaml_list)
 
     def get_testjob_metadata(self, job_id):
         """
@@ -498,7 +485,7 @@ class ResultsAPI(ExposedAPI):
         except TestJob.DoesNotExist:
             raise xmlrpc.client.Fault(404, "Specified job not found.")
 
-        return yaml_dump(yaml_list)
+        return yaml_safe_dump(yaml_list)
 
     def get_testsuite_results_yaml(self, job_id, suite_name, limit=None, offset=None):
         """
@@ -546,7 +533,7 @@ class ResultsAPI(ExposedAPI):
         except TestSuite.DoesNotExist:
             raise xmlrpc.client.Fault(404, "Specified test suite not found.")
 
-        return yaml_dump(yaml_list)
+        return yaml_safe_dump(yaml_list)
 
     def get_testsuite_results_csv(self, job_id, suite_name, limit=None, offset=None):
         """
@@ -693,7 +680,7 @@ class ResultsAPI(ExposedAPI):
         except TestCase.DoesNotExist:
             raise xmlrpc.client.Fault(404, "Specified test case not found.")
 
-        return yaml_dump(yaml_list)
+        return yaml_safe_dump(yaml_list)
 
     def get_testcase_results_csv(self, job_id, suite_name, case_name):
         """
