@@ -1,31 +1,17 @@
-# -*- coding: utf-8 -*-
 # Copyright (C) 2015-2018 Linaro Limited
 #
 # Author: Stevan Radakovic <stevan.radakovic@linaro.org>
 #
-# This file is part of LAVA.
-#
-# LAVA is free software: you can redistribute it and/or modify
-# it under the terms of the GNU Affero General Public License version 3
-# as published by the Free Software Foundation
-#
-# LAVA is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU Affero General Public License
-# along with LAVA.  If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-2.0-or-later
 
 
 import decimal
 import logging
 import os
-from collections import OrderedDict  # pylint: disable=unused-import
 from urllib.parse import quote
 
-from lava_common.compat import yaml_dump, yaml_load
 from lava_common.version import __version__
+from lava_common.yaml import yaml_safe_dump, yaml_safe_load
 from lava_results_app.models import TestCase, TestSet, TestSuite
 
 
@@ -74,8 +60,8 @@ def create_metadata_store(results, job):
     meta_filename = os.path.join(job.output_dir, "metadata", stub)
     os.makedirs(os.path.dirname(meta_filename), mode=0o755, exist_ok=True)
     if os.path.exists(meta_filename):
-        with open(meta_filename, "r") as existing_store:
-            data = yaml_load(existing_store)
+        with open(meta_filename) as existing_store:
+            data = yaml_safe_load(existing_store)
         if data is None:
             data = {}
         data.update(results["extra"])
@@ -83,7 +69,7 @@ def create_metadata_store(results, job):
         data = results["extra"]
     try:
         with open(meta_filename, "w") as extra_store:
-            yaml_dump(data, extra_store)
+            yaml_safe_dump(data, extra_store)
     except OSError as exc:  # LAVA-847
         msg = "[%d] Unable to create metadata store: %s" % (job.id, exc)
         logger.error(msg)
@@ -116,7 +102,7 @@ def map_scanned_results(results, job, starttc, endtc, meta_filename):
     if "extra" in results:
         results["extra"] = meta_filename
 
-    metadata = yaml_dump(results)
+    metadata = yaml_safe_dump(results)
     if len(metadata) > 4096:  # bug 2471 - test_length unit test
         msg = "[%d] Result metadata is too long. %s" % (job.id, metadata)
         logger.warning(msg)
@@ -129,9 +115,15 @@ def map_scanned_results(results, job, starttc, endtc, meta_filename):
         }
         if "error_type" in results:
             stripped_results["error_type"] = results["error_type"]
-        metadata = yaml_dump(stripped_results)
+        metadata = yaml_safe_dump(stripped_results)
         if len(metadata) > 4096:
             metadata = ""
+        # Try to keep partial error msg
+        elif "error_msg" in results:
+            stripped_results["error_msg"] = results["error_msg"][:256]
+            stripped_results_str = yaml_safe_dump(stripped_results)
+            if len(stripped_results_str) < 4096:
+                metadata = stripped_results_str
 
     suite, _ = TestSuite.objects.get_or_create(name=results["definition"], job=job)
     testset = _check_for_testset(results, suite)
