@@ -1,21 +1,8 @@
-# -*- coding: utf-8 -*-
 # Copyright (C) 2015-2019 Linaro Limited
 #
 # Author: Stevan Radakovic <stevan.radakovic@linaro.org>
 #
-# This file is part of LAVA.
-#
-# LAVA is free software: you can redistribute it and/or modify
-# it under the terms of the GNU Affero General Public License version 3
-# as published by the Free Software Foundation
-#
-# LAVA is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU Affero General Public License
-# along with LAVA.  If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-2.0-or-later
 
 """
 Database models of the LAVA Results
@@ -40,16 +27,17 @@ from django.contrib.contenttypes import fields
 from django.contrib.contenttypes.models import ContentType
 from django.core.validators import MaxValueValidator, MinValueValidator
 from django.db import connection, models, transaction
-from django.db.models import Case, IntegerField, Lookup, Q, Sum, When
+from django.db.models import Count, Lookup, Q
 from django.db.models.fields import Field
 from django.db.models.signals import pre_save
 from django.dispatch import receiver
 from django.urls import reverse
 from django.utils import timezone
 from django.utils.translation import gettext_lazy as _
+from psycopg2.extensions import quote_ident
 
-from lava_common.compat import yaml_load
 from lava_common.decorators import nottest
+from lava_common.yaml import yaml_safe_load
 from lava_results_app.utils import help_max_length
 from lava_scheduler_app.managers import (
     RestrictedTestCaseQuerySet,
@@ -111,46 +99,64 @@ class QueryMaterializedView(MaterializedView):
     class Meta:
         abstract = True
 
-    CREATE_VIEW = "CREATE MATERIALIZED VIEW %s%s AS %s;"
-    DROP_VIEW = "DROP MATERIALIZED VIEW IF EXISTS %s%s;"
-    REFRESH_VIEW = "REFRESH MATERIALIZED VIEW %s%s;"
-    VIEW_EXISTS = "SELECT EXISTS(SELECT * FROM pg_class WHERE relname='%s%s');"
     QUERY_VIEW_PREFIX = "query_"
 
     @classmethod
     def create(cls, query):
         # Check if view for this query exists.
-        cursor = connection.cursor()
-        if not cls.view_exists(query.id):  # create view
+        if cls.view_exists(query.id):
+            return
+
+        with connection.cursor() as cursor:
             sql, params = Query.get_queryset(
                 query.content_type, query.querycondition_set.all(), query.limit
             ).query.sql_with_params()
 
-            sql = sql.replace("%s", "'%s'")
-            query_str = sql % params
-
+            query_id_str = f"{cls.QUERY_VIEW_PREFIX}{query.id}"
+            # Must be quoted separately as otherwise
+            # will be quoted with single quotes making
+            # it incompatible with table_name identifier
+            query_id_str_quoted = quote_ident(query_id_str, cursor.cursor)
             # TODO: handle potential exceptions here. what to do if query
             # view is not created? - new field update_status?
-            query_str = cls.CREATE_VIEW % (cls.QUERY_VIEW_PREFIX, query.id, query_str)
-            cursor.execute(query_str)
+            cursor.execute(
+                f"CREATE MATERIALIZED VIEW {query_id_str_quoted} AS {sql}",
+                params,
+            )
 
     @classmethod
     def refresh(cls, query_id):
-        refresh_sql = cls.REFRESH_VIEW % (cls.QUERY_VIEW_PREFIX, query_id)
-        cursor = connection.cursor()
-        cursor.execute(refresh_sql)
+        with connection.cursor() as cursor:
+            query_id_str = f"{cls.QUERY_VIEW_PREFIX}{query_id}"
+            # Must be quoted separately as otherwise
+            # will be quoted with single quotes making
+            # it incompatible with table_name identifier
+            query_id_str_quoted = quote_ident(query_id_str, cursor.cursor)
+            cursor.execute(
+                f"REFRESH MATERIALIZED VIEW {query_id_str_quoted}",
+            )
 
     @classmethod
     def drop(cls, query_id):
-        drop_sql = cls.DROP_VIEW % (cls.QUERY_VIEW_PREFIX, query_id)
-        cursor = connection.cursor()
-        cursor.execute(drop_sql)
+        with connection.cursor() as cursor:
+            query_id_str = f"{cls.QUERY_VIEW_PREFIX}{query_id}"
+            # Must be quoted separately as otherwise
+            # will be quoted with single quotes making
+            # it incompatible with table_name identifier
+            query_id_str_quoted = quote_ident(query_id_str, cursor.cursor)
+            cursor.execute(
+                f"DROP MATERIALIZED VIEW IF EXISTS {query_id_str_quoted}",
+            )
 
     @classmethod
     def view_exists(cls, query_id):
-        cursor = connection.cursor()
-        cursor.execute(cls.VIEW_EXISTS % (cls.QUERY_VIEW_PREFIX, query_id))
-        return cursor.fetchone()[0]
+        with connection.cursor() as cursor:
+            query_id_str = f"{cls.QUERY_VIEW_PREFIX}{query_id}"
+            cursor.execute(
+                "SELECT EXISTS(SELECT 1 FROM pg_class WHERE relname=%s)",
+                (query_id_str,),
+            )
+            return cursor.fetchone()[0]
 
     def get_queryset(self):
         return QueryMaterializedView.objects.all()
@@ -164,9 +170,20 @@ class TestSuite(models.Model, Queryable):
     Directly linked to a single TestJob, the job can have multiple TestSets.
     """
 
+    class Meta:
+        constraints = (
+            models.UniqueConstraint(
+                fields=("job", "name"), name="lava_results_app_testsuite_job_name_uniq"
+            ),
+        )
+
     objects = models.Manager.from_queryset(RestrictedTestSuiteQuerySet)()
 
-    job = models.ForeignKey(TestJob, on_delete=models.CASCADE)
+    job = models.ForeignKey(
+        TestJob,
+        on_delete=models.CASCADE,
+        db_index=False,  # UniqueConstraint defined in Meta
+    )
     name = models.CharField(
         verbose_name="Suite name", blank=True, null=True, default=None, max_length=200
     )
@@ -175,33 +192,21 @@ class TestSuite(models.Model, Queryable):
         if not hasattr(self, "_testcase_count"):
             res = self.testcase_set.values("result")
             res = res.aggregate(
-                PASS=Sum(
-                    Case(
-                        When(result=TestCase.RESULT_PASS, then=1),
-                        default=0,
-                        output_field=IntegerField(),
-                    )
+                PASS=Count(
+                    "pk",
+                    filter=Q(result=TestCase.RESULT_PASS),
                 ),
-                FAIL=Sum(
-                    Case(
-                        When(result=TestCase.RESULT_FAIL, then=1),
-                        default=0,
-                        output_field=IntegerField(),
-                    )
+                FAIL=Count(
+                    "pk",
+                    filter=Q(result=TestCase.RESULT_FAIL),
                 ),
-                SKIP=Sum(
-                    Case(
-                        When(result=TestCase.RESULT_SKIP, then=1),
-                        default=0,
-                        output_field=IntegerField(),
-                    )
+                SKIP=Count(
+                    "pk",
+                    filter=Q(result=TestCase.RESULT_SKIP),
                 ),
-                UNKNOWN=Sum(
-                    Case(
-                        When(result=TestCase.RESULT_UNKNOWN, then=1),
-                        default=0,
-                        output_field=IntegerField(),
-                    )
+                UNKNOWN=Count(
+                    "pk",
+                    filter=Q(result=TestCase.RESULT_UNKNOWN),
                 ),
             )
             self._testcase_count = {k.lower(): (v or 0) for (k, v) in res.items()}
@@ -277,6 +282,14 @@ class TestSet(models.Model):
     Not all cases have a TestSet
     """
 
+    class Meta:
+        constraints = (
+            models.UniqueConstraint(
+                fields=("suite", "name"),
+                name="lava_results_app_testset_name_suite_uniq",
+            ),
+        )
+
     id = models.AutoField(primary_key=True)
 
     name = models.CharField(
@@ -284,7 +297,10 @@ class TestSet(models.Model):
     )
 
     suite = models.ForeignKey(
-        TestSuite, related_name="test_sets", on_delete=models.CASCADE
+        TestSuite,
+        related_name="test_sets",
+        on_delete=models.CASCADE,
+        db_index=False,  # UniqueConstraint defined in Meta
     )
 
     def get_absolute_url(self):
@@ -305,6 +321,15 @@ class TestCase(models.Model, Queryable):
     lava-test-case or action result
     """
 
+    class Meta:
+        constraints = (
+            models.UniqueConstraint(
+                fields=("suite",),
+                condition=models.Q(name="job"),
+                name="lava_results_app_testcase_job_uniq",
+            ),
+        )
+
     objects = models.Manager.from_queryset(RestrictedTestCaseQuerySet)()
 
     RESULT_PASS = 0
@@ -399,7 +424,7 @@ class TestCase(models.Model, Queryable):
         if not self.metadata:
             return None
         try:
-            ret = yaml_load(self.metadata)
+            ret = yaml_safe_load(self.metadata)
         except yaml.YAMLError:
             return None
         return ret
@@ -523,7 +548,6 @@ class TestData(models.Model):
 
 
 class QueryGroup(models.Model):
-
     name = models.SlugField(max_length=1024, unique=True)
 
     def __str__(self):
@@ -532,7 +556,6 @@ class QueryGroup(models.Model):
 
 def TestJobViewFactory(query):
     class TestJobMaterializedView(QueryMaterializedView, TestJob):
-
         objects = models.Manager.from_queryset(RestrictedTestJobQuerySet)()
 
         class Meta(QueryMaterializedView.Meta):
@@ -543,7 +566,6 @@ def TestJobViewFactory(query):
 
 def TestCaseViewFactory(query):
     class TestCaseMaterializedView(QueryMaterializedView, TestCase):
-
         objects = models.Manager.from_queryset(RestrictedTestCaseQuerySet)()
 
         class Meta(QueryMaterializedView.Meta):
@@ -554,7 +576,6 @@ def TestCaseViewFactory(query):
 
 def TestSuiteViewFactory(query):
     class TestSuiteMaterializedView(QueryMaterializedView, TestSuite):
-
         objects = models.Manager.from_queryset(RestrictedTestSuiteQuerySet)()
 
         class Meta(QueryMaterializedView.Meta):
@@ -564,8 +585,11 @@ def TestSuiteViewFactory(query):
 
 
 class Query(models.Model):
-
-    owner = models.ForeignKey(User, on_delete=models.CASCADE)
+    owner = models.ForeignKey(
+        User,
+        on_delete=models.CASCADE,
+        db_index=False,  # UniqueConstraint defined in Meta
+    )
 
     group = models.ForeignKey(
         Group, default=None, null=True, blank=True, on_delete=models.SET_NULL
@@ -604,7 +628,12 @@ class Query(models.Model):
         return "~%s/%s" % (self.owner.username, self.name)
 
     class Meta:
-        unique_together = ("owner", "name")
+        constraints = (
+            models.UniqueConstraint(
+                fields=("owner", "name"),
+                name="lava_results_app_query_owner_name_uniq",
+            ),
+        )
         verbose_name = "query"
         verbose_name_plural = "queries"
 
@@ -690,7 +719,6 @@ class Query(models.Model):
         filters = {}
 
         for condition in conditions:
-
             try:
                 relation_string = QueryCondition.RELATION_MAP[
                     content_type.model_class()
@@ -705,11 +733,9 @@ class Query(models.Model):
             if condition.table.model_class() == NamedTestAttribute:
                 # For custom attributes, need two filters since
                 # we're comparing the key(name) and the value.
-                filter_key_name = "{0}__name".format(relation_string)
-                filter_key_value = "{0}__value".format(relation_string)
-                filter_key_value = "{0}__{1}".format(
-                    filter_key_value, condition.operator
-                )
+                filter_key_name = f"{relation_string}__name"
+                filter_key_value = f"{relation_string}__value"
+                filter_key_value = "{}__{}".format(filter_key_value, condition.operator)
 
                 filters[filter_key_name] = condition.field
                 filters[filter_key_value] = condition.value
@@ -718,7 +744,7 @@ class Query(models.Model):
                 if condition.table == content_type:
                     filter_key = condition.field
                 else:
-                    filter_key = "{0}__{1}".format(relation_string, condition.field)
+                    filter_key = f"{relation_string}__{condition.field}"
                 # Handle conditions through relations.
                 fk_model = _get_foreign_key_model(
                     condition.table.model_class(), condition.field
@@ -727,21 +753,21 @@ class Query(models.Model):
                 # have 'name' as the default search field.
                 if fk_model:
                     if fk_model == User:
-                        filter_key = "{0}__username".format(filter_key)
+                        filter_key = f"{filter_key}__username"
                     elif fk_model == Device:
-                        filter_key = "{0}__hostname".format(filter_key)
+                        filter_key = f"{filter_key}__hostname"
                     else:
-                        filter_key = "{0}__name".format(filter_key)
+                        filter_key = f"{filter_key}__name"
 
                 # Handle conditions with choice fields.
                 condition_field_obj = condition.table.model_class()._meta.get_field(
                     condition.field
                 )
                 if condition_field_obj.choices:
-                    choices_reverse = dict(
-                        (value, key)
+                    choices_reverse = {
+                        value: key
                         for key, value in dict(condition_field_obj.choices).items()
-                    )
+                    }
                     try:
                         condition.value = choices_reverse[condition.value]
                     except KeyError:
@@ -759,7 +785,7 @@ class Query(models.Model):
                         condition.value = True
 
                 # Add operator.
-                filter_key = "{0}__{1}".format(filter_key, condition.operator)
+                filter_key = f"{filter_key}__{condition.operator}"
 
                 filters[filter_key] = condition.value
 
@@ -780,7 +806,6 @@ class Query(models.Model):
         return query_results
 
     def refresh_view(self):
-
         if self.is_live:
             raise RefreshLiveQueryError("Refreshing live query not permitted.")
 
@@ -835,7 +860,6 @@ class Query(models.Model):
                 condition.operator = condition_fields[1]
                 condition.value = condition_fields[2]
             elif len(condition_fields) == 4:
-
                 try:
                     content_type = Query.get_content_type(condition_fields[0])
                 except ContentType.DoesNotExist:
@@ -953,7 +977,6 @@ def limit_update_signal(sender, instance, **kwargs):
 
 
 class QueryCondition(models.Model):
-
     table = models.ForeignKey(
         ContentType,
         limit_choices_to=Q(model__in=["testsuite", "testjob", "namedtestattribute"])
@@ -1125,9 +1148,7 @@ class QueryCondition(models.Model):
                 cls.LT,
             ]
 
-        operators = dict(
-            [(i, operator_dict[i]) for i in operator_keys if i in operator_dict]
-        )
+        operators = {i: operator_dict[i] for i in operator_keys if i in operator_dict}
 
         return operators
 
@@ -1141,24 +1162,31 @@ def _get_foreign_key_model(model, fieldname):
         and direct
         and isinstance(field_object, models.ForeignKey)
     ):
-        return field_object.rel.to
+        return field_object.remote_field.model
     return None
 
 
 class QueryOmitResult(models.Model):
-
-    query = models.ForeignKey(Query, on_delete=models.CASCADE)
+    query = models.ForeignKey(
+        Query,
+        on_delete=models.CASCADE,
+        db_index=False,  # UniqueConstraint defined in Meta
+    )
 
     content_type = models.ForeignKey(ContentType, on_delete=models.CASCADE)
     object_id = models.PositiveIntegerField()
     content_object = fields.GenericForeignKey("content_type", "object_id")
 
     class Meta:
-        unique_together = ("object_id", "query", "content_type")
+        constraints = (
+            models.UniqueConstraint(
+                fields=("query", "content_type", "object_id"),
+                name="lava_results_app_query_omit_result_content_type_object_id_uniq",
+            ),
+        )
 
 
 class ChartGroup(models.Model):
-
     name = models.SlugField(max_length=1024, unique=True)
 
     def __str__(self):
@@ -1182,7 +1210,6 @@ CHART_VISIBILITY = (
 
 
 class Chart(models.Model):
-
     name = models.SlugField(max_length=1024, unique=True)
 
     chart_group = models.ForeignKey(
@@ -1368,7 +1395,6 @@ class ChartQuery(models.Model):
         return data
 
     def get_user_chart_data(self, user):
-
         data = {}
         try:
             chart_user = ChartQueryUser.objects.get(chart_query=self, user=user)
@@ -1383,10 +1409,8 @@ class ChartQuery(models.Model):
         return data
 
     def get_chart_passfail_data(self, user, query_results):
-
         data = []
         for item in query_results:
-
             # Set attribute based on xaxis_attribute.
             attribute = item.get_xaxis_attribute(self.xaxis_attribute)
             # If xaxis attribute is set and this query item does not have
@@ -1399,7 +1423,6 @@ class ChartQuery(models.Model):
 
             passfail_results = item.get_passfail_results()
             for result in passfail_results:
-
                 if result:
                     chart_item = {
                         "id": result,
@@ -1424,10 +1447,8 @@ class ChartQuery(models.Model):
         return data
 
     def get_chart_measurement_data(self, user, query_results):
-
         data = []
         for item in query_results:
-
             # Set attribute based on xaxis_attribute.
             attribute = item.get_xaxis_attribute(self.xaxis_attribute)
             # If xaxis attribute is set and this query item does not have
@@ -1440,7 +1461,6 @@ class ChartQuery(models.Model):
 
             measurement_results = item.get_measurement_results()
             for result in measurement_results:
-
                 if result:
                     chart_item = {
                         "id": result,
@@ -1456,10 +1476,8 @@ class ChartQuery(models.Model):
         return data
 
     def get_chart_attributes_data(self, user, query_results):
-
         data = []
         for item in query_results:
-
             attribute_results = item.get_attribute_results(self.attributes)
             for result in attribute_results:
                 if result:
@@ -1485,9 +1503,19 @@ class ChartQuery(models.Model):
 
 class ChartQueryUser(models.Model):
     class Meta:
-        unique_together = ("chart_query", "user")
+        constraints = (
+            models.UniqueConstraint(
+                fields=("chart_query", "user"),
+                name="lava_results_app_chart_query_user_uniq",
+            ),
+        )
 
-    chart_query = models.ForeignKey(ChartQuery, null=False, on_delete=models.CASCADE)
+    chart_query = models.ForeignKey(
+        ChartQuery,
+        null=False,
+        on_delete=models.CASCADE,
+        db_index=False,  # UniqueConstraint defined in Meta
+    )
 
     user = models.ForeignKey(User, null=False, on_delete=models.CASCADE)
 
