@@ -1,26 +1,13 @@
-# -*- coding: utf-8 -*-
 # Copyright (C) 2015-2018 Linaro Limited
 #
 # Author: Stevan Radakovic <stevan.radakovic@linaro.org>
 #
-# This file is part of LAVA.
-#
-# LAVA is free software: you can redistribute it and/or modify
-# it under the terms of the GNU Affero General Public License version 3
-# as published by the Free Software Foundation
-#
-# LAVA is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU Affero General Public License
-# along with LAVA.  If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-2.0-or-later
 
 import contextlib
 import datetime
+from json import dumps as json_dumps
 
-import simplejson
 from django import forms
 from django.conf import settings
 from django.db import models
@@ -93,9 +80,7 @@ class QueryConditionForm(forms.ModelForm):
         if "query" in self.initial and self.initial["query"].__class__ == Query:
             condition_choices = QueryCondition.get_condition_choices()
 
-            self.fields["condition_choices"].initial = simplejson.dumps(
-                condition_choices
-            )
+            self.fields["condition_choices"].initial = json_dumps(condition_choices)
 
     def save(self, commit=True, **kwargs):
         return super().save(commit=commit, **kwargs)
