@@ -1,28 +1,14 @@
-# -*- coding: utf-8 -*-
 # Copyright (C) 2015-2018 Linaro Limited
 #
 # Author: Stevan Radakovic <stevan.radakovic@linaro.org>
 #
-# This file is part of LAVA.
-#
-# LAVA is free software: you can redistribute it and/or modify
-# it under the terms of the GNU Affero General Public License version 3
-# as published by the Free Software Foundation
-#
-# LAVA is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU Affero General Public License
-# along with LAVA.  If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-2.0-or-later
 
 import csv
 import os
 import shutil
 import tempfile
 
-import simplejson
 from django.contrib import messages
 from django.contrib.auth.decorators import login_required
 from django.contrib.auth.models import Group
@@ -31,10 +17,11 @@ from django.core.exceptions import FieldDoesNotExist, FieldError, PermissionDeni
 from django.db import IntegrityError
 from django.db.models import Q
 from django.db.utils import ProgrammingError
-from django.http import Http404, HttpResponse, HttpResponseRedirect
-from django.shortcuts import get_object_or_404, loader
+from django.http import Http404, HttpResponse, HttpResponseRedirect, JsonResponse
+from django.shortcuts import get_object_or_404, render
 from django.template import defaultfilters
 from django.urls import reverse
+from django.views.decorators.http import require_POST
 from django_tables2 import RequestConfig
 
 from lava_results_app.models import (
@@ -54,6 +41,7 @@ from lava_results_app.views.query.forms import QueryConditionForm, QueryForm
 from lava_results_app.views.query.tables import (
     GroupQueryTable,
     OtherQueryTable,
+    QueryConditionsTable,
     QueryTestCaseTable,
     QueryTestJobTable,
     QueryTestSuiteTable,
@@ -138,9 +126,7 @@ class QueryResultView(LavaView):
 
 @BreadCrumb("Queries", parent=index)
 def query_list(request):
-
     group_tables = {}
-    terms_data = search_data = discrete_data = {}
     for group in QueryGroup.objects.all():
         if group.query_set.count():
             prefix = "group_%s_" % group.id
@@ -150,9 +136,6 @@ def query_list(request):
             table = GroupQueryTable(
                 group_view.get_table_data(prefix), request=request, prefix=prefix
             )
-            search_data.update(table.prepare_search_data(group_view))
-            discrete_data.update(table.prepare_discrete_data(group_view))
-            terms_data.update(table.prepare_terms_data(group_view))
             group_tables[group.name] = table
             config = RequestConfig(request, paginate={"per_page": table.length})
             config.configure(table)
@@ -164,9 +147,6 @@ def query_list(request):
     )
     config = RequestConfig(request, paginate={"per_page": other_query_table.length})
     config.configure(other_query_table)
-    search_data.update(other_query_table.prepare_search_data(other_view))
-    discrete_data.update(other_query_table.prepare_discrete_data(other_view))
-    terms_data.update(other_query_table.prepare_terms_data(other_view))
 
     prefix = "user_"
     view = UserQueryView(request, model=Query, table_class=UserQueryTable)
@@ -175,31 +155,22 @@ def query_list(request):
     )
     config = RequestConfig(request, paginate={"per_page": user_query_table.length})
     config.configure(user_query_table)
-    search_data.update(user_query_table.prepare_search_data(view))
-    discrete_data.update(user_query_table.prepare_discrete_data(view))
-    terms_data.update(user_query_table.prepare_terms_data(view))
 
-    template = loader.get_template("lava_results_app/query_list.html")
-    return HttpResponse(
-        template.render(
-            {
-                "user_query_table": user_query_table,
-                "other_query_table": other_query_table,
-                "search_data": search_data,
-                "discrete_data": discrete_data,
-                "terms_data": terms_data,
-                "group_tables": group_tables,
-                "bread_crumb_trail": BreadCrumbTrail.leading_to(query_list),
-                "context_help": ["lava-queries-charts"],
-            },
-            request=request,
-        )
+    return render(
+        request,
+        "lava_results_app/query_list.html",
+        {
+            "user_query_table": user_query_table,
+            "other_query_table": other_query_table,
+            "group_tables": group_tables,
+            "bread_crumb_trail": BreadCrumbTrail.leading_to(query_list),
+            "context_help": ["lava-queries-charts"],
+        },
     )
 
 
 @BreadCrumb("Query ~{username}/{name}", parent=query_list, needs=["username", "name"])
 def query_display(request, username, name):
-
     query = get_object_or_404(Query, owner__username=username, name=name)
 
     if not request.user.is_superuser:
@@ -229,34 +200,26 @@ def query_display(request, username, name):
     omitted = [
         result.content_object for result in QueryOmitResult.objects.filter(query=query)
     ]
-    template = loader.get_template("lava_results_app/query_display.html")
-    return HttpResponse(
-        template.render(
-            {
-                "query": query,
-                "entity": query.content_type.model,
-                "conditions": Query.serialize_conditions(
-                    query.querycondition_set.all()
-                ),
-                "omitted": omitted,
-                "query_table": table,
-                "terms_data": table.prepare_terms_data(view),
-                "search_data": table.prepare_search_data(view),
-                "discrete_data": table.prepare_discrete_data(view),
-                "bread_crumb_trail": BreadCrumbTrail.leading_to(
-                    query_display, username=username, name=name
-                ),
-                "context_help": ["lava-queries-charts"],
-            },
-            request=request,
-        )
+    return render(
+        request,
+        "lava_results_app/query_display.html",
+        {
+            "query": query,
+            "entity": query.content_type.model,
+            "conditions": Query.serialize_conditions(query.querycondition_set.all()),
+            "omitted": omitted,
+            "query_table": table,
+            "bread_crumb_trail": BreadCrumbTrail.leading_to(
+                query_display, username=username, name=name
+            ),
+            "context_help": ["lava-queries-charts"],
+        },
     )
 
 
 @BreadCrumb("Custom Query", parent=query_list)
 @login_required
 def query_custom(request):
-
     try:
         content_type = Query.get_content_type(request.GET.get("entity"))
     except InvalidContentTypeError as e:
@@ -293,52 +256,49 @@ def query_custom(request):
 
     config = RequestConfig(request, paginate={"per_page": table.length})
     config.configure(table)
-    template = loader.get_template("lava_results_app/query_custom.html")
-    return HttpResponse(
-        template.render(
-            {
-                "query_table": table,
-                "conditions": conditions,
-                "terms_data": table.prepare_terms_data(view),
-                "search_data": table.prepare_search_data(view),
-                "discrete_data": table.prepare_discrete_data(view),
-                "bread_crumb_trail": BreadCrumbTrail.leading_to(query_custom),
-                "context_help": ["lava-queries-charts"],
-            },
-            request=request,
-        )
+    return render(
+        request,
+        "lava_results_app/query_custom.html",
+        {
+            "query_table": table,
+            "conditions": conditions,
+            "bread_crumb_trail": BreadCrumbTrail.leading_to(query_custom),
+            "context_help": ["lava-queries-charts"],
+        },
     )
 
 
 @BreadCrumb("Query ~{username}/{name}", parent=query_list, needs=["username", "name"])
 @login_required
 def query_detail(request, username, name):
-
     query = get_object_or_404(Query, owner__username=username, name=name)
     query_conditions = Query.serialize_conditions(query.querycondition_set.all())
-    template = loader.get_template("lava_results_app/query_detail.html")
-    return HttpResponse(
-        template.render(
-            {
-                "query": query,
-                "query_conditions": query_conditions,
-                "bread_crumb_trail": BreadCrumbTrail.leading_to(
-                    query_detail, username=username, name=name
-                ),
-                "context_help": ["lava-queries-charts"],
-                "condition_form": QueryConditionForm(
-                    instance=None, initial={"query": query, "table": query.content_type}
-                ),
-            },
-            request=request,
-        )
+    query_conditions_table = QueryConditionsTable(
+        query.querycondition_set.select_related("table").all(), request=request
+    )
+    config = RequestConfig(request)
+    config.configure(query_conditions_table)
+    return render(
+        request,
+        "lava_results_app/query_detail.html",
+        {
+            "query": query,
+            "query_conditions": query_conditions,
+            "query_conditions_table": query_conditions_table,
+            "bread_crumb_trail": BreadCrumbTrail.leading_to(
+                query_detail, username=username, name=name
+            ),
+            "context_help": ["lava-queries-charts"],
+            "condition_form": QueryConditionForm(
+                instance=None, initial={"query": query, "table": query.content_type}
+            ),
+        },
     )
 
 
 @BreadCrumb("Add new", parent=query_list)
 @login_required
 def query_add(request):
-
     query = Query()
     query.owner = request.user
 
@@ -352,7 +312,6 @@ def query_add(request):
 @login_required
 @ownership_required
 def query_edit(request, username, name):
-
     query = get_object_or_404(Query, owner__username=username, name=name)
 
     return query_form(
@@ -365,7 +324,6 @@ def query_edit(request, username, name):
 @login_required
 @ownership_required
 def query_delete(request, username, name):
-
     query = get_object_or_404(Query, owner__username=username, name=name)
     query.is_archived = True
     query.save()
@@ -424,13 +382,14 @@ def query_export_custom(request):
 @login_required
 @ownership_required
 def query_toggle_published(request, username, name):
-
     query = get_object_or_404(Query, owner__username=username, name=name)
 
     query.is_published = not query.is_published
     query.save()
 
-    return HttpResponseRedirect(query.get_absolute_url() + "/+detail")
+    return HttpResponseRedirect(
+        reverse("lava.results.query_detail", args=(username, name))
+    )
 
 
 @BreadCrumb("Copy", parent=query_detail, needs=["username", "name"])
@@ -451,7 +410,6 @@ def query_copy(request, username, name):
 @login_required
 @ownership_required
 def query_refresh(request, name, username):
-
     query = get_object_or_404(Query, owner__username=username, name=name)
 
     success = True
@@ -466,25 +424,23 @@ def query_refresh(request, name, username):
         error_msg = "%s<br>Please contact system administrator." % str(e)
 
     last_updated = defaultfilters.date(query.last_updated, "DATETIME_FORMAT")
-    return HttpResponse(
-        simplejson.dumps([success, str(last_updated), error_msg]),
-        content_type="application/json",
+    return JsonResponse(
+        [success, str(last_updated), error_msg],
+        safe=False,
     )
 
 
 @login_required
 def query_group_list(request):
-
     term = request.GET["term"]
     groups = [
         str(group.name) for group in QueryGroup.objects.filter(name__istartswith=term)
     ]
-    return HttpResponse(simplejson.dumps(groups), content_type="application/json")
+    return JsonResponse(groups, safe=False)
 
 
 @login_required
 def query_add_group(request, username, name):
-
     if request.method != "POST":
         raise PermissionDenied
 
@@ -504,13 +460,14 @@ def query_add_group(request, username, name):
         if not old_group.query_set.count():
             old_group.delete()
 
-    return HttpResponseRedirect(query.get_absolute_url() + "/+detail")
+    return HttpResponseRedirect(
+        reverse("lava.results.query_detail", args=(username, name))
+    )
 
 
 @login_required
 @ownership_required
 def query_select_group(request, username, name):
-
     if request.method != "POST":
         raise PermissionDenied
 
@@ -528,23 +485,23 @@ def query_select_group(request, username, name):
 
     query.save()
 
-    return HttpResponseRedirect(query.get_absolute_url() + "/+detail")
+    return HttpResponseRedirect(
+        reverse("lava.results.query_detail", args=(username, name))
+    )
 
 
 @login_required
 def get_query_group_names(request):
-
     term = request.GET["term"]
     groups = []
     for group in Group.objects.filter(user=request.user, name__istartswith=term):
         groups.append({"id": group.id, "name": group.name, "label": group.name})
-    return HttpResponse(simplejson.dumps(groups), content_type="application/json")
+    return JsonResponse(groups, safe=False)
 
 
 @login_required
 @ownership_required
 def query_add_condition(request, username, name):
-
     query = get_object_or_404(Query, owner__username=username, name=name)
 
     return query_condition_form(request, query, BreadCrumbTrail.leading_to(query_add))
@@ -553,7 +510,6 @@ def query_add_condition(request, username, name):
 @login_required
 @ownership_required
 def query_edit_condition(request, username, name, id):
-
     query = get_object_or_404(Query, owner__username=username, name=name)
     query_condition = get_object_or_404(QueryCondition, id=id)
 
@@ -562,21 +518,22 @@ def query_edit_condition(request, username, name, id):
     )
 
 
+@require_POST
 @login_required
 @ownership_required
 def query_remove_condition(request, username, name, id):
-
-    query = get_object_or_404(Query, owner__username=username, name=name)
+    get_object_or_404(Query, owner__username=username, name=name)
     query_condition = get_object_or_404(QueryCondition, id=id)
     query_condition.delete()
 
-    return HttpResponseRedirect(query.get_absolute_url() + "/+detail")
+    return HttpResponseRedirect(
+        reverse("lava.results.query_detail", args=(username, name))
+    )
 
 
 @login_required
 @ownership_required
 def query_omit_result(request, username, name, id):
-
     query = get_object_or_404(Query, owner__username=username, name=name)
     result_object = get_object_or_404(query.content_type.model_class(), id=id)
 
@@ -592,7 +549,6 @@ def query_omit_result(request, username, name, id):
 @login_required
 @ownership_required
 def query_include_result(request, username, name, id):
-
     query = get_object_or_404(Query, owner__username=username, name=name)
     result_object = get_object_or_404(query.content_type.model_class(), id=id)
 
@@ -608,7 +564,6 @@ def query_include_result(request, username, name, id):
 
 
 def get_query_names(request):
-
     term = request.GET["term"]
     result = []
 
@@ -629,13 +584,11 @@ def get_query_names(request):
                 "content_type": query.content_type.model_class().__name__,
             }
         )
-    return HttpResponse(simplejson.dumps(list(result)), content_type="application/json")
+    return JsonResponse(result, safe=False)
 
 
 def query_form(request, bread_crumb_trail, instance=None, is_copy=False):
-
     if request.method == "POST":
-
         form = QueryForm(request.user, request.POST, instance=instance, is_copy=is_copy)
         if form.is_valid():
             query = form.save()
@@ -647,7 +600,12 @@ def query_form(request, bread_crumb_trail, instance=None, is_copy=False):
                     condition.query = query
                     condition.save()
 
-            return HttpResponseRedirect(query.get_absolute_url() + "/+detail")
+            return HttpResponseRedirect(
+                reverse(
+                    "lava.results.query_detail",
+                    args=(request.user.username, query.name),
+                )
+            )
 
     else:
         form = QueryForm(request.user, instance=instance, is_copy=is_copy)
@@ -657,23 +615,21 @@ def query_form(request, bread_crumb_trail, instance=None, is_copy=False):
     if is_copy:
         query_name = instance.name
         instance.name = None
-    template = loader.get_template("lava_results_app/query_form.html")
-    return HttpResponse(
-        template.render(
-            {
-                "bread_crumb_trail": bread_crumb_trail,
-                "is_copy": is_copy,
-                "query_name": query_name,
-                "form": form,
-                "context_help": ["lava-queries-charts"],
-            },
-            request=request,
-        )
+
+    return render(
+        request,
+        "lava_results_app/query_form.html",
+        {
+            "bread_crumb_trail": bread_crumb_trail,
+            "is_copy": is_copy,
+            "query_name": query_name,
+            "form": form,
+            "context_help": ["lava-queries-charts"],
+        },
     )
 
 
 def query_condition_form(request, query, bread_crumb_trail, instance=None):
-
     form = QueryConditionForm(request.POST, instance=instance)
 
     if form.is_valid():
@@ -684,9 +640,7 @@ def query_condition_form(request, query, bread_crumb_trail, instance=None):
             content_type="application/json",
         )
     else:
-        return HttpResponse(
-            simplejson.dumps(["fail", form.errors]), content_type="application/json"
-        )
+        return JsonResponse(["fail", form.errors], safe=False)
 
 
 def _remove_dir(path):
@@ -701,7 +655,6 @@ def _remove_dir(path):
 
 
 def _export_query(query_results, content_type, filename):
-
     tmp_dir = tempfile.mkdtemp()
     file_path = os.path.join(tmp_dir, "%s.csv" % filename)
 
@@ -711,6 +664,7 @@ def _export_query(query_results, content_type, filename):
     # Remove non-relevant columns for CSV file.
     removed_fields = [
         # TestJob fields:
+        "token",
         "user_id",
         "actual_device_id",
         "definition",
@@ -726,11 +680,11 @@ def _export_query(query_results, content_type, filename):
         "notification",
         "testjobmaterializedview",
         "testjobuser",
+        # TODO: Remove once pipeline_compatibility gets removed
         "pipeline_compatibility",
         # TestSuite fields:
         "job_id",
         # TestCase fields:
-        "actionlevels",
         "suite_id",
         "test_set_id",
     ]
@@ -759,7 +713,7 @@ def _export_query(query_results, content_type, filename):
 
     response = HttpResponse(content_type="text/csv")
     response["Content-Disposition"] = "attachment; filename=%s.csv" % filename
-    with open(file_path, "r") as csv_file:
+    with open(file_path) as csv_file:
         response.write(csv_file.read())
 
     _remove_dir(tmp_dir)
