@@ -1,22 +1,9 @@
-# -*- coding: utf-8 -*-
 # Copyright (C) 2017-2018 Linaro Limited
 #
 # Author: Neil Williams <neil.williams@linaro.org>
 #         Senthil Kumaran <senthil.kumaran@linaro.org>
 #
-# This file is part of LAVA.
-#
-# LAVA is free software: you can redistribute it and/or modify
-# it under the terms of the GNU Affero General Public License version 3
-# as published by the Free Software Foundation
-#
-# LAVA is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU Affero General Public License
-# along with LAVA.  If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-2.0-or-later
 from __future__ import annotations
 
 import os
@@ -29,7 +16,8 @@ from django.core.exceptions import PermissionDenied
 from django.db import transaction
 from django.db.models import Count
 
-from lava_common.compat import yaml_safe_dump, yaml_safe_load
+from lava_common.yaml import yaml_safe_dump, yaml_safe_load
+from lava_results_app.models import TestCase
 from lava_scheduler_app.dbutils import (
     active_device_types,
     device_type_summary,
@@ -64,7 +52,7 @@ def check_perm(perm):
             if not self.user.has_perm(perm):
                 raise xmlrpc.client.Fault(
                     403,
-                    "User '%s' is missing permission %s ." % (self.user.username, perm),
+                    "User '%s' is missing permission %s." % (self.user.username, perm),
                 )
             return f(self, *args, **kwargs)
 
@@ -660,7 +648,7 @@ class SchedulerAPI(ExposedV2API):
                 raise xmlrpc.client.Fault(404, "Device '%s' was not found." % hostname)
             if device.can_change(self.user):
                 device.health = Device.HEALTH_MAINTENANCE
-                device.save()
+                device.save(update_fields=["health"])
             else:
                 raise xmlrpc.client.Fault(
                     403,
@@ -707,7 +695,7 @@ class SchedulerAPI(ExposedV2API):
                 raise xmlrpc.client.Fault(404, "Device '%s' was not found." % hostname)
             if device.can_change(self.user):
                 device.health = Device.HEALTH_UNKNOWN
-                device.save()
+                device.save(update_fields=["health"])
             else:
                 raise xmlrpc.client.Fault(
                     403,
@@ -792,7 +780,7 @@ class SchedulerAPI(ExposedV2API):
         _state, submitter_id, is_pipeline, id, failure_comment,
         multinode_definition, priority, _actual_device_cache,
         original_definition, status, health_check, description,
-        start_time, target_group, pipeline_compatibility, submit_time,
+        start_time, target_group, submit_time,
         is_public, _old_status, actual_device_id, definition, sub_id,
         requested_device_type_id, end_time, absolute_url, submitter_username
         """
@@ -806,6 +794,21 @@ class SchedulerAPI(ExposedV2API):
                 403, "Job '%s' not available to user '%s'." % (job_id, self.user)
             )
 
+        metadata = None
+        try:
+            job_case = TestCase.objects.get(
+                suite__job=job, suite__name="lava", name="job"
+            )
+        except TestCase.DoesNotExist:
+            job_case = None
+        if job_case:
+            metadata = job_case.action_metadata
+        if not metadata:
+            # if job_case exists but metadata is still None due to job error
+            metadata = {}
+
+        job.error_msg = metadata.get("error_msg")
+        job.error_type = metadata.get("error_type")
         job.status = job.get_legacy_status_display()
         job.state = job.get_state_display()
         job.health = job.get_health_display()
